@@ -40,6 +40,7 @@ class AlbumCoverLoader : public QObject {
   static QString ImageCacheDir();
 
   void SetDesiredHeight(int height) { height_ = height; }
+  void SetScaleOutputImage(bool scale) { scale_ = scale; }
   void SetPadOutputImage(bool padding) { padding_ = padding; }
   void SetDefaultOutputImage(const QImage& image);
 
@@ -53,13 +54,12 @@ class AlbumCoverLoader : public QObject {
 
   void Clear();
 
-  static QPixmap TryLoadPixmap(const QString& automatic, const QString& manual);
-
-  static const char* kManuallyUnsetCover;
-  static const char* kEmbeddedCover;
+  static QPixmap TryLoadPixmap(const QString& automatic, const QString& manual,
+                               const QString& filename = QString());
 
  signals:
   void ImageLoaded(quint64 id, const QImage& image);
+  void ImageLoaded(quint64 id, const QImage& scaled, const QImage& original);
 
  protected slots:
   void ProcessTasks();
@@ -95,11 +95,11 @@ class AlbumCoverLoader : public QObject {
   void NextState(Task* task);
   TryLoadResult TryLoadImage(const Task& task);
   QImage ScaleAndPad(const QImage& image) const;
-  QImage LoadFromTaglib(const QString& filename) const;
 
   bool stop_requested_;
 
   int height_;
+  bool scale_;
   bool padding_;
   QImage default_;
 
