@@ -31,7 +31,7 @@
 #include <QVariant>
 
 const char* Database::kDatabaseFilename = "clementine.db";
-const int Database::kSchemaVersion = 23;
+const int Database::kSchemaVersion = 28;
 const char* Database::kMagicAllSongsTables = "%allsongstables";
 
 int Database::sNextConnectionId = 1;
@@ -326,7 +326,8 @@ Database::Database(QObject* parent, const QString& database_name)
   : QObject(parent),
     mutex_(QMutex::Recursive),
     injected_database_name_(database_name),
-    query_hash_(0)
+    query_hash_(0),
+    startup_schema_version_(-1)
 {
   {
     QMutexLocker l(&sNextConnectionIdMutex);
@@ -425,21 +426,8 @@ QSqlDatabase Database::Connect() {
     }
   }
 
-  // Get the database's schema version
-  QSqlQuery q("SELECT version FROM schema_version", db);
-  int schema_version = 0;
-  if (q.next())
-    schema_version = q.value(0).toInt();
-
-  if (schema_version > kSchemaVersion) {
-    qWarning() << "The database schema (version" << schema_version << ") is newer than I was expecting";
-    return db;
-  }
-  if (schema_version < kSchemaVersion) {
-    // Update the schema
-    for (int v=schema_version+1 ; v<= kSchemaVersion ; ++v) {
-      UpdateDatabaseSchema(v, db);
-    }
+  if(startup_schema_version_ == -1) {
+    UpdateMainSchema(&db);
   }
 
   // We might have to initialise the schema in some attached databases now, if
@@ -458,6 +446,27 @@ QSqlDatabase Database::Connect() {
   return db;
 }
 
+void Database::UpdateMainSchema(QSqlDatabase* db) {
+  // Get the database's schema version
+  QSqlQuery q("SELECT version FROM schema_version", *db);
+  int schema_version = 0;
+  if (q.next())
+    schema_version = q.value(0).toInt();
+
+  startup_schema_version_ = schema_version;
+
+  if (schema_version > kSchemaVersion) {
+    qWarning() << "The database schema (version" << schema_version << ") is newer than I was expecting";
+    return;
+  }
+  if (schema_version < kSchemaVersion) {
+    // Update the schema
+    for (int v=schema_version+1 ; v<= kSchemaVersion ; ++v) {
+      UpdateDatabaseSchema(v, *db);
+    }
+  }
+}
+
 void Database::RecreateAttachedDb(const QString& database_name) {
   if (!attached_databases_.contains(database_name)) {
     qWarning() << "Attached database does not exist:" << database_name;
@@ -513,22 +522,27 @@ void Database::ExecFromFile(const QString &filename, QSqlDatabase &db) {
 void Database::ExecCommands(const QString &schema, QSqlDatabase &db) {
   // Run each command
   QStringList commands(schema.split(";\n\n"));
+
+  // We don't want this list to reflect possible DB schema changes
+  // so we initialize it before executing any statements.
+  QStringList tables = SongsTables(db);
+
   foreach (const QString& command, commands) {
     // There are now lots of "songs" tables that need to have the same schema:
     // songs, magnatune_songs, and device_*_songs.  We allow a magic value
     // in the schema files to update all songs tables at once.
     if (command.contains(kMagicAllSongsTables)) {
-      foreach (const QString& table, SongsTables(db)) {
+      foreach (const QString& table, tables) {
         qDebug() << "Updating" << table << "for" << kMagicAllSongsTables;
         QString new_command(command);
         new_command.replace(kMagicAllSongsTables, table);
         QSqlQuery query(db.exec(new_command));
-        if (CheckErrors(query.lastError()))
+        if (CheckErrors(query))
           qFatal("Unable to update music library database");
       }
     } else {
       QSqlQuery query(db.exec(command));
-      if (CheckErrors(query.lastError()))
+      if (CheckErrors(query))
         qFatal("Unable to update music library database");
     }
   }
@@ -536,18 +550,38 @@ void Database::ExecCommands(const QString &schema, QSqlDatabase &db) {
 
 QStringList Database::SongsTables(QSqlDatabase& db) const {
   QStringList ret;
+
+  // look for the tables in the main db
   foreach (const QString& table, db.tables()) {
     if (table == "songs" || table.endsWith("_songs"))
       ret << table;
   }
+
+  // look for the tables in attached dbs
+  foreach (const QString& key, attached_databases_.keys()) {
+    QSqlQuery q(QString("SELECT NAME FROM %1.sqlite_master"
+                        " WHERE type='table' AND name='songs' OR name LIKE '%songs'").arg(key), db);
+    if (q.exec()) {
+      while(q.next()) {
+        QString tab_name = key + "." + q.value(0).toString();
+        ret << tab_name;
+      }
+    }
+  }
+
   return ret;
 }
 
-bool Database::CheckErrors(const QSqlError& error) {
-  if (error.isValid()) {
-    qDebug() << error;
-    emit Error("LibraryBackend: " + error.text());
+bool Database::CheckErrors(const QSqlQuery& query) {
+  QSqlError last_error = query.lastError();
+  if (last_error.isValid()) {
+    qDebug() << "db error: " << last_error;
+    qDebug() << "faulty query: " << query.lastQuery();
+    qDebug() << "bound values: " << query.boundValues();
+
+    emit Error("LibraryBackend: " + last_error.text());
     return true;
   }
+
   return false;
 }
