@@ -18,6 +18,10 @@
 #include "globalshortcutbackend.h"
 #include "globalshortcuts.h"
 
+#include <QFuture>
+#include <QFutureWatcher>
+#include <QtConcurrentRun>
+
 GlobalShortcutBackend::GlobalShortcutBackend(GlobalShortcuts *parent)
   : QObject(parent),
     manager_(parent),
@@ -36,8 +40,3 @@ void GlobalShortcutBackend::Unregister() {
   DoUnregister();
   active_ = false;
 }
-
-void GlobalShortcutBackend::Reregister() {
-  Unregister();
-  Register();
-}
