@@ -18,6 +18,10 @@
 #include "gnomeglobalshortcutbackend.h"
 #include "globalshortcuts.h"
 
+#ifdef QT_DBUS_LIB
+#  include "dbus/gnomesettingsdaemon.h"
+#endif
+
 #include <QAction>
 #include <QtDebug>
 
@@ -36,22 +40,27 @@ GnomeGlobalShortcutBackend::GnomeGlobalShortcutBackend(GlobalShortcuts* parent)
 }
 
 bool GnomeGlobalShortcutBackend::DoRegister() {
-  qDebug() << __PRETTY_FUNCTION__;
 #ifdef QT_DBUS_LIB
+  qDebug() << __PRETTY_FUNCTION__ << "- starting";
   // Check if the GSD service is available
-  if (!QDBusConnection::sessionBus().interface()->isServiceRegistered(kGsdService))
+  if (!QDBusConnection::sessionBus().interface()->isServiceRegistered(kGsdService)) {
+    qDebug() << __PRETTY_FUNCTION__ << "- gnome settings daemon not registered";
     return false;
+  }
 
   if (!interface_) {
-    interface_ = new QDBusInterface(
-        kGsdService, kGsdPath, kGsdInterface, QDBusConnection::sessionBus(), this);
+    interface_ = new OrgGnomeSettingsDaemonMediaKeysInterface(
+        kGsdService, kGsdPath, QDBusConnection::sessionBus(), this);
   }
 
   connect(interface_, SIGNAL(MediaPlayerKeyPressed(QString,QString)),
           this, SLOT(GnomeMediaKeyPressed(QString,QString)));
 
+  qDebug() << __PRETTY_FUNCTION__ << "- complete";
+
   return true;
 #else // QT_DBUS_LIB
+  qDebug() << __PRETTY_FUNCTION__ << "- dbus not available";
   return false;
 #endif
 }
