@@ -29,20 +29,25 @@
 #import <Foundation/NSTimer.h>
 #import <Foundation/NSURL.h>
 #import <AppKit/NSNibDeclarations.h>
-#import <Sparkle/SUUpdater.h>
 
 #import <Kernel/AvailabilityMacros.h>
 
 #include "config.h"
 #include "globalshortcuts.h"
+#include "mac_delegate.h"
 #include "mac_startup.h"
 #include "macglobalshortcutbackend.h"
 #include "utilities.h"
 
+#ifdef HAVE_SPARKLE
+#import <Sparkle/SUUpdater.h>
+#endif
+
 #include <QCoreApplication>
 #include <QDir>
 #include <QEvent>
 #include <QFile>
+#include <QSettings>
 
 #include <QtDebug>
 
@@ -63,22 +68,6 @@
 - (void) mediaKeyEvent: (int)key state: (BOOL)state repeat: (BOOL)repeat;
 @end
 
-#ifdef SNOW_LEOPARD
-@interface AppDelegate :NSObject <NSApplicationDelegate> {
-#else
-@interface AppDelegate :NSObject {
-#endif
-  PlatformInterface* application_handler_;
-  NSMenu* dock_menu_;
-}
-
-- (id) initWithHandler: (PlatformInterface*)handler;
-// NSApplicationDelegate
-- (BOOL) applicationShouldHandleReopen: (NSApplication*)app hasVisibleWindows:(BOOL)flag;
-- (NSMenu*) applicationDockMenu: (NSApplication*)sender;
-- (void) setDockMenu: (NSMenu*)menu;
-@end
-
 @implementation AppDelegate
 
 - (id) init {
@@ -177,8 +166,10 @@ void MacMain() {
   [[NSAutoreleasePool alloc] init];
   // Creates and sets the magic global variable so QApplication will find it.
   [MacApplication sharedApplication];
-  // Creates and sets the magic global variable for Sparkle.
-  [[SUUpdater sharedUpdater] setDelegate: NSApp];
+  #ifdef HAVE_SPARKLE
+    // Creates and sets the magic global variable for Sparkle.
+    [[SUUpdater sharedUpdater] setDelegate: NSApp];
+  #endif
 }
 
 void SetShortcutHandler(MacGlobalShortcutBackend* handler) {
@@ -190,7 +181,9 @@ void SetApplicationHandler(PlatformInterface* handler) {
 }
 
 void CheckForUpdates() {
+  #ifdef HAVE_SPARKLE
   [[SUUpdater sharedUpdater] checkForUpdates: NSApp];
+  #endif
 }
 
 QString GetBundlePath() {
@@ -226,30 +219,58 @@ QString GetApplicationSupportPath() {
   return ret;
 }
 
+QString GetMusicDirectory() {
+  NSAutoreleasePool* pool = [NSAutoreleasePool alloc];
+  [pool init];
+  NSArray* paths = NSSearchPathForDirectoriesInDomains(
+      NSMusicDirectory,
+      NSUserDomainMask,
+      YES);
+  QString ret;
+  if ([paths count] > 0) {
+    NSString* user_path = [paths objectAtIndex:0];
+    ret = QString::fromUtf8([user_path UTF8String]);
+  } else {
+    ret = "~/Music";
+  }
+  [pool drain];
+  return ret;
+}
+
 bool MigrateLegacyConfigFiles() {
+  bool moved_dir = false;
   QString old_config_dir = QString("%1/.config/%2").arg(
       QDir::homePath(), QCoreApplication::organizationName());
-  if (!QFile::exists(old_config_dir)) {
-    return false;
-  }
-  QString new_config_dir = Utilities::GetConfigPath(Utilities::Path_Root);
-  // Create ~/Library/Application Support which should already exist anyway.
-  QDir::root().mkpath(GetApplicationSupportPath());
+  if (QFile::exists(old_config_dir)) {
+    QString new_config_dir = Utilities::GetConfigPath(Utilities::Path_Root);
+    // Create ~/Library/Application Support which should already exist anyway.
+    QDir::root().mkpath(GetApplicationSupportPath());
 
-  qDebug() << "Move from:" << old_config_dir
-           << "to:" << new_config_dir;
+    qDebug() << "Move from:" << old_config_dir
+             << "to:" << new_config_dir;
 
-  NSFileManager* file_manager = [[NSFileManager alloc] init];
-  NSError* error;
-  bool ret = [file_manager moveItemAtPath:
-      [NSString stringWithUTF8String: old_config_dir.toUtf8().constData()]
-      toPath:[NSString stringWithUTF8String: new_config_dir.toUtf8().constData()]
-      error: &error];
-  if (!ret) {
-    qWarning() << [[error localizedDescription] UTF8String];
-    return false;
+    NSFileManager* file_manager = [[NSFileManager alloc] init];
+    NSError* error;
+    bool ret = [file_manager moveItemAtPath:
+        [NSString stringWithUTF8String: old_config_dir.toUtf8().constData()]
+        toPath:[NSString stringWithUTF8String: new_config_dir.toUtf8().constData()]
+        error: &error];
+    if (!ret) {
+      qWarning() << [[error localizedDescription] UTF8String];
+    }
+    moved_dir = true;
   }
-  return true;
+
+  QString old_config_path = QDir::homePath() + "/Library/Preferences/com.davidsansome.Clementine.plist";
+  if (QFile::exists(old_config_path)) {
+    QSettings settings;
+    bool ret = QFile::rename(old_config_path, settings.fileName());
+    if (ret) {
+      qWarning() << "Migrated old config file: " << old_config_path << "to: " << settings.fileName();
+    }
+  }
+
+  return moved_dir;
 }
 
 }  // namespace mac
