@@ -18,30 +18,14 @@
 #include "config.h"
 #include "player.h"
 #include "engines/enginebase.h"
+#include "engines/gstengine.h"
+#include "library/librarybackend.h"
 #include "playlist/playlist.h"
 #include "playlist/playlistitem.h"
 #include "playlist/playlistmanager.h"
-#include "radio/lastfmservice.h"
-#include "mpris_common.h"
 
-#ifdef HAVE_GSTREAMER
-#  include "engines/gstengine.h"
-#endif
-#ifdef HAVE_LIBVLC
-#  include "engines/vlcengine.h"
-#endif
-#ifdef HAVE_LIBXINE
-#  include "engines/xine-engine.h"
-#endif
-#ifdef HAVE_QT_PHONON
-#  include "engines/phononengine.h"
-#endif
-
-#ifdef Q_WS_X11
-#  include "mpris.h"
-#  include "mpris2.h"
-#  include <QDBusConnection>
-#  include <QImage>
+#ifdef HAVE_LIBLASTFM
+#  include "radio/lastfmservice.h"
 #endif
 
 #include <QtDebug>
@@ -52,77 +36,29 @@
 using boost::shared_ptr;
 
 
-Player::Player(MainWindow* main_window, PlaylistManager* playlists,
-               LastFMService* lastfm, Engine::Type engine, QObject* parent)
-  : QObject(parent),
-    art_loader_(new mpris::ArtLoader(this)),
-    mpris1_(NULL),
-    mpris2_(NULL),
+Player::Player(PlaylistManagerInterface* playlists, LastFMService* lastfm,
+               QObject* parent)
+  : PlayerInterface(parent),
     playlists_(playlists),
     lastfm_(lastfm),
-    engine_(CreateEngine(engine)),
+    engine_(new GstEngine),
     stream_change_type_(Engine::First),
     last_state_(Engine::Empty),
     volume_before_mute_(50)
 {
-  // Loads album art and saves it to a file in /tmp for MPRIS clients and those
-  // objects which need cover art's filepath, not the image itself
-  connect(playlists, SIGNAL(CurrentSongChanged(Song)),
-          art_loader_, SLOT(LoadArt(Song)));
-
-#ifdef Q_WS_X11
-  // MPRIS DBus interface.
-  qDBusRegisterMetaType<QImage>();
-  qDBusRegisterMetaType<TrackMetadata>();
-  qDBusRegisterMetaType<TrackIds>();
-
-  //MPRIS 1.0 implementation
-  mpris1_ = new mpris::Mpris1(this, art_loader_, this);
-
-  //MPRIS 2.0 implementation
-  mpris2_ = new mpris::Mpris2(main_window, this, art_loader_, mpris1_, this);
-#endif
-
   settings_.beginGroup("Player");
 
   SetVolume(settings_.value("volume", 50).toInt());
 
   connect(engine_.get(), SIGNAL(Error(QString)), SIGNAL(Error(QString)));
+
+  connect(engine_.get(), SIGNAL(ValidSongRequested(QUrl)), SLOT(ValidSongRequested(QUrl)));
+  connect(engine_.get(), SIGNAL(InvalidSongRequested(QUrl)), SLOT(InvalidSongRequested(QUrl)));
 }
 
 Player::~Player() {
 }
 
-EngineBase* Player::CreateEngine(Engine::Type engine) {
-  switch(engine) {
-#ifdef HAVE_GSTREAMER
-    case Engine::Type_GStreamer:
-      return new GstEngine();
-      break;
-#endif
-#ifdef HAVE_LIBVLC
-    case Engine::Type_VLC:
-      return new VlcEngine();
-      break;
-#endif
-#ifdef HAVE_LIBXINE
-    case Engine::Type_Xine:
-      return new XineEngine();
-      break;
-#endif
-#ifdef HAVE_QT_PHONON
-    case Engine::Type_QtPhonon:
-      return new PhononEngine();
-      break;
-#endif
-    default:
-      qFatal("Selected engine not compiled in");
-      break;
-  }
-  /* NOT REACHED */
-  return NULL;
-}
-
 void Player::Init() {
   if (!engine_->Init())
     qFatal("Error initialising audio engine");
@@ -149,7 +85,7 @@ void Player::HandleSpecialLoad(const PlaylistItem::SpecialLoadResult &result) {
 
   case PlaylistItem::SpecialLoadResult::TrackAvailable: {
     // Might've been an async load, so check we're still on the same item
-    int current_index = playlists_->active()->current_index();
+    int current_index = playlists_->active()->current_row();
     if (current_index == -1)
       return;
 
@@ -157,7 +93,10 @@ void Player::HandleSpecialLoad(const PlaylistItem::SpecialLoadResult &result) {
     if (!item || item->Url() != result.original_url_)
       return;
 
-    engine_->Play(result.media_url_, stream_change_type_);
+    engine_->Play(result.media_url_, stream_change_type_,
+                  item->Metadata().has_cue(),
+                  item->Metadata().beginning_nanosec(),
+                  item->Metadata().end_nanosec());
 
     current_item_ = item;
     loading_async_ = QUrl();
@@ -175,7 +114,7 @@ void Player::Next() {
   NextInternal(Engine::Manual);
 }
 
-void Player::NextInternal(Engine::TrackChangeType change) {
+void Player::NextInternal(Engine::TrackChangeFlags change) {
   if (playlists_->active()->stop_after_current()) {
     playlists_->active()->StopAfter(-1);
     Stop();
@@ -196,10 +135,10 @@ void Player::NextInternal(Engine::TrackChangeType change) {
   NextItem(change);
 }
 
-void Player::NextItem(Engine::TrackChangeType change) {
-  int i = playlists_->active()->next_index();
+void Player::NextItem(Engine::TrackChangeFlags change) {
+  int i = playlists_->active()->next_row();
   if (i == -1) {
-    playlists_->active()->set_current_index(i);
+    playlists_->active()->set_current_row(i);
     emit PlaylistFinished();
     Stop();
     return;
@@ -215,6 +154,14 @@ void Player::TrackEnded() {
     return;
   }
 
+  if (current_item_ && current_item_->IsLocalLibraryItem() &&
+      current_item_->Metadata().id() != -1 && !playlists_->active()->has_scrobbled()) {
+    // The track finished before its scrobble point (30 seconds), so increment
+    // the play count now.
+    playlists_->library_backend()->IncrementPlayCountAsync(
+        current_item_->Metadata().id());
+  }
+
   NextInternal(Engine::Auto);
 }
 
@@ -229,7 +176,7 @@ void Player::PlayPause() {
     // Stopping seems like a reasonable thing to do (especially on mac where there
     // is no media key for stop).
     if (current_item_->options() & PlaylistItem::PauseDisabled) {
-      engine_->Stop();
+      Stop();
     } else {
       engine_->Pause();
     }
@@ -242,8 +189,8 @@ void Player::PlayPause() {
     if (playlists_->active()->rowCount() == 0)
       break;
 
-             int i = playlists_->active()->current_index();
-    if (i == -1) i = playlists_->active()->last_played_index();
+             int i = playlists_->active()->current_row();
+    if (i == -1) i = playlists_->active()->last_played_row();
     if (i == -1) i = 0;
 
     PlayAt(i, Engine::First, true);
@@ -253,17 +200,14 @@ void Player::PlayPause() {
 }
 
 void Player::Stop() {
-  if (engine_->position() != engine_->length()) {
-    emit TrackSkipped(current_item_);
-  }
   engine_->Stop();
-  playlists_->active()->set_current_index(-1);
+  playlists_->active()->set_current_row(-1);
   current_item_.reset();
 }
 
 void Player::Previous() {
-  int i = playlists_->active()->previous_index();
-  playlists_->active()->set_current_index(i);
+  int i = playlists_->active()->previous_row();
+  playlists_->active()->set_current_row(i);
   if (i == -1) {
     Stop();
     return;
@@ -299,14 +243,19 @@ int Player::GetVolume() const {
   return engine_->volume();
 }
 
-void Player::PlayAt(int index, Engine::TrackChangeType change, bool reshuffle) {
-  if (change == Engine::Manual && engine_->position() != engine_->length()) {
+void Player::PlayAt(int index, Engine::TrackChangeFlags change, bool reshuffle) {
+  if (change == Engine::Manual && engine_->position_nanosec() != engine_->length_nanosec()) {
     emit TrackSkipped(current_item_);
   }
 
+  if (current_item_ && current_item_->Metadata().IsOnSameAlbum(
+        playlists_->active()->item_at(index)->Metadata())) {
+    change |= Engine::SameAlbum;
+  }
+
   if (reshuffle)
-    playlists_->active()->set_current_index(-1);
-  playlists_->active()->set_current_index(index);
+    playlists_->active()->set_current_row(-1);
+  playlists_->active()->set_current_row(index);
 
   current_item_ = playlists_->active()->current_item();
 
@@ -320,23 +269,45 @@ void Player::PlayAt(int index, Engine::TrackChangeType change, bool reshuffle) {
   }
   else {
     loading_async_ = QUrl();
-    engine_->Play(current_item_->Url(), change);
+    engine_->Play(current_item_->Url(), change,
+                  current_item_->Metadata().has_cue(),
+                  current_item_->Metadata().beginning_nanosec(),
+                  current_item_->Metadata().end_nanosec());
 
+#ifdef HAVE_LIBLASTFM
     if (lastfm_->IsScrobblingEnabled())
       lastfm_->NowPlaying(current_item_->Metadata());
+#endif
   }
 }
 
 void Player::CurrentMetadataChanged(const Song& metadata) {
+  // those things might have changed (especially when a previously invalid
+  // song was reloaded) so we push the latest version into Engine
+  engine_->RefreshMarkers(metadata.beginning_nanosec(), metadata.end_nanosec());
+
+#ifdef HAVE_LIBLASTFM
   lastfm_->NowPlaying(metadata);
+#endif
 }
 
-void Player::Seek(int seconds) {
-  int msec = qBound(0, seconds * 1000, int(engine_->length()));
-  engine_->Seek(msec);
+void Player::SeekTo(int seconds) {
+  qint64 nanosec = qBound(0ll, qint64(seconds) * kNsecPerSec,
+                          engine_->length_nanosec());
+  engine_->Seek(nanosec);
 
   // If we seek the track we don't want to submit it to last.fm
   playlists_->active()->set_scrobbled(true);
+
+  emit Seeked(nanosec / 1000);
+}
+
+void Player::SeekForward() {
+  SeekTo(engine()->position_nanosec() / kNsecPerSec + 5);
+}
+
+void Player::SeekBackward() {
+  SeekTo(engine()->position_nanosec() / kNsecPerSec - 5);
 }
 
 void Player::EngineMetadataReceived(const Engine::SimpleMetaBundle& bundle) {
@@ -403,7 +374,7 @@ void Player::Pause() {
 void Player::Play() {
   switch (GetState()) {
     case Engine::Playing:
-      Seek(0);
+      SeekTo(0);
       break;
     case Engine::Paused:
       engine_->Unpause();
@@ -420,48 +391,76 @@ void Player::ShowOSD() {
 }
 
 void Player::TrackAboutToEnd() {
+  const bool current_contains_multiple_tracks =
+      current_item_->options() & PlaylistItem::ContainsMultipleTracks;
+  const bool has_next_row = playlists_->active()->next_row() != -1;
+  PlaylistItemPtr next_item;
+
+  if (has_next_row) {
+    next_item = playlists_->active()->item_at(playlists_->active()->next_row());
+  }
+
   if (engine_->is_autocrossfade_enabled()) {
     // Crossfade is on, so just start playing the next track.  The current one
     // will fade out, and the new one will fade in
 
     // But, if there's no next track and we don't want to fade out, then do
     // nothing and just let the track finish to completion.
-    if (!engine_->is_fadeout_enabled() &&
-        playlists_->active()->next_index() == -1)
+    if (!engine_->is_fadeout_enabled() && !has_next_row)
       return;
 
-    NextInternal(Engine::Auto);
-  } else {
-    // Crossfade is off, so start preloading the next track so we don't get a
-    // gap between songs.
-    if (current_item_->options() & PlaylistItem::ContainsMultipleTracks)
+    // If the next track is on the same album (or same cue file), and the
+    // user doesn't want to crossfade between tracks on the same album, then
+    // don't do this automatic crossfading.
+    if (engine_->crossfade_same_album() ||
+        current_contains_multiple_tracks ||
+        !has_next_row ||
+        !next_item ||
+        !current_item_->Metadata().IsOnSameAlbum(next_item->Metadata())) {
+      TrackEnded();
       return;
-    if (playlists_->active()->next_index() == -1)
-      return;
-
-    shared_ptr<PlaylistItem> item = playlists_->active()->item_at(
-        playlists_->active()->next_index());
-    if (!item)
-      return;
-
-    QUrl url = item->Url();
-
-    // Get the actual track URL rather than the stream URL.
-    if (item->options() & PlaylistItem::ContainsMultipleTracks) {
-      PlaylistItem::SpecialLoadResult result = item->LoadNext();
-      switch (result.type_) {
-      case PlaylistItem::SpecialLoadResult::NoMoreTracks:
-        return;
-
-      case PlaylistItem::SpecialLoadResult::WillLoadAsynchronously:
-        loading_async_ = item->Url();
-        return;
-
-      case PlaylistItem::SpecialLoadResult::TrackAvailable:
-        url = result.media_url_;
-        break;
-      }
     }
-    engine_->StartPreloading(url);
   }
+
+  // Crossfade is off, so start preloading the next track so we don't get a
+  // gap between songs.
+  if (current_contains_multiple_tracks || !has_next_row)
+    return;
+
+  if (!next_item)
+    return;
+
+  QUrl url = next_item->Url();
+
+  // Get the actual track URL rather than the stream URL.
+  if (next_item->options() & PlaylistItem::ContainsMultipleTracks) {
+    PlaylistItem::SpecialLoadResult result = next_item->LoadNext();
+    switch (result.type_) {
+    case PlaylistItem::SpecialLoadResult::NoMoreTracks:
+      return;
+
+    case PlaylistItem::SpecialLoadResult::WillLoadAsynchronously:
+      loading_async_ = next_item->Url();
+      return;
+
+    case PlaylistItem::SpecialLoadResult::TrackAvailable:
+      url = result.media_url_;
+      break;
+    }
+  }
+  engine_->StartPreloading(url, next_item->Metadata().has_cue(),
+                           next_item->Metadata().beginning_nanosec(),
+                           next_item->Metadata().end_nanosec());
+}
+
+void Player::ValidSongRequested(const QUrl& url) {
+  emit SongChangeRequestProcessed(url, true);
+}
+
+void Player::InvalidSongRequested(const QUrl& url) {
+  // first send the notification to others...
+  emit SongChangeRequestProcessed(url, false);
+  // ... and now when our listeners have completed their processing of the
+  // current item we can change the current item by skipping to the next song
+  NextItem(Engine::Auto);
 }
