@@ -23,37 +23,90 @@
 
 #include <boost/scoped_ptr.hpp>
 
+#include "config.h"
 #include "core/albumcoverloader.h"
 #include "core/song.h"
 #include "engines/engine_fwd.h"
 #include "playlist/playlistitem.h"
 
-class PlaylistManager;
-class Settings;
 class LastFMService;
+class PlaylistManagerInterface;
+class Settings;
 class MainWindow;
 
-namespace mpris {
-  class Mpris1;
-  class Mpris2;
-  class ArtLoader;
-}
 
-#ifdef Q_WS_X11
-# include <QDBusArgument>
-  QDBusArgument& operator<< (QDBusArgument& arg, const QImage& image);
-  const QDBusArgument& operator>> (const QDBusArgument& arg, QImage& image);
-#endif
-
-class Player : public QObject {
+class PlayerInterface : public QObject {
   Q_OBJECT
 
- public:
-  Player(MainWindow* main_window, PlaylistManager* playlists,
-         LastFMService* lastfm, Engine::Type engine, QObject* parent = 0);
+public:
+  PlayerInterface(QObject* parent = 0) : QObject(parent) {}
+
+  virtual EngineBase* engine() const = 0;
+  virtual Engine::State GetState() const = 0;
+  virtual int GetVolume() const = 0;
+
+  virtual PlaylistItemPtr GetCurrentItem() const = 0;
+  virtual PlaylistItemPtr GetItemAt(int pos) const = 0;
+  virtual PlaylistManagerInterface* playlists() const = 0;
+
+public slots:
+  virtual void ReloadSettings() = 0;
+
+  // Manual track change to the specified track
+  virtual void PlayAt(int i, Engine::TrackChangeFlags change, bool reshuffle) = 0;
+
+  // If there's currently a song playing, pause it, otherwise play the track
+  // that was playing last, or the first one on the playlist
+  virtual void PlayPause() = 0;
+
+  // Skips this track.  Might load more of the current radio station.
+  virtual void Next() = 0;
+
+  virtual void Previous() = 0;
+  virtual void SetVolume(int value) = 0;
+  virtual void VolumeUp() = 0;
+  virtual void VolumeDown() = 0;
+  virtual void SeekTo(int seconds) = 0;
+  // Moves the position of the currently playing song five seconds forward.
+  virtual void SeekForward() = 0;
+  // Moves the position of the currently playing song five seconds backwards.
+  virtual void SeekBackward() = 0;
+
+  virtual void HandleSpecialLoad(const PlaylistItem::SpecialLoadResult& result) = 0;
+  virtual void CurrentMetadataChanged(const Song& metadata) = 0;
+
+  virtual void Mute() = 0;
+  virtual void Pause() = 0;
+  virtual void Stop() = 0;
+  virtual void Play() = 0;
+  virtual void ShowOSD() = 0;
+
+signals:
+  void Playing();
+  void Paused();
+  void Stopped();
+  void PlaylistFinished();
+  void VolumeChanged(int volume);
+  void Error(const QString& message);
+  void TrackSkipped(PlaylistItemPtr old_track);
+  // Emitted when there's a manual change to the current's track position.
+  void Seeked(qlonglong microseconds);
+
+  // Emitted when Player has processed a request to play another song. This contains
+  // the URL of the song and a flag saying whether it was able to play the song.
+  void SongChangeRequestProcessed(const QUrl& url, bool valid);
+
+  void ForceShowOSD(Song);
+};
+
+class Player : public PlayerInterface {
+  Q_OBJECT
+
+public:
+  Player(PlaylistManagerInterface* playlists, LastFMService* lastfm,
+         QObject* parent = 0);
   ~Player();
 
-  EngineBase* CreateEngine(Engine::Type engine);
   void Init();
 
   EngineBase* engine() const { return engine_.get(); }
@@ -62,29 +115,21 @@ class Player : public QObject {
 
   PlaylistItemPtr GetCurrentItem() const { return current_item_; }
   PlaylistItemPtr GetItemAt(int pos) const;
-  PlaylistManager* playlists() const { return playlists_; }
-  mpris::ArtLoader* ArtLoader() const { return art_loader_; }
+  PlaylistManagerInterface* playlists() const { return playlists_; }
 
- public slots:
+public slots:
   void ReloadSettings();
 
-  // Manual track change to the specified track
-  void PlayAt(int i, Engine::TrackChangeType change, bool reshuffle);
-
-  // If there's currently a song playing, pause it, otherwise play the track
-  // that was playing last, or the first one on the playlist
+  void PlayAt(int i, Engine::TrackChangeFlags change, bool reshuffle);
   void PlayPause();
-
-  // Skips this track.  Might load more of the current radio station.
   void Next();
-
   void Previous();
   void SetVolume(int value);
   void VolumeUp() { SetVolume(GetVolume() + 5); }
   void VolumeDown() { SetVolume(GetVolume() - 5); }
-  void Seek(int seconds);
-  void SeekForward() { Seek(+5); }
-  void SeekBackward() { Seek(-5); }
+  void SeekTo(int seconds);
+  void SeekForward();
+  void SeekBackward();
 
   void HandleSpecialLoad(const PlaylistItem::SpecialLoadResult& result);
   void CurrentMetadataChanged(const Song& metadata);
@@ -95,17 +140,6 @@ class Player : public QObject {
   void Play();
   void ShowOSD();
 
- signals:
-  void Playing();
-  void Paused();
-  void Stopped();
-  void PlaylistFinished();
-  void VolumeChanged(int volume);
-  void Error(const QString& message);
-  void TrackSkipped(PlaylistItemPtr old_track);
-
-  void ForceShowOSD(Song);
-
  private slots:
   void EngineStateChanged(Engine::State);
   void EngineMetadataReceived(const Engine::SimpleMetaBundle& bundle);
@@ -113,24 +147,22 @@ class Player : public QObject {
   void TrackEnded();
   // Play the next item on the playlist - disregarding radio stations like
   // last.fm that might have more tracks.
-  void NextItem(Engine::TrackChangeType change);
+  void NextItem(Engine::TrackChangeFlags change);
 
-  void NextInternal(Engine::TrackChangeType);
+  void NextInternal(Engine::TrackChangeFlags);
+
+  void ValidSongRequested(const QUrl&);
+  void InvalidSongRequested(const QUrl&);
 
  private:
-  mpris::ArtLoader* art_loader_;
-
-  mpris::Mpris1* mpris1_;
-  mpris::Mpris2* mpris2_;
-
-  PlaylistManager* playlists_;
+  PlaylistManagerInterface* playlists_;
   LastFMService* lastfm_;
   QSettings settings_;
 
   PlaylistItemPtr current_item_;
 
   boost::scoped_ptr<EngineBase> engine_;
-  Engine::TrackChangeType stream_change_type_;
+  Engine::TrackChangeFlags stream_change_type_;
   Engine::State last_state_;
 
   QUrl loading_async_;
