@@ -22,28 +22,31 @@
 
 #include <sys/stat.h>
 
-#include <taglib/aifffile.h>
-#include <taglib/asffile.h>
-#include <taglib/commentsframe.h>
-#include <taglib/fileref.h>
-#include <taglib/flacfile.h>
-#include <taglib/id3v1genres.h>
-#include <taglib/id3v2tag.h>
-#include <taglib/mp4file.h>
-#include <taglib/mpcfile.h>
-#include <taglib/mpegfile.h>
-#include <taglib/oggfile.h>
-#include <taglib/oggflacfile.h>
-#include <taglib/speexfile.h>
-#include <taglib/tag.h>
-#include <taglib/textidentificationframe.h>
-#include <taglib/trueaudiofile.h>
-#include <taglib/tstring.h>
-#include <taglib/vorbisfile.h>
-#include <taglib/wavfile.h>
+#include <aifffile.h>
+#include <asffile.h>
+#include <attachedpictureframe.h>
+#include <commentsframe.h>
+#include <fileref.h>
+#include <flacfile.h>
+#include <id3v1genres.h>
+#include <id3v2tag.h>
+#include <mp4file.h>
+#include <mpcfile.h>
+#include <mpegfile.h>
+#include <oggfile.h>
+#include <oggflacfile.h>
+#include <speexfile.h>
+#include <tag.h>
+#include <textidentificationframe.h>
+#include <trueaudiofile.h>
+#include <tstring.h>
+#include <vorbisfile.h>
+#include <wavfile.h>
 
-#include "radio/fixlastfm.h"
-#include <lastfm/Track>
+#ifdef HAVE_LIBLASTFM
+  #include "radio/fixlastfm.h"
+  #include <lastfm/Track>
+#endif
 
 #include <QFile>
 #include <QFileInfo>
@@ -87,11 +90,12 @@ static QStringList Updateify(const QStringList& list) {
 const QStringList Song::kColumns = QStringList()
     << "title" << "album" << "artist" << "albumartist" << "composer" << "track"
     << "disc" << "bpm" << "year" << "genre" << "comment" << "compilation"
-    << "length" << "bitrate" << "samplerate" << "directory" << "filename"
+    << "bitrate" << "samplerate" << "directory" << "filename"
     << "mtime" << "ctime" << "filesize" << "sampler" << "art_automatic"
     << "art_manual" << "filetype" << "playcount" << "lastplayed" << "rating"
     << "forced_compilation_on" << "forced_compilation_off"
-    << "effective_compilation" << "skipcount" << "score";
+    << "effective_compilation" << "skipcount" << "score" << "beginning" << "length"
+    << "cue_path";
 
 const QString Song::kColumnSpec = Song::kColumns.join(", ");
 const QString Song::kBindSpec = Prepend(":", Song::kColumns).join(", ");
@@ -106,6 +110,9 @@ const QString Song::kFtsColumnSpec = Song::kFtsColumns.join(", ");
 const QString Song::kFtsBindSpec = Prepend(":", Song::kFtsColumns).join(", ");
 const QString Song::kFtsUpdateSpec = Updateify(Song::kFtsColumns).join(", ");
 
+const QString Song::kManuallyUnsetCover = "(unset)";
+const QString Song::kEmbeddedCover = "(embedded)";
+
 QString Song::JoinSpec(const QString& table) {
   return Prepend(table + ".", kColumns).join(", ");
 }
@@ -156,7 +163,8 @@ Song::Private::Private()
     skipcount_(0),
     lastplayed_(-1),
     score_(0),
-    length_(-1),
+    beginning_(0),
+    end_(-1),
     bitrate_(-1),
     samplerate_(-1),
     directory_id_(-1),
@@ -164,12 +172,17 @@ Song::Private::Private()
     ctime_(-1),
     filesize_(-1),
     filetype_(Type_Unknown),
-    init_from_file_(false)
+    init_from_file_(false),
+    suspicious_tags_(false)
 {
 }
 
 TagLib::FileRef* TagLibFileRefFactory::GetFileRef(const QString& filename) {
+#ifdef Q_OS_WIN32
+  return new TagLib::FileRef(filename.toStdWString().c_str());
+#else
   return new TagLib::FileRef(QFile::encodeName(filename).constData());
+#endif
 }
 
 Song::Song()
@@ -189,15 +202,30 @@ Song::Song(FileRefFactory* factory)
       factory_(factory) {
 }
 
-void Song::Init(const QString& title, const QString& artist, const QString& album, int length) {
+void Song::Init(const QString& title, const QString& artist,
+                const QString& album, qint64 length_nanosec) {
   d->valid_ = true;
+
   d->title_ = title;
   d->artist_ = artist;
   d->album_ = album;
-  d->length_ = length;
+
+  set_length_nanosec(length_nanosec);
 }
 
-void Song::set_genre(int id) {
+void Song::Init(const QString& title, const QString& artist,
+                const QString& album, qint64 beginning, qint64 end) {
+  d->valid_ = true;
+
+  d->title_ = title;
+  d->artist_ = artist;
+  d->album_ = album;
+
+  d->beginning_ = beginning;
+  d->end_ = end;
+}
+
+void Song::set_genre_id3(int id) {
   set_genre(TStringToQString(TagLib::ID3v1::genre(id)));
 }
 
@@ -218,24 +246,39 @@ QString Song::Decode(const QString& tag, const QTextCodec* codec) {
   return codec->toUnicode(tag.toUtf8());
 }
 
+bool Song::HasProperMediaFile() const {
+#ifndef QT_NO_DEBUG_OUTPUT
+  if (qApp->thread() == QThread::currentThread())
+    qWarning() << Q_FUNC_INFO << "on GUI thread!";
+#endif
+
+  QMutexLocker l(&taglib_mutex_);
+  scoped_ptr<TagLib::FileRef> fileref(factory_->GetFileRef(d->filename_));
+
+  return !fileref->isNull() && fileref->tag();
+}
+
 void Song::InitFromFile(const QString& filename, int directory_id) {
 #ifndef QT_NO_DEBUG_OUTPUT
   if (qApp->thread() == QThread::currentThread())
     qWarning() << Q_FUNC_INFO << "on GUI thread!";
 #endif
+
+  d->init_from_file_ = true;
+
   d->filename_ = filename;
   d->directory_id_ = directory_id;
 
+  QFileInfo info(filename);
+  d->basefilename_ = info.fileName();
+
   QMutexLocker l(&taglib_mutex_);
   scoped_ptr<TagLib::FileRef> fileref(factory_->GetFileRef(filename));
 
-  if(fileref->isNull())
+  if(fileref->isNull()) {
     return;
+  }
 
-  d->init_from_file_ = true;
-
-  QFileInfo info(filename);
-  d->basefilename_ = info.fileName();
   d->filesize_ = info.size();
   d->mtime_ = info.lastModified().toTime_t();
   d->ctime_ = info.created().toTime_t();
@@ -246,11 +289,22 @@ void Song::InitFromFile(const QString& filename, int directory_id) {
   TagLib::Tag* tag = fileref->tag();
   QTextCodec* codec = NULL;
   if (tag) {
-    codec = detector.Guess(*fileref);
-    d->title_ = Decode(tag->title(), codec);
-    d->artist_ = Decode(tag->artist(), codec);
-    d->album_ = Decode(tag->album(), codec);
-    d->genre_ = Decode(tag->genre(), codec);
+    TagLib::MPEG::File* file = dynamic_cast<TagLib::MPEG::File*>(fileref->file());
+    if (file && (file->ID3v2Tag() || file->ID3v1Tag())) {
+      codec = detector.Guess(*fileref);
+    }
+    if (codec &&
+        codec->name() != "UTF-8" &&
+        codec->name() != "ISO-8859-1") {
+      // Mark tags where we detect an unusual codec as suspicious.
+      d->suspicious_tags_ = true;
+    }
+
+
+    d->title_ = Decode(tag->title());
+    d->artist_ = Decode(tag->artist());
+    d->album_ = Decode(tag->album());
+    d->genre_ = Decode(tag->genre());
     d->year_ = tag->year();
     d->track_ = tag->track();
 
@@ -270,24 +324,24 @@ void Song::InitFromFile(const QString& filename, int directory_id) {
         d->bpm_ = TStringToQString(map["TBPM"].front()->toString()).trimmed().toFloat();
 
       if (!map["TCOM"].isEmpty())
-        d->composer_ = Decode(map["TCOM"].front()->toString(), codec);
+        d->composer_ = Decode(map["TCOM"].front()->toString());
 
       if (!map["TPE2"].isEmpty()) // non-standard: Apple, Microsoft
-        d->albumartist_ = Decode(map["TPE2"].front()->toString(), codec);
+        d->albumartist_ = Decode(map["TPE2"].front()->toString());
 
       if (!map["TCMP"].isEmpty())
         compilation = TStringToQString(map["TCMP"].front()->toString()).trimmed();
 
       if (!map["APIC"].isEmpty())
-        d->art_automatic_ = AlbumCoverLoader::kEmbeddedCover;
+        set_embedded_cover();
 
       // Find a suitable comment tag.  For now we ignore iTunNORM comments.
       for (int i=0 ; i<map["COMM"].size() ; ++i) {
         const TagLib::ID3v2::CommentsFrame* frame =
             dynamic_cast<const TagLib::ID3v2::CommentsFrame*>(map["COMM"][i]);
 
-        if (TStringToQString(frame->description()) != "iTunNORM") {
-          d->comment_ = Decode(frame->text(), codec);
+        if (frame && TStringToQString(frame->description()) != "iTunNORM") {
+          d->comment_ = Decode(frame->text());
           break;
         }
       }
@@ -297,7 +351,7 @@ void Song::InitFromFile(const QString& filename, int directory_id) {
         const TagLib::ID3v2::UserTextIdentificationFrame* frame =
             dynamic_cast<const TagLib::ID3v2::UserTextIdentificationFrame*>(map["TXXX"][i]);
 
-        if (frame->description().startsWith("FMPS_")) {
+        if (frame && frame->description().startsWith("FMPS_")) {
           ParseFMPSFrame(TStringToQString(frame->description()),
                          TStringToQString(frame->fieldList()[1]));
         }
@@ -305,16 +359,16 @@ void Song::InitFromFile(const QString& filename, int directory_id) {
     }
   } else if (TagLib::Ogg::Vorbis::File* file = dynamic_cast<TagLib::Ogg::Vorbis::File*>(fileref->file())) {
     if (file->tag()) {
-      ParseOggTag(file->tag()->fieldListMap(), codec, &disc, &compilation);
+      ParseOggTag(file->tag()->fieldListMap(), NULL, &disc, &compilation);
     }
-    d->comment_ = Decode(tag->comment(), codec);
+    d->comment_ = Decode(tag->comment());
   } else if (TagLib::FLAC::File* file = dynamic_cast<TagLib::FLAC::File*>(fileref->file())) {
     if ( file->xiphComment() ) {
-      ParseOggTag(file->xiphComment()->fieldListMap(), codec, &disc, &compilation);
+      ParseOggTag(file->xiphComment()->fieldListMap(), NULL, &disc, &compilation);
     }
-    d->comment_ = Decode(tag->comment(), codec);
+    d->comment_ = Decode(tag->comment());
   } else if (tag) {
-    d->comment_ = Decode(tag->comment(), codec);
+    d->comment_ = Decode(tag->comment());
   }
 
   if ( !disc.isEmpty() )   {
@@ -337,12 +391,24 @@ void Song::InitFromFile(const QString& filename, int directory_id) {
 
   if (fileref->audioProperties()) {
     d->bitrate_    = fileref->audioProperties()->bitrate();
-    d->length_     = fileref->audioProperties()->length();
     d->samplerate_ = fileref->audioProperties()->sampleRate();
+    set_length_nanosec(fileref->audioProperties()->length() * kNsecPerSec);
   }
 
   // Get the filetype if we can
   GuessFileType(fileref.get());
+
+  // Set integer fields to -1 if they're not valid
+  #define intval(x) (x <= 0 ? -1 : x)
+  d->track_ = intval(d->track_);
+  d->disc_ = intval(d->disc_);
+  d->bpm_ = intval(d->bpm_);
+  d->year_ = intval(d->year_);
+  d->bitrate_ = intval(d->bitrate_);
+  d->samplerate_ = intval(d->samplerate_);
+  d->lastplayed_ = intval(d->lastplayed_);
+  d->rating_ = intval(d->rating_);
+  #undef intval
 }
 
 void Song::ParseFMPSFrame(const QString& name, const QString& value) {
@@ -396,6 +462,9 @@ void Song::ParseOggTag(const TagLib::Ogg::FieldListMap& map, const QTextCodec* c
 
   if (!map["COMPILATION"].isEmpty() )
     *compilation = TStringToQString( map["COMPILATION"].front() ).trimmed();
+
+  if (!map["COVERART"].isEmpty())
+    set_embedded_cover();
 }
 
 void Song::GuessFileType(TagLib::FileRef* fileref) {
@@ -432,9 +501,10 @@ void Song::InitFromQuery(const SqlRow& q, int col) {
 
   d->init_from_file_ = true;
 
-  #define tostr(n) (q.value(n).isNull() ? QString::null : q.value(n).toString())
-  #define toint(n) (q.value(n).isNull() ? -1 : q.value(n).toInt())
-  #define tofloat(n) (q.value(n).isNull() ? -1 : q.value(n).toDouble())
+  #define tostr(n)      (q.value(n).isNull() ? QString::null : q.value(n).toString())
+  #define toint(n)      (q.value(n).isNull() ? -1 : q.value(n).toInt())
+  #define tolonglong(n) (q.value(n).isNull() ? -1 : q.value(n).toLongLong())
+  #define tofloat(n)    (q.value(n).isNull() ? -1 : q.value(n).toDouble())
 
   d->id_ = toint(col + 0);
   d->title_ = tostr(col + 1);
@@ -450,41 +520,48 @@ void Song::InitFromQuery(const SqlRow& q, int col) {
   d->comment_ = tostr(col + 11);
   d->compilation_ = q.value(col + 12).toBool();
 
-  d->length_ = toint(col + 13);
-  d->bitrate_ = toint(col + 14);
-  d->samplerate_ = toint(col + 15);
+  d->bitrate_ = toint(col + 13);
+  d->samplerate_ = toint(col + 14);
 
-  d->directory_id_ = toint(col + 16);
-  d->filename_ = tostr(col + 17);
+  d->directory_id_ = toint(col + 15);
+  d->filename_ = tostr(col + 16);
   d->basefilename_ = QFileInfo(d->filename_).fileName();
-  d->mtime_ = toint(col + 18);
-  d->ctime_ = toint(col + 19);
-  d->filesize_ = toint(col + 20);
+  d->mtime_ = toint(col + 17);
+  d->ctime_ = toint(col + 18);
+  d->filesize_ = toint(col + 19);
 
-  d->sampler_ = q.value(col + 21).toBool();
+  d->sampler_ = q.value(col + 20).toBool();
 
-  d->art_automatic_ = q.value(col + 22).toString();
-  d->art_manual_ = q.value(col + 23).toString();
+  d->art_automatic_ = q.value(col + 21).toString();
+  d->art_manual_ = q.value(col + 22).toString();
 
-  d->filetype_ = FileType(q.value(col + 24).toInt());
-  d->playcount_ = q.value(col + 25).isNull() ? 0 : q.value(col + 25).toInt();
-  d->lastplayed_ = toint(col + 26);
-  d->rating_ = tofloat(col + 27);
+  d->filetype_ = FileType(q.value(col + 23).toInt());
+  d->playcount_ = q.value(col + 24).isNull() ? 0 : q.value(col + 24).toInt();
+  d->lastplayed_ = toint(col + 25);
+  d->rating_ = tofloat(col + 26);
 
-  d->forced_compilation_on_ = q.value(col + 28).toBool();
-  d->forced_compilation_off_ = q.value(col + 29).toBool();
+  d->forced_compilation_on_ = q.value(col + 27).toBool();
+  d->forced_compilation_off_ = q.value(col + 28).toBool();
 
-  // effective_compilation = 30
+  // effective_compilation = 29
 
-  d->skipcount_ = q.value(col + 31).isNull() ? 0 : q.value(col + 31).toInt();
+  d->skipcount_ = q.value(col + 30).isNull() ? 0 : q.value(col + 30).toInt();
+  d->score_ = q.value(col + 31).isNull() ? 0 : q.value(col + 31).toInt();
 
-  d->score_ = q.value(col + 32).isNull() ? 0 : q.value(col + 32).toInt();
+  // do not move those statements - beginning must be initialized before
+  // length is!
+  d->beginning_ = q.value(col + 32).isNull() ? 0 : q.value(col + 32).toLongLong();
+  set_length_nanosec(tolonglong(col + 33));
+
+  d->cue_path_ = tostr(col + 34);
 
   #undef tostr
   #undef toint
+  #undef tolonglong
   #undef tofloat
 }
 
+#ifdef HAVE_LIBLASTFM
 void Song::InitFromLastFM(const lastfm::Track& track) {
   d->valid_ = true;
   d->filetype_ = Type_Stream;
@@ -493,8 +570,10 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
   d->album_ = track.album();
   d->artist_ = track.artist();
   d->track_ = track.trackNumber();
-  d->length_ = track.duration();
+
+  set_length_nanosec(track.duration() * kNsecPerSec);
 }
+#endif // HAVE_LIBLASTFM
 
 #ifdef HAVE_LIBGPOD
   void Song::InitFromItdb(const Itdb_Track* track) {
@@ -512,7 +591,7 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
     d->genre_ = QString::fromUtf8(track->genre);
     d->comment_ = QString::fromUtf8(track->comment);
     d->compilation_ = track->compilation;
-    d->length_ = track->tracklen / 1000;
+    set_length_nanosec(track->tracklen * kNsecPerMsec);
     d->bitrate_ = track->bitrate;
     d->samplerate_ = track->samplerate;
     d->mtime_ = track->time_modified;
@@ -542,7 +621,7 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
     track->genre = strdup(d->genre_.toUtf8().constData());
     track->comment = strdup(d->comment_.toUtf8().constData());
     track->compilation = d->compilation_;
-    track->tracklen = d->length_ * 1000;
+    track->tracklen = length_nanosec() / kNsecPerMsec;
     track->bitrate = d->bitrate_;
     track->samplerate = d->samplerate_;
     track->time_modified = d->mtime_;
@@ -571,7 +650,7 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
     d->basefilename_ = d->filename_;
 
     d->track_ = track->tracknumber;
-    d->length_ = track->duration / 1000;
+    set_length_nanosec(track->duration * kNsecPerMsec);
     d->samplerate_ = track->samplerate;
     d->bitrate_ = track->bitrate;
     d->filesize_ = track->filesize;
@@ -611,7 +690,7 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
     track->filename = strdup(d->basefilename_.toUtf8().constData());
 
     track->tracknumber = d->track_;
-    track->duration = d->length_ * 1000;
+    track->duration = length_nanosec() / kNsecPerMsec;
     track->samplerate = d->samplerate_;
     track->nochannels = 0;
     track->wavecodec = 0;
@@ -745,7 +824,7 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
         d->filename_ = item_value.toString();
 
       else if (wcscmp(name, g_wszWMDMDuration) == 0)
-        d->length_ = item_value.toULongLong() / 10000000ll;
+        set_length_nanosec(item_value.toULongLong() * 1e2);
 
       else if (wcscmp(name, L"WMDM/FileSize") == 0)
         d->filesize_ = item_value.toULongLong();
@@ -809,7 +888,7 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
       if (!d->title_.isEmpty() || !d->artist_.isEmpty() ||
           !d->album_.isEmpty() || !d->comment_.isEmpty() ||
           !d->genre_.isEmpty() || d->track_ != -1 || d->year_ != -1 ||
-          d->length_ != -1) {
+          length_nanosec() != -1) {
         d->filetype_ = Song::Type_Unknown;
         break;
       }
@@ -843,7 +922,7 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
     AddWmdmItem(metadata, g_wszWMDMComposer, d->composer_);
     AddWmdmItem(metadata, g_wszWMDMBitrate, d->bitrate_);
     AddWmdmItem(metadata, g_wszWMDMFileName, d->basefilename_);
-    AddWmdmItem(metadata, g_wszWMDMDuration, qint64(d->length_) * 10000000ll);
+    AddWmdmItem(metadata, g_wszWMDMDuration, qint64(length_nanosec()) / 1e2);
     AddWmdmItem(metadata, L"WMDM/FileSize", d->filesize_);
 
     WMDM_FORMATCODE format;
@@ -864,13 +943,13 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
 #endif // Q_OS_WIN32
 
 void Song::MergeFromSimpleMetaBundle(const Engine::SimpleMetaBundle &bundle) {
-  d->valid_ = true;
-
   if (d->init_from_file_) {
     // This Song was already loaded using taglib. Our tags are probably better than the engine's.
     return;
   }
 
+  d->valid_ = true;
+
   UniversalEncodingHandler detector(NS_FILTER_NON_CJK);
   QTextCodec* codec = detector.Guess(bundle);
 
@@ -881,7 +960,7 @@ void Song::MergeFromSimpleMetaBundle(const Engine::SimpleMetaBundle &bundle) {
   if (!bundle.genre.isEmpty()) d->genre_ = Decode(bundle.genre, codec);
   if (!bundle.bitrate.isEmpty()) d->bitrate_ = bundle.bitrate.toInt();
   if (!bundle.samplerate.isEmpty()) d->samplerate_ = bundle.samplerate.toInt();
-  if (!bundle.length.isEmpty()) d->length_ = bundle.length.toInt();
+  if (!bundle.length.isEmpty()) set_length_nanosec(bundle.length.toLongLong());
   if (!bundle.year.isEmpty()) d->year_ = bundle.year.toInt();
   if (!bundle.tracknr.isEmpty()) d->track_ = bundle.tracknr.toInt();
 }
@@ -906,7 +985,6 @@ void Song::BindToQuery(QSqlQuery *query) const {
   query->bindValue(":comment", strval(d->comment_));
   query->bindValue(":compilation", d->compilation_ ? 1 : 0);
 
-  query->bindValue(":length", intval(d->length_));
   query->bindValue(":bitrate", intval(d->bitrate_));
   query->bindValue(":samplerate", intval(d->samplerate_));
 
@@ -931,9 +1009,13 @@ void Song::BindToQuery(QSqlQuery *query) const {
   query->bindValue(":effective_compilation", is_compilation() ? 1 : 0);
 
   query->bindValue(":skipcount", d->skipcount_);
-
   query->bindValue(":score", d->score_);
 
+  query->bindValue(":beginning", d->beginning_);
+  query->bindValue(":length", intval(length_nanosec()));
+
+  query->bindValue(":cue_path", d->cue_path_);
+
   #undef intval
   #undef notnullintval
 }
@@ -948,22 +1030,30 @@ void Song::BindToFtsQuery(QSqlQuery *query) const {
   query->bindValue(":ftscomment", d->comment_);
 }
 
+#ifdef HAVE_LIBLASTFM
 void Song::ToLastFM(lastfm::Track* track) const {
   lastfm::MutableTrack mtrack(*track);
 
   mtrack.setArtist(d->artist_);
   mtrack.setAlbum(d->album_);
   mtrack.setTitle(d->title_);
-  mtrack.setDuration(d->length_);
+  mtrack.setDuration(length_nanosec() / kNsecPerSec);
   mtrack.setTrackNumber(d->track_);
   mtrack.setSource(lastfm::Track::Player);
 }
+#endif // HAVE_LIBLASTFM
+
+QUrl Song::url() const {
+  return QFile::exists(filename())
+             ? QUrl::fromLocalFile(filename())
+             : filename();
+}
 
 QString Song::PrettyTitle() const {
   QString title(d->title_);
 
   if (title.isEmpty())
-    title = QFileInfo(d->filename_).baseName();
+    title = d->basefilename_;
 
   return title;
 }
@@ -972,7 +1062,7 @@ QString Song::PrettyTitleWithArtist() const {
   QString title(d->title_);
 
   if (title.isEmpty())
-    title = QFileInfo(d->filename_).baseName();
+    title = d->basefilename_;
 
   if (!d->artist_.isEmpty())
     title = d->artist_ + " - " + title;
@@ -981,10 +1071,10 @@ QString Song::PrettyTitleWithArtist() const {
 }
 
 QString Song::PrettyLength() const {
-  if (d->length_ == -1)
+  if (length_nanosec() == -1)
     return QString::null;
 
-  return Utilities::PrettyTime(d->length_);
+  return Utilities::PrettyTimeNanosec(length_nanosec());
 }
 
 QString Song::PrettyYear() const {
@@ -998,7 +1088,7 @@ QString Song::TitleWithCompilationArtist() const {
   QString title(d->title_);
 
   if (title.isEmpty())
-    title = QFileInfo(d->filename_).baseName();
+    title = d->basefilename_;
 
   if (is_compilation() && !d->artist_.isEmpty() && !d->artist_.toLower().contains("various"))
     title = d->artist_ + " - " + title;
@@ -1019,12 +1109,13 @@ bool Song::IsMetadataEqual(const Song& other) const {
          d->genre_ == other.d->genre_ &&
          d->comment_ == other.d->comment_ &&
          d->compilation_ == other.d->compilation_ &&
-         d->length_ == other.d->length_ &&
+         d->beginning_ == other.d->beginning_ &&
+         length_nanosec() == other.length_nanosec() &&
          d->bitrate_ == other.d->bitrate_ &&
          d->samplerate_ == other.d->samplerate_ &&
-         d->sampler_ == other.d->sampler_ &&
          d->art_automatic_ == other.d->art_automatic_ &&
-         d->art_manual_ == other.d->art_manual_;
+         d->art_manual_ == other.d->art_manual_ &&
+         d->cue_path_ == other.d->cue_path_;
 }
 
 void Song::SetTextFrame(const QString& id, const QString& value,
@@ -1050,8 +1141,8 @@ TagLib::String QStringToTaglibString(const QString& s) {
 }
 
 bool Song::IsEditable() const {
-  return d->valid_ && !d->filename_.isNull() &&
-         d->filetype_ != Type_Stream && d->filetype_ != Type_Unknown;
+  return d->valid_ && !d->filename_.isNull() && !is_stream() &&
+         d->filetype_ != Type_Unknown && !has_cue();
 }
 
 bool Song::Save() const {
@@ -1115,3 +1206,74 @@ QFuture<bool> Song::BackgroundSave() const {
   QFuture<bool> future = QtConcurrent::run(&Song::Save, Song(*this));
   return future;
 }
+
+bool Song::operator==(const Song& other) const {
+  // TODO: this isn't working for radios
+  return filename() == other.filename() &&
+         beginning_nanosec() == other.beginning_nanosec();
+}
+
+QImage Song::LoadEmbeddedArt(const QString& filename) {
+  QImage ret;
+  if (filename.isEmpty())
+    return ret;
+
+  QMutexLocker l(&taglib_mutex_);
+
+#ifdef Q_OS_WIN32
+  TagLib::FileRef ref(filename.toStdWString().c_str());
+#else
+  TagLib::FileRef ref(QFile::encodeName(filename).constData());
+#endif
+
+  if (ref.isNull() || !ref.file())
+    return ret;
+
+  // MP3
+  TagLib::MPEG::File* file = dynamic_cast<TagLib::MPEG::File*>(ref.file());
+  if (file && file->ID3v2Tag()) {
+    TagLib::ID3v2::FrameList apic_frames = file->ID3v2Tag()->frameListMap()["APIC"];
+    if (apic_frames.isEmpty())
+      return ret;
+
+    TagLib::ID3v2::AttachedPictureFrame* pic =
+        static_cast<TagLib::ID3v2::AttachedPictureFrame*>(apic_frames.front());
+
+    ret.loadFromData((const uchar*) pic->picture().data(), pic->picture().size());
+    return ret;
+  }
+
+  // Ogg vorbis/flac/speex
+  TagLib::Ogg::XiphComment* xiph_comment =
+      dynamic_cast<TagLib::Ogg::XiphComment*>(ref.file()->tag());
+
+  if (xiph_comment) {
+    TagLib::Ogg::FieldListMap map = xiph_comment->fieldListMap();
+
+    // Ogg lacks a definitive standard for embedding cover art, but it seems
+    // b64 encoding a field called COVERART is the general convention
+    if (!map.contains("COVERART"))
+      return ret;
+
+    QByteArray image_data_b64(map["COVERART"].toString().toCString());
+    QByteArray image_data = QByteArray::fromBase64(image_data_b64);
+
+    if (!ret.loadFromData(image_data))
+      ret.loadFromData(image_data_b64); //maybe it's not b64 after all
+  }
+
+  return ret;
+}
+
+bool Song::IsOnSameAlbum(const Song& other) const {
+  if (is_compilation() != other.is_compilation())
+    return false;
+
+  if (has_cue() && other.has_cue() && cue_path() == other.cue_path())
+    return true;
+
+  if (is_compilation() && album() == other.album())
+    return true;
+
+  return album() == other.album() && artist() == other.artist();
+}
