@@ -27,9 +27,10 @@
 #include <QSqlQuery>
 #include <QString>
 
-#include <taglib/xiphcomment.h>
+#include <xiphcomment.h>
 
 #include "config.h"
+#include "timeconstants.h"
 #include "engines/engine_fwd.h"
 
 #ifdef HAVE_LIBGPOD
@@ -46,9 +47,11 @@
 
 class SqlRow;
 
-namespace lastfm {
-  class Track;
-}
+#ifdef HAVE_LIBLASTFM
+  namespace lastfm {
+    class Track;
+  }
+#endif
 
 namespace TagLib {
   class FileRef;
@@ -70,6 +73,7 @@ class TagLibFileRefFactory : public FileRefFactory {
   virtual TagLib::FileRef* GetFileRef(const QString& filename);
 };
 
+
 class Song {
  public:
   Song();
@@ -86,6 +90,9 @@ class Song {
   static const QString kFtsBindSpec;
   static const QString kFtsUpdateSpec;
 
+  static const QString kManuallyUnsetCover;
+  static const QString kEmbeddedCover;
+
   static QString JoinSpec(const QString& table);
 
   // Don't change these values - they're stored in the database
@@ -108,11 +115,22 @@ class Song {
   static QString TextForFiletype(FileType type);
   QString TextForFiletype() const { return TextForFiletype(filetype()); }
 
+  // Helper function to load embedded cover art from a music file.  This is not
+  // actually used by the Song class, but instead it is called by
+  // AlbumCoverLoader and is here so it can lock on the taglib mutex.
+  static QImage LoadEmbeddedArt(const QString& filename);
+  // Checks if this Song can be properly initialized from it's media file.
+  // This requires the 'filename' attribute to be set first.
+  bool HasProperMediaFile() const;
+
   // Constructors
-  void Init(const QString& title, const QString& artist, const QString& album, int length);
+  void Init(const QString& title, const QString& artist, const QString& album, qint64 length_nanosec);
+  void Init(const QString& title, const QString& artist, const QString& album, qint64 beginning, qint64 end);
   void InitFromFile(const QString& filename, int directory_id);
   void InitFromQuery(const SqlRow& query, int col = 0);
+#ifdef HAVE_LIBLASTFM
   void InitFromLastFM(const lastfm::Track& track);
+#endif
 
   void MergeFromSimpleMetaBundle(const Engine::SimpleMetaBundle& bundle);
 
@@ -131,13 +149,15 @@ class Song {
   void ToWmdm(IWMDMMetaData* metadata) const;
 #endif
 
-  static QString Decode(const TagLib::String& tag, const QTextCodec* codec);
-  static QString Decode(const QString& tag, const QTextCodec* codec);
+  static QString Decode(const TagLib::String& tag, const QTextCodec* codec = NULL);
+  static QString Decode(const QString& tag, const QTextCodec* codec = NULL);
 
   // Save
   void BindToQuery(QSqlQuery* query) const;
   void BindToFtsQuery(QSqlQuery* query) const;
+#ifdef HAVE_LIBLASTFM
   void ToLastFM(lastfm::Track* track) const;
+#endif
 
   // Simple accessors
   bool is_valid() const { return d->valid_; }
@@ -164,21 +184,44 @@ class Song {
   int lastplayed() const { return d->lastplayed_; }
   int score() const { return d->score_; }
 
-  int length() const { return d->length_; }
+  const QString& cue_path() const { return d->cue_path_; }
+  bool has_cue() const { return !d->cue_path_.isEmpty(); }
+
+  qint64 beginning_nanosec() const { return d->beginning_; }
+  qint64 end_nanosec() const { return d->end_; }
+
+  qint64 length_nanosec() const { return d->end_ - d->beginning_; }
+
   int bitrate() const { return d->bitrate_; }
   int samplerate() const { return d->samplerate_; }
 
   int directory_id() const { return d->directory_id_; }
   const QString& filename() const { return d->filename_; }
+  // Returns this Song's URL which may point either to a file or to another type
+  // of stream.
+  QUrl url() const;
   const QString& basefilename() const { return d->basefilename_; }
   uint mtime() const { return d->mtime_; }
   uint ctime() const { return d->ctime_; }
   int filesize() const { return d->filesize_; }
   FileType filetype() const { return d->filetype_; }
+  bool is_stream() const { return d->filetype_ == Type_Stream; }
 
   const QString& art_automatic() const { return d->art_automatic_; }
   const QString& art_manual() const { return d->art_manual_; }
 
+  // Returns true if this Song had it's cover manually unset by user.
+  bool has_manually_unset_cover() const { return d->art_manual_ == kManuallyUnsetCover; }
+  // This method represents an explicit request to unset this song's
+  // cover.
+  void manually_unset_cover() { d->art_manual_ = kManuallyUnsetCover; }
+
+  // Returns true if this song (it's media file) has an embedded cover.
+  bool has_embedded_cover() const { return d->art_automatic_ == kEmbeddedCover; }
+  // Sets a flag saying that this song (it's media file) has an embedded
+  // cover.
+  void set_embedded_cover() { d->art_automatic_ = kEmbeddedCover; }
+
   const QImage& image() const { return d->image_; }
 
   // Pretty accessors
@@ -207,11 +250,13 @@ class Song {
   void set_bpm(float v) { d->bpm_ = v; }
   void set_year(int v) { d->year_ = v; }
   void set_genre(const QString& v) { d->genre_ = v; }
-  void set_genre(int id);
+  void set_genre_id3(int id);
   void set_comment(const QString& v) { d->comment_ = v; }
   void set_compilation(bool v) { d->compilation_ = v; }
   void set_sampler(bool v) { d->sampler_ = v; }
-  void set_length(int v) { d->length_ = v; }
+  void set_beginning_nanosec(qint64 v) { d->beginning_ = qMax(0ll, v); }
+  void set_end_nanosec(qint64 v) { d->end_ = v; }
+  void set_length_nanosec(qint64 v) { d->end_ = d->beginning_ + v; }
   void set_bitrate(int v) { d->bitrate_ = v; }
   void set_samplerate(int v) { d->samplerate_ = v; }
   void set_mtime(int v) { d->mtime_ = v; }
@@ -228,6 +273,7 @@ class Song {
   void set_skipcount(int v) { d->skipcount_ = v; }
   void set_lastplayed(int v) { d->lastplayed_ = v; }
   void set_score(int v) { d->score_ = qBound(0, v, 100); }
+  void set_cue_path(const QString& v) { d->cue_path_ = v; }
 
   // Setters that should only be used by tests
   void set_filename(const QString& v) { d->filename_ = v; }
@@ -236,6 +282,9 @@ class Song {
 
   // Comparison functions
   bool IsMetadataEqual(const Song& other) const;
+  bool IsOnSameAlbum(const Song& other) const;
+
+  bool operator==(const Song& other) const;
 
  private:
   void GuessFileType(TagLib::FileRef* fileref);
@@ -275,7 +324,19 @@ class Song {
     int lastplayed_;
     int score_;
 
-    int length_;  // Seconds.
+    // The beginning of the song in seconds. In case of single-part media
+    // streams, this will equal to 0. In case of multi-part streams on the
+    // other hand, this will mark the beginning of a section represented by
+    // this Song object. This is always greater than 0.
+    qint64 beginning_;
+    // The end of the song in seconds. In case of single-part media
+    // streams, this will equal to the song's length. In case of multi-part
+    // streams on the other hand, this will mark the end of a section
+    // represented by this Song object.
+    // This may be negative indicating that the length of this song is
+    // unknown.
+    qint64 end_;
+
     int bitrate_;
     int samplerate_;
 
@@ -287,6 +348,9 @@ class Song {
     int filesize_;
     FileType filetype_;
 
+    // If the song has a CUE, this contains it's path.
+    QString cue_path_;
+
     // Filenames to album art for this song.
     QString art_automatic_; // Guessed by LibraryWatcher
     QString art_manual_;    // Set by the user - should take priority
@@ -295,6 +359,8 @@ class Song {
 
     // Whether this song was loaded from a file using taglib.
     bool init_from_file_;
+    // Whether our encoding guesser thinks these tags might be incorrectly encoded.
+    bool suspicious_tags_;
   };
 
   void ParseOggTag(const TagLib::Ogg::FieldListMap& map, const QTextCodec* codec, QString* disc, QString* compilation);
