@@ -20,6 +20,7 @@
 #include "library/librarybackend.h"
 #include "library/sqlrow.h"
 #include "playlistparsers/parserbase.h"
+#include "playlistparsers/cueparser.h"
 #include "playlistparsers/playlistparser.h"
 #include "radio/fixlastfm.h"
 
@@ -39,6 +40,7 @@ SongLoader::SongLoader(LibraryBackendInterface* library, QObject *parent)
   : QObject(parent),
     timeout_timer_(new QTimer(this)),
     playlist_parser_(new PlaylistParser(library, this)),
+    cue_parser_(new CueParser(library, this)),
     timeout_(kDefaultTimeout),
     state_(WaitingForType),
     success_(false),
@@ -52,18 +54,14 @@ SongLoader::SongLoader(LibraryBackendInterface* library, QObject *parent)
 
   timeout_timer_->setSingleShot(true);
 
-#ifdef HAVE_GSTREAMER
   connect(timeout_timer_, SIGNAL(timeout()), SLOT(Timeout()));
-#endif
 }
 
 SongLoader::~SongLoader() {
-#ifdef HAVE_GSTREAMER
   if (pipeline_) {
     state_ = Finished;
     gst_element_set_state(pipeline_.get(), GST_STATE_NULL);
   }
-#endif // HAVE_GSTREAMER
 }
 
 SongLoader::Result SongLoader::Load(const QUrl& url) {
@@ -80,15 +78,8 @@ SongLoader::Result SongLoader::Load(const QUrl& url) {
     return Success;
   }
 
-#ifdef HAVE_GSTREAMER
   timeout_timer_->start(timeout_);
   return LoadRemote();
-#else
-  // If we don't have GStreamer we can't check the type of remote playlists,
-  // so just assume it's a raw stream and get on with our lives.
-  AddAsRawStream();
-  return Success;
-#endif
 }
 
 SongLoader::Result SongLoader::LoadLocal(const QString& filename, bool block,
@@ -114,7 +105,7 @@ SongLoader::Result SongLoader::LoadLocal(const QString& filename, bool block,
     return Error;
   QByteArray data(file.read(PlaylistParser::kMagicSize));
 
-  ParserBase* parser = playlist_parser_->MaybeGetParserForMagic(data);
+  ParserBase* parser = playlist_parser_->ParserForMagic(data);
   if (!parser) {
     // Check the file extension as well, maybe the magic failed, or it was a
     // basic M3U file which is just a plain list of filenames.
@@ -141,17 +132,46 @@ SongLoader::Result SongLoader::LoadLocal(const QString& filename, bool block,
 
   // Not a playlist, so just assume it's a song
   QFileInfo info(filename);
+
   LibraryQuery query;
   query.SetColumnSpec("%songs_table.ROWID, " + Song::kColumnSpec);
   query.AddWhere("filename", info.canonicalFilePath());
-  Song song;
+
+  SongList song_list;
+
   if (library_->ExecQuery(&query) && query.Next()) {
-    song.InitFromQuery(query);
+    // we may have many results when the file has many sections
+    do {
+      Song song;
+      song.InitFromQuery(query);
+
+      song_list << song;
+    } while(query.Next());
   } else {
-    song.InitFromFile(filename, -1);
+    QString matching_cue = filename.section('.', 0, -2) + ".cue";
+
+    // it's a cue - create virtual tracks
+    if(QFile::exists(matching_cue)) {
+      QFile cue(matching_cue);
+      cue.open(QIODevice::ReadOnly);
+
+      song_list = cue_parser_->Load(&cue, matching_cue, QDir(filename.section('/', 0, -2)));
+
+    // it's a normal media file
+    } else {
+      Song song;
+      song.InitFromFile(filename, -1);
+
+      song_list << song;
+
+    }
   }
-  if (song.is_valid())
-    songs_ << song;
+
+  foreach(const Song& song, song_list) {
+    if (song.is_valid())
+      songs_ << song;
+  }
+
   return Success;
 }
 
@@ -163,18 +183,20 @@ void SongLoader::LoadPlaylistAndEmit(ParserBase* parser, const QString& filename
 void SongLoader::LoadPlaylist(ParserBase* parser, const QString& filename) {
   QFile file(filename);
   file.open(QIODevice::ReadOnly);
-  songs_ = parser->Load(&file, QFileInfo(filename).path());
+  songs_ = parser->Load(&file, filename, QFileInfo(filename).path());
 }
 
 static bool CompareSongs(const Song& left, const Song& right) {
   // Order by artist, album, disc, track
   if (left.artist() < right.artist()) return true;
-  if (left.artist() > right.artist()) return true;
+  if (left.artist() > right.artist()) return false;
   if (left.album() < right.album()) return true;
-  if (left.album() > right.album()) return true;
+  if (left.album() > right.album()) return false;
   if (left.disc() < right.disc()) return true;
-  if (left.disc() > right.disc()) return true;
-  return left.track() < right.track();
+  if (left.disc() > right.disc()) return false;
+  if (left.track() < right.track()) return true;
+  if (left.track() > right.track()) return false;
+  return left.filename() < right.filename();
 }
 
 void SongLoader::LoadLocalDirectoryAndEmit(const QString& filename) {
@@ -210,7 +232,6 @@ void SongLoader::Timeout() {
 }
 
 void SongLoader::StopTypefind() {
-#ifdef HAVE_GSTREAMER
   // Destroy the pipeline
   if (pipeline_) {
     gst_element_set_state(pipeline_.get(), GST_STATE_NULL);
@@ -231,12 +252,10 @@ void SongLoader::StopTypefind() {
     // It wasn't a playlist - just put the URL in as a stream
     AddAsRawStream();
   }
-#endif // HAVE_GSTREAMER
 
   emit LoadFinished(success_);
 }
 
-#ifdef HAVE_GSTREAMER
 SongLoader::Result SongLoader::LoadRemote() {
   qDebug() << "Loading remote file" << url_;
 
@@ -409,7 +428,7 @@ void SongLoader::EndOfStreamReached() {
 }
 
 void SongLoader::MagicReady() {
-  parser_ = playlist_parser_->MaybeGetParserForMagic(buffer_, mime_type_);
+  parser_ = playlist_parser_->ParserForMagic(buffer_, mime_type_);
 
   if (!parser_) {
     qWarning() << url_.toString() << "is text, but not a recognised playlist";
@@ -437,4 +456,3 @@ void SongLoader::StopTypefindAsync(bool success) {
 
   metaObject()->invokeMethod(this, "StopTypefind", Qt::QueuedConnection);
 }
-#endif // HAVE_GSTREAMER
