@@ -21,15 +21,13 @@
 #include <QObject>
 #include <QUrl>
 
-#include "config.h"
 #include "song.h"
 
 #include <boost/shared_ptr.hpp>
 
-#ifdef HAVE_GSTREAMER
-# include <gst/gst.h>
-#endif
+#include <gst/gst.h>
 
+class CueParser;
 class LibraryBackendInterface;
 class ParserBase;
 class PlaylistParser;
@@ -79,7 +77,6 @@ private:
 
   void AddAsRawStream();
 
-#ifdef HAVE_GSTREAMER
   Result LoadRemote();
 
   // GStreamer callbacks
@@ -92,7 +89,6 @@ private:
   void ErrorMessageReceived(GstMessage* msg);
   void EndOfStreamReached();
   void MagicReady();
-#endif // HAVE_GSTREAMER
 
 private:
   static QSet<QString> sRawUriSchemes;
@@ -102,6 +98,7 @@ private:
 
   QTimer* timeout_timer_;
   PlaylistParser* playlist_parser_;
+  CueParser* cue_parser_;
 
   // For async loads
   int timeout_;
@@ -112,9 +109,7 @@ private:
   QByteArray buffer_;
   LibraryBackendInterface* library_;
 
-#ifdef HAVE_GSTREAMER
   boost::shared_ptr<GstElement> pipeline_;
-#endif
 };
 
 #endif // SONGLOADER_H
