@@ -15,15 +15,19 @@
    along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "timeconstants.h"
 #include "utilities.h"
 
-#include <QtGlobal>
 #include <QCoreApplication>
 #include <QDateTime>
+#include <QDesktopServices>
 #include <QDir>
 #include <QIODevice>
 #include <QStringList>
 #include <QTemporaryFile>
+#include <QUrl>
+#include <QtDebug>
+#include <QtGlobal>
 
 #if defined(Q_OS_UNIX)
 #  include <sys/statvfs.h>
@@ -43,6 +47,10 @@ static QString tr(const char* str) {
   return QCoreApplication::translate("", str);
 }
 
+QString PrettyTimeDelta(int seconds) {
+  return (seconds >= 0 ? "+" : "-") + PrettyTime(seconds);
+}
+
 QString PrettyTime(int seconds) {
   // last.fm sometimes gets the track length wrong, so you end up with
   // negative times.
@@ -61,6 +69,10 @@ QString PrettyTime(int seconds) {
   return ret;
 }
 
+QString PrettyTimeNanosec(qint64 nanoseconds) {
+  return PrettyTime(nanoseconds / kNsecPerSec);
+}
+
 QString WordyTime(quint64 seconds) {
   quint64 days = seconds / (60*60*24);
 
@@ -74,6 +86,10 @@ QString WordyTime(quint64 seconds) {
   return parts.join(" ");
 }
 
+QString WordyTimeNanosec(qint64 nanoseconds) {
+  return WordyTime(nanoseconds / kNsecPerSec);
+}
+
 QString Ago(int seconds_since_epoch, const QLocale& locale) {
   const QDateTime now = QDateTime::currentDateTime();
   const QDateTime then = QDateTime::fromTime_t(seconds_since_epoch);
@@ -87,7 +103,7 @@ QString Ago(int seconds_since_epoch, const QLocale& locale) {
   if (days_ago <= 7)
     return tr("%1 days ago").arg(days_ago);
 
-  return then.date().toString(locale.dateTimeFormat());
+  return then.date().toString(locale.dateFormat());
 }
 
 QString PrettySize(quint64 bytes) {
@@ -136,10 +152,13 @@ quint64 FileSystemFreeSpace(const QString& path) {
   return 0;
 }
 
-QString MakeTempDir() {
+QString MakeTempDir(const QString template_name) {
   QString path;
   {
     QTemporaryFile tempfile;
+    if (!template_name.isEmpty())
+      tempfile.setFileTemplate(template_name);
+
     tempfile.open();
     path = tempfile.fileName();
   }
@@ -152,15 +171,38 @@ QString MakeTempDir() {
 
 void RemoveRecursive(const QString& path) {
   QDir dir(path);
-  foreach (const QString& child, dir.entryList(QDir::NoDotAndDotDot | QDir::Dirs))
+  foreach (const QString& child, dir.entryList(QDir::NoDotAndDotDot | QDir::Dirs | QDir::Hidden))
     RemoveRecursive(path + "/" + child);
 
-  foreach (const QString& child, dir.entryList(QDir::NoDotAndDotDot | QDir::Files))
+  foreach (const QString& child, dir.entryList(QDir::NoDotAndDotDot | QDir::Files | QDir::Hidden))
     QFile::remove(path + "/" + child);
 
   dir.rmdir(path);
 }
 
+bool CopyRecursive(const QString& source, const QString& destination) {
+  // Make the destination directory
+  QString dir_name = source.section('/', -1, -1);
+  QString dest_path = destination + "/" + dir_name;
+  QDir().mkpath(dest_path);
+
+  QDir dir(source);
+  foreach (const QString& child, dir.entryList(QDir::NoDotAndDotDot | QDir::Dirs)) {
+    if (!CopyRecursive(source + "/" + child, dest_path)) {
+      qWarning() << "Failed to copy dir" << source + "/" + child << "to" << dest_path;
+      return false;
+    }
+  }
+
+  foreach (const QString& child, dir.entryList(QDir::NoDotAndDotDot | QDir::Files)) {
+    if (!QFile::copy(source + "/" + child, dest_path + "/" + child)) {
+      qWarning() << "Failed to copy file" << source + "/" + child << "to" << dest_path;
+      return false;
+    }
+  }
+  return true;
+}
+
 bool Copy(QIODevice* source, QIODevice* destination) {
   if (!source->open(QIODevice::ReadOnly))
     return false;
@@ -220,12 +262,43 @@ QString GetConfigPath(ConfigPath config) {
       return GetConfigPath(Path_Root) +
           QString("/gst-registry-%1-bin").arg(QCoreApplication::applicationVersion());
 
+    case Path_Scripts:
+      return GetConfigPath(Path_Root) + "/scripts";
+
+    case Path_DefaultMusicLibrary:
+      #ifdef Q_OS_DARWIN
+        return mac::GetMusicDirectory();
+      #else
+        return QDir::homePath();
+      #endif
+
     default:
       qFatal("%s", Q_FUNC_INFO);
       return QString::null;
   }
 }
 
+void OpenInFileBrowser(const QStringList& filenames) {
+  QSet<QString> dirs;
+
+  foreach (const QString& filename, filenames) {
+    // Ignore things that look like URLs
+    if (filename.contains("://"))
+      continue;
+
+    if (!QFile::exists(filename))
+      continue;
+
+    const QString directory = QFileInfo(filename).dir().path();
+
+    if (dirs.contains(directory))
+      continue;
+    dirs.insert(directory);
+
+    QDesktopServices::openUrl(QUrl::fromLocalFile(directory));
+  }
+}
+
 }  // namespace Utilities
 
 
