@@ -28,8 +28,11 @@ class QIODevice;
 
 namespace Utilities {
   QString PrettyTime(int seconds);
+  QString PrettyTimeDelta(int seconds);
+  QString PrettyTimeNanosec(qint64 nanoseconds);
   QString PrettySize(quint64 bytes);
   QString WordyTime(quint64 seconds);
+  QString WordyTimeNanosec(qint64 nanoseconds);
   QString Ago(int seconds_since_epoch, const QLocale& locale);
 
   QString ColorToRgba(const QColor& color);
@@ -37,16 +40,21 @@ namespace Utilities {
   quint64 FileSystemCapacity(const QString& path);
   quint64 FileSystemFreeSpace(const QString& path);
 
-  QString MakeTempDir();
+  QString MakeTempDir(const QString template_name = QString());
   void RemoveRecursive(const QString& path);
+  bool CopyRecursive(const QString& source, const QString& destination);
   bool Copy(QIODevice* source, QIODevice* destination);
 
+  void OpenInFileBrowser(const QStringList& filenames);
+
 
   enum ConfigPath {
     Path_Root,
     Path_AlbumCovers,
     Path_NetworkCache,
     Path_GstreamerRegistry,
+    Path_DefaultMusicLibrary,
+    Path_Scripts,
   };
   QString GetConfigPath(ConfigPath config);
 }
