@@ -47,8 +47,12 @@ void DeviceKitLister::Init() {
       OrgFreedesktopUDisksInterface::staticInterfaceName(),
       "/org/freedesktop/UDisks", QDBusConnection::systemBus()));
 
-  if (!interface_->isValid()) {
-    qWarning() << "Error connecting to the DeviceKit-disks DBUS service";
+  // Get all the devices currently attached
+  QDBusPendingReply<QList<QDBusObjectPath> > reply = interface_->EnumerateDevices();
+  reply.waitForFinished();
+
+  if (!reply.isValid()) {
+    qWarning() << "Error enumerating DeviceKit-disks devices:" << reply.error().name() << reply.error().message();
     interface_.reset();
     return;
   }
@@ -58,15 +62,6 @@ void DeviceKitLister::Init() {
   connect(interface_.get(), SIGNAL(DeviceRemoved(QDBusObjectPath)), SLOT(DBusDeviceRemoved(QDBusObjectPath)));
   connect(interface_.get(), SIGNAL(DeviceChanged(QDBusObjectPath)), SLOT(DBusDeviceChanged(QDBusObjectPath)));
 
-  // Get all the devices currently attached
-  QDBusPendingReply<QList<QDBusObjectPath> > reply = interface_->EnumerateDevices();
-  reply.waitForFinished();
-
-  if (!reply.isValid()) {
-    qWarning() << "Error enumerating DeviceKit-disks devices:" << reply.error().name() << reply.error().message();
-    return;
-  }
-
   // Get information about each one
   QMap<QString, DeviceData> device_data;
   foreach (const QDBusObjectPath& path, reply.value()) {
