@@ -21,8 +21,10 @@
 #include "deviceview.h"
 #include "core/deletefiles.h"
 #include "core/mergedproxymodel.h"
+#include "core/mimedata.h"
 #include "library/librarydirectorymodel.h"
 #include "library/librarymodel.h"
+#include "library/libraryview.h"
 #include "ui/iconloader.h"
 #include "ui/organisedialog.h"
 #include "ui/organiseerrordialog.h"
@@ -209,10 +211,12 @@ void DeviceView::contextMenuEvent(QContextMenuEvent* e) {
         IconLoader::Load("configure"), tr("Device properties..."), this, SLOT(Properties()));
 
     // Library menu
-    load_action_ = library_menu_->addAction(IconLoader::Load("media-playback-start"),
-        tr("Load"), this, SLOT(Load()));
     add_to_playlist_action_ = library_menu_->addAction(IconLoader::Load("media-playback-start"),
-        tr("Add to playlist"), this, SLOT(AddToPlaylist()));
+        tr("Append to current playlist"), this, SLOT(AddToPlaylist()));
+    load_action_ = library_menu_->addAction(IconLoader::Load("media-playback-start"),
+        tr("Replace current playlist"), this, SLOT(Load()));
+    open_in_new_playlist_ = library_menu_->addAction(IconLoader::Load("document-new"),
+        tr("Open in new playlist"), this, SLOT(OpenInNewPlaylist()));
     library_menu_->addSeparator();
     organise_action_ = library_menu_->addAction(IconLoader::Load("edit-copy"),
         tr("Copy to library..."), this, SLOT(Organise()));
@@ -331,12 +335,6 @@ void DeviceView::mouseDoubleClickEvent(QMouseEvent *event) {
       menu_index_ = merged_index;
       Connect();
     }
-    return;
-  }
-
-  QModelIndex library_index = MapToLibrary(merged_index);
-  if (library_index.isValid()) {
-    emit DoubleClicked(GetSelectedSongs());
   }
 }
 
@@ -359,11 +357,23 @@ SongList DeviceView::GetSelectedSongs() const {
 }
 
 void DeviceView::Load() {
-  emit Load(GetSelectedSongs());
+  QMimeData* data = model()->mimeData(selectedIndexes());
+  if (MimeData* mime_data = qobject_cast<MimeData*>(data)) {
+    mime_data->clear_first_ = true;
+  }
+  emit AddToPlaylistSignal(data);
 }
 
 void DeviceView::AddToPlaylist() {
-  emit AddToPlaylist(GetSelectedSongs());
+  emit AddToPlaylistSignal(model()->mimeData(selectedIndexes()));
+}
+
+void DeviceView::OpenInNewPlaylist() {
+  QMimeData* data = model()->mimeData(selectedIndexes());
+  if (MimeData* mime_data = qobject_cast<MimeData*>(data)) {
+    mime_data->open_in_new_playlist_ = true;
+  }
+  emit AddToPlaylistSignal(data);
 }
 
 void DeviceView::Delete() {
