@@ -24,7 +24,6 @@
 #include "config.h"
 #include "gstengine.h"
 #include "gstenginepipeline.h"
-#include "core/boundfuturewatcher.h"
 #include "core/utilities.h"
 
 #ifdef HAVE_IMOBILEDEVICE
@@ -64,7 +63,6 @@ const char* GstEngine::kHypnotoadPipeline =
       "band0=-24 band1=-3 band2=7.5 band3=12 band4=8 "
       "band5=6 band6=5 band7=6 band8=0 band9=-24";
 
-
 GstEngine::GstEngine()
   : Engine::Base(),
     delayq_(g_queue_new()),
@@ -74,13 +72,13 @@ GstEngine::GstEngine()
     rg_mode_(0),
     rg_preamp_(0.0),
     rg_compression_(true),
-    buffer_duration_ms_(1000),
+    buffer_duration_nanosec_(1 * kNsecPerSec), // 1s
     seek_timer_(new QTimer(this)),
     timer_id_(-1),
     next_element_id_(0)
 {
   seek_timer_->setSingleShot(true);
-  seek_timer_->setInterval(kSeekDelay);
+  seek_timer_->setInterval(kSeekDelayNanosec / kNsecPerMsec);
   connect(seek_timer_, SIGNAL(timeout()), SLOT(SeekNow()));
 
   ReloadSettings();
@@ -167,116 +165,26 @@ void GstEngine::ReloadSettings() {
   rg_preamp_ = s.value("rgpreamp", 0.0).toDouble();
   rg_compression_ = s.value("rgcompression", true).toBool();
 
-  buffer_duration_ms_ = s.value("bufferduration", 1000).toInt();
+  buffer_duration_nanosec_ = s.value("bufferduration", 1000).toLongLong() * kNsecPerMsec;
 }
 
 
-bool GstEngine::CanDecode(const QUrl &url) {
-  EnsureInitialised();
-
-  // We had some bug reports claiming that video files cause crashes in canDecode(),
-  // so don't try to decode them
-  if ( url.path().toLower().endsWith( ".mov" ) ||
-       url.path().toLower().endsWith( ".avi" ) ||
-       url.path().toLower().endsWith( ".wmv" ) )
-    return false;
-
-  can_decode_success_ = false;
-  can_decode_last_ = false;
-
-  // Create the pipeline
-  shared_ptr<GstElement> pipeline(gst_pipeline_new("pipeline"),
-                                  boost::bind(gst_object_unref, _1));
-  if (!pipeline) return false;
-  GstElement* src = CreateElement("giosrc", pipeline.get());    if (!src) return false;
-  GstElement* bin = CreateElement("decodebin2", pipeline.get()); if (!bin) return false;
-
-  gst_element_link(src, bin);
-  g_signal_connect(G_OBJECT(bin), "new-decoded-pad", G_CALLBACK(CanDecodeNewPadCallback), this);
-  g_signal_connect(G_OBJECT(bin), "no-more-pads", G_CALLBACK(CanDecodeLastCallback), this);
-
-  // These handlers just print out errors to stderr
-  gst_bus_set_sync_handler(gst_pipeline_get_bus(GST_PIPELINE(pipeline.get())), CanDecodeBusCallbackSync, 0);
-  gst_bus_add_watch(gst_pipeline_get_bus(GST_PIPELINE(pipeline.get())), CanDecodeBusCallback, 0);
-
-  // Set the file we're testing
-  g_object_set(G_OBJECT(src), "location", url.toEncoded().constData(), NULL);
-
-  // Start the pipeline playing
-  gst_element_set_state(pipeline.get(), GST_STATE_PLAYING);
-
-  // Wait until found audio stream
-  int count = 0;
-  while (!can_decode_success_ && !can_decode_last_ && count < 100) {
-    count++;
-    usleep(1000);
-  }
-
-  // Stop playing
-  gst_element_set_state(pipeline.get(), GST_STATE_NULL);
-
-  return can_decode_success_;
-}
-
-
-
-void GstEngine::CanDecodeNewPadCallback(GstElement*, GstPad* pad, gboolean, gpointer self) {
-  GstEngine* instance = reinterpret_cast<GstEngine*>(self);
-
-  GstCaps* caps = gst_pad_get_caps(pad);
-  if (gst_caps_get_size(caps) > 0) {
-    GstStructure* str = gst_caps_get_structure(caps, 0);
-    if (g_strrstr(gst_structure_get_name( str ), "audio" ))
-      instance->can_decode_success_ = true;
-  }
-  gst_caps_unref(caps);
-}
-
-void GstEngine::CanDecodeLastCallback(GstElement*, gpointer self) {
-  GstEngine* instance = reinterpret_cast<GstEngine*>(self);
-  instance->can_decode_last_ = true;
-}
-
-void GstEngine::PrintGstError(GstMessage *msg) {
-  GError* error;
-  gchar* debugs;
-
-  gst_message_parse_error(msg, &error, &debugs);
-  qDebug() << error->message;
-  qDebug() << debugs;
-
-  g_error_free(error);
-  free(debugs);
-}
-
-GstBusSyncReply GstEngine::CanDecodeBusCallbackSync(GstBus*, GstMessage* msg, gpointer) {
-  if (GST_MESSAGE_TYPE(msg) == GST_MESSAGE_ERROR)
-    PrintGstError(msg);
-  return GST_BUS_PASS;
-}
-
-gboolean GstEngine::CanDecodeBusCallback(GstBus*, GstMessage* msg, gpointer) {
-  if (GST_MESSAGE_TYPE(msg) == GST_MESSAGE_ERROR)
-    PrintGstError(msg);
-  return GST_BUS_DROP;
-}
-
-
-uint GstEngine::position() const {
+qint64 GstEngine::position_nanosec() const {
   if (!current_pipeline_)
     return 0;
 
-  return uint(current_pipeline_->position() / GST_MSECOND);
+  qint64 result = current_pipeline_->position() - beginning_nanosec_;
+  return qint64(qMax(0ll, result));
 }
 
-uint GstEngine::length() const {
+qint64 GstEngine::length_nanosec() const {
   if (!current_pipeline_)
     return 0;
 
-  return uint(current_pipeline_->length() / GST_MSECOND);
+  qint64 result = end_nanosec_ - beginning_nanosec_;
+  return qint64(qMax(0ll, result));
 }
 
-
 Engine::State GstEngine::state() const {
   if (!current_pipeline_)
     return url_.isEmpty() ? Engine::Empty : Engine::Idle;
@@ -290,18 +198,18 @@ Engine::State GstEngine::state() const {
   }
 }
 
-void GstEngine::ConsumeBuffer(GstBuffer *buffer, GstEnginePipeline* pipeline) {
+void GstEngine::ConsumeBuffer(GstBuffer* buffer, int pipeline_id) {
   // Schedule this to run in the GUI thread.  The buffer gets added to the
   // queue and unreffed by UpdateScope.
   if (!QMetaObject::invokeMethod(this, "AddBufferToScope",
                                  Q_ARG(GstBuffer*, buffer),
-                                 Q_ARG(GstEnginePipeline*, pipeline))) {
+                                 Q_ARG(int, pipeline_id))) {
     qWarning() << "Failed to invoke AddBufferToScope on GstEngine";
   }
 }
 
-void GstEngine::AddBufferToScope(GstBuffer* buf, GstEnginePipeline* pipeline) {
-  if (current_pipeline_.get() != pipeline) {
+void GstEngine::AddBufferToScope(GstBuffer* buf, int pipeline_id) {
+  if (!current_pipeline_ || current_pipeline_->id() != pipeline_id) {
     gst_buffer_unref(buf);
     return;
   }
@@ -399,79 +307,65 @@ void GstEngine::UpdateScope() {
   }
 }
 
-void GstEngine::StartPreloading(const QUrl& url) {
+void GstEngine::StartPreloading(const QUrl& url, bool force_stop_at_end,
+                                qint64 beginning_nanosec, qint64 end_nanosec) {
   EnsureInitialised();
 
-  if (autocrossfade_enabled_) {
-    // Have to create a new pipeline so we can crossfade between the two
+  QUrl gst_url = FixupUrl(url);
 
-    preload_pipeline_ = CreatePipeline(url);
-    if (!preload_pipeline_)
-      return;
-
-    // We don't want to get metadata messages before the track starts playing -
-    // we reconnect this in GstEngine::Load
-    disconnect(preload_pipeline_.get(), SIGNAL(MetadataFound(Engine::SimpleMetaBundle)), this, 0);
-
-    preloaded_url_ = url;
-    preload_pipeline_->SetState(GST_STATE_PAUSED);
-  } else {
-    // No crossfading, so we can just queue the new URL in the existing
-    // pipeline and get gapless playback (hopefully)
-    if (current_pipeline_)
-      current_pipeline_->SetNextUrl(url);
-  }
+  // No crossfading, so we can just queue the new URL in the existing
+  // pipeline and get gapless playback (hopefully)
+  if (current_pipeline_)
+    current_pipeline_->SetNextUrl(gst_url, beginning_nanosec,
+        force_stop_at_end ? end_nanosec : 0);
 }
 
-bool GstEngine::Load(const QUrl& url, Engine::TrackChangeType change) {
-  EnsureInitialised();
-
-  Engine::Base::Load(url, change);
-
-  // Clementine just crashes when asked to load a file that doesn't exist on
-  // Windows, so check for that here.  This is definitely the wrong place for
-  // this "fix"...
-  if (url.scheme() == "file" && !QFile::exists(url.toLocalFile()))
-    return false;
-
-  QUrl gst_url = url;
+QUrl GstEngine::FixupUrl(const QUrl& url) {
+  QUrl copy = url;
 
   // It's a file:// url with a hostname set.  QUrl::fromLocalFile does this
   // when given a \\host\share\file path on Windows.  Munge it back into a
   // path that gstreamer will recognise.
   if (url.scheme() == "file" && !url.host().isEmpty()) {
-    gst_url.setPath("//" + gst_url.host() + gst_url.path());
-    gst_url.setHost(QString());
+    copy.setPath("//" + copy.host() + copy.path());
+    copy.setHost(QString());
   }
 
-  const bool crossfade = current_pipeline_ &&
-                         ((crossfade_enabled_ && change == Engine::Manual) ||
-                          (autocrossfade_enabled_ && change == Engine::Auto));
+  return copy;
+}
+
+bool GstEngine::Load(const QUrl& url, Engine::TrackChangeFlags change,
+                     bool force_stop_at_end,
+                     quint64 beginning_nanosec, qint64 end_nanosec) {
+  EnsureInitialised();
+
+  Engine::Base::Load(url, change, force_stop_at_end, beginning_nanosec, end_nanosec);
+
+  QUrl gst_url = FixupUrl(url);
+
+  bool crossfade = current_pipeline_ &&
+                   ((crossfade_enabled_ && change & Engine::Manual) ||
+                    (autocrossfade_enabled_ && change & Engine::Auto));
+
+  if (change & Engine::Auto && change & Engine::SameAlbum && !crossfade_same_album_)
+    crossfade = false;
 
   if (!crossfade && current_pipeline_ && current_pipeline_->url() == gst_url &&
-      change == Engine::Auto) {
+      change & Engine::Auto) {
     // We're not crossfading, and the pipeline is already playing the URI we
     // want, so just do nothing.
     return true;
   }
 
-  shared_ptr<GstEnginePipeline> pipeline;
-  if (preload_pipeline_ && preloaded_url_ == gst_url) {
-    pipeline = preload_pipeline_;
-    connect(preload_pipeline_.get(),
-            SIGNAL(MetadataFound(Engine::SimpleMetaBundle)),
-            SLOT(NewMetaData(Engine::SimpleMetaBundle)));
-  } else {
-    pipeline = CreatePipeline(gst_url);
-    if (!pipeline)
-      return false;
-  }
+  shared_ptr<GstEnginePipeline> pipeline = CreatePipeline(gst_url,
+      force_stop_at_end ? end_nanosec : 0);
+  if (!pipeline)
+    return false;
 
   if (crossfade)
     StartFadeout();
 
   current_pipeline_ = pipeline;
-  preload_pipeline_.reset();
 
   SetVolume(volume_);
   SetEqualizerEnabled(equalizer_enabled_);
@@ -479,7 +373,7 @@ bool GstEngine::Load(const QUrl& url, Engine::TrackChangeType change) {
 
   // Maybe fade in this track
   if (crossfade)
-    current_pipeline_->StartFader(fadeout_duration_, QTimeLine::Forward);
+    current_pipeline_->StartFader(fadeout_duration_nanosec_, QTimeLine::Forward);
 
   return true;
 }
@@ -490,20 +384,20 @@ void GstEngine::StartFadeout() {
   fadeout_pipeline_->RemoveAllBufferConsumers();
   ClearScopeBuffers();
 
-  fadeout_pipeline_->StartFader(fadeout_duration_, QTimeLine::Backward);
+  fadeout_pipeline_->StartFader(fadeout_duration_nanosec_, QTimeLine::Backward);
   connect(fadeout_pipeline_.get(), SIGNAL(FaderFinished()), SLOT(FadeoutFinished()));
 }
 
 
-bool GstEngine::Play( uint offset ) {
+bool GstEngine::Play(quint64 offset_nanosec) {
   EnsureInitialised();
 
   if (!current_pipeline_)
     return false;
 
   QFuture<GstStateChangeReturn> future = current_pipeline_->SetState(GST_STATE_PLAYING);
-  BoundFutureWatcher<GstStateChangeReturn, uint>* watcher =
-      new BoundFutureWatcher<GstStateChangeReturn, uint>(offset, this);
+  PlayFutureWatcher* watcher = new PlayFutureWatcher(
+        PlayFutureWatcherArg(offset_nanosec, current_pipeline_->id()), this);
   watcher->setFuture(future);
   connect(watcher, SIGNAL(finished()), SLOT(PlayDone()));
 
@@ -511,21 +405,23 @@ bool GstEngine::Play( uint offset ) {
 }
 
 void GstEngine::PlayDone() {
-  BoundFutureWatcher<GstStateChangeReturn, uint>* watcher =
-      static_cast<BoundFutureWatcher<GstStateChangeReturn, uint>*>(sender());
+  PlayFutureWatcher* watcher = static_cast<PlayFutureWatcher*>(sender());
   watcher->deleteLater();
-  GstStateChangeReturn ret = watcher->result();
 
-  if (!current_pipeline_)
+  GstStateChangeReturn ret = watcher->result();
+  quint64 offset_nanosec = watcher->data().first;
+
+  if (!current_pipeline_ || watcher->data().second != current_pipeline_->id()) {
     return;
+  }
 
   if (ret == GST_STATE_CHANGE_FAILURE) {
     // Failure, but we got a redirection URL - try loading that instead
     QUrl redirect_url = current_pipeline_->redirect_url();
     if (!redirect_url.isEmpty() && redirect_url != current_pipeline_->url()) {
       qDebug() << "Redirecting to" << redirect_url;
-      current_pipeline_ = CreatePipeline(redirect_url);
-      Play(watcher->data());
+      current_pipeline_ = CreatePipeline(redirect_url, end_nanosec_);
+      Play(offset_nanosec);
       return;
     }
 
@@ -539,11 +435,14 @@ void GstEngine::PlayDone() {
 
   current_sample_ = 0;
 
-  if (watcher->data()) {
-    Seek(watcher->data());
+  // initial offset
+  if(offset_nanosec != 0 || beginning_nanosec_ != 0) {
+    Seek(offset_nanosec);
   }
 
   emit StateChanged(Engine::Playing);
+  // we've successfully started playing a media stream with this url
+  emit ValidSongRequested(url_);
 }
 
 
@@ -551,6 +450,7 @@ void GstEngine::Stop() {
   StopTimers();
 
   url_ = QUrl(); // To ensure we return Empty from state()
+  beginning_nanosec_ = end_nanosec_ = 0;
 
   if (fadeout_enabled_ && current_pipeline_)
     StartFadeout();
@@ -561,6 +461,7 @@ void GstEngine::Stop() {
 
 void GstEngine::FadeoutFinished() {
   fadeout_pipeline_.reset();
+  emit FadeoutFinishedSignal();
 }
 
 void GstEngine::Pause() {
@@ -587,11 +488,11 @@ void GstEngine::Unpause() {
   }
 }
 
-void GstEngine::Seek(uint ms) {
+void GstEngine::Seek(quint64 offset_nanosec) {
   if (!current_pipeline_)
     return;
 
-  seek_pos_ = ms;
+  seek_pos_ = beginning_nanosec_ + offset_nanosec;
   waiting_to_seek_ = true;
 
   if (!seek_timer_->isActive()) {
@@ -607,7 +508,7 @@ void GstEngine::SeekNow() {
   if (!current_pipeline_)
     return;
 
-  if (current_pipeline_->Seek(seek_pos_ * GST_MSECOND))
+  if (current_pipeline_->Seek(seek_pos_))
     ClearScopeBuffers();
   else
     qDebug() << "Seek failed";
@@ -637,7 +538,7 @@ void GstEngine::SetVolumeSW( uint percent ) {
 void GstEngine::StartTimers() {
   StopTimers();
 
-  timer_id_ = startTimer(kTimerInterval);
+  timer_id_ = startTimer(kTimerIntervalNanosec / kNsecPerMsec);
 }
 
 void GstEngine::StopTimers() {
@@ -656,36 +557,65 @@ void GstEngine::timerEvent(QTimerEvent* e) {
   // we are fading
   PruneScope();
 
-  // Emit TrackAboutToEnd when we're a few seconds away from finishing
   if (current_pipeline_) {
-    const qint64 nanosec_position = current_pipeline_->position();
-    const qint64 nanosec_length = current_pipeline_->length();
-    const qint64 remaining = (nanosec_length - nanosec_position) / 1000000;
-    const qint64 fudge = kTimerInterval + 100; // Mmm fudge
-    const qint64 gap = autocrossfade_enabled_ ? fadeout_duration_ : kPreloadGap;
+    const qint64 current_position = position_nanosec();
+    const qint64 current_length = length_nanosec();
 
-    if (nanosec_length > 0 && remaining < gap + fudge)
-      EmitAboutToEnd();
+    const qint64 remaining = current_length - current_position;
+
+    const qint64 fudge = kTimerIntervalNanosec + 100 * kNsecPerMsec; // Mmm fudge
+    const qint64 gap = autocrossfade_enabled_ ? fadeout_duration_nanosec_ : kPreloadGapNanosec;
+
+    // only if we know the length of the current stream...
+    if(current_length > 0) {
+      // emit TrackAboutToEnd when we're a few seconds away from finishing
+      if (remaining < gap + fudge) {
+        EmitAboutToEnd();
+      }
+    }
   }
 }
 
-void GstEngine::HandlePipelineError(const QString& message) {
+void GstEngine::HandlePipelineError(int pipeline_id, const QString& message,
+                                    int domain, int error_code) {
+  if (!current_pipeline_.get() || current_pipeline_->id() != pipeline_id)
+    return;
+
   qWarning() << "Gstreamer error:" << message;
 
   current_pipeline_.reset();
-  emit Error(message);
+
   emit StateChanged(Engine::Empty);
+  // unable to play media stream with this url
+  emit InvalidSongRequested(url_);
+
+  // TODO: the types of errors listed below won't be shown to user - they will 
+  // get logged and the current song will be skipped; instead of maintaining 
+  // the list we should probably:
+  // - don't report any engine's errors to user (always just log and skip)
+  // - come up with a less intrusive error box (not a dialog but a notification
+  //   popup of some kind) and then report all errors
+  if(!(domain == GST_RESOURCE_ERROR && error_code == GST_RESOURCE_ERROR_NOT_FOUND) &&
+     !(domain == GST_STREAM_ERROR && error_code == GST_STREAM_ERROR_TYPE_NOT_FOUND) &&
+     !(domain == GST_RESOURCE_ERROR && error_code == GST_RESOURCE_ERROR_OPEN_READ)) {
+    emit Error(message);
+  }
 }
 
+void GstEngine::EndOfStreamReached(int pipeline_id, bool has_next_track) {
+  if (!current_pipeline_.get() || current_pipeline_->id() != pipeline_id)
+    return;
 
-void GstEngine::EndOfStreamReached(bool has_next_track) {
   if (!has_next_track)
     current_pipeline_.reset();
   ClearScopeBuffers();
   emit TrackEnded();
 }
 
-void GstEngine::NewMetaData(const Engine::SimpleMetaBundle& bundle) {
+void GstEngine::NewMetaData(int pipeline_id, const Engine::SimpleMetaBundle& bundle) {
+  if (!current_pipeline_.get() || current_pipeline_->id() != pipeline_id)
+    return;
+
   emit MetaData(bundle);
 }
 
@@ -744,22 +674,24 @@ shared_ptr<GstEnginePipeline> GstEngine::CreatePipeline() {
   shared_ptr<GstEnginePipeline> ret(new GstEnginePipeline(this));
   ret->set_output_device(sink_, device_);
   ret->set_replaygain(rg_enabled_, rg_mode_, rg_preamp_, rg_compression_);
-  ret->set_buffer_duration_ms(buffer_duration_ms_);
+  ret->set_buffer_duration_nanosec(buffer_duration_nanosec_);
 
   ret->AddBufferConsumer(this);
-  foreach (BufferConsumer* consumer, buffer_consumers_)
+  foreach (BufferConsumer* consumer, buffer_consumers_) {
     ret->AddBufferConsumer(consumer);
+  }
 
-  connect(ret.get(), SIGNAL(EndOfStreamReached(bool)), SLOT(EndOfStreamReached(bool)));
-  connect(ret.get(), SIGNAL(Error(QString)), SLOT(HandlePipelineError(QString)));
-  connect(ret.get(), SIGNAL(MetadataFound(Engine::SimpleMetaBundle)),
-          SLOT(NewMetaData(Engine::SimpleMetaBundle)));
+  connect(ret.get(), SIGNAL(EndOfStreamReached(int, bool)), SLOT(EndOfStreamReached(int, bool)));
+  connect(ret.get(), SIGNAL(Error(int, QString,int,int)), SLOT(HandlePipelineError(int, QString,int,int)));
+  connect(ret.get(), SIGNAL(MetadataFound(int, Engine::SimpleMetaBundle)),
+          SLOT(NewMetaData(int, Engine::SimpleMetaBundle)));
   connect(ret.get(), SIGNAL(destroyed()), SLOT(ClearScopeBuffers()));
 
   return ret;
 }
 
-shared_ptr<GstEnginePipeline> GstEngine::CreatePipeline(const QUrl& url) {
+shared_ptr<GstEnginePipeline> GstEngine::CreatePipeline(const QUrl& url,
+                                                        qint64 end_nanosec) {
   shared_ptr<GstEnginePipeline> ret = CreatePipeline();
 
   if (url.scheme() == "hypnotoad") {
@@ -767,7 +699,7 @@ shared_ptr<GstEnginePipeline> GstEngine::CreatePipeline(const QUrl& url) {
     return ret;
   }
 
-  if (!ret->InitFromUrl(url))
+  if (!ret->InitFromUrl(url, end_nanosec))
     ret.reset();
 
   return ret;
@@ -864,12 +796,12 @@ void GstEngine::BackgroundStreamPlayDone() {
 }
 
 int GstEngine::AddBackgroundStream(const QUrl& url) {
-  shared_ptr<GstEnginePipeline> pipeline = CreatePipeline(url);
+  shared_ptr<GstEnginePipeline> pipeline = CreatePipeline(url, 0);
   if (!pipeline) {
     return -1;
   }
   pipeline->SetVolume(30);
-  pipeline->SetNextUrl(url);
+  pipeline->SetNextUrl(url, 0, 0);
   return AddBackgroundStream(pipeline);
 }
 
@@ -879,7 +811,7 @@ void GstEngine::StopBackgroundStream(int id) {
 
 void GstEngine::BackgroundStreamFinished() {
   GstEnginePipeline* pipeline = qobject_cast<GstEnginePipeline*>(sender());
-  pipeline->SetNextUrl(pipeline->url());
+  pipeline->SetNextUrl(pipeline->url(), 0, 0);
 }
 
 void GstEngine::SetBackgroundStreamVolume(int id, int volume) {
