@@ -22,8 +22,10 @@
 #ifndef AMAROK_GSTENGINE_H
 #define AMAROK_GSTENGINE_H
 
-#include "enginebase.h"
 #include "bufferconsumer.h"
+#include "enginebase.h"
+#include "core/boundfuturewatcher.h"
+#include "core/timeconstants.h"
 
 #include <QFuture>
 #include <QHash>
@@ -66,14 +68,12 @@ class GstEngine : public Engine::Base, public BufferConsumer {
   bool Init();
   void EnsureInitialised() { initialising_.waitForFinished(); }
 
-  bool CanDecode(const QUrl& url);
-
   int AddBackgroundStream(const QUrl& url);
   void StopBackgroundStream(int id);
   void SetBackgroundStreamVolume(int id, int volume);
 
-  uint position() const;
-  uint length() const;
+  qint64 position_nanosec() const;
+  qint64 length_nanosec() const;
   Engine::State state() const;
   const Engine::Scope& scope();
 
@@ -83,16 +83,19 @@ class GstEngine : public Engine::Base, public BufferConsumer {
   GstElement* CreateElement(const QString& factoryName, GstElement* bin = 0);
 
   // BufferConsumer
-  void ConsumeBuffer(GstBuffer *buffer, GstEnginePipeline* pipeline);
+  void ConsumeBuffer(GstBuffer *buffer, int pipeline_id);
 
  public slots:
-  void StartPreloading(const QUrl &);
-  bool Load(const QUrl&, Engine::TrackChangeType change);
-  bool Play(uint offset);
+  void StartPreloading(const QUrl& url, bool force_stop_at_end,
+                       qint64 beginning_nanosec, qint64 end_nanosec);
+  bool Load(const QUrl&, Engine::TrackChangeFlags change,
+            bool force_stop_at_end,
+            quint64 beginning_nanosec, qint64 end_nanosec);
+  bool Play(quint64 offset_nanosec);
   void Stop();
   void Pause();
   void Unpause();
-  void Seek(uint ms);
+  void Seek(quint64 offset_nanosec);
 
   /** Set whether equalizer is enabled */
   void SetEqualizerEnabled(bool);
@@ -110,11 +113,11 @@ class GstEngine : public Engine::Base, public BufferConsumer {
   void timerEvent(QTimerEvent*);
 
  private slots:
-  void EndOfStreamReached(bool has_next_track);
-  void HandlePipelineError(const QString& message);
-  void NewMetaData(const Engine::SimpleMetaBundle& bundle);
+  void EndOfStreamReached(int pipeline_id, bool has_next_track);
+  void HandlePipelineError(int pipeline_id, const QString& message, int domain, int error_code);
+  void NewMetaData(int pipeline_id, const Engine::SimpleMetaBundle& bundle);
   void ClearScopeBuffers();
-  void AddBufferToScope(GstBuffer* buf, GstEnginePipeline* pipeline);
+  void AddBufferToScope(GstBuffer* buf, int pipeline_id);
   void FadeoutFinished();
   void SeekNow();
   void BackgroundStreamFinished();
@@ -122,12 +125,8 @@ class GstEngine : public Engine::Base, public BufferConsumer {
   void PlayDone();
 
  private:
-  // Callbacks
-  static void CanDecodeNewPadCallback(GstElement*, GstPad*, gboolean, gpointer);
-  static void CanDecodeLastCallback(GstElement*, gpointer);
-  static GstBusSyncReply CanDecodeBusCallbackSync(GstBus*, GstMessage*, gpointer);
-  static gboolean CanDecodeBusCallback(GstBus*, GstMessage*, gpointer);
-  static void PrintGstError(GstMessage* msg);
+  typedef QPair<quint64, int> PlayFutureWatcherArg;
+  typedef BoundFutureWatcher<GstStateChangeReturn, PlayFutureWatcherArg> PlayFutureWatcher;
 
   static void SetEnv(const char* key, const QString& value);
   PluginDetailsList GetPluginList(const QString& classname) const;
@@ -139,17 +138,19 @@ class GstEngine : public Engine::Base, public BufferConsumer {
   void StopTimers();
 
   boost::shared_ptr<GstEnginePipeline> CreatePipeline();
-  boost::shared_ptr<GstEnginePipeline> CreatePipeline(const QUrl& url);
+  boost::shared_ptr<GstEnginePipeline> CreatePipeline(const QUrl& url, qint64 end_nanosec);
 
   void UpdateScope();
   qint64 PruneScope();
 
   int AddBackgroundStream(boost::shared_ptr<GstEnginePipeline> pipeline);
 
+  static QUrl FixupUrl(const QUrl& url);
+
  private:
-  static const int kTimerInterval = 1000; // msec
-  static const int kPreloadGap = 1000; // msec
-  static const int kSeekDelay = 100; // msec
+  static const int kTimerIntervalNanosec = 1000 * kNsecPerMsec; // 1s
+  static const int kPreloadGapNanosec = 1000 * kNsecPerMsec; // 1s
+  static const int kSeekDelayNanosec = 100 * kNsecPerMsec; // 100msec
 
   static const char* kHypnotoadPipeline;
 
@@ -160,7 +161,6 @@ class GstEngine : public Engine::Base, public BufferConsumer {
 
   boost::shared_ptr<GstEnginePipeline> current_pipeline_;
   boost::shared_ptr<GstEnginePipeline> fadeout_pipeline_;
-  boost::shared_ptr<GstEnginePipeline> preload_pipeline_;
   QUrl preloaded_url_;
 
   QList<BufferConsumer*> buffer_consumers_;
@@ -178,7 +178,7 @@ class GstEngine : public Engine::Base, public BufferConsumer {
   float rg_preamp_;
   bool rg_compression_;
 
-  int buffer_duration_ms_;
+  qint64 buffer_duration_nanosec_;
 
   mutable bool can_decode_success_;
   mutable bool can_decode_last_;
@@ -186,7 +186,7 @@ class GstEngine : public Engine::Base, public BufferConsumer {
   // Hack to stop seeks happening too often
   QTimer* seek_timer_;
   bool waiting_to_seek_;
-  uint seek_pos_;
+  quint64 seek_pos_;
 
   int timer_id_;
   int next_element_id_;
