@@ -32,12 +32,14 @@ const int GstEnginePipeline::kEqBandCount = 10;
 const int GstEnginePipeline::kEqBandFrequencies[] = {
   60, 170, 310, 600, 1000, 3000, 6000, 12000, 14000, 16000};
 
+int GstEnginePipeline::sId = 1;
 GstElementDeleter* GstEnginePipeline::sElementDeleter = NULL;
 
 
 GstEnginePipeline::GstEnginePipeline(GstEngine* engine)
   : QObject(NULL),
     engine_(engine),
+    id_(sId++),
     valid_(false),
     sink_(GstEngine::kAutoSink),
     segment_start_(0),
@@ -48,8 +50,15 @@ GstEnginePipeline::GstEnginePipeline(GstEngine* engine)
     rg_mode_(0),
     rg_preamp_(0.0),
     rg_compression_(true),
-    buffer_duration_ms_(1000),
+    buffer_duration_nanosec_(1 * kNsecPerSec),
+    end_offset_nanosec_(-1),
+    next_beginning_offset_nanosec_(-1),
+    next_end_offset_nanosec_(-1),
+    ignore_next_seek_(false),
     ignore_tags_(false),
+    pipeline_is_initialised_(false),
+    pipeline_is_connected_(false),
+    pending_seek_nanosec_(-1),
     volume_percent_(100),
     volume_modifier_(1.0),
     fader_(NULL),
@@ -86,8 +95,8 @@ void GstEnginePipeline::set_replaygain(bool enabled, int mode, float preamp,
   rg_compression_ = compression;
 }
 
-void GstEnginePipeline::set_buffer_duration_ms(int buffer_duration_ms) {
-  buffer_duration_ms_ = buffer_duration_ms;
+void GstEnginePipeline::set_buffer_duration_nanosec(qint64 buffer_duration_nanosec) {
+  buffer_duration_nanosec_ = buffer_duration_nanosec;
 }
 
 bool GstEnginePipeline::ReplaceDecodeBin(GstElement* new_bin) {
@@ -104,6 +113,7 @@ bool GstEnginePipeline::ReplaceDecodeBin(GstElement* new_bin) {
   uridecodebin_ = new_bin;
   segment_start_ = 0;
   segment_start_received_ = false;
+  pipeline_is_connected_ = false;
   gst_bin_add(GST_BIN(pipeline_), uridecodebin_);
 
   return true;
@@ -112,7 +122,7 @@ bool GstEnginePipeline::ReplaceDecodeBin(GstElement* new_bin) {
 bool GstEnginePipeline::ReplaceDecodeBin(const QUrl& url) {
   GstElement* new_bin = engine_->CreateElement("uridecodebin");
   g_object_set(G_OBJECT(new_bin), "uri", url.toEncoded().constData(), NULL);
-  g_object_set(G_OBJECT(new_bin), "buffer-duration", buffer_duration_ms_ * 1000 * 1000, NULL);
+  g_object_set(G_OBJECT(new_bin), "buffer-duration", buffer_duration_nanosec_, NULL);
   g_object_set(G_OBJECT(new_bin), "download", true, NULL);
   g_object_set(G_OBJECT(new_bin), "use-buffering", true, NULL);
   g_signal_connect(G_OBJECT(new_bin), "drained", G_CALLBACK(SourceDrainedCallback), this);
@@ -123,16 +133,20 @@ bool GstEnginePipeline::ReplaceDecodeBin(const QUrl& url) {
 GstElement* GstEnginePipeline::CreateDecodeBinFromString(const char* pipeline) {
   GError* error = NULL;
   GstElement* bin = gst_parse_bin_from_description(pipeline, TRUE, &error);
+
   if (error) {
     QString message = QString::fromLocal8Bit(error->message);
+    int domain = error->domain;
+    int code = error->code;
     g_error_free(error);
 
     qWarning() << message;
-    emit Error(message);
-    return NULL;
-  }
+    emit Error(id(), message, domain, code);
 
-  return bin;
+    return NULL;
+  } else {
+    return bin;
+  }
 }
 
 bool GstEnginePipeline::Init() {
@@ -237,10 +251,11 @@ bool GstEnginePipeline::InitFromString(const QString& pipeline) {
   return gst_element_link(new_bin, audiobin_);
 }
 
-bool GstEnginePipeline::InitFromUrl(const QUrl &url) {
+bool GstEnginePipeline::InitFromUrl(const QUrl &url, qint64 end_nanosec) {
   pipeline_ = gst_pipeline_new("pipeline");
 
   url_ = url;
+  end_offset_nanosec_ = end_nanosec;
 
   // Decode bin
   if (!ReplaceDecodeBin(url)) return false;
@@ -271,6 +286,10 @@ gboolean GstEnginePipeline::BusCallback(GstBus*, GstMessage* msg, gpointer self)
       instance->TagMessageReceived(msg);
       break;
 
+    case GST_MESSAGE_STATE_CHANGED:
+      instance->StateChangedMessageReceived(msg);
+      break;
+
     default:
       break;
   }
@@ -280,9 +299,10 @@ gboolean GstEnginePipeline::BusCallback(GstBus*, GstMessage* msg, gpointer self)
 
 GstBusSyncReply GstEnginePipeline::BusCallbackSync(GstBus*, GstMessage* msg, gpointer self) {
   GstEnginePipeline* instance = reinterpret_cast<GstEnginePipeline*>(self);
+
   switch (GST_MESSAGE_TYPE(msg)) {
     case GST_MESSAGE_EOS:
-      emit instance->EndOfStreamReached(false);
+      emit instance->EndOfStreamReached(instance->id(), false);
       break;
 
     case GST_MESSAGE_TAG:
@@ -297,9 +317,14 @@ GstBusSyncReply GstEnginePipeline::BusCallbackSync(GstBus*, GstMessage* msg, gpo
       instance->ElementMessageReceived(msg);
       break;
 
+    case GST_MESSAGE_STATE_CHANGED:
+      instance->StateChangedMessageReceived(msg);
+      break;
+
     default:
       break;
   }
+
   return GST_BUS_PASS;
 }
 
@@ -323,7 +348,8 @@ void GstEnginePipeline::ErrorMessageReceived(GstMessage* msg) {
   gst_message_parse_error(msg, &error, &debugs);
   QString message = QString::fromLocal8Bit(error->message);
   QString debugstr = QString::fromLocal8Bit(debugs);
-
+  int domain = error->domain;
+  int code = error->code;
   g_error_free(error);
   free(debugs);
 
@@ -336,7 +362,7 @@ void GstEnginePipeline::ErrorMessageReceived(GstMessage* msg) {
   }
 
   qDebug() << debugstr;
-  emit Error(message);
+  emit Error(id(), message, domain, code);
 }
 
 void GstEnginePipeline::TagMessageReceived(GstMessage* msg) {
@@ -356,7 +382,7 @@ void GstEnginePipeline::TagMessageReceived(GstMessage* msg) {
 
   if (!bundle.title.isEmpty() || !bundle.artist.isEmpty() ||
       !bundle.comment.isEmpty() || !bundle.album.isEmpty())
-    emit MetadataFound(bundle);
+    emit MetadataFound(id(), bundle);
 }
 
 QString GstEnginePipeline::ParseTag(GstTagList* list, const char* tag) const {
@@ -371,6 +397,27 @@ QString GstEnginePipeline::ParseTag(GstTagList* list, const char* tag) const {
   return ret.trimmed();
 }
 
+void GstEnginePipeline::StateChangedMessageReceived(GstMessage* msg) {
+  if (msg->src != GST_OBJECT(pipeline_)) {
+    // We only care about state changes of the whole pipeline.
+    return;
+  }
+
+  GstState old_state, new_state, pending;
+  gst_message_parse_state_changed(msg, &old_state, &new_state, &pending);
+
+  if (!pipeline_is_initialised_ && (new_state == GST_STATE_PAUSED || new_state == GST_STATE_PLAYING)) {
+    pipeline_is_initialised_ = true;
+    if (pending_seek_nanosec_ != -1 && pipeline_is_connected_) {
+      QMetaObject::invokeMethod(this, "Seek", Qt::QueuedConnection,
+                                Q_ARG(qint64, pending_seek_nanosec_));
+    }
+  }
+
+  if (pipeline_is_initialised_ && new_state != GST_STATE_PAUSED && new_state != GST_STATE_PLAYING) {
+    pipeline_is_initialised_ = false;
+  }
+}
 
 void GstEnginePipeline::NewPadCallback(GstElement*, GstPad* pad, gpointer self) {
   GstEnginePipeline* instance = reinterpret_cast<GstEnginePipeline*>(self);
@@ -384,8 +431,13 @@ void GstEnginePipeline::NewPadCallback(GstElement*, GstPad* pad, gpointer self)
   gst_pad_link(pad, audiopad);
 
   gst_object_unref(audiopad);
-}
 
+  instance->pipeline_is_connected_ = true;
+  if (instance->pending_seek_nanosec_ != -1 && instance->pipeline_is_initialised_) {
+    QMetaObject::invokeMethod(instance, "Seek", Qt::QueuedConnection,
+                              Q_ARG(qint64, instance->pending_seek_nanosec_));
+  }
+}
 
 bool GstEnginePipeline::HandoffCallback(GstPad*, GstBuffer* buf, gpointer self) {
   GstEnginePipeline* instance = reinterpret_cast<GstEnginePipeline*>(self);
@@ -398,7 +450,41 @@ bool GstEnginePipeline::HandoffCallback(GstPad*, GstBuffer* buf, gpointer self)
 
   foreach (BufferConsumer* consumer, consumers) {
     gst_buffer_ref(buf);
-    consumer->ConsumeBuffer(buf, instance);
+    consumer->ConsumeBuffer(buf, instance->id());
+  }
+
+  // Calculate the end time of this buffer so we can stop playback if it's
+  // after the end time of this song.
+  if (instance->end_offset_nanosec_ > 0) {
+    quint64 start_time = GST_BUFFER_TIMESTAMP(buf) - instance->segment_start_;
+    quint64 duration = GST_BUFFER_DURATION(buf);
+    quint64 end_time = start_time + duration;
+
+    if (end_time > instance->end_offset_nanosec_) {
+      if (instance->has_next_valid_url()) {
+        if (instance->next_url_ == instance->url_ &&
+            instance->next_beginning_offset_nanosec_ == instance->end_offset_nanosec_) {
+          // The "next" song is actually the next segment of this file - so
+          // cheat and keep on playing, but just tell the Engine we've moved on.
+          instance->end_offset_nanosec_ = instance->next_end_offset_nanosec_;
+          instance->next_url_ = QUrl();
+          instance->next_beginning_offset_nanosec_ = 0;
+          instance->next_end_offset_nanosec_ = 0;
+
+          // GstEngine will try to seek to the start of the new section, but
+          // we're already there so ignore it.
+          instance->ignore_next_seek_ = true;
+
+          emit instance->EndOfStreamReached(instance->id(), true);
+        } else {
+          // We have a next song but we can't cheat, so move to it normally.
+          instance->TransitionToNext();
+        }
+      } else {
+        // There's no next song
+        emit instance->EndOfStreamReached(instance->id(), false);
+      }
+    }
   }
 
   return true;
@@ -422,28 +508,35 @@ bool GstEnginePipeline::EventHandoffCallback(GstPad*, GstEvent* e, gpointer self
 void GstEnginePipeline::SourceDrainedCallback(GstURIDecodeBin* bin, gpointer self) {
   GstEnginePipeline* instance = reinterpret_cast<GstEnginePipeline*>(self);
 
-  if (instance->next_url_.isValid()) {
-    GstElement* old_decode_bin = instance->uridecodebin_;
-
-    instance->ignore_tags_ = true;
-
-    instance->ReplaceDecodeBin(instance->next_url_);
-    gst_element_set_state(instance->uridecodebin_, GST_STATE_PLAYING);
-
-    instance->url_ = instance->next_url_;
-    instance->next_url_ = QUrl();
-
-    // This just tells the UI that we've moved on to the next song
-    emit instance->EndOfStreamReached(true);
-
-    // This has to happen *after* the gst_element_set_state on the new bin to
-    // fix an occasional race condition deadlock.
-    sElementDeleter->DeleteElementLater(old_decode_bin);
-
-    instance->ignore_tags_ = false;
+  if (instance->has_next_valid_url()) {
+    instance->TransitionToNext();
   }
 }
 
+void GstEnginePipeline::TransitionToNext() {
+  GstElement* old_decode_bin = uridecodebin_;
+
+  ignore_tags_ = true;
+
+  ReplaceDecodeBin(next_url_);
+  gst_element_set_state(uridecodebin_, GST_STATE_PLAYING);
+
+  url_ = next_url_;
+  end_offset_nanosec_ = next_end_offset_nanosec_;
+  next_url_ = QUrl();
+  next_beginning_offset_nanosec_ = 0;
+  next_end_offset_nanosec_ = 0;
+
+  // This just tells the UI that we've moved on to the next song
+  emit EndOfStreamReached(id(), true);
+
+  // This has to happen *after* the gst_element_set_state on the new bin to
+  // fix an occasional race condition deadlock.
+  sElementDeleter->DeleteElementLater(old_decode_bin);
+
+  ignore_tags_ = false;
+}
+
 qint64 GstEnginePipeline::position() const {
   GstFormat fmt = GST_FORMAT_TIME;
   gint64 value = 0;
@@ -452,7 +545,6 @@ qint64 GstEnginePipeline::position() const {
   return value;
 }
 
-
 qint64 GstEnginePipeline::length() const {
   GstFormat fmt = GST_FORMAT_TIME;
   gint64 value = 0;
@@ -461,7 +553,6 @@ qint64 GstEnginePipeline::length() const {
   return value;
 }
 
-
 GstState GstEnginePipeline::state() const {
   GstState s, sp;
   if (gst_element_get_state(pipeline_, &s, &sp, kGstStateTimeoutNanosecs) ==
@@ -476,6 +567,17 @@ QFuture<GstStateChangeReturn> GstEnginePipeline::SetState(GstState state) {
 }
 
 bool GstEnginePipeline::Seek(qint64 nanosec) {
+  if (ignore_next_seek_) {
+    ignore_next_seek_ = false;
+    return true;
+  }
+
+  if (!pipeline_is_connected_ || !pipeline_is_initialised_) {
+    pending_seek_nanosec_ = nanosec;
+    return true;
+  }
+
+  pending_seek_nanosec_ = -1;
   return gst_element_seek_simple(pipeline_, GST_FORMAT_TIME,
                                  GST_SEEK_FLAG_FLUSH, nanosec);
 }
@@ -485,7 +587,6 @@ void GstEnginePipeline::SetEqualizerEnabled(bool enabled) {
   UpdateEqualizer();
 }
 
-
 void GstEnginePipeline::SetEqualizerParams(int preamp, const QList<int>& band_gains) {
   eq_preamp_ = preamp;
   eq_band_gains_ = band_gains;
@@ -529,9 +630,11 @@ void GstEnginePipeline::UpdateVolume() {
   g_object_set(G_OBJECT(volume_), "volume", vol, NULL);
 }
 
-void GstEnginePipeline::StartFader(int duration_msec,
+void GstEnginePipeline::StartFader(qint64 duration_nanosec,
                                    QTimeLine::Direction direction,
                                    QTimeLine::CurveShape shape) {
+  const int duration_msec = duration_nanosec / kNsecPerMsec;
+
   // If there's already another fader running then start from the same time
   // that one was already at.
   int start_time = direction == QTimeLine::Forward ? 0 : duration_msec;
@@ -584,3 +687,11 @@ void GstEnginePipeline::RemoveAllBufferConsumers() {
   QMutexLocker l(&buffer_consumers_mutex_);
   buffer_consumers_.clear();
 }
+
+void GstEnginePipeline::SetNextUrl(const QUrl& url,
+                                   qint64 beginning_nanosec,
+                                   qint64 end_nanosec) {
+  next_url_ = url;
+  next_beginning_offset_nanosec_ = beginning_nanosec;
+  next_end_offset_nanosec_ = end_nanosec;
+}
