@@ -43,13 +43,16 @@ class GstEnginePipeline : public QObject {
   GstEnginePipeline(GstEngine* engine);
   ~GstEnginePipeline();
 
+  // Globally unique across all pipelines.
+  int id() const { return id_; }
+
   // Call these setters before Init
   void set_output_device(const QString& sink, const QString& device);
   void set_replaygain(bool enabled, int mode, float preamp, bool compression);
-  void set_buffer_duration_ms(int duration_ms);
+  void set_buffer_duration_nanosec(qint64 duration_nanosec);
 
   // Creates the pipeline, returns false on error
-  bool InitFromUrl(const QUrl& url);
+  bool InitFromUrl(const QUrl& url, qint64 end_nanosec);
   bool InitFromString(const QString& pipeline);
 
   // BufferConsumers get fed audio data.  Thread-safe.
@@ -59,23 +62,30 @@ class GstEnginePipeline : public QObject {
 
   // Control the music playback
   QFuture<GstStateChangeReturn> SetState(GstState state);
-  bool Seek(qint64 nanosec);
+  Q_INVOKABLE bool Seek(qint64 nanosec);
   void SetEqualizerEnabled(bool enabled);
   void SetEqualizerParams(int preamp, const QList<int>& band_gains);
   void SetVolume(int percent);
-  void StartFader(int duration_msec,
+  void StartFader(qint64 duration_nanosec,
                   QTimeLine::Direction direction = QTimeLine::Forward,
                   QTimeLine::CurveShape shape = QTimeLine::LinearCurve);
 
   // If this is set then it will be loaded automatically when playback finishes
   // for gapless playback
-  void SetNextUrl(const QUrl& url) { next_url_ = url; }
+  void SetNextUrl(const QUrl& url, qint64 beginning_nanosec, qint64 end_nanosec);
+  bool has_next_valid_url() const { return next_url_.isValid(); }
 
   // Get information about the music playback
   QUrl url() const { return url_; }
   bool is_valid() const { return valid_; }
+  // Please note that this method (unlike GstEngine's.position()) is
+  // multiple-section media unaware.
   qint64 position() const;
+  // Please note that this method (unlike GstEngine's.length()) is
+  // multiple-section media unaware.
   qint64 length() const;
+  // Returns this pipeline's state. May return GST_STATE_NULL if the state check
+  // timed out. The timeout value is a reasonable default.
   GstState state() const;
   qint64 segment_start() const { return segment_start_; }
 
@@ -85,9 +95,11 @@ class GstEnginePipeline : public QObject {
   void SetVolumeModifier(qreal mod);
 
  signals:
-  void EndOfStreamReached(bool has_next_track);
-  void MetadataFound(const Engine::SimpleMetaBundle& bundle);
-  void Error(const QString& message);
+  void EndOfStreamReached(int pipeline_id, bool has_next_track);
+  void MetadataFound(int pipeline_id, const Engine::SimpleMetaBundle& bundle);
+  // This indicates an error, delegated from GStreamer, in the pipeline.
+  // The message, domain and error_code are related to GStreamer's GError.
+  void Error(int pipeline_id, const QString& message, int domain, int error_code);
   void FaderFinished();
 
  protected:
@@ -102,9 +114,12 @@ class GstEnginePipeline : public QObject {
   static bool HandoffCallback(GstPad*, GstBuffer*, gpointer);
   static bool EventHandoffCallback(GstPad*, GstEvent*, gpointer);
   static void SourceDrainedCallback(GstURIDecodeBin*, gpointer);
+
   void TagMessageReceived(GstMessage*);
   void ErrorMessageReceived(GstMessage*);
   void ElementMessageReceived(GstMessage*);
+  void StateChangedMessageReceived(GstMessage*);
+
   QString ParseTag(GstTagList* list, const char* tag) const;
 
   bool Init();
@@ -115,6 +130,8 @@ class GstEnginePipeline : public QObject {
   bool ReplaceDecodeBin(GstElement* new_bin);
   bool ReplaceDecodeBin(const QUrl& url);
 
+  void TransitionToNext();
+
  private slots:
   void FaderTimelineFinished();
 
@@ -128,6 +145,14 @@ class GstEnginePipeline : public QObject {
 
   GstEngine* engine_;
 
+  // Using == to compare two pipelines is a bad idea, because new ones often
+  // get created in the same address as old ones.  This ID will be unique for
+  // each pipeline.
+  // Threading warning: access to the static ID field isn't protected by a
+  // mutex because all pipeline creation is currently done in the main thread.
+  static int sId;
+  int id_;
+
   // General settings for the pipeline
   bool valid_;
   QString sink_;
@@ -149,13 +174,27 @@ class GstEnginePipeline : public QObject {
   int rg_mode_;
   float rg_preamp_;
   bool rg_compression_;
-  quint64 buffer_duration_ms_;
+  quint64 buffer_duration_nanosec_;
 
   // The URL that is currently playing, and the URL that is to be preloaded
   // when the current track is close to finishing.
   QUrl url_;
   QUrl next_url_;
 
+  // If this is > 0 then the pipeline will be forced to stop when playback goes
+  // past this position.
+  qint64 end_offset_nanosec_;
+
+  // We store the beginning and end for the preloading song too, so we can just
+  // carry on without reloading the file if the sections carry on from each
+  // other.
+  qint64 next_beginning_offset_nanosec_;
+  qint64 next_end_offset_nanosec_;
+
+  // Set temporarily when moving to the next contiguous section in a multi-part
+  // file.
+  bool ignore_next_seek_;
+
   // Set temporarily when switching out the decode bin, so metadata doesn't
   // get sent while the Player still thinks it's playing the last song
   bool ignore_tags_;
@@ -164,6 +203,13 @@ class GstEnginePipeline : public QObject {
   // callers can pick it up after the state change to PLAYING fails.
   QUrl redirect_url_;
 
+  // Seeking while the pipeline is in the READY state doesn't work, so we have
+  // to wait until it goes to PAUSED or PLAYING.
+  // Also we have to wait for the decodebin to be connected.
+  bool pipeline_is_initialised_;
+  bool pipeline_is_connected_;
+  qint64 pending_seek_nanosec_;
+
   int volume_percent_;
   qreal volume_modifier_;
 
