@@ -48,7 +48,7 @@ Library::Library(BackgroundThread<Database>* db_thread, TaskManager* task_manage
   using smart_playlists::Search;
   using smart_playlists::SearchTerm;
 
-  model_ = new LibraryModel(backend_, this);
+  model_ = new LibraryModel(backend_, task_manager_, this);
   model_->set_show_smart_playlists(true);
   model_->set_default_smart_playlists(LibraryModel::DefaultGenerators()
     << (LibraryModel::GeneratorList()
@@ -90,6 +90,9 @@ Library::Library(BackgroundThread<Database>* db_thread, TaskManager* task_manage
                   Search::Sort_Random, SearchTerm::Field_Title), true))
     )
   );
+
+  // full rescan revisions
+  full_rescan_revisions_[26] = tr("CUE sheet support");
 }
 
 void Library::set_watcher_factory(BackgroundThreadFactory<LibraryWatcher>* factory) {
@@ -145,6 +148,13 @@ void Library::IncrementalScan() {
   watcher_->Worker()->IncrementalScanAsync();
 }
 
+void Library::FullScan() {
+  if (!watcher_->Worker())
+    return;
+
+  watcher_->Worker()->FullScanAsync();
+}
+
 void Library::PauseWatcher() {
   if (!watcher_->Worker())
     return;
