@@ -22,12 +22,13 @@
 #include "core/scopedtransaction.h"
 #include "smartplaylists/search.h"
 
-#include <QDir>
-#include <QVariant>
-#include <QSettings>
-#include <QtDebug>
 #include <QCoreApplication>
 #include <QDateTime>
+#include <QDir>
+#include <QFileInfo>
+#include <QSettings>
+#include <QVariant>
+#include <QtDebug>
 
 const char* LibraryBackend::kNewScoreSql =
     "case when playcount <= 0 then (%1 * 100 + score) / 2"
@@ -67,6 +68,11 @@ void LibraryBackend::IncrementSkipCountAsync(int id, float progress) {
                              Q_ARG(int, id), Q_ARG(float, progress));
 }
 
+void LibraryBackend::ResetStatisticsAsync(int id) {
+  metaObject()->invokeMethod(this, "ResetStatistics", Qt::QueuedConnection,
+                             Q_ARG(int, id));
+}
+
 void LibraryBackend::UpdateSongRatingAsync(int id, float rating) {
   metaObject()->invokeMethod(this, "UpdateSongRating", Qt::QueuedConnection,
                              Q_ARG(int, id), Q_ARG(float, rating));
@@ -93,7 +99,7 @@ void LibraryBackend::ChangeDirPath(int id, const QString &new_path) {
   q.bindValue(":path", new_path);
   q.bindValue(":id", id);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return;
+  if (db_->CheckErrors(q)) return;
 
   const int path_len = new_path.length();
 
@@ -103,7 +109,7 @@ void LibraryBackend::ChangeDirPath(int id, const QString &new_path) {
   q.bindValue(":path", new_path);
   q.bindValue(":id", id);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return;
+  if (db_->CheckErrors(q)) return;
 
   // Do the songs table
   q = QSqlQuery(QString("UPDATE %1 SET filename=:path || substr(filename, %2)"
@@ -111,7 +117,7 @@ void LibraryBackend::ChangeDirPath(int id, const QString &new_path) {
   q.bindValue(":path", new_path);
   q.bindValue(":id", id);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return;
+  if (db_->CheckErrors(q)) return;
 
   t.Commit();
 }
@@ -124,7 +130,7 @@ DirectoryList LibraryBackend::GetAllDirectories() {
 
   QSqlQuery q(QString("SELECT ROWID, path FROM %1").arg(dirs_table_), db);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return ret;
+  if (db_->CheckErrors(q)) return ret;
 
   while (q.next()) {
     Directory dir;
@@ -147,7 +153,7 @@ SubdirectoryList LibraryBackend::SubdirsInDirectory(int id, QSqlDatabase &db) {
                       " WHERE directory = :dir").arg(subdirs_table_), db);
   q.bindValue(":dir", id);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return SubdirectoryList();
+  if (db_->CheckErrors(q)) return SubdirectoryList();
 
   SubdirectoryList subdirs;
   while (q.next()) {
@@ -167,24 +173,26 @@ void LibraryBackend::UpdateTotalSongCount() {
 
   QSqlQuery q(QString("SELECT COUNT(*) FROM %1").arg(songs_table_), db);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return;
+  if (db_->CheckErrors(q)) return;
   if (!q.next()) return;
 
   emit TotalSongCountUpdated(q.value(0).toInt());
 }
 
-void LibraryBackend::AddDirectory(const QString &path) {
+void LibraryBackend::AddDirectory(const QString& path) {
+  QString canonical_path = QFileInfo(path).canonicalFilePath();
+
   QMutexLocker l(db_->Mutex());
   QSqlDatabase db(db_->Connect());
 
   QSqlQuery q(QString("INSERT INTO %1 (path, subdirs)"
                       " VALUES (:path, 1)").arg(dirs_table_), db);
-  q.bindValue(":path", path);
+  q.bindValue(":path", canonical_path);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return;
+  if (db_->CheckErrors(q)) return;
 
   Directory dir;
-  dir.path = path;
+  dir.path = canonical_path;
   dir.id = q.lastInsertId().toInt();
 
   emit DirectoryDiscovered(dir, SubdirectoryList());
@@ -204,14 +212,14 @@ void LibraryBackend::RemoveDirectory(const Directory& dir) {
               .arg(subdirs_table_), db);
   q.bindValue(":id", dir.id);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return;
+  if (db_->CheckErrors(q)) return;
 
   // Now remove the directory itself
   q = QSqlQuery(QString("DELETE FROM %1 WHERE ROWID = :id")
                 .arg(dirs_table_), db);
   q.bindValue(":id", dir.id);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return;
+  if (db_->CheckErrors(q)) return;
 
   emit DirectoryDeleted(dir);
 
@@ -227,7 +235,7 @@ SongList LibraryBackend::FindSongsInDirectory(int id) {
               .arg(songs_table_), db);
   q.bindValue(":directory", id);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return SongList();
+  if (db_->CheckErrors(q)) return SongList();
 
   SongList ret;
   while (q.next()) {
@@ -261,26 +269,26 @@ void LibraryBackend::AddOrUpdateSubdirs(const SubdirectoryList& subdirs) {
       delete_query.bindValue(":id", subdir.directory_id);
       delete_query.bindValue(":path", subdir.path);
       delete_query.exec();
-      db_->CheckErrors(delete_query.lastError());
+      db_->CheckErrors(delete_query);
     } else {
       // See if this subdirectory already exists in the database
       find_query.bindValue(":id", subdir.directory_id);
       find_query.bindValue(":path", subdir.path);
       find_query.exec();
-      if (db_->CheckErrors(find_query.lastError())) continue;
+      if (db_->CheckErrors(find_query)) continue;
 
       if (find_query.next()) {
         update_query.bindValue(":mtime", subdir.mtime);
         update_query.bindValue(":id", subdir.directory_id);
         update_query.bindValue(":path", subdir.path);
         update_query.exec();
-        db_->CheckErrors(update_query.lastError());
+        db_->CheckErrors(update_query);
       } else {
         add_query.bindValue(":id", subdir.directory_id);
         add_query.bindValue(":path", subdir.path);
         add_query.bindValue(":mtime", subdir.mtime);
         add_query.exec();
-        db_->CheckErrors(add_query.lastError());
+        db_->CheckErrors(add_query);
       }
     }
   }
@@ -316,7 +324,7 @@ void LibraryBackend::AddOrUpdateSongs(const SongList& songs) {
     if (!dirs_table_.isEmpty()) {
       check_dir.bindValue(":id", song.directory_id());
       check_dir.exec();
-      if (db_->CheckErrors(check_dir.lastError())) continue;
+      if (db_->CheckErrors(check_dir)) continue;
 
       if (!check_dir.next())
         continue; // Directory didn't exist
@@ -328,7 +336,7 @@ void LibraryBackend::AddOrUpdateSongs(const SongList& songs) {
       // Insert the row and create a new ID
       song.BindToQuery(&add_song);
       add_song.exec();
-      if (db_->CheckErrors(add_song.lastError())) continue;
+      if (db_->CheckErrors(add_song)) continue;
 
       // Get the new ID
       const int id = add_song.lastInsertId().toInt();
@@ -337,7 +345,7 @@ void LibraryBackend::AddOrUpdateSongs(const SongList& songs) {
       add_song_fts.bindValue(":id", id);
       song.BindToFtsQuery(&add_song_fts);
       add_song_fts.exec();
-      if (db_->CheckErrors(add_song_fts.lastError())) continue;
+      if (db_->CheckErrors(add_song_fts)) continue;
 
       Song copy(song);
       copy.set_id(id);
@@ -352,12 +360,12 @@ void LibraryBackend::AddOrUpdateSongs(const SongList& songs) {
       song.BindToQuery(&update_song);
       update_song.bindValue(":id", song.id());
       update_song.exec();
-      if (db_->CheckErrors(update_song.lastError())) continue;
+      if (db_->CheckErrors(update_song)) continue;
 
       song.BindToFtsQuery(&update_song_fts);
       update_song_fts.bindValue(":id", song.id());
       update_song_fts.exec();
-      if (db_->CheckErrors(update_song_fts.lastError())) continue;
+      if (db_->CheckErrors(update_song_fts)) continue;
 
       deleted_songs << old_song;
       added_songs << song;
@@ -387,7 +395,7 @@ void LibraryBackend::UpdateMTimesOnly(const SongList& songs) {
     q.bindValue(":mtime", song.mtime());
     q.bindValue(":id", song.id());
     q.exec();
-    db_->CheckErrors(q.lastError());
+    db_->CheckErrors(q);
   }
   transaction.Commit();
 }
@@ -405,11 +413,11 @@ void LibraryBackend::DeleteSongs(const SongList &songs) {
   foreach (const Song& song, songs) {
     remove.bindValue(":id", song.id());
     remove.exec();
-    db_->CheckErrors(remove.lastError());
+    db_->CheckErrors(remove);
 
     remove_fts.bindValue(":id", song.id());
     remove_fts.exec();
-    db_->CheckErrors(remove_fts.lastError());
+    db_->CheckErrors(remove_fts);
   }
   transaction.Commit();
 
@@ -418,9 +426,9 @@ void LibraryBackend::DeleteSongs(const SongList &songs) {
   UpdateTotalSongCountAsync();
 }
 
-QStringList LibraryBackend::GetAllArtists(const QueryOptions& opt) {
+QStringList LibraryBackend::GetAll(const QString& column, const QueryOptions& opt) {
   LibraryQuery query(opt);
-  query.SetColumnSpec("DISTINCT artist");
+  query.SetColumnSpec("DISTINCT " + column);
   query.AddCompilationRequirement(false);
 
   QMutexLocker l(db_->Mutex());
@@ -433,6 +441,10 @@ QStringList LibraryBackend::GetAllArtists(const QueryOptions& opt) {
   return ret;
 }
 
+QStringList LibraryBackend::GetAllArtists(const QueryOptions& opt) {
+  return GetAll("artist", opt);
+}
+
 QStringList LibraryBackend::GetAllArtistsWithAlbums(const QueryOptions& opt) {
   LibraryQuery query(opt);
   query.SetColumnSpec("DISTINCT artist");
@@ -514,7 +526,7 @@ SongList LibraryBackend::GetSongsByForeignId(
                       " WHERE %2.%3 IN (%4) AND %1.ROWID = %2.ROWID")
               .arg(songs_table_, table, column, in), db);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return SongList();
+  if (db_->CheckErrors(q)) return SongList();
 
   QVector<Song> ret(ids.count());
   while (q.next()) {
@@ -541,7 +553,7 @@ SongList LibraryBackend::GetSongsById(const QStringList& ids, QSqlDatabase& db)
   QSqlQuery q(QString("SELECT ROWID, " + Song::kColumnSpec + " FROM %1"
                       " WHERE ROWID IN (%2)").arg(songs_table_, in), db);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return SongList();
+  if (db_->CheckErrors(q)) return SongList();
 
   SongList ret;
   while (q.next()) {
@@ -552,6 +564,36 @@ SongList LibraryBackend::GetSongsById(const QStringList& ids, QSqlDatabase& db)
   return ret;
 }
 
+Song LibraryBackend::GetSongByFilename(const QString& filename, qint64 beginning) {
+  LibraryQuery query;
+  query.SetColumnSpec("%songs_table.ROWID, " + Song::kColumnSpec);
+  query.AddWhere("filename", filename);
+  query.AddWhere("beginning", beginning);
+
+  Song song;
+  if (ExecQuery(&query) && query.Next()) {
+    song.InitFromQuery(query);
+  }
+  return song;
+}
+
+SongList LibraryBackend::GetSongsByFilename(const QString& filename) {
+  LibraryQuery query;
+  query.SetColumnSpec("%songs_table.ROWID, " + Song::kColumnSpec);
+  query.AddWhere("filename", filename);
+
+  SongList songlist;
+  if (ExecQuery(&query)) {
+    while(query.Next()) {
+      Song song;
+      song.InitFromQuery(query);
+
+      songlist << song;
+    }
+  }
+  return songlist;
+}
+
 bool LibraryBackend::HasCompilations(const QueryOptions& opt) {
   LibraryQuery query(opt);
   query.SetColumnSpec("%songs_table.ROWID");
@@ -595,7 +637,7 @@ void LibraryBackend::UpdateCompilations() {
 
   QSqlQuery q("SELECT artist, album, filename, sampler FROM songs ORDER BY album", db);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return;
+  if (db_->CheckErrors(q)) return;
 
   QMap<QString, CompilationInfo> compilation_info;
   while (q.next()) {
@@ -679,7 +721,7 @@ void LibraryBackend::UpdateCompilations(QSqlQuery& find_songs, QSqlQuery& update
   update.bindValue(":sampler", sampler);
   update.bindValue(":album", album);
   update.exec();
-  db_->CheckErrors(update.lastError());
+  db_->CheckErrors(update);
 }
 
 LibraryBackend::AlbumList LibraryBackend::GetAlbums(const QString& artist,
@@ -793,7 +835,7 @@ void LibraryBackend::UpdateManualAlbumArt(const QString &artist,
     q.bindValue(":artist", artist);
 
   q.exec();
-  db_->CheckErrors(q.lastError());
+  db_->CheckErrors(q);
 
 
   // Now get the updated songs
@@ -848,7 +890,7 @@ void LibraryBackend::ForceCompilation(const QString& artist, const QString& albu
     q.bindValue(":artist", artist);
 
   q.exec();
-  db_->CheckErrors(q.lastError());
+  db_->CheckErrors(q);
 
   // Now get the updated songs
   if (!ExecQuery(&query)) return;
@@ -881,7 +923,7 @@ SongList LibraryBackend::FindSongs(const smart_playlists::Search& search) {
   SongList ret;
   QSqlQuery query(sql, db);
   query.exec();
-  if (db_->CheckErrors(query.lastError()))
+  if (db_->CheckErrors(query))
     return ret;
 
   // Read the results
@@ -907,7 +949,7 @@ void LibraryBackend::IncrementPlayCount(int id) {
   q.bindValue(":now", QDateTime::currentDateTime().toTime_t());
   q.bindValue(":id", id);
   q.exec();
-  if (db_->CheckErrors(q.lastError()))
+  if (db_->CheckErrors(q))
     return;
 
   Song new_song = GetSongById(id, db);
@@ -927,7 +969,27 @@ void LibraryBackend::IncrementSkipCount(int id, float progress) {
                       " WHERE ROWID = :id").arg(songs_table_), db);
   q.bindValue(":id", id);
   q.exec();
-  if (db_->CheckErrors(q.lastError()))
+  if (db_->CheckErrors(q))
+    return;
+
+  Song new_song = GetSongById(id, db);
+  emit SongsStatisticsChanged(SongList() << new_song);
+}
+
+void LibraryBackend::ResetStatistics(int id) {
+  if (id == -1)
+    return;
+
+  QMutexLocker l(db_->Mutex());
+  QSqlDatabase db(db_->Connect());
+
+  QSqlQuery q(QString(
+      "UPDATE %1 SET playcount = 0, skipcount = 0,"
+      "              lastplayed = -1, score = 0"
+      " WHERE ROWID = :id").arg(songs_table_), db);
+  q.bindValue(":id", id);
+  q.exec();
+  if (db_->CheckErrors(q))
     return;
 
   Song new_song = GetSongById(id, db);
@@ -946,7 +1008,7 @@ void LibraryBackend::UpdateSongRating(int id, float rating) {
   q.bindValue(":rating", rating);
   q.bindValue(":id", id);
   q.exec();
-  if (db_->CheckErrors(q.lastError()))
+  if (db_->CheckErrors(q))
     return;
 
   Song new_song = GetSongById(id, db);
@@ -961,12 +1023,12 @@ void LibraryBackend::DeleteAll() {
 
     QSqlQuery q("DELETE FROM " + songs_table_, db);
     q.exec();
-    if (db_->CheckErrors(q.lastError()))
+    if (db_->CheckErrors(q))
       return;
 
     q = QSqlQuery("DELETE FROM " + fts_table_, db);
     q.exec();
-    if (db_->CheckErrors(q.lastError()))
+    if (db_->CheckErrors(q))
       return;
 
     t.Commit();
