@@ -82,6 +82,14 @@ class LibraryBackendInterface : public QObject {
 
   virtual Song GetSongById(int id) = 0;
 
+  // Returns all sections of a song with the given filename. If there's just one section
+  // the resulting list will have it's size equal to 1.
+  virtual SongList GetSongsByFilename(const QString& filename) = 0;
+  // Returns a section of a song with the given filename and beginning. If the section
+  // is not present in library, returns invalid song.
+  // Using default beginning value is suitable when searching for single-section songs.
+  virtual Song GetSongByFilename(const QString& filename, qint64 beginning = 0) = 0;
+
   virtual void AddDirectory(const QString& path) = 0;
   virtual void RemoveDirectory(const Directory& dir) = 0;
 
@@ -114,6 +122,7 @@ class LibraryBackend : public LibraryBackendInterface {
   DirectoryList GetAllDirectories();
   void ChangeDirPath(int id, const QString& new_path);
 
+  QStringList GetAll(const QString& column, const QueryOptions& opt = QueryOptions());
   QStringList GetAllArtists(const QueryOptions& opt = QueryOptions());
   QStringList GetAllArtistsWithAlbums(const QueryOptions& opt = QueryOptions());
   SongList GetSongs(const QString& artist, const QString& album, const QueryOptions& opt = QueryOptions());
@@ -134,6 +143,9 @@ class LibraryBackend : public LibraryBackendInterface {
   SongList GetSongsByForeignId(const QStringList& ids, const QString& table,
                                const QString& column);
 
+  SongList GetSongsByFilename(const QString& filename);
+  Song GetSongByFilename(const QString& filename, qint64 beginning = 0);
+
   void AddDirectory(const QString& path);
   void RemoveDirectory(const Directory& dir);
 
@@ -142,6 +154,7 @@ class LibraryBackend : public LibraryBackendInterface {
 
   void IncrementPlayCountAsync(int id);
   void IncrementSkipCountAsync(int id, float progress);
+  void ResetStatisticsAsync(int id);
   void UpdateSongRatingAsync(int id, float rating);
 
   void DeleteAll();
@@ -158,6 +171,7 @@ class LibraryBackend : public LibraryBackendInterface {
   void ForceCompilation(const QString& artist, const QString& album, bool on);
   void IncrementPlayCount(int id);
   void IncrementSkipCount(int id, float progress);
+  void ResetStatistics(int id);
   void UpdateSongRating(int id, float rating);
 
  signals:
