@@ -17,11 +17,13 @@
 
 #include "libraryconfig.h"
 #include "librarydirectorymodel.h"
+#include "librarymodel.h"
 #include "libraryview.h"
 #include "librarywatcher.h"
 #include "ui_libraryconfig.h"
 #include "playlist/playlistdelegates.h"
 #include "ui/iconloader.h"
+#include "core/utilities.h"
 
 #include <QFileDialog>
 #include <QSettings>
@@ -68,7 +70,8 @@ void LibraryConfig::Add() {
   QSettings settings;
   settings.beginGroup(kSettingsGroup);
 
-  QString path(settings.value("last_path", QDir::homePath()).toString());
+  QString path(settings.value("last_path",
+      Utilities::GetConfigPath(Utilities::Path_DefaultMusicLibrary)).toString());
   path = QFileDialog::getExistingDirectory(this, tr("Add directory..."), path);
 
   if (!path.isNull()) {
@@ -90,12 +93,18 @@ void LibraryConfig::Save() {
   QSettings s;
   s.beginGroup(LibraryView::kSettingsGroup);
   s.setValue("auto_open", ui_->auto_open->isChecked());
-  s.setValue("autoclear_playlist", ui_->auto_load->isChecked());
+  s.setValue("pretty_covers", ui_->pretty_covers->isChecked());
+  s.setValue("show_dividers", ui_->show_dividers->isChecked());
   s.endGroup();
 
   s.beginGroup(LibraryWatcher::kSettingsGroup);
   s.setValue("startup_scan", ui_->startup_scan->isChecked());
   s.setValue("monitor", ui_->monitor->isChecked());
+  
+  QString filter_text = ui_->cover_art_patterns->text();
+  QStringList filters = filter_text.split(',', QString::SkipEmptyParts);
+  s.setValue("cover_art_patterns", filters);
+  
   s.endGroup();
 }
 
@@ -107,11 +116,17 @@ void LibraryConfig::Load() {
   QSettings s;
   s.beginGroup(LibraryView::kSettingsGroup);
   ui_->auto_open->setChecked(s.value("auto_open", true).toBool());
-  ui_->auto_load->setChecked(s.value("autoclear_playlist", false).toBool());
+  ui_->pretty_covers->setChecked(s.value("pretty_covers", true).toBool());
+  ui_->show_dividers->setChecked(s.value("show_dividers", true).toBool());
   s.endGroup();
 
   s.beginGroup(LibraryWatcher::kSettingsGroup);
   ui_->startup_scan->setChecked(s.value("startup_scan", true).toBool());
   ui_->monitor->setChecked(s.value("monitor", true).toBool());
+  
+  QStringList filters = s.value("cover_art_patterns",
+      QStringList() << "front" << "cover").toStringList();
+  ui_->cover_art_patterns->setText(filters.join(","));
+  
   s.endGroup();
 }
