@@ -17,6 +17,7 @@
 
 #include "libraryfilterwidget.h"
 #include "librarymodel.h"
+#include "libraryquery.h"
 #include "groupbydialog.h"
 #include "ui_libraryfilterwidget.h"
 #include "ui/iconloader.h"
@@ -45,7 +46,6 @@ LibraryFilterWidget::LibraryFilterWidget(QWidget *parent)
   filter_delay_->setSingleShot(true);
 
   // Icons
-  ui_->clear->setIcon(IconLoader::Load("edit-clear-locationbar-ltr"));
   ui_->options->setIcon(IconLoader::Load("configure"));
 
   // Filter by age
@@ -74,7 +74,6 @@ LibraryFilterWidget::LibraryFilterWidget(QWidget *parent)
   connect(ui_->filter_age_month, SIGNAL(triggered()), filter_age_mapper_, SLOT(map()));
   connect(ui_->filter_age_three_months, SIGNAL(triggered()), filter_age_mapper_, SLOT(map()));
   connect(ui_->filter_age_year, SIGNAL(triggered()), filter_age_mapper_, SLOT(map()));
-  connect(ui_->clear, SIGNAL(clicked()), SLOT(ClearFilter()));
 
   // "Group by ..."
   ui_->group_by_artist->setProperty("group_by", QVariant::fromValue(
@@ -116,18 +115,22 @@ LibraryFilterWidget::LibraryFilterWidget(QWidget *parent)
   MacLineEdit* lineedit = new MacLineEdit(this);
   ui_->horizontalLayout->insertWidget(1, lineedit);
   filter_ = lineedit;
-  ui_->clear->setHidden(true);
 #else
   filter_ = ui_->filter;
 #endif
 
-  connect(filter_->object(), SIGNAL(textChanged(QString)), SLOT(FilterTextChanged(QString)));
+  connect(filter_->widget(), SIGNAL(textChanged(QString)), SLOT(FilterTextChanged(QString)));
 }
 
 LibraryFilterWidget::~LibraryFilterWidget() {
   delete ui_;
 }
 
+void LibraryFilterWidget::FocusOnFilter(QKeyEvent *event) {
+  ui_->filter->setFocus(Qt::OtherFocusReason);
+  QApplication::sendEvent(ui_->filter, event);
+}
+
 void LibraryFilterWidget::SetLibraryModel(LibraryModel *model) {
   if (model_) {
     disconnect(model_, 0, this, 0);
@@ -191,13 +194,15 @@ void LibraryFilterWidget::GroupingChanged(const LibraryModel::Grouping& g) {
   ui_->group_by_advanced->setChecked(true);
 }
 
-void LibraryFilterWidget::ClearFilter() {
-  filter_->clear();
-  filter_->setFocus();
+void LibraryFilterWidget::SetFilterHint(const QString& hint) {
+  filter_->set_hint(hint);
 }
 
-void LibraryFilterWidget::SetFilterHint(const QString& hint) {
-  filter_->SetHint(hint);
+void LibraryFilterWidget::SetQueryMode(QueryOptions::QueryMode query_mode) {
+  filter_->clear();
+  filter_->set_enabled(query_mode == QueryOptions::QueryMode_All);
+
+  model_->SetFilterQueryMode(query_mode);
 }
 
 void LibraryFilterWidget::SetAgeFilterEnabled(bool enabled) {
@@ -212,11 +217,6 @@ void LibraryFilterWidget::AddMenuAction(QAction* action) {
   library_menu_->addAction(action);
 }
 
-void LibraryFilterWidget::AppendAndFocus(const QString& text) {
-  ui_->filter->setText(ui_->filter->text() + text);
-  ui_->filter->setFocus();
-}
-
 void LibraryFilterWidget::keyReleaseEvent(QKeyEvent* e) {
   switch (e->key()) {
     case Qt::Key_Up:
@@ -228,6 +228,11 @@ void LibraryFilterWidget::keyReleaseEvent(QKeyEvent* e) {
       emit DownPressed();
       e->accept();
       break;
+
+    case Qt::Key_Escape:
+      ui_->filter->LineEditInterface::clear();
+      e->accept();
+      break;
   }
 
   QWidget::keyReleaseEvent(e);
