@@ -29,6 +29,8 @@ class LineEditInterface;
 class SettingsDialog;
 class Ui_LibraryFilterWidget;
 
+struct QueryOptions;
+
 class QMenu;
 class QActionGroup;
 class QSignalMapper;
@@ -45,13 +47,15 @@ class LibraryFilterWidget : public QWidget {
   void SetFilterHint(const QString& hint);
   void SetAgeFilterEnabled(bool enabled);
   void SetGroupByEnabled(bool enabled);
+
   void AddMenuAction(QAction* action);
 
   void SetSettingsGroup(const QString& group) { settings_group_ = group; }
   void SetLibraryModel(LibraryModel* model);
 
  public slots:
-  void AppendAndFocus(const QString& text);
+  void SetQueryMode(QueryOptions::QueryMode view);
+  void FocusOnFilter(QKeyEvent* e);
 
  signals:
   void UpPressed();
@@ -64,7 +68,6 @@ class LibraryFilterWidget : public QWidget {
  private slots:
   void GroupingChanged(const LibraryModel::Grouping& g);
   void GroupByClicked(QAction* action);
-  void ClearFilter();
 
   void FilterTextChanged(const QString& text);
   void FilterDelayTimeout();
@@ -86,6 +89,9 @@ class LibraryFilterWidget : public QWidget {
 
   QString settings_group_;
 
+  // Since on Mac ui_->filter is nonexistent and on other platforms
+  // filter_ == ui_->filter, make sure to always use this field and
+  // not the one in the ui_ object!
   LineEditInterface* filter_;
 };
 
