@@ -19,8 +19,11 @@
 #include "librarybackend.h"
 #include "libraryitem.h"
 #include "librarydirectorymodel.h"
+#include "libraryview.h"
 #include "sqlrow.h"
+#include "core/albumcoverloader.h"
 #include "core/database.h"
+#include "core/taskmanager.h"
 #include "playlist/songmimedata.h"
 #include "smartplaylists/generator.h"
 #include "smartplaylists/generatormimedata.h"
@@ -49,9 +52,11 @@ const int LibraryModel::kSmartPlaylistsVersion = 3;
 typedef QFuture<SqlRowList> RootQueryFuture;
 typedef QFutureWatcher<SqlRowList> RootQueryWatcher;
 
-LibraryModel::LibraryModel(LibraryBackend* backend, QObject* parent)
+LibraryModel::LibraryModel(LibraryBackend* backend, TaskManager* task_manager,
+                           QObject* parent)
   : SimpleTreeModel<LibraryItem>(new LibraryItem(this), parent),
     backend_(backend),
+    task_manager_(task_manager),
     dir_model_(new LibraryDirectoryModel(backend, this)),
     show_smart_playlists_(false),
     show_various_artists_(true),
@@ -60,20 +65,43 @@ LibraryModel::LibraryModel(LibraryBackend* backend, QObject* parent)
     album_icon_(":/icons/22x22/x-clementine-album.png"),
     no_cover_icon_(":nocover.png"),
     playlists_dir_icon_(IconLoader::Load("folder-sound")),
-    playlist_icon_(":/icons/22x22/x-clementine-albums.png")
+    playlist_icon_(":/icons/22x22/x-clementine-albums.png"),
+    init_task_id_(-1),
+    pretty_cover_size_(32, 32),
+    use_pretty_covers_(false),
+    show_dividers_(true)
 {
   root_->lazy_loaded = true;
 
   group_by_[0] = GroupBy_Artist;
   group_by_[1] = GroupBy_Album;
   group_by_[2] = GroupBy_None;
+  
+  no_cover_icon_pretty_ = QImage(":nocover.png").scaled(pretty_cover_size_,
+                                                    Qt::KeepAspectRatio,
+                                                    Qt::SmoothTransformation);
+
 }
 
 LibraryModel::~LibraryModel() {
   delete root_;
 }
 
-void LibraryModel::Init() {
+void LibraryModel::set_pretty_covers(bool use_pretty_covers) {
+  if (use_pretty_covers != use_pretty_covers_) {
+    use_pretty_covers_ = use_pretty_covers;
+    Reset();
+  }
+}
+
+void LibraryModel::set_show_dividers(bool show_dividers) {
+  if (show_dividers != show_dividers_) {
+    show_dividers_ = show_dividers;
+    Reset();
+  }
+}
+
+void LibraryModel::Init(bool async) {
   connect(backend_, SIGNAL(SongsDiscovered(SongList)), SLOT(SongsDiscovered(SongList)));
   connect(backend_, SIGNAL(SongsDeleted(SongList)), SLOT(SongsDeleted(SongList)));
   connect(backend_, SIGNAL(SongsStatisticsChanged(SongList)), SLOT(SongsStatisticsChanged(SongList)));
@@ -82,7 +110,20 @@ void LibraryModel::Init() {
 
   backend_->UpdateTotalSongCountAsync();
 
-  Reset();
+  if (async) {
+    // Show a loading indicator in the model.
+    LibraryItem* loading = new LibraryItem(LibraryItem::Type_LoadingIndicator, root_);
+    loading->display_text = tr("Loading...");
+    loading->lazy_loaded = true;
+    reset();
+
+    // Show a loading indicator in the status bar too.
+    init_task_id_ = task_manager_->StartTask(tr("Loading songs"));
+
+    ResetAsync();
+  } else {
+    Reset();
+  }
 }
 
 void LibraryModel::SongsDiscovered(const SongList& songs) {
@@ -334,8 +375,53 @@ void LibraryModel::SongsDeleted(const SongList& songs) {
   }
 }
 
+QVariant LibraryModel::AlbumIcon(const QModelIndex& index, int role) const {  
+  
+  // the easiest way to get from *here* to working out what album art an index 
+  // represents seems to be to get the node's child songs and look at their metadata.
+  // if none is found, return the generic CD icon
+
+  // Cache the art in the item's metadata field
+  LibraryItem* item = IndexToItem(index);
+  if (!item)
+    return no_cover_icon_pretty_;
+  if (!item->metadata.image().isNull())
+    return item->metadata.image();
+  
+  SongList songs = GetChildSongs(index);
+  if (!songs.isEmpty()) {
+    const Song& s = songs.first();
+    QPixmap pixmap = AlbumCoverLoader::TryLoadPixmap(
+      s.art_automatic(), s.art_manual(), s.filename());
+
+    if (!pixmap.isNull()) {
+      QImage image = pixmap.toImage().scaled(
+            pretty_cover_size_, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+      item->metadata.set_image(image);
+      return image;
+    }
+  }
+  return no_cover_icon_pretty_;
+}
+
 QVariant LibraryModel::data(const QModelIndex& index, int role) const {
   const LibraryItem* item = IndexToItem(index);
+  
+  // Handle a special case for returning album artwork instead of a generic CD icon.
+  // this is here instead of in the other data() function to let us use the
+  // QModelIndex& version of GetChildSongs, which satisfies const-ness, instead
+  // of the LibraryItem* version, which doesn't.
+  if (use_pretty_covers_) {    
+    bool album_node = false;
+    if (role == Qt::DecorationRole && item->type == LibraryItem::Type_Container) {
+      GroupBy container_type = group_by_[item->container_level];
+      album_node = container_type == GroupBy_Album 
+                    || container_type == GroupBy_YearAlbum
+                    || container_type == GroupBy_AlbumArtist;
+    }
+    if (album_node)
+      return AlbumIcon(index, role);
+  }
 
   return data(item, role);
 }
@@ -387,6 +473,31 @@ QVariant LibraryModel::data(const LibraryItem* item, int role) const {
     case Role_Artist:
       return item->metadata.artist();
 
+    case Role_Editable:
+      if (!item->lazy_loaded) {
+        const_cast<LibraryModel*>(this)->LazyPopulate(
+              const_cast<LibraryItem*>(item), true);
+      }
+
+      if(item->type == LibraryItem::Type_Container) {
+        // if we have even one non editable item as a child, we ourselves
+        // are not available for edit
+        if(!item->children.isEmpty()) {
+          foreach(LibraryItem* child, item->children) {
+            if(!data(child, role).toBool()) {
+              return false;
+            }
+          }
+          return true;
+        } else {
+          return false;
+        }
+      } else if(item->type == LibraryItem::Type_Song) {
+        return item->metadata.IsEditable();
+      } else {
+        return false;
+      }
+
     case Role_SortText:
       return item->SortText();
   }
@@ -505,6 +616,11 @@ void LibraryModel::ResetAsyncQueryFinished() {
       container_nodes_[0][item->key] = item;
   }
 
+  if (init_task_id_ != -1) {
+    task_manager_->SetTaskFinished(init_task_id_);
+    init_task_id_ = -1;
+  }
+
   reset();
 }
 
@@ -529,7 +645,7 @@ void LibraryModel::BeginReset() {
   }
 
   // Smart playlists?
-  if (show_smart_playlists_ && query_options_.filter.isEmpty())
+  if (show_smart_playlists_ && query_options_.filter().isEmpty())
     CreateSmartPlaylists();
 }
 
@@ -750,7 +866,7 @@ void LibraryModel::FinishItem(GroupBy type,
     endInsertRows();
 
   // Create the divider entry if we're supposed to
-  if (create_divider) {
+  if (create_divider && show_dividers_) {
     QString divider_key = DividerKey(type, item);
     item->sort_text.prepend(divider_key);
 
@@ -829,6 +945,7 @@ Qt::ItemFlags LibraryModel::flags(const QModelIndex& index) const {
            Qt::ItemIsDragEnabled;
   case LibraryItem::Type_Divider:
   case LibraryItem::Type_Root:
+  case LibraryItem::Type_LoadingIndicator:
   default:
     return Qt::ItemIsEnabled;
   }
@@ -850,6 +967,7 @@ QMimeData* LibraryModel::mimeData(const QModelIndexList& indexes) const {
 
     GeneratorMimeData* data = new GeneratorMimeData(generator);
     data->setData(kSmartPlaylistsMimeType, QByteArray());
+    data->name_for_new_playlist_ = this->data(indexes.first()).toString();
     return data;
   }
 
@@ -858,12 +976,13 @@ QMimeData* LibraryModel::mimeData(const QModelIndexList& indexes) const {
   QSet<int> song_ids;
 
   data->backend = backend_;
-
+  
   foreach (const QModelIndex& index, indexes) {
     GetChildSongs(IndexToItem(index), &urls, &data->songs, &song_ids);
   }
 
   data->setUrls(urls);
+  data->name_for_new_playlist_ = LibraryView::GetNameForNewPlaylist(data->songs);
 
   return data;
 }
@@ -921,12 +1040,17 @@ SongList LibraryModel::GetChildSongs(const QModelIndex &index) const {
 }
 
 void LibraryModel::SetFilterAge(int age) {
-  query_options_.max_age = age;
+  query_options_.set_max_age(age);
   ResetAsync();
 }
 
 void LibraryModel::SetFilterText(const QString& text) {
-  query_options_.filter = text;
+  query_options_.set_filter(text);
+  ResetAsync();
+}
+
+void LibraryModel::SetFilterQueryMode(QueryOptions::QueryMode query_mode) {
+  query_options_.set_query_mode(query_mode);
   ResetAsync();
 }
 
