@@ -44,7 +44,8 @@ class LibraryModel : public SimpleTreeModel<LibraryItem> {
   Q_ENUMS(GroupBy);
 
  public:
-  LibraryModel(LibraryBackend* backend, QObject* parent = 0);
+  LibraryModel(LibraryBackend* backend, TaskManager* task_manager,
+               QObject* parent = 0);
   ~LibraryModel();
 
   static const char* kSmartPlaylistsMimeType;
@@ -59,6 +60,7 @@ class LibraryModel : public SimpleTreeModel<LibraryItem> {
     Role_Key,
     Role_Artist,
     Role_IsDivider,
+    Role_Editable,
 
     LastRole
   };
@@ -128,6 +130,12 @@ class LibraryModel : public SimpleTreeModel<LibraryItem> {
   QMimeData* mimeData(const QModelIndexList& indexes) const;
   bool canFetchMore(const QModelIndex &parent) const;
 
+  // Whether or not to use album cover art, if it exists, in the library view
+  void set_pretty_covers(bool use_pretty_covers);
+
+  //Whether or not to show letters heading in the library view
+  void set_show_dividers(bool show_dividers);
+
  signals:
   void TotalSongCountUpdated(int count);
   void GroupingChanged(const LibraryModel::Grouping& g);
@@ -135,8 +143,10 @@ class LibraryModel : public SimpleTreeModel<LibraryItem> {
  public slots:
   void SetFilterAge(int age);
   void SetFilterText(const QString& text);
+  void SetFilterQueryMode(QueryOptions::QueryMode query_mode);
+
   void SetGroupBy(const LibraryModel::Grouping& g);
-  void Init();
+  void Init(bool async = true);
   void Reset();
   void ResetAsync();
 
@@ -155,8 +165,6 @@ class LibraryModel : public SimpleTreeModel<LibraryItem> {
   void ResetAsyncQueryFinished();
 
  private:
-  void Initialise();
-
   // Provides some optimisations for loading the list of items in the root.
   // This gets called a lot when filtering the playlist, so it's nice to be
   // able to do it in a background thread.
@@ -209,11 +217,13 @@ class LibraryModel : public SimpleTreeModel<LibraryItem> {
   QString DividerDisplayText(GroupBy type, const QString& key) const;
 
   // Helpers
+  QVariant AlbumIcon(const QModelIndex& index, int role) const;
   QVariant data(const LibraryItem* item, int role) const;
   bool CompareItems(const LibraryItem* a, const LibraryItem* b) const;
 
  private:
   LibraryBackend* backend_;
+  TaskManager* task_manager_;
   LibraryDirectoryModel* dir_model_;
   bool show_smart_playlists_;
   DefaultGenerators default_smart_playlists_;
@@ -241,9 +251,18 @@ class LibraryModel : public SimpleTreeModel<LibraryItem> {
 
   QIcon artist_icon_;
   QIcon album_icon_;
+  // used as a generic icon to show when no cover art is found,
+  // fixed to the same size as the artwork (32x32)
+  QImage no_cover_icon_pretty_;
   QIcon no_cover_icon_;
   QIcon playlists_dir_icon_;
   QIcon playlist_icon_;
+
+  int init_task_id_;
+  
+  QSize pretty_cover_size_;
+  bool use_pretty_covers_;
+  bool show_dividers_;
 };
 
 Q_DECLARE_METATYPE(LibraryModel::Grouping);
