@@ -21,9 +21,13 @@
 #include "libraryitem.h"
 #include "librarybackend.h"
 #include "core/deletefiles.h"
+#include "core/mimedata.h"
 #include "core/musicstorage.h"
+#include "core/utilities.h"
 #include "devices/devicemanager.h"
 #include "devices/devicestatefiltermodel.h"
+#include "scripting/scriptmanager.h"
+#include "scripting/uiinterface.h"
 #include "smartplaylists/wizard.h"
 #include "ui/iconloader.h"
 #include "ui/organisedialog.h"
@@ -31,10 +35,14 @@
 
 #include <QPainter>
 #include <QContextMenuEvent>
+#include <QHelpEvent>
 #include <QMenu>
 #include <QMessageBox>
-#include <QSortFilterProxyModel>
+#include <QSet>
 #include <QSettings>
+#include <QSortFilterProxyModel>
+#include <QToolTip>
+#include <QWhatsThis>
 
 using smart_playlists::Wizard;
 
@@ -77,8 +85,53 @@ void LibraryItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &o
   }
 }
 
+bool LibraryItemDelegate::helpEvent(QHelpEvent *event, QAbstractItemView *view,
+                                     const QStyleOptionViewItem &option,
+                                     const QModelIndex &index) {
+  Q_UNUSED(option);
+
+  if (!event || !view)
+    return false;
+
+  QHelpEvent *he = static_cast<QHelpEvent*>(event);
+  QString text = displayText(index.data(), QLocale::system());
+
+  if (text.isEmpty() || !he)
+    return false;
+
+  switch (event->type()) {
+    case QEvent::ToolTip: {
+      QRect displayed_text;
+      QSize real_text;
+      bool is_elided = false;
+
+      real_text = sizeHint(option, index);
+      displayed_text = view->visualRect(index);
+      is_elided = displayed_text.width() < real_text.width();
+      if(is_elided) {
+        QToolTip::showText(he->globalPos(), text, view);
+      } else { // in case that another text was previously displayed
+        QToolTip::hideText();
+      }
+      return true;
+    }
+
+    case QEvent::QueryWhatsThis:
+      return true;
+
+    case QEvent::WhatsThis:
+      QWhatsThis::showText(he->globalPos(), text, view);
+      return true;
+
+    default:
+      break;
+  }
+  return false;
+}
+
 LibraryView::LibraryView(QWidget* parent)
   : AutoExpandingTreeView(parent),
+    scripts_(NULL),
     library_(NULL),
     total_song_count_(-1),
     nomusic_(":nomusic.png"),
@@ -94,6 +147,7 @@ LibraryView::LibraryView(QWidget* parent)
   setSelectionMode(QAbstractItemView::ExtendedSelection);
 
   ReloadSettings();
+  setStyleSheet("QTreeView::item{padding-top:1px;}");
 }
 
 LibraryView::~LibraryView() {
@@ -104,20 +158,29 @@ void LibraryView::ReloadSettings() {
   s.beginGroup(kSettingsGroup);
 
   SetAutoOpen(s.value("auto_open", true).toBool());
+  if (library_ != NULL) {
+    library_->set_pretty_covers(s.value("pretty_covers", true).toBool());
+    library_->set_show_dividers(s.value("show_dividers", true).toBool());
+  }
 }
 
 void LibraryView::SetTaskManager(TaskManager *task_manager) {
   task_manager_ = task_manager;
 }
 
-void LibraryView::SetLibrary(LibraryModel *library) {
+void LibraryView::SetLibrary(LibraryModel* library) {
   library_ = library;
+  ReloadSettings();
 }
 
-void LibraryView::SetDeviceManager(DeviceManager *device_manager) {
+void LibraryView::SetDeviceManager(DeviceManager* device_manager) {
   devices_ = device_manager;
 }
 
+void LibraryView::SetScriptManager(ScriptManager* scripts) {
+  scripts_ = scripts;
+}
+
 void LibraryView::TotalSongCountUpdated(int count) {
   bool old = total_song_count_;
   total_song_count_ = count;
@@ -169,12 +232,18 @@ void LibraryView::mouseReleaseEvent(QMouseEvent* e) {
 }
 
 void LibraryView::contextMenuEvent(QContextMenuEvent *e) {
-  if (!context_menu_) {
+  if(!context_menu_) {
     context_menu_ = new QMenu(this);
-    load_ = context_menu_->addAction(IconLoader::Load("media-playback-start"),
-        tr("Load"), this, SLOT(Load()));
     add_to_playlist_ = context_menu_->addAction(IconLoader::Load("media-playback-start"),
-        tr("Add to playlist"), this, SLOT(AddToPlaylist()));
+        tr("Append to current playlist"), this, SLOT(AddToPlaylist()));
+    load_ = context_menu_->addAction(IconLoader::Load("media-playback-start"),
+        tr("Replace current playlist"), this, SLOT(Load()));
+    open_in_new_playlist_ = context_menu_->addAction(IconLoader::Load("document-new"),
+        tr("Open in new playlist"), this, SLOT(OpenInNewPlaylist()));
+
+    context_menu_->addSeparator();
+    add_to_playlist_enqueue_ = context_menu_->addAction(IconLoader::Load("go-next"),
+        tr("Queue track"), this, SLOT(AddToPlaylistEnqueue()));
 
     context_menu_->addSeparator();
     new_smart_playlist_ = context_menu_->addAction(IconLoader::Load("document-new"),
@@ -195,6 +264,10 @@ void LibraryView::contextMenuEvent(QContextMenuEvent *e) {
     context_menu_->addSeparator();
     edit_track_ = context_menu_->addAction(IconLoader::Load("edit-rename"),
       tr("Edit track information..."), this, SLOT(EditTracks()));
+    edit_tracks_ = context_menu_->addAction(IconLoader::Load("edit-rename"),
+      tr("Edit tracks information..."), this, SLOT(EditTracks()));
+    show_in_browser_ = context_menu_->addAction(IconLoader::Load("document-open-folder"),
+      tr("Show in file browser..."), this, SLOT(ShowInBrowser()));
 
     context_menu_->addSeparator();
     show_in_various_ = context_menu_->addAction(
@@ -202,9 +275,15 @@ void LibraryView::contextMenuEvent(QContextMenuEvent *e) {
     no_show_in_various_ = context_menu_->addAction(
         tr("Don't show in various artists"), this, SLOT(NoShowInVarious()));
 
+    context_menu_->addSeparator();
+
     copy_to_device_->setDisabled(devices_->connected_devices_model()->rowCount() == 0);
     connect(devices_->connected_devices_model(), SIGNAL(IsEmptyChanged(bool)),
             copy_to_device_, SLOT(setDisabled(bool)));
+
+    if(scripts_) {
+      scripts_->ui()->RegisterActionLocation("library_context_menu", context_menu_, NULL);
+    }
   }
 
   context_menu_index_ = indexAt(e->pos());
@@ -214,33 +293,74 @@ void LibraryView::contextMenuEvent(QContextMenuEvent *e) {
   context_menu_index_ = qobject_cast<QSortFilterProxyModel*>(model())
                         ->mapToSource(context_menu_index_);
 
-  const int type = library_->data(context_menu_index_, LibraryModel::Role_Type).toInt();
-  const bool enable_various = type == LibraryItem::Type_Container ||
-                              type == LibraryItem::Type_Song;
-  const bool smart_playlist = type == LibraryItem::Type_SmartPlaylist;
-  const bool show_smart     = type == LibraryItem::Type_PlaylistContainer ||
-                              smart_playlist;
-  const bool enable_add     = type == LibraryItem::Type_Container ||
-                              type == LibraryItem::Type_Song ||
-                              smart_playlist;
+  QModelIndexList selected_indexes =
+      qobject_cast<QSortFilterProxyModel*>(model())->mapSelectionToSource(
+          selectionModel()->selection()).indexes();
 
-  load_->setEnabled(enable_add);
-  add_to_playlist_->setEnabled(enable_add);
-  edit_track_->setEnabled(enable_add);
-  show_in_various_->setEnabled(enable_various);
-  no_show_in_various_->setEnabled(enable_various);
+  // number of smart playlists selected
+  int smart_playlists = 0;
+  // is the smart playlists header selected?
+  int smart_playlists_header = 0;
+  // number of non smart playlists selected
+  int regular_elements = 0;
+  // number of editable non smart playlists selected
+  int regular_editable = 0;
 
-  new_smart_playlist_->setVisible(show_smart);
-  edit_smart_playlist_->setVisible(show_smart);
-  delete_smart_playlist_->setVisible(show_smart);
-  organise_->setVisible(!show_smart);
-  copy_to_device_->setVisible(!show_smart);
-  delete_->setVisible(!show_smart);
-  show_in_various_->setVisible(!show_smart);
-  no_show_in_various_->setVisible(!show_smart);
+  foreach(const QModelIndex& index, selected_indexes) {
+    int type = library_->data(index, LibraryModel::Role_Type).toInt();
 
-  edit_smart_playlist_->setEnabled(smart_playlist);
-  delete_smart_playlist_->setEnabled(smart_playlist);
+    if(type == LibraryItem::Type_SmartPlaylist) {
+      smart_playlists++;
+    } else if(type == LibraryItem::Type_PlaylistContainer) {
+      smart_playlists_header++;
+    } else {
+      regular_elements++;
+    }
+
+    if(library_->data(index, LibraryModel::Role_Editable).toBool()) {
+      regular_editable++;
+    }
+  }
+
+  // TODO: check if custom plugin actions should be enabled / visible
+  const int songs_selected     = smart_playlists + smart_playlists_header + regular_elements;
+  const bool regular_elements_only = songs_selected == regular_elements && regular_elements > 0;
+  const bool smart_playlists_only = songs_selected == smart_playlists + smart_playlists_header;
+  const bool only_smart_playlist_selected = smart_playlists == 1 && songs_selected == 1;
+
+  // in all modes
+  load_->setEnabled(songs_selected);
+  add_to_playlist_->setEnabled(songs_selected);
+  open_in_new_playlist_->setEnabled(songs_selected);
+  add_to_playlist_enqueue_->setEnabled(songs_selected);
+
+  // allow mixed smart playlists / regular elements selected
+  show_in_browser_->setVisible(!smart_playlists_only);
+  edit_tracks_->setVisible(!smart_playlists_only && regular_editable > 1);
+  // if neither edit_track not edit_tracks are available, we show disabled
+  // edit_track element
+  edit_track_->setVisible(!smart_playlists_only && (regular_editable <= 1));
+  edit_track_->setEnabled(regular_editable == 1);
+
+  // only when no smart playlists selected
+  organise_->setVisible(regular_elements_only);
+  copy_to_device_->setVisible(regular_elements_only);
+  delete_->setVisible(regular_elements_only);
+  show_in_various_->setVisible(regular_elements_only);
+  no_show_in_various_->setVisible(regular_elements_only);
+
+  // only when all selected items are editable
+  organise_->setEnabled(regular_elements == regular_editable);
+  copy_to_device_->setEnabled(regular_elements == regular_editable);
+  delete_->setEnabled(regular_elements == regular_editable);
+
+  // only when no regular elements selected
+  new_smart_playlist_->setVisible(smart_playlists_only);
+  edit_smart_playlist_->setVisible(smart_playlists_only);
+  delete_smart_playlist_->setVisible(smart_playlists_only);
+
+  edit_smart_playlist_->setEnabled(only_smart_playlist_selected);
+  delete_smart_playlist_->setEnabled(only_smart_playlist_selected);
 
   context_menu_->popup(e->globalPos());
 }
@@ -271,17 +391,31 @@ void LibraryView::ShowInVarious(bool on) {
 }
 
 void LibraryView::Load() {
-  if (!context_menu_index_.isValid())
-    return;
-
-  emit Load(selectedIndexes());
+  QMimeData* data = model()->mimeData(selectedIndexes());
+  if (MimeData* mime_data = qobject_cast<MimeData*>(data)) {
+    mime_data->clear_first_ = true;
+  }
+  emit AddToPlaylistSignal(data);
 }
 
 void LibraryView::AddToPlaylist() {
-  if (!context_menu_index_.isValid())
-    return;
+  emit AddToPlaylistSignal(model()->mimeData(selectedIndexes()));
+}
 
-  emit AddToPlaylist(selectedIndexes());
+void LibraryView::AddToPlaylistEnqueue() {
+  QMimeData* data = model()->mimeData(selectedIndexes());
+  if (MimeData* mime_data = qobject_cast<MimeData*>(data)) {
+    mime_data->enqueue_now_ = true;
+  }
+  emit AddToPlaylistSignal(data);
+}
+
+void LibraryView::OpenInNewPlaylist() {
+  QMimeData* data = model()->mimeData(selectedIndexes());
+  if (MimeData* mime_data = qobject_cast<MimeData*>(data)) {
+    mime_data->open_in_new_playlist_ = true;
+  }
+  emit AddToPlaylistSignal(data);
 }
 
 void LibraryView::keyboardSearch(const QString& search) {
@@ -355,18 +489,6 @@ void LibraryView::CopyToDevice() {
   organise_dialog_->show();
 }
 
-void LibraryView::keyPressEvent(QKeyEvent* e) {
-  switch (e->key()) {
-    case Qt::Key_Enter:
-    case Qt::Key_Return:
-      if (currentIndex().isValid())
-        emit doubleClicked(currentIndex());
-      break;
-  }
-
-  QTreeView::keyPressEvent(e);
-}
-
 void LibraryView::DeleteFinished(const SongList& songs_with_errors) {
   if (songs_with_errors.isEmpty())
     return;
@@ -376,16 +498,6 @@ void LibraryView::DeleteFinished(const SongList& songs_with_errors) {
   // It deletes itself when the user closes it
 }
 
-void LibraryView::UpAndFocus() {
-  setCurrentIndex(moveCursor(QAbstractItemView::MoveUp, Qt::NoModifier));
-  setFocus();
-}
-
-void LibraryView::DownAndFocus() {
-  setCurrentIndex(moveCursor(QAbstractItemView::MoveDown, Qt::NoModifier));
-  setFocus();
-}
-
 void LibraryView::FilterReturnPressed() {
   if (!currentIndex().isValid()) {
     // Pick the first thing that isn't a divider
@@ -434,3 +546,49 @@ void LibraryView::EditSmartPlaylistFinished() {
   const Wizard* wizard = qobject_cast<Wizard*>(sender());
   library_->UpdateGenerator(context_menu_index_, wizard->CreateGenerator());
 }
+
+QString LibraryView::GetNameForNewPlaylist(const SongList& songs) {
+  if (songs.isEmpty()) {
+    return tr("Playlist");
+  }
+
+  QSet<QString> artists;
+  QSet<QString> albums;
+
+  foreach(const Song& song, songs) {
+    artists << (song.artist().isEmpty()
+                    ? tr("Unknown")
+                    : song.artist());
+    albums << (song.album().isEmpty()
+                    ? tr("Unknown")
+                    : song.album());
+
+    if(artists.size() > 1) {
+      break;
+    }
+  }
+
+  bool various_artists = artists.size() > 1;
+
+  QString result;
+  if(various_artists) {
+    result = tr("Various artists");
+  } else {
+    result = artists.values().first();
+  }
+
+  if(!various_artists && albums.size() == 1) {
+    result += " - " + albums.toList().first();
+  }
+
+  return result;
+}
+
+void LibraryView::ShowInBrowser() {
+  QStringList filenames;
+  foreach (const Song& song, GetSelectedSongs()) {
+    filenames << song.filename();
+  }
+
+  Utilities::OpenInFileBrowser(filenames);
+}
