@@ -29,14 +29,23 @@
 class DeviceManager;
 class LibraryModel;
 class OrganiseDialog;
+class ScriptManager;
 class TaskManager;
 
+class QMimeData;
+
 namespace smart_playlists { class Wizard; }
 
 class LibraryItemDelegate : public QStyledItemDelegate {
- public:
+  Q_OBJECT
+
+public:
   LibraryItemDelegate(QObject* parent);
   void paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const;
+
+public slots:
+  bool helpEvent(QHelpEvent *event, QAbstractItemView *view,
+                 const QStyleOptionViewItem &option, const QModelIndex &index);
 };
 
 class LibraryView : public AutoExpandingTreeView {
@@ -48,6 +57,12 @@ class LibraryView : public AutoExpandingTreeView {
 
   static const char* kSettingsGroup;
 
+  // Returns Songs currently selected in the library view. Please note that the
+  // selection is recursive meaning that if for example an album is selected
+  // this will return all of it's songs.
+  SongList GetSelectedSongs() const;
+
+  void SetScriptManager(ScriptManager* scripts);
   void SetTaskManager(TaskManager* task_manager);
   void SetLibrary(LibraryModel* library);
   void SetDeviceManager(DeviceManager* device_manager);
@@ -56,33 +71,35 @@ class LibraryView : public AutoExpandingTreeView {
   void keyboardSearch(const QString &search);
   void scrollTo(const QModelIndex& index, ScrollHint hint = EnsureVisible);
 
+  // Returns a pretty automatic name for playlist created from the given list of
+  // songs.
+  static QString GetNameForNewPlaylist(const SongList& songs);
+
  public slots:
   void TotalSongCountUpdated(int count);
   void ReloadSettings();
 
-  void UpAndFocus();
-  void DownAndFocus();
   void FilterReturnPressed();
 
  signals:
   void ShowConfigDialog();
-  void Load(const QModelIndexList& indexes);
-  void AddToPlaylist(const QModelIndexList& indexes);
 
  protected:
   // QWidget
   void paintEvent(QPaintEvent* event);
   void mouseReleaseEvent(QMouseEvent* e);
   void contextMenuEvent(QContextMenuEvent* e);
-  void keyPressEvent(QKeyEvent* e);
 
  private slots:
   void Load();
   void AddToPlaylist();
+  void AddToPlaylistEnqueue();
+  void OpenInNewPlaylist();
   void Organise();
   void CopyToDevice();
   void Delete();
   void EditTracks();
+  void ShowInBrowser();
   void ShowInVarious();
   void NoShowInVarious();
 
@@ -98,9 +115,10 @@ class LibraryView : public AutoExpandingTreeView {
  private:
   void RecheckIsEmpty();
   void ShowInVarious(bool on);
-  SongList GetSelectedSongs() const;
 
  private:
+  ScriptManager* scripts_;
+
   LibraryModel* library_;
   DeviceManager* devices_;
   TaskManager* task_manager_;
@@ -113,10 +131,14 @@ class LibraryView : public AutoExpandingTreeView {
   QModelIndex context_menu_index_;
   QAction* load_;
   QAction* add_to_playlist_;
+  QAction* add_to_playlist_enqueue_;
+  QAction* open_in_new_playlist_;
   QAction* organise_;
   QAction* copy_to_device_;
   QAction* delete_;
   QAction* edit_track_;
+  QAction* edit_tracks_;
+  QAction* show_in_browser_;
   QAction* show_in_various_;
   QAction* no_show_in_various_;
 
