@@ -18,20 +18,23 @@
 #include "librarywatcher.h"
 #include "librarybackend.h"
 #include "core/taskmanager.h"
+#include "playlistparsers/cueparser.h"
 
 #include <QFileSystemWatcher>
+#include <QDateTime>
 #include <QDirIterator>
 #include <QtDebug>
 #include <QThread>
 #include <QDateTime>
-#include <QTimer>
+#include <QHash>
+#include <QSet>
 #include <QSettings>
+#include <QTimer>
 
-#include <taglib/fileref.h>
-#include <taglib/tag.h>
+#include <fileref.h>
+#include <tag.h>
 
 QStringList LibraryWatcher::sValidImages;
-QStringList LibraryWatcher::sValidPlaylists;
 
 const char* LibraryWatcher::kSettingsGroup = "LibraryWatcher";
 
@@ -44,14 +47,14 @@ LibraryWatcher::LibraryWatcher(QObject* parent)
     monitor_(true),
     rescan_timer_(new QTimer(this)),
     rescan_paused_(false),
-    total_watches_(0)
+    total_watches_(0),
+    cue_parser_(new CueParser(backend_, this))
 {
   rescan_timer_->setInterval(1000);
   rescan_timer_->setSingleShot(true);
 
   if (sValidImages.isEmpty()) {
     sValidImages << "jpg" << "png" << "gif" << "jpeg";
-    sValidPlaylists << "m3u" << "pls";
   }
 
   ReloadSettings();
@@ -59,12 +62,13 @@ LibraryWatcher::LibraryWatcher(QObject* parent)
   connect(rescan_timer_, SIGNAL(timeout()), SLOT(RescanPathsNow()));
 }
 
-LibraryWatcher::ScanTransaction::ScanTransaction(LibraryWatcher* watcher,
-                                                 int dir, bool incremental)
+LibraryWatcher::ScanTransaction::ScanTransaction(LibraryWatcher* watcher, int dir,
+                                                 bool incremental, bool ignores_mtime)
   : progress_(0),
     progress_max_(0),
     dir_(dir),
     incremental_(incremental),
+    ignores_mtime_(ignores_mtime),
     watcher_(watcher),
     cached_songs_dirty_(true),
     known_subdirs_dirty_(true)
@@ -220,7 +224,7 @@ void LibraryWatcher::ScanSubdirectory(
     }
   }
 
-  if (!force_noincremental && t->is_incremental() &&
+  if (!t->ignores_mtime() && !force_noincremental && t->is_incremental() &&
       subdir.mtime == path_info.lastModified().toTime_t()) {
     // The directory hasn't changed since last time
     t->AddToProgress(1);
@@ -278,12 +282,19 @@ void LibraryWatcher::ScanSubdirectory(
   // Ask the database for a list of files in this directory
   SongList songs_in_db = t->FindSongsInSubdirectory(path);
 
+  QSet<QString> cues_processed;
+
   // Now compare the list from the database with the list of files on disk
   foreach (const QString& file, files_on_disk) {
     if (stop_requested_) return;
 
+    // associated cue
+    QString matching_cue = NoExtensionPart(file) + ".cue";
+
     Song matching_song;
     if (FindSongByPath(songs_in_db, file, &matching_song)) {
+      uint matching_cue_mtime = GetMtimeForCue(matching_cue);
+
       // The song is in the database and still on disk.
       // Check the mtime to see if it's been changed since it was added.
       QFileInfo file_info(file);
@@ -295,57 +306,57 @@ void LibraryWatcher::ScanSubdirectory(
         continue;
       }
 
-      bool changed = matching_song.mtime() != file_info.lastModified().toTime_t();
+      // cue sheet's path from library (if any)
+      QString song_cue = matching_song.cue_path();
+      uint song_cue_mtime = GetMtimeForCue(song_cue);
+
+      bool cue_deleted = song_cue_mtime == 0 && matching_song.has_cue();
+      bool cue_added = matching_cue_mtime != 0 && !matching_song.has_cue();
+
+      // watch out for cue songs which have their mtime equal to qMax(media_file_mtime, cue_sheet_mtime)
+      bool changed = (matching_song.mtime() != qMax(file_info.lastModified().toTime_t(), song_cue_mtime))
+                     || cue_deleted || cue_added;
 
       // Also want to look to see whether the album art has changed
       QString image = ImageForSong(file, album_art);
       if ((matching_song.art_automatic().isEmpty() && !image.isEmpty()) ||
-          (!matching_song.art_automatic().isEmpty() && !QFile::exists(matching_song.art_automatic()))) {
+          (!matching_song.art_automatic().isEmpty()
+           && !matching_song.has_embedded_cover()
+           && !QFile::exists(matching_song.art_automatic()))) {
         changed = true;
       }
 
-      if (changed) {
+      // the song's changed - reread the metadata from file
+      if (t->ignores_mtime() || changed) {
         qDebug() << file << "changed";
 
-        // It's changed - reread the metadata from the file
-        Song song_on_disk;
-        song_on_disk.InitFromFile(file, t->dir());
-        if (!song_on_disk.is_valid())
-          continue;
-        song_on_disk.set_id(matching_song.id());
-        song_on_disk.set_art_automatic(image);
-
-        // Preserve user-settable fields
-        song_on_disk.set_playcount(matching_song.playcount());
-        song_on_disk.set_skipcount(matching_song.skipcount());
-        song_on_disk.set_lastplayed(matching_song.lastplayed());
-        song_on_disk.set_rating(matching_song.rating());
-        song_on_disk.set_score(matching_song.score());
-        song_on_disk.set_art_manual(matching_song.art_manual());
-
-        if (!matching_song.IsMetadataEqual(song_on_disk)) {
-          qDebug() << file << "metadata changed";
-          // Update the song in the DB
-          t->new_songs << song_on_disk;
+        // if cue associated...
+        if(!cue_deleted && (matching_song.has_cue() || cue_added)) {
+          UpdateCueAssociatedSongs(file, path, matching_cue, image, t);
+        // if no cue or it's about to lose it...
         } else {
-          // Only the mtimes changed
-          t->touched_songs << song_on_disk;
+          UpdateNonCueAssociatedSong(file, matching_song, image, cue_deleted, t);
         }
       }
     } else {
       // The song is on disk but not in the DB
+      SongList song_list = ScanNewFile(file, path, matching_cue, &cues_processed);
 
-      Song song;
-      song.InitFromFile(file, t->dir());
-      if (!song.is_valid())
+      if(song_list.isEmpty()) {
         continue;
+      }
+
       qDebug() << file << "created";
+      // choose an image for the song(s)
+      QString image = ImageForSong(file, album_art);
 
-      // Choose an image for the song
-      if (song.art_automatic().isEmpty())
-        song.set_art_automatic(ImageForSong(file, album_art));
+      foreach (Song song, song_list) {
+        song.set_directory_id(t->dir());
+        if (song.art_automatic().isEmpty())
+          song.set_art_automatic(image);
 
-      t->new_songs << song;
+        t->new_songs << song;
+      }
     }
   }
 
@@ -379,6 +390,149 @@ void LibraryWatcher::ScanSubdirectory(
   }
 }
 
+void LibraryWatcher::UpdateCueAssociatedSongs(const QString& file, const QString& path,
+                                              const QString& matching_cue, const QString& image,
+                                              ScanTransaction* t) {
+  QFile cue(matching_cue);
+  cue.open(QIODevice::ReadOnly);
+
+  SongList old_sections = backend_->GetSongsByFilename(file);
+
+  QHash<quint64, Song> sections_map;
+  foreach(const Song& song, old_sections) {
+    sections_map[song.beginning_nanosec()] = song;
+  }
+
+  QSet<int> used_ids;
+
+  // update every song that's in the cue and library
+  foreach(Song cue_song, cue_parser_->Load(&cue, matching_cue, path)) {
+    cue_song.set_directory_id(t->dir());
+
+    Song matching = sections_map[cue_song.beginning_nanosec()];
+    // a new section
+    if(!matching.is_valid()) {
+      t->new_songs << cue_song;
+    // changed section
+    } else {
+      PreserveUserSetData(file, image, matching, &cue_song, t);
+      used_ids.insert(matching.id());
+    }
+  }
+
+  // sections that are now missing
+  foreach(const Song& matching, old_sections) {
+    if(!used_ids.contains(matching.id())) {
+      t->deleted_songs << matching;
+    }
+  }
+}
+
+
+void LibraryWatcher::UpdateNonCueAssociatedSong(const QString& file, const Song& matching_song,
+                                                const QString& image, bool cue_deleted,
+                                                ScanTransaction* t) {
+  // if a cue got deleted, we turn it's first section into the new
+  // 'raw' (cueless) song and we just remove the rest of the sections
+  // from the library
+  if(cue_deleted) {
+    foreach(const Song& song, backend_->GetSongsByFilename(file)) {
+      if(!song.IsMetadataEqual(matching_song)) {
+        t->deleted_songs << song;
+      }
+    }
+  }
+
+  Song song_on_disk;
+  song_on_disk.InitFromFile(file, t->dir());
+
+  if(song_on_disk.is_valid()) {
+    PreserveUserSetData(file, image, matching_song, &song_on_disk, t);
+  }
+}
+
+SongList LibraryWatcher::ScanNewFile(const QString& file, const QString& path,
+                                     const QString& matching_cue, QSet<QString>* cues_processed) {
+  SongList song_list;
+
+  uint matching_cue_mtime = GetMtimeForCue(matching_cue);
+  // if it's a cue - create virtual tracks
+  if(matching_cue_mtime) {
+    // don't process the same cue many times
+    if(cues_processed->contains(matching_cue))
+      return song_list;
+
+    QFile cue(matching_cue);
+    cue.open(QIODevice::ReadOnly);
+
+    // Ignore FILEs pointing to other media files. Also, watch out for incorrect
+    // media files. Playlist parser for CUEs considers every entry in sheet
+    // valid and we don't want invalid media getting into library!
+    foreach(const Song& cue_song, cue_parser_->Load(&cue, matching_cue, path)) {
+      if(cue_song.filename() == file && cue_song.HasProperMediaFile()) {
+        song_list << cue_song;
+      }
+    }
+
+    if(!song_list.isEmpty()) {
+      *cues_processed << matching_cue;
+    }
+
+  // it's a normal media file
+  } else {
+    Song song;
+    song.InitFromFile(file, -1);
+
+    if (song.is_valid()) {
+      song_list << song;
+    }
+
+  }
+
+  return song_list;
+}
+
+void LibraryWatcher::PreserveUserSetData(const QString& file, const QString& image,
+                                         const Song& matching_song, Song* out, ScanTransaction* t) {
+  out->set_id(matching_song.id());
+
+  // Previous versions of Clementine incorrectly overwrote this and
+  // stored it in the DB, so we can't rely on matching_song to
+  // know if it has embedded artwork or not, but we can check here.
+  if (!out->has_embedded_cover())
+    out->set_art_automatic(image);
+
+  out->set_playcount(matching_song.playcount());
+  out->set_skipcount(matching_song.skipcount());
+  out->set_lastplayed(matching_song.lastplayed());
+  out->set_rating(matching_song.rating());
+  out->set_score(matching_song.score());
+  out->set_art_manual(matching_song.art_manual());
+
+  if (!matching_song.IsMetadataEqual(*out)) {
+    qDebug() << file << "metadata changed";
+
+    // Update the song in the DB
+    t->new_songs << *out;
+  } else {
+    // Only the mtime's changed
+    t->touched_songs << *out;
+  }
+}
+
+uint LibraryWatcher::GetMtimeForCue(const QString& cue_path) {
+  // slight optimisation
+  if(cue_path.isEmpty()) {
+    return 0;
+  }
+
+  QDateTime cue_last_modified = QFileInfo(cue_path).lastModified();
+
+  return cue_last_modified.isValid()
+             ? cue_last_modified.toTime_t()
+             : 0;
+}
+
 void LibraryWatcher::AddWatch(QFileSystemWatcher* w, const QString& path) {
   if (!QFile::exists(path))
     return;
@@ -450,13 +604,38 @@ void LibraryWatcher::RescanPathsNow() {
 }
 
 QString LibraryWatcher::PickBestImage(const QStringList& images) {
+  
   // This is used when there is more than one image in a directory.
-  // Just pick the biggest image.
+  // Pick the biggest image that matches the most important filter
+  
+  QStringList filtered;
+  
+  foreach(const QString& filter_text, best_image_filters_) {
+    // the images in the images list are represented by a full path, 
+    // so we need to isolate just the filename
+    foreach(const QString& image, images) {
+      QFileInfo file_info(image);
+      QString filename(file_info.fileName());
+      if (filename.contains(filter_text, Qt::CaseInsensitive))
+        filtered << image;
+    }
+
+    /* We assume the filters are give in the order best to worst, so 
+      if we've got a result, we go with it. Otherwise we might
+      start capturing more generic rules */
+    if (!filtered.isEmpty())
+      break;
+  }
+  
+  if (filtered.isEmpty()){
+    // the filter was too restrictive, just use the original list
+    filtered = images;
+  }
 
   int biggest_size = 0;
   QString biggest_path;
 
-  foreach (const QString& path, images) {
+  foreach (const QString& path, filtered) {
     QImage image(path);
     if (image.isNull())
       continue;
@@ -497,7 +676,16 @@ void LibraryWatcher::ReloadSettings() {
   s.beginGroup(kSettingsGroup);
   scan_on_startup_ = s.value("startup_scan", true).toBool();
   monitor_ = s.value("monitor", true).toBool();
-
+  
+  best_image_filters_.clear();
+  QStringList filters = s.value("cover_art_patterns",
+      QStringList() << "front" << "cover").toStringList();
+  foreach(const QString& filter, filters) {
+    QString s = filter.trimmed();
+    if (!s.isEmpty())
+      best_image_filters_ << s;
+  }
+  
   if (!monitor_ && was_monitoring_before) {
     // Remove all directories from all QFileSystemWatchers
     foreach (const DirData& data, watched_dirs_.values()) {
@@ -529,9 +717,22 @@ void LibraryWatcher::IncrementalScanAsync() {
   QMetaObject::invokeMethod(this, "IncrementalScanNow", Qt::QueuedConnection);
 }
 
+void LibraryWatcher::FullScanAsync() {
+  QMetaObject::invokeMethod(this, "FullScanNow", Qt::QueuedConnection);
+}
+
 void LibraryWatcher::IncrementalScanNow() {
+  PerformScan(true, false);
+}
+
+void LibraryWatcher::FullScanNow() {
+  PerformScan(false, true);
+}
+
+void LibraryWatcher::PerformScan(bool incremental, bool ignore_mtimes) {
   foreach (const DirData& data, watched_dirs_.values()) {
-    ScanTransaction transaction(this, data.dir.id, true);
+    ScanTransaction transaction(this, data.dir.id,
+                                incremental, ignore_mtimes);
     SubdirectoryList subdirs(transaction.GetAllSubdirs());
     transaction.AddToProgressMax(subdirs.count());
 
@@ -541,5 +742,6 @@ void LibraryWatcher::IncrementalScanNow() {
       ScanSubdirectory(subdir.path, subdir, &transaction);
     }
   }
+
   emit CompilationsNeedUpdating();
 }
