@@ -28,6 +28,7 @@
 class QFileSystemWatcher;
 class QTimer;
 
+class CueParser;
 class LibraryBackend;
 class TaskManager;
 
@@ -42,8 +43,9 @@ class LibraryWatcher : public QObject {
   void set_backend(LibraryBackend* backend) { backend_ = backend; }
   void set_task_manager(TaskManager* task_manager) { task_manager_ = task_manager; }
   void set_device_name(const QString& device_name) { device_name_ = device_name; }
-
+  
   void IncrementalScanAsync();
+  void FullScanAsync();
   void SetRescanPausedAsync(bool pause);
   void ReloadSettingsAsync();
 
@@ -77,7 +79,8 @@ class LibraryWatcher : public QObject {
   // LibraryBackend::FindSongsInDirectory.
   class ScanTransaction {
    public:
-    ScanTransaction(LibraryWatcher* watcher, int dir, bool incremental);
+    ScanTransaction(LibraryWatcher* watcher, int dir,
+                    bool incremental, bool ignores_mtime = false);
     ~ScanTransaction();
 
     SongList FindSongsInSubdirectory(const QString& path);
@@ -91,6 +94,7 @@ class LibraryWatcher : public QObject {
 
     int dir() const { return dir_; }
     bool is_incremental() const { return incremental_; }
+    bool ignores_mtime() const { return ignores_mtime_; }
 
     SongList deleted_songs;
     SongList new_songs;
@@ -107,7 +111,15 @@ class LibraryWatcher : public QObject {
     int progress_max_;
 
     int dir_;
+    // Incremental scan enters a directory only if it has changed since the
+    // last scan.
     bool incremental_;
+    // This type of scan updates every file in a folder that's
+    // being scanned. Even if it detects the file hasn't changed since
+    // the last scan. Also, since it's ignoring mtimes on folders too,
+    // it will go as deep in the folder hierarchy as it's possible.
+    bool ignores_mtime_;
+
     LibraryWatcher* watcher_;
 
     SongList cached_songs_;
@@ -120,17 +132,41 @@ class LibraryWatcher : public QObject {
  private slots:
   void DirectoryChanged(const QString& path);
   void IncrementalScanNow();
+  void FullScanNow();
   void RescanPathsNow();
   void ScanSubdirectory(const QString& path, const Subdirectory& subdir,
                         ScanTransaction* t, bool force_noincremental = false);
 
  private:
   static bool FindSongByPath(const SongList& list, const QString& path, Song* out);
+  inline static QString NoExtensionPart( const QString &fileName );
   inline static QString ExtensionPart( const QString &fileName );
   inline static QString DirectoryPart( const QString &fileName );
-  static QString PickBestImage(const QStringList& images);
-  static QString ImageForSong(const QString& path, QMap<QString, QStringList>& album_art);
+  QString PickBestImage(const QStringList& images);
+  QString ImageForSong(const QString& path, QMap<QString, QStringList>& album_art);
   void AddWatch(QFileSystemWatcher* w, const QString& path);
+  uint GetMtimeForCue(const QString& cue_path);
+  void PerformScan(bool incremental, bool ignore_mtimes);
+
+  // Updates the sections of a cue associated and altered (according to mtime)
+  // media file during a scan.
+  void UpdateCueAssociatedSongs(const QString& file, const QString& path,
+                                const QString& matching_cue, const QString& image,
+                                ScanTransaction* t);
+  // Updates a single non-cue associated and altered (according to mtime) song
+  // during a scan.
+  void UpdateNonCueAssociatedSong(const QString& file, const Song& matching_song,
+                                  const QString& image, bool cue_deleted,
+                                  ScanTransaction* t)  ;
+  // Updates a new song with some metadata taken from it's equivalent old 
+  // song (for example rating and score).
+  void PreserveUserSetData(const QString& file, const QString& image,
+                           const Song& matching_song, Song* out, ScanTransaction* t);
+  // Scans a single media file that's present on the disk but not yet in the library.
+  // It may result in a multiple files added to the library when the media file
+  // has many sections (like a CUE related media file).
+  SongList ScanNewFile(const QString& file, const QString& path,
+                       const QString& matching_cue, QSet<QString>* cues_processed);
 
  private:
   // One of these gets stored for each Directory we're watching
@@ -142,6 +178,14 @@ class LibraryWatcher : public QObject {
   LibraryBackend* backend_;
   TaskManager* task_manager_;
   QString device_name_;
+  
+  /* A list of words use to try to identify the (likely) best image 
+   * found in an directory to use as cover artwork.
+   * e.g. using ["front", "cover"] would identify front.jpg and
+   * exclude back.jpg.
+   */
+  QStringList best_image_filters_; 
+  
   bool stop_requested_;
   bool scan_on_startup_;
   bool monitor_;
@@ -153,14 +197,18 @@ class LibraryWatcher : public QObject {
 
   int total_watches_;
 
+  CueParser* cue_parser_;
+
   static QStringList sValidImages;
-  static QStringList sValidPlaylists;
 
   #ifdef Q_OS_DARWIN
   static const int kMaxWatches = 100;
   #endif
 };
 
+inline QString LibraryWatcher::NoExtensionPart( const QString &fileName ) {
+  return fileName.contains( '.' ) ? fileName.section( '.', 0, -2 ) : "";
+}
 // Thanks Amarok
 inline QString LibraryWatcher::ExtensionPart( const QString &fileName ) {
   return fileName.contains( '.' ) ? fileName.mid( fileName.lastIndexOf('.') + 1 ).toLower() : "";
