@@ -15,35 +15,46 @@
    along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <QtGlobal>
+
 #ifdef Q_OS_WIN32
-#  define _WIN32_WINNT 0x0500
+#  define _WIN32_WINNT 0x0600
 #  include <windows.h>
 #  include <iostream>
 #endif // Q_OS_WIN32
 
 #include "config.h"
+#include "core/artloader.h"
 #include "core/commandlineoptions.h"
 #include "core/database.h"
 #include "core/encoding.h"
 #include "core/mac_startup.h"
 #include "core/network.h"
+#include "core/networkproxyfactory.h"
 #include "core/player.h"
 #include "core/potranslator.h"
 #include "core/song.h"
+#include "core/taskmanager.h"
 #include "core/utilities.h"
 #include "engines/enginebase.h"
 #include "library/directory.h"
-#include "radio/lastfmservice.h"
+#include "playlist/playlist.h"
+#include "playlist/playlistmanager.h"
+#include "radio/radiomodel.h"
 #include "smartplaylists/generator.h"
 #include "ui/equalizer.h"
 #include "ui/iconloader.h"
 #include "ui/mainwindow.h"
+#include "ui/systemtrayicon.h"
+#include "version.h"
+#include "widgets/osd.h"
 
 #include "qtsingleapplication.h"
 #include "qtsinglecoreapplication.h"
 
 #include <QDir>
 #include <QLibraryInfo>
+#include <QNetworkProxyFactory>
 #include <QSqlDatabase>
 #include <QSqlQuery>
 #include <QTranslator>
@@ -51,16 +62,38 @@
 
 #include <glib-object.h>
 #include <glib/gutils.h>
+#include <gst/gst.h>
+
+#include <boost/scoped_ptr.hpp>
+using boost::scoped_ptr;
 
 #include <echonest/Config.h>
 
-
-
-#ifdef HAVE_GSTREAMER
-#  include <gst/gst.h>
-   class GstEnginePipeline;
+#ifdef Q_OS_DARWIN
+  #include <sys/resource.h>
+  #include <sys/sysctl.h>
 #endif
 
+#ifdef HAVE_LIBLASTFM
+  #include "radio/lastfmservice.h"
+#else
+  class LastFMService;
+#endif
+
+#ifdef HAVE_DBUS
+  #include "core/mpris.h"
+  #include "core/mpris2.h"
+  #include "dbus/metatypes.h"
+  #include <QDBusArgument>
+  #include <QDBusConnection>
+  #include <QImage>
+
+  QDBusArgument& operator<< (QDBusArgument& arg, const QImage& image);
+  const QDBusArgument& operator>> (const QDBusArgument& arg, QImage& image);
+#endif
+
+class GstEnginePipeline;
+
 // Load sqlite plugin on windows and mac.
 #ifdef HAVE_STATIC_SQLITE
 # include <QtPlugin>
@@ -100,15 +133,35 @@ int main(int argc, char *argv[]) {
   // Do Mac specific startup to get media keys working.
   // This must go before QApplication initialisation.
   mac::MacMain();
+
+  {
+    // Bump the soft limit for the number of file descriptors from the default of 256 to
+    // the maximum (usually 10240).
+    struct rlimit limit;
+    getrlimit(RLIMIT_NOFILE, &limit);
+
+    // getrlimit() lies about the hard limit so we have to check sysctl.
+    int max_fd = 0;
+    size_t len = sizeof(max_fd);
+    sysctlbyname("kern.maxfilesperproc", &max_fd, &len, NULL, 0);
+
+    limit.rlim_cur = max_fd;
+    int ret = setrlimit(RLIMIT_NOFILE, &limit);
+
+    if (ret == 0) {
+      qDebug() << "Max fd:" << max_fd;
+    }
+  }
 #endif
 
   QCoreApplication::setApplicationName("Clementine");
-  QCoreApplication::setApplicationVersion(CLEMENTINE_VERSION_STRING);
+  QCoreApplication::setApplicationVersion(CLEMENTINE_VERSION_DISPLAY);
   QCoreApplication::setOrganizationName("Clementine");
-  QCoreApplication::setOrganizationDomain("davidsansome.com");
+  QCoreApplication::setOrganizationDomain("clementine-player.org");
 
 #ifdef Q_OS_DARWIN
   // Must happen after QCoreApplication::setOrganizationName().
+  setenv("XDG_CONFIG_HOME", Utilities::GetConfigPath(Utilities::Path_Root).toLocal8Bit().constData(), 1);
   if (mac::MigrateLegacyConfigFiles()) {
     QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE");
     db.setDatabaseName(Utilities::GetConfigPath(
@@ -134,6 +187,7 @@ int main(int argc, char *argv[]) {
   qRegisterMetaType<DirectoryList>("DirectoryList");
   qRegisterMetaType<Subdirectory>("Subdirectory");
   qRegisterMetaType<SubdirectoryList>("SubdirectoryList");
+  qRegisterMetaType<Song>("Song");
   qRegisterMetaType<SongList>("SongList");
   qRegisterMetaType<PlaylistItemList>("PlaylistItemList");
   qRegisterMetaType<Engine::State>("Engine::State");
@@ -144,26 +198,32 @@ int main(int argc, char *argv[]) {
   qRegisterMetaType<QNetworkReply*>("QNetworkReply*");
   qRegisterMetaType<QNetworkReply**>("QNetworkReply**");
   qRegisterMetaType<smart_playlists::GeneratorPtr>("smart_playlists::GeneratorPtr");
+  qRegisterMetaType<ColumnAlignmentMap>("ColumnAlignmentMap");
+  qRegisterMetaTypeStreamOperators<QMap<int, int> >("ColumnAlignmentMap");
 
-#ifdef HAVE_GSTREAMER
   qRegisterMetaType<GstBuffer*>("GstBuffer*");
   qRegisterMetaType<GstElement*>("GstElement*");
   qRegisterMetaType<GstEnginePipeline*>("GstEnginePipeline*");
-#endif
 
+#ifdef HAVE_LIBLASTFM
   lastfm::ws::ApiKey = LastFMService::kApiKey;
   lastfm::ws::SharedSecret = LastFMService::kSecret;
+#endif
 
-  // Parse commandline options - need to do this before starting the
-  // QApplication so it works without an X server
   CommandlineOptions options(argc, argv);
-  if (!options.Parse())
-    return 1;
 
   {
     // Only start a core application now so we can check if there's another
     // Clementine running without needing an X server.
+    // This MUST be done before parsing the commandline options so QTextCodec
+    // gets the right system locale for filenames.
     QtSingleCoreApplication a(argc, argv);
+
+    // Parse commandline options - need to do this before starting the
+    // full QApplication so it works without an X server
+    if (!options.Parse())
+      return 1;
+
     if (a.isRunning()) {
       if (options.is_empty()) {
         qDebug() << "Clementine is already running - activating existing window";
@@ -186,7 +246,11 @@ int main(int argc, char *argv[]) {
 #endif
 
   a.setQuitOnLastWindowClosed(false);
-  a.isRunning(); // Otherwise QtLocalPeer won't lock the lockfile :S
+
+  // Do this check again because another instance might have started by now
+  if (a.isRunning() && a.sendMessage(options.Serialize(), 5000)) {
+    return 0;
+  }
 
 #ifndef Q_OS_DARWIN
   // Gnome on Ubuntu has menu icons disabled by default.  I think that's a bad
@@ -220,12 +284,62 @@ int main(int argc, char *argv[]) {
   Echonest::Config::instance()->setAPIKey("DFLFLJBUF4EGTXHIG");
   Echonest::Config::instance()->setNetworkAccessManager(new NetworkAccessManager);
 
+  // Network proxy
+  QNetworkProxyFactory::setApplicationProxyFactory(
+      NetworkProxyFactory::Instance());
+
   // Seed the random number generator
   srand(time(NULL));
 
-  // Window
-  MainWindow w(options.engine());
+  // Create some key objects
+  scoped_ptr<BackgroundThread<Database> > database(
+      new BackgroundThreadImplementation<Database, Database>(NULL));
+  database->Start(true);
+  TaskManager task_manager;
+  PlaylistManager playlists(&task_manager, NULL);
+  RadioModel radio_model(database.get(), &task_manager, NULL);
 
+  // Get the last.fm service if it's available
+  LastFMService* lastfm_service = NULL;
+#ifdef HAVE_LIBLASTFM
+  lastfm_service = RadioModel::Service<LastFMService>();
+#endif
+
+  // Create the player
+  Player player(&playlists, lastfm_service);
+
+  // Create the tray icon and OSD
+  scoped_ptr<SystemTrayIcon> tray_icon(SystemTrayIcon::CreateSystemTrayIcon());
+  OSD osd(tray_icon.get());
+
+  ArtLoader art_loader;
+
+#ifdef HAVE_DBUS
+  qDBusRegisterMetaType<QImage>();
+  qDBusRegisterMetaType<TrackMetadata>();
+  qDBusRegisterMetaType<TrackIds>();
+  qDBusRegisterMetaType<QList<QByteArray> >();
+
+  mpris::Mpris mpris(&player, &art_loader);
+
+  QObject::connect(&playlists, SIGNAL(CurrentSongChanged(Song)), &art_loader, SLOT(LoadArt(Song)));
+  QObject::connect(&art_loader, SIGNAL(ThumbnailLoaded(Song, QString, QImage)),
+                   &osd, SLOT(CoverArtPathReady(Song, QString)));
+#endif
+
+  // Window
+  MainWindow w(
+      database.get(),
+      &task_manager,
+      &playlists,
+      &radio_model,
+      &player,
+      tray_icon.get(),
+      &osd,
+      &art_loader);
+#ifdef HAVE_DBUS
+  QObject::connect(&mpris, SIGNAL(RaiseMainWindow()), &w, SLOT(Raise()));
+#endif
   QObject::connect(&a, SIGNAL(messageReceived(QByteArray)), &w, SLOT(CommandlineOptionsReceived(QByteArray)));
   w.CommandlineOptionsReceived(options);
 
