@@ -18,6 +18,7 @@
 #include "playlist.h"
 #include "playlistbackend.h"
 #include "playlistfilter.h"
+#include "playlistitemmimedata.h"
 #include "playlistundocommands.h"
 #include "queue.h"
 #include "songloaderinserter.h"
@@ -40,20 +41,20 @@
 #include "smartplaylists/generatorinserter.h"
 #include "smartplaylists/generatormimedata.h"
 
-#include <QtDebug>
 #include <QApplication>
-#include <QMimeData>
 #include <QBuffer>
-#include <QFileInfo>
 #include <QDirIterator>
-#include <QUndoStack>
+#include <QFileInfo>
+#include <QMimeData>
+#include <QMutableListIterator>
 #include <QSortFilterProxyModel>
+#include <QUndoStack>
+#include <QtConcurrentRun>
+#include <QtDebug>
 
 #include <boost/bind.hpp>
 #include <algorithm>
 
-#include <lastfm/ScrobblePoint>
-
 using smart_playlists::Generator;
 using smart_playlists::GeneratorInserter;
 using smart_playlists::GeneratorPtr;
@@ -63,6 +64,14 @@ using boost::shared_ptr;
 const char* Playlist::kRowsMimetype = "application/x-clementine-playlist-rows";
 const char* Playlist::kPlayNowMimetype = "application/x-clementine-play-now";
 
+const int Playlist::kInvalidSongPriority = 200;
+const QRgb Playlist::kInvalidSongColor = qRgb(0xC0, 0xC0, 0xC0);
+
+const int Playlist::kDynamicHistoryPriority = 100;
+const QRgb Playlist::kDynamicHistoryColor = qRgb(0x80, 0x80, 0x80);
+
+const char* Playlist::kSettingsGroup = "Playlist";
+
 Playlist::Playlist(PlaylistBackend* backend,
                    TaskManager* task_manager,
                    LibraryBackend* library,
@@ -101,6 +110,20 @@ Playlist::Playlist(PlaylistBackend* backend,
           SLOT(TracksEnqueued(const QModelIndex&, int, int)));
 
   connect(queue_, SIGNAL(layoutChanged()), SLOT(QueueLayoutChanged()));
+
+  column_alignments_[Column_Length]
+      = column_alignments_[Column_Track]
+      = column_alignments_[Column_Disc]
+      = column_alignments_[Column_Year]
+      = column_alignments_[Column_BPM]
+      = column_alignments_[Column_Bitrate]
+      = column_alignments_[Column_Samplerate]
+      = column_alignments_[Column_Filesize]
+      = column_alignments_[Column_PlayCount]
+      = column_alignments_[Column_SkipCount]
+      = (Qt::AlignRight | Qt::AlignVCenter);
+
+  column_alignments_[Column_Score] = (Qt::AlignCenter);
 }
 
 Playlist::~Playlist() {
@@ -109,12 +132,14 @@ Playlist::~Playlist() {
 }
 
 template<typename T>
-QModelIndex InsertSongItems(Playlist* playlist, const SongList& songs, int pos) {
+void Playlist::InsertSongItems(const SongList& songs, int pos, bool play_now, bool enqueue) {
   PlaylistItemList items;
+
   foreach (const Song& song, songs) {
     items << PlaylistItemPtr(new T(song));
   }
-  return playlist->InsertItems(items, pos);
+
+  InsertItems(items, pos, play_now, enqueue);
 }
 
 QVariant Playlist::headerData(int section, Qt::Orientation, int role) const {
@@ -210,7 +235,8 @@ QVariant Playlist::data(const QModelIndex& index, int role) const {
 
     case Role_CanSetRating:
       return index.column() == Column_Rating &&
-             items_[index.row()]->IsLocalLibraryItem();
+             items_[index.row()]->IsLocalLibraryItem() &&
+             items_[index.row()]->Metadata().id() != -1;
 
     case Qt::EditRole:
     case Qt::ToolTipRole:
@@ -221,14 +247,10 @@ QVariant Playlist::data(const QModelIndex& index, int role) const {
       // Don't forget to change Playlist::CompareItems when adding new columns
       switch (index.column()) {
         case Column_Title:
-          if (!song.title().isEmpty())
-            return song.title();
-          if (!song.basefilename().isEmpty())
-            return song.basefilename();
-          return song.filename();
+          return song.PrettyTitle();
         case Column_Artist:       return song.artist();
         case Column_Album:        return song.album();
-        case Column_Length:       return song.length();
+        case Column_Length:       return song.length_nanosec();
         case Column_Track:        return song.track();
         case Column_Disc:         return song.disc();
         case Column_Year:         return song.year();
@@ -260,27 +282,18 @@ QVariant Playlist::data(const QModelIndex& index, int role) const {
     }
 
     case Qt::TextAlignmentRole:
-      switch (index.column()) {
-        case Column_Length:
-        case Column_Track:
-        case Column_Disc:
-        case Column_Year:
-        case Column_BPM:
-        case Column_Bitrate:
-        case Column_Samplerate:
-        case Column_Filesize:
-        case Column_PlayCount:
-        case Column_SkipCount:
-          return QVariant(Qt::AlignRight | Qt::AlignVCenter);
-        case Column_Score:
-          return QVariant(Qt::AlignCenter | Qt::AlignVCenter);
-        default:
-          return QVariant(Qt::AlignLeft | Qt::AlignVCenter);
-      }
+      return QVariant(column_alignments_.value(index.column(), (Qt::AlignLeft | Qt::AlignVCenter)));
 
     case Qt::ForegroundRole:
-      if (items_[index.row()]->IsDynamicHistory())
-        return QApplication::palette().brush(QPalette::Disabled, QPalette::Text);
+      if (items_[index.row()]->HasCurrentForegroundColor()) {
+        return QBrush(items_[index.row()]->GetCurrentForegroundColor());
+      }
+      return QVariant();
+
+    case Qt::BackgroundRole:
+      if (items_[index.row()]->HasCurrentBackgroundColor()) {
+        return QBrush(items_[index.row()]->GetCurrentBackgroundColor());
+      }
       return QVariant();
 
     default:
@@ -333,11 +346,15 @@ void Playlist::ItemReloadComplete() {
   }
 }
 
-int Playlist::current_index() const {
+int Playlist::current_row() const {
   return current_item_index_.isValid() ? current_item_index_.row() : -1;
 }
 
-int Playlist::last_played_index() const {
+const QModelIndex Playlist::current_index() const {
+  return current_item_index_;
+}
+
+int Playlist::last_played_row() const {
   return last_played_item_index_.isValid() ? last_played_item_index_.row() : -1;
 }
 
@@ -433,9 +450,9 @@ int Playlist::PreviousVirtualIndex(int i) const {
   return -1;
 }
 
-int Playlist::next_index() const {
+int Playlist::next_row() const {
   // Did we want to stop after this track?
-  if (stop_after_.isValid() && current_index() == stop_after_.row())
+  if (stop_after_.isValid() && current_row() == stop_after_.row())
     return -1;
 
   // Any queued items take priority
@@ -461,13 +478,13 @@ int Playlist::next_index() const {
   }
 
   // Still off the end?  Then just give up
-  if (next_virtual_index >= virtual_items_.count())
+  if (next_virtual_index < 0 || next_virtual_index >= virtual_items_.count())
     return -1;
 
   return virtual_items_[next_virtual_index];
 }
 
-int Playlist::previous_index() const {
+int Playlist::previous_row() const {
   int prev_virtual_index = PreviousVirtualIndex(current_virtual_index_);
   if (prev_virtual_index < 0) {
     // We've gone off the beginning of the playlist.
@@ -492,7 +509,7 @@ int Playlist::previous_index() const {
   return virtual_items_[prev_virtual_index];
 }
 
-void Playlist::set_current_index(int i) {
+void Playlist::set_current_row(int i) {
   QModelIndex old_current = current_item_index_;
   ClearStreamMetadata();
 
@@ -515,15 +532,17 @@ void Playlist::set_current_index(int i) {
 
   if (old_current.isValid()) {
     if (dynamic_playlist_) {
-      items_[old_current.row()]->SetDynamicHistory(true);
+      items_[old_current.row()]->SetForegroundColor(kDynamicHistoryPriority,
+                                                    kDynamicHistoryColor);
     }
 
     emit dataChanged(old_current, old_current.sibling(old_current.row(), ColumnCount-1));
   }
 
   if (current_item_index_.isValid()) {
-    emit dataChanged(current_item_index_, current_item_index_.sibling(current_item_index_.row(), ColumnCount-1));
-    emit CurrentSongChanged(current_item_metadata());
+    InformOfCurrentSongChange(current_item_index_,
+                              current_item_index_.sibling(current_item_index_.row(), ColumnCount-1),
+                              current_item_metadata());
   }
 
   // Update the virtual index
@@ -553,7 +572,7 @@ void Playlist::set_current_index(int i) {
       connect(inserter, SIGNAL(Error(QString)), SIGNAL(LoadTracksError(QString)));
       connect(inserter, SIGNAL(PlayRequested(QModelIndex)), SIGNAL(PlayRequested(QModelIndex)));
 
-      inserter->Load(this, -1, false, dynamic_playlist_, count);
+      inserter->Load(this, -1, false, false, dynamic_playlist_, count);
     }
 
     // Remove the first item
@@ -592,23 +611,36 @@ bool Playlist::dropMimeData(const QMimeData* data, Qt::DropAction action, int ro
 
   using smart_playlists::GeneratorMimeData;
 
+  bool play_now = false;
+  bool enqueue_now = false;
+  if (const MimeData* mime_data = qobject_cast<const MimeData*>(data)) {
+    if (mime_data->clear_first_) {
+      Clear();
+    }
+    play_now = mime_data->play_now_;
+    enqueue_now = mime_data->enqueue_now_;
+  }
+
   if (const SongMimeData* song_data = qobject_cast<const SongMimeData*>(data)) {
     // Dragged from a library
     // We want to check if these songs are from the actual local file backend,
     // if they are we treat them differently.
     if (song_data->backend && song_data->backend->songs_table() == Library::kSongsTable)
-      InsertSongItems<LibraryPlaylistItem>(this, song_data->songs, row);
+      InsertSongItems<LibraryPlaylistItem>(song_data->songs, row, play_now, enqueue_now);
     else if (song_data->backend && song_data->backend->songs_table() == MagnatuneService::kSongsTable)
-      InsertSongItems<MagnatunePlaylistItem>(this, song_data->songs, row);
+      InsertSongItems<MagnatunePlaylistItem>(song_data->songs, row, play_now, enqueue_now);
     else if (song_data->backend && song_data->backend->songs_table() == JamendoService::kSongsTable)
-      InsertSongItems<JamendoPlaylistItem>(this, song_data->songs, row);
+      InsertSongItems<JamendoPlaylistItem>(song_data->songs, row, play_now, enqueue_now);
     else
-      InsertSongItems<SongPlaylistItem>(this, song_data->songs, row);
+      InsertSongItems<SongPlaylistItem>(song_data->songs, row, play_now, enqueue_now);
   } else if (const RadioMimeData* radio_data = qobject_cast<const RadioMimeData*>(data)) {
     // Dragged from the Radio pane
-    InsertRadioStations(radio_data->items, row, data->hasFormat(kPlayNowMimetype));
+    InsertRadioStations(radio_data->model, radio_data->indexes,
+                        row, play_now, enqueue_now);
   } else if (const GeneratorMimeData* generator_data = qobject_cast<const GeneratorMimeData*>(data)) {
-    InsertSmartPlaylist(generator_data->generator_, row, data->hasFormat(kPlayNowMimetype));
+    InsertSmartPlaylist(generator_data->generator_, row, play_now, enqueue_now);
+  } else if (const PlaylistItemMimeData* item_data = qobject_cast<const PlaylistItemMimeData*>(data)) {
+    InsertItems(item_data->items_, row, play_now, enqueue_now);
   } else if (data->hasFormat(kRowsMimetype)) {
     // Dragged from the playlist
     // Rearranging it is tricky...
@@ -642,26 +674,29 @@ bool Playlist::dropMimeData(const QMimeData* data, Qt::DropAction action, int ro
     }
   } else if (data->hasUrls()) {
     // URL list dragged from the file list or some other app
-    InsertUrls(data->urls(), false, row);
+    InsertUrls(data->urls(), row, play_now, enqueue_now);
   }
 
   return true;
 }
 
-void Playlist::InsertUrls(const QList<QUrl> &urls, bool play_now, int pos) {
-  SongLoaderInserter* inserter = new SongLoaderInserter(task_manager_, library_, this);
+void Playlist::InsertUrls(const QList<QUrl> &urls, int pos, bool play_now, bool enqueue) {
+  SongLoaderInserter* inserter = new SongLoaderInserter(task_manager_, library_);
   connect(inserter, SIGNAL(Error(QString)), SIGNAL(LoadTracksError(QString)));
-  connect(inserter, SIGNAL(PlayRequested(QModelIndex)), SIGNAL(PlayRequested(QModelIndex)));
 
-  inserter->Load(this, pos, play_now, urls);
+  inserter->Load(this, pos, play_now, enqueue, urls);
 }
 
-void Playlist::InsertSmartPlaylist(GeneratorPtr generator, int pos, bool play_now) {
+void Playlist::InsertSmartPlaylist(GeneratorPtr generator, int pos, bool play_now, bool enqueue) {
+  // Hack: If the generator hasn't got a library set then use the main one
+  if (!generator->library()) {
+    generator->set_library(library_);
+  }
+
   GeneratorInserter* inserter = new GeneratorInserter(task_manager_, library_, this);
   connect(inserter, SIGNAL(Error(QString)), SIGNAL(LoadTracksError(QString)));
-  connect(inserter, SIGNAL(PlayRequested(QModelIndex)), SIGNAL(PlayRequested(QModelIndex)));
 
-  inserter->Load(this, pos, play_now, generator);
+  inserter->Load(this, pos, play_now, enqueue, generator);
 
   if (generator->is_dynamic()) {
     TurnOnDynamicPlaylist(generator);
@@ -693,7 +728,7 @@ void Playlist::MoveItemsWithoutUndo(const QList<int> &source_rows, int pos) {
   // Put the items back in
   const int start = pos == -1 ? items_.count() : pos;
   for (int i=start ; i<start+moved_items.count() ; ++i) {
-    moved_items[i - start]->SetDynamicHistory(false);
+    moved_items[i - start]->RemoveForegroundColor(kDynamicHistoryPriority);
     items_.insert(i, moved_items[i - start]);
   }
 
@@ -715,7 +750,7 @@ void Playlist::MoveItemsWithoutUndo(const QList<int> &source_rows, int pos) {
       changePersistentIndex(pidx, index(pidx.row() + d, pidx.column(), QModelIndex()));
     }
   }
-  current_virtual_index_ = virtual_items_.indexOf(current_index());
+  current_virtual_index_ = virtual_items_.indexOf(current_row());
 
   layoutChanged();
   Save();
@@ -759,26 +794,61 @@ void Playlist::MoveItemsWithoutUndo(int start, const QList<int>& dest_rows) {
       changePersistentIndex(pidx, index(pidx.row() + d, pidx.column(), QModelIndex()));
     }
   }
-  current_virtual_index_ = virtual_items_.indexOf(current_index());
+  current_virtual_index_ = virtual_items_.indexOf(current_row());
 
   layoutChanged();
   Save();
 }
 
-QModelIndex Playlist::InsertItems(const PlaylistItemList& items, int pos) {
-  if (items.isEmpty())
-    return QModelIndex();
+void Playlist::InsertItems(const PlaylistItemList& itemsIn, int pos, bool play_now, bool enqueue) {
+  if (itemsIn.isEmpty())
+    return;
+
+  PlaylistItemList items = itemsIn;
+
+  // exercise vetoes
+  SongList songs;
+
+  foreach(PlaylistItemPtr item, items) {
+    songs << item.get()->Metadata();
+  }
+
+  QList<Song> vetoed;
+  foreach(SongInsertVetoListener* listener, veto_listeners_) {
+    foreach(const Song& song, listener->AboutToInsertSongs(GetAllSongs(), songs)) {
+      vetoed.append(song);
+    }
+  }
+
+  if(!vetoed.isEmpty()) {
+    QMutableListIterator<PlaylistItemPtr> it(items);
+    while (it.hasNext()) {
+      PlaylistItemPtr item = it.next();
+      const Song& current = item.get()->Metadata();
+
+      if(vetoed.contains(current)) {
+        vetoed.removeOne(current);
+        it.remove();
+      }
+    }
+
+    // check for empty items once again after veto
+    if(items.isEmpty()) {
+      return;
+    }
+  }
 
   const int start = pos == -1 ? items_.count() : pos;
-  undo_stack_->push(new PlaylistUndoCommands::InsertItems(this, items, pos));
+  undo_stack_->push(new PlaylistUndoCommands::InsertItems(this, items, pos, enqueue));
 
-  return index(start, 0);
+  if (play_now)
+    emit PlayRequested(index(start, 0));
 }
 
-QModelIndex Playlist::InsertItemsWithoutUndo(const PlaylistItemList& items,
-                                             int pos) {
+void Playlist::InsertItemsWithoutUndo(const PlaylistItemList& items,
+                                      int pos, bool enqueue) {
   if (items.isEmpty())
-    return QModelIndex();
+    return;
 
   const int start = pos == -1 ? items_.count() : pos;
   const int end = start + items.count() - 1;
@@ -804,21 +874,27 @@ QModelIndex Playlist::InsertItemsWithoutUndo(const PlaylistItemList& items,
   }
   endInsertRows();
 
+  if (enqueue) {
+    QModelIndexList indexes;
+    for (int i=start ; i<=end ; ++i) {
+      indexes << index(i, 0);
+    }
+    queue_->ToggleTracks(indexes);
+  }
+
   Save();
   ReshuffleIndices();
-
-  return index(start, 0);
 }
 
-QModelIndex Playlist::InsertLibraryItems(const SongList& songs, int pos) {
-  return InsertSongItems<LibraryPlaylistItem>(this, songs, pos);
+void Playlist::InsertLibraryItems(const SongList& songs, int pos, bool play_now, bool enqueue) {
+  InsertSongItems<LibraryPlaylistItem>(songs, pos, play_now, enqueue);
 }
 
-QModelIndex Playlist::InsertSongs(const SongList& songs, int pos) {
-  return InsertSongItems<SongPlaylistItem>(this, songs, pos);
+void Playlist::InsertSongs(const SongList& songs, int pos, bool play_now, bool enqueue) {
+  InsertSongItems<SongPlaylistItem>(songs, pos, play_now, enqueue);
 }
 
-QModelIndex Playlist::InsertSongsOrLibraryItems(const SongList& songs, int pos) {
+void Playlist::InsertSongsOrLibraryItems(const SongList& songs, int pos, bool play_now, bool enqueue) {
   PlaylistItemList items;
   foreach (const Song& song, songs) {
     if (song.id() == -1)
@@ -826,29 +902,37 @@ QModelIndex Playlist::InsertSongsOrLibraryItems(const SongList& songs, int pos)
     else
       items << PlaylistItemPtr(new LibraryPlaylistItem(song));
   }
-  return InsertItems(items, pos);
+  InsertItems(items, pos, play_now, enqueue);
 }
 
-QModelIndex Playlist::InsertRadioStations(const QList<RadioItem*>& items, int pos, bool play_now) {
+void Playlist::InsertRadioStations(const RadioModel* model,
+                                   const QModelIndexList& items,
+                                   int pos, bool play_now, bool enqueue) {
   PlaylistItemList playlist_items;
   QList<QUrl> song_urls;
-  foreach (RadioItem* item, items) {
-    if (!item->playable)
-      continue;
 
-    if (item->use_song_loader) {
-      song_urls << item->Url();
-    } else {
-      playlist_items << shared_ptr<PlaylistItem>(
-          new RadioPlaylistItem(item->service, item->Url(), item->Title(), item->Artist()));
+  foreach (const QModelIndex& item, items) {
+    switch (item.data(RadioModel::Role_PlayBehaviour).toInt()) {
+    case RadioModel::PlayBehaviour_SingleItem:
+      playlist_items << shared_ptr<PlaylistItem>(new RadioPlaylistItem(
+          model->ServiceForIndex(item),
+          item.data(RadioModel::Role_Url).toUrl(),
+          item.data(RadioModel::Role_Title).toString(),
+          item.data(RadioModel::Role_Artist).toString()));
+      break;
+
+    case RadioModel::PlayBehaviour_UseSongLoader:
+      song_urls << item.data(RadioModel::Role_Url).toUrl();
+      break;
     }
   }
 
   if (!song_urls.isEmpty()) {
-    InsertUrls(song_urls, play_now, pos);
+    InsertUrls(song_urls, pos, play_now, enqueue);
+    play_now = false;
   }
 
-  return InsertItems(playlist_items, pos);
+  InsertItems(playlist_items, pos, play_now, enqueue);
 }
 
 QMimeData* Playlist::mimeData(const QModelIndexList& indexes) const {
@@ -891,7 +975,7 @@ bool Playlist::CompareItems(int column, Qt::SortOrder order,
     case Column_Title:        strcmp(title);
     case Column_Artist:       strcmp(artist);
     case Column_Album:        strcmp(album);
-    case Column_Length:       cmp(length);
+    case Column_Length:       cmp(length_nanosec);
     case Column_Track:        cmp(track);
     case Column_Disc:         cmp(disc);
     case Column_Year:         cmp(year);
@@ -1020,7 +1104,7 @@ void Playlist::Save() const {
   if (!backend_ || is_loading_)
     return;
 
-  backend_->SavePlaylistAsync(id_, items_, last_played_index(), dynamic_playlist_);
+  backend_->SavePlaylistAsync(id_, items_, last_played_row(), dynamic_playlist_);
 }
 
 namespace {
@@ -1046,14 +1130,29 @@ void Playlist::ItemsLoaded() {
   watcher->deleteLater();
 
   PlaylistItemList items = watcher->future().results();
+
+  // backend returns empty elements for library items which it couldn't 
+  // match (because they got deleted); we don't need those
+  QMutableListIterator<PlaylistItemPtr> it(items);
+  while (it.hasNext()) {
+    PlaylistItemPtr item = it.next();
+
+    if (item->IsLocalLibraryItem() && item->Metadata().filename().isEmpty()) {
+      it.remove();
+    }
+  }
+
   is_loading_ = true;
   InsertItems(items, 0);
   is_loading_ = false;
 
   PlaylistBackend::Playlist p = backend_->GetPlaylist(id_);
 
-  last_played_item_index_ =
-      p.last_played == -1 ? QModelIndex() : index(p.last_played);
+  // the newly loaded list of items might be shorter than it was before so
+  // look out for a bad last_played index
+  last_played_item_index_ = p.last_played == -1 || p.last_played >= rowCount()
+                                ? QModelIndex()
+                                : index(p.last_played);
 
   if (!p.dynamic_type.isEmpty()) {
     GeneratorPtr gen = Generator::Create(p.dynamic_type);
@@ -1074,6 +1173,41 @@ void Playlist::ItemsLoaded() {
       }
     }
   }
+
+  emit RestoreFinished();
+
+  QSettings s;
+  s.beginGroup(kSettingsGroup);
+
+  // should we gray out deleted songs asynchronously on startup?
+  if(s.value("greyoutdeleted", false).toBool()) {
+    QtConcurrent::run(this, &Playlist::InvalidateDeletedSongs);
+  }
+}
+
+static bool DescendingIntLessThan(int a, int b) {
+  return a > b;
+}
+
+void Playlist::RemoveItemsWithoutUndo(const QList<int>& indicesIn) {
+  // Sort the indices descending because removing elements 'backwards'
+  // is easier - indices don't 'move' in the process.
+  QList<int> indices = indicesIn;
+  qSort(indices.begin(), indices.end(), DescendingIntLessThan);
+
+  for(int j = 0; j < indices.count(); j++) {
+    int beginning = indices[j], end = indices[j];
+
+    // Splits the indices into sequences. For example this: [1, 2, 4],
+    // will get split into [1, 2] and [4].
+    while(j != indices.count() - 1 && indices[j] == indices[j + 1] + 1) {
+      beginning--;
+      j++;
+    }
+
+    // Remove the current sequence.
+    removeRows(beginning, end - beginning + 1);
+  }
 }
 
 bool Playlist::removeRows(int row, int count, const QModelIndex& parent) {
@@ -1118,10 +1252,10 @@ PlaylistItemList Playlist::RemoveItemsWithoutUndo(int row, int count) {
   }
 
   // Reset current_virtual_index_
-  if (current_index() == -1)
+  if (current_row() == -1)
     current_virtual_index_ = -1;
   else
-    current_virtual_index_ = virtual_items_.indexOf(current_index());
+    current_virtual_index_ = virtual_items_.indexOf(current_row());
 
   Save();
   return ret;
@@ -1156,8 +1290,9 @@ void Playlist::SetStreamMetadata(const QUrl& url, const Song& song) {
   current_item_->SetTemporaryMetadata(song);
   UpdateScrobblePoint();
 
-  emit dataChanged(index(current_item_index_.row(), 0), index(current_item_index_.row(), ColumnCount-1));
-  emit CurrentSongChanged(song);
+  InformOfCurrentSongChange(index(current_item_index_.row(), 0),
+                            index(current_item_index_.row(), ColumnCount-1),
+                            song);
 }
 
 void Playlist::ClearStreamMetadata() {
@@ -1190,10 +1325,16 @@ Song Playlist::current_item_metadata() const {
 }
 
 void Playlist::UpdateScrobblePoint() {
-  int length = qMin(current_item_metadata().length(), 240);
+  const qint64 length = current_item_metadata().length_nanosec();
+
+  if (length == 0) {
+    scrobble_point_ = 240ll * kNsecPerSec; // 4 minutes
+  } else {
+    scrobble_point_ = qBound(31ll * kNsecPerSec,
+                             length/2,
+                             240ll * kNsecPerSec);
+  }
 
-  ScrobblePoint point(length / 2);
-  scrobble_point_ = point;
   has_scrobbled_ = false;
 }
 
@@ -1258,11 +1399,40 @@ void Playlist::RemoveItemsNotInQueue() {
 
 void Playlist::ReloadItems(const QList<int>& rows) {
   foreach (int row, rows) {
-    item_at(row)->Reload();
-    emit dataChanged(index(row, 0), index(row, ColumnCount-1));
+    PlaylistItemPtr item = item_at(row);
+
+    item->Reload();
+    InformOfCurrentSongChange(index(row, 0), index(row, ColumnCount-1),
+                              item->Metadata());
   }
 }
 
+void Playlist::RateSong(const QModelIndex& index, double rating) {
+  int row = index.row();
+
+  if(has_item_at(row)) {
+    PlaylistItemPtr item = item_at(row);
+    if (item && item->IsLocalLibraryItem() && item->Metadata().id() != -1) {
+      library_->UpdateSongRatingAsync(item->Metadata().id(), rating);
+    }
+  }
+}
+
+void Playlist::AddSongInsertVetoListener(SongInsertVetoListener* listener) {
+  veto_listeners_.append(listener);
+  connect(listener, SIGNAL(destroyed()), this, SLOT(SongInsertVetoListenerDestroyed()));
+}
+
+void Playlist::RemoveSongInsertVetoListener(SongInsertVetoListener* listener) {
+  disconnect(listener, SIGNAL(destroyed()), this, SLOT(SongInsertVetoListenerDestroyed()));
+  veto_listeners_.removeAll(listener);
+}
+
+void Playlist::SongInsertVetoListenerDestroyed() {
+  // qobject_cast returns NULL here for Python SIP listeners.
+  veto_listeners_.removeAll(static_cast<SongInsertVetoListener*>(sender()));
+}
+
 void Playlist::Shuffle() {
   layoutAboutToBeChanged();
 
@@ -1279,7 +1449,7 @@ void Playlist::Shuffle() {
         changePersistentIndex(pidx, index(i, pidx.column(), QModelIndex()));
     }
   }
-  current_virtual_index_ = virtual_items_.indexOf(current_index());
+  current_virtual_index_ = virtual_items_.indexOf(current_row());
 
   layoutChanged();
 
@@ -1293,8 +1463,8 @@ void Playlist::Shuffle() {
 void Playlist::ReshuffleIndices() {
   if (!is_shuffled_) {
     std::sort(virtual_items_.begin(), virtual_items_.end());
-    if (current_index() != -1)
-      current_virtual_index_ = virtual_items_.indexOf(current_index());
+    if (current_row() != -1)
+      current_virtual_index_ = virtual_items_.indexOf(current_row());
   } else {
     QList<int>::iterator begin = virtual_items_.begin();
     if (current_virtual_index_ != -1)
@@ -1324,10 +1494,14 @@ SongList Playlist::GetAllSongs() const {
   return ret;
 }
 
+PlaylistItemList Playlist::GetAllItems() const {
+  return items_;
+}
+
 quint64 Playlist::GetTotalLength() const {
   quint64 ret = 0;
   foreach (PlaylistItemPtr item, items_) {
-    int length = item->Metadata().length();
+    quint64 length = item->Metadata().length_nanosec();
     if (length > 0)
       ret += length;
   }
@@ -1382,3 +1556,77 @@ void Playlist::ItemChanged(PlaylistItemPtr item) {
     }
   }
 }
+
+void Playlist::set_column_alignments(const ColumnAlignmentMap& column_alignments) {
+  column_alignments_ = column_alignments;
+}
+
+void Playlist::set_column_align_left(int column) {
+  column_alignments_[column] = (Qt::AlignLeft | Qt::AlignVCenter);
+}
+
+void Playlist::set_column_align_center(int column) {
+  column_alignments_[column] = Qt::AlignCenter;
+}
+
+void Playlist::set_column_align_right(int column) {
+  column_alignments_[column] = (Qt::AlignRight | Qt::AlignVCenter);
+}
+
+void Playlist::InformOfCurrentSongChange(const QModelIndex& top_left, const QModelIndex& bottom_right,
+                                         const Song& metadata) {
+  emit dataChanged(top_left, bottom_right);
+  // if the song is invalid, we won't play it - there's no point in
+  // informing anybody about the change
+  if(metadata.is_valid()) {
+    emit CurrentSongChanged(metadata);
+  }
+}
+
+void Playlist::InvalidateDeletedSongs() {
+  QList<int> invalidated_rows;
+
+  for (int row = 0; row < items_.count(); ++row) {
+    PlaylistItemPtr item = items_[row];
+    Song song = item->Metadata();
+
+    if(!song.is_stream()) {
+      bool exists = QFile::exists(song.filename());
+
+      if(!exists && !item->HasForegroundColor(kInvalidSongPriority)) {
+        // gray out the song if it's not there
+        item->SetForegroundColor(kInvalidSongPriority, kInvalidSongColor);
+        invalidated_rows.append(row);
+      } else if(exists && item->HasForegroundColor(kInvalidSongPriority)) {
+        item->RemoveForegroundColor(kInvalidSongPriority);
+        invalidated_rows.append(row);
+      }
+    }
+  }
+
+  ReloadItems(invalidated_rows);
+}
+
+bool Playlist::ApplyValidityOnCurrentSong(const QUrl& url, bool valid) {
+  PlaylistItemPtr current = current_item();
+
+  if(current) {
+    Song current_song = current->Metadata();
+
+    // if validity has changed, reload the item
+    if(!current_song.is_stream() &&
+        current_song.filename() == url.toLocalFile() &&
+        current_song.is_valid() != QFile::exists(current_song.filename())) {
+      ReloadItems(QList<int>() << current_row());
+    }
+
+    // gray out the song if it's now broken; otherwise undo the gray color
+    if(valid) {
+      current->RemoveForegroundColor(kInvalidSongPriority);
+    } else {
+      current->SetForegroundColor(kInvalidSongPriority, kInvalidSongColor);
+    }
+  }
+
+  return current;
+}
