@@ -26,14 +26,13 @@
 #include "playlistitem.h"
 #include "playlistsequence.h"
 #include "core/song.h"
-#include "radio/radioitem.h"
 #include "smartplaylists/generator_fwd.h"
 
 class LibraryBackend;
 class PlaylistBackend;
 class PlaylistFilter;
 class Queue;
-class RadioService;
+class RadioModel;
 class TaskManager;
 
 class QSortFilterProxyModel;
@@ -45,6 +44,23 @@ namespace PlaylistUndoCommands {
   class MoveItems;
 }
 
+typedef QMap<int, Qt::Alignment> ColumnAlignmentMap;
+Q_DECLARE_METATYPE(Qt::Alignment);
+Q_DECLARE_METATYPE(ColumnAlignmentMap);
+
+// Objects that may prevent a song being added to the playlist. When there
+// is something about to be inserted into it, Playlist notifies all of it's
+// listeners about the fact and every one of them picks 'invalid' songs.
+class SongInsertVetoListener : public QObject {
+  Q_OBJECT
+
+public:
+  // Listener returns a list of 'invalid' songs. 'old_songs' are songs that are
+  // currently in the playlist and 'new_songs' are the songs about to be added if
+  // nobody exercises a veto.
+  virtual SongList AboutToInsertSongs(const SongList& old_songs, const SongList& new_songs) = 0;
+};
+
 class Playlist : public QAbstractListModel {
   Q_OBJECT
 
@@ -105,6 +121,14 @@ class Playlist : public QAbstractListModel {
   static const char* kRowsMimetype;
   static const char* kPlayNowMimetype;
 
+  static const int kInvalidSongPriority;
+  static const QRgb kInvalidSongColor;
+
+  static const int kDynamicHistoryPriority;
+  static const QRgb kDynamicHistoryColor;
+
+  static const char* kSettingsGroup;
+
   static bool CompareItems(int column, Qt::SortOrder order,
                            PlaylistItemPtr a, PlaylistItemPtr b);
 
@@ -121,14 +145,20 @@ class Playlist : public QAbstractListModel {
   Queue* queue() const { return queue_; }
 
   int id() const { return id_; }
-  int current_index() const;
-  int last_played_index() const;
-  int next_index() const;
-  int previous_index() const;
+
+  int current_row() const;
+  int last_played_row() const;
+  int next_row() const;
+  int previous_row() const;
+
+  const QModelIndex current_index() const;
+
   bool stop_after_current() const;
   bool is_dynamic() const { return dynamic_playlist_; }
 
   const PlaylistItemPtr& item_at(int index) const { return items_[index]; }
+  const bool has_item_at(int index) const { return index >= 0 && index < rowCount(); }
+
   PlaylistItemPtr current_item() const { return current_item_; }
 
   PlaylistItem::Options current_item_options() const;
@@ -137,6 +167,7 @@ class Playlist : public QAbstractListModel {
   PlaylistItemList library_items_by_id(int id) const;
 
   SongList GetAllSongs() const;
+  PlaylistItemList GetAllItems() const;
   quint64 GetTotalLength() const; // in seconds
 
   void set_sequence(PlaylistSequence* v);
@@ -144,22 +175,50 @@ class Playlist : public QAbstractListModel {
 
   QUndoStack* undo_stack() const { return undo_stack_; }
 
+  ColumnAlignmentMap column_alignments() const { return column_alignments_; }
+  void set_column_alignments(const ColumnAlignmentMap& column_alignments);
+  void set_column_align_left(int column);
+  void set_column_align_center(int column);
+  void set_column_align_right(int column);
+
   // Scrobbling
-  int scrobble_point() const { return scrobble_point_; }
+  qint64 scrobble_point_nanosec() const { return scrobble_point_; }
   bool has_scrobbled() const { return has_scrobbled_; }
   void set_scrobbled(bool v) { has_scrobbled_ = v; }
 
   // Changing the playlist
-  QModelIndex InsertItems(const PlaylistItemList& items, int pos = -1);
-  QModelIndex InsertLibraryItems(const SongList& items, int pos = -1);
-  QModelIndex InsertSongs(const SongList& items, int pos = -1);
-  QModelIndex InsertSongsOrLibraryItems(const SongList& items, int pos = -1);
-  QModelIndex InsertRadioStations(const QList<RadioItem*>& items, int pos = -1, bool play_now = false);
-  void InsertSmartPlaylist(smart_playlists::GeneratorPtr generator, int pos = -1, bool play_now = false);
-  void InsertUrls(const QList<QUrl>& urls, bool play_now, int pos = -1);
+  void InsertItems              (const PlaylistItemList& items,     int pos = -1, bool play_now = false, bool enqueue = false);
+  void InsertLibraryItems       (const SongList& items,             int pos = -1, bool play_now = false, bool enqueue = false);
+  void InsertSongs              (const SongList& items,             int pos = -1, bool play_now = false, bool enqueue = false);
+  void InsertSongsOrLibraryItems(const SongList& items,             int pos = -1, bool play_now = false, bool enqueue = false);
+  void InsertSmartPlaylist      (smart_playlists::GeneratorPtr gen, int pos = -1, bool play_now = false, bool enqueue = false);
+  void InsertUrls               (const QList<QUrl>& urls,           int pos = -1, bool play_now = false, bool enqueue = false);
+  // Removes items with given indices from the playlist. This operation is not undoable.
+  void RemoveItemsWithoutUndo   (const QList<int>& indices);
+
+  // If this playlist contains the current item, this method will apply the "valid" flag on it.
+  // If the "valid" flag is false, the song will be greyed out. Otherwise the grey color will
+  // be undone.
+  // If the song is a local file and it's valid but non existent or invalid but exists, the
+  // song will be reloaded to even out the situation because obviously something has changed.
+  // This returns true if this playlist had current item when the method was invoked.
+  bool ApplyValidityOnCurrentSong(const QUrl& url, bool valid);
+  // Grays out and reloads all deleted songs in all playlists. Also, "ungreys" those songs
+  // which were once deleted but now got restored somehow.
+  void InvalidateDeletedSongs();
+
   void StopAfter(int row);
   void ReloadItems(const QList<int>& rows);
 
+  // Changes rating of a song to the given value asynchronously
+  void RateSong(const QModelIndex& index, double rating);
+
+  // Registers an object which will get notifications when new songs
+  // are about to be inserted into this playlist.
+  void AddSongInsertVetoListener(SongInsertVetoListener* listener);
+  // Unregisters a SongInsertVetoListener object.
+  void RemoveSongInsertVetoListener(SongInsertVetoListener* listener);
+
   // QAbstractListModel
   int rowCount(const QModelIndex& = QModelIndex()) const { return items_.count(); }
   int columnCount(const QModelIndex& = QModelIndex()) const { return ColumnCount; }
@@ -175,7 +234,7 @@ class Playlist : public QAbstractListModel {
   bool removeRows(int row, int count, const QModelIndex& parent = QModelIndex());
 
  public slots:
-  void set_current_index(int index);
+  void set_current_row(int index);
   void Paused();
   void Playing();
   void Stopped();
@@ -194,6 +253,7 @@ class Playlist : public QAbstractListModel {
   void TurnOffDynamicPlaylist();
 
  signals:
+  void RestoreFinished();
   void CurrentSongChanged(const Song& metadata);
   void EditingFinished(const QModelIndex& index);
   void PlayRequested(const QModelIndex& index);
@@ -214,15 +274,26 @@ class Playlist : public QAbstractListModel {
   bool FilterContainsVirtualIndex(int i) const;
   void TurnOnDynamicPlaylist(smart_playlists::GeneratorPtr gen);
 
+  void InsertRadioStations(const RadioModel* model,
+                           const QModelIndexList& items,
+                           int pos, bool play_now, bool enqueue);
+
+  template<typename T>
+  void InsertSongItems(const SongList& songs, int pos, bool play_now, bool enqueue);
+
   // Modify the playlist without changing the undo stack.  These are used by
   // our friends in PlaylistUndoCommands
-  QModelIndex InsertItemsWithoutUndo(const PlaylistItemList& items, int pos);
+  void InsertItemsWithoutUndo(const PlaylistItemList& items, int pos,
+                              bool enqueue = false);
   PlaylistItemList RemoveItemsWithoutUndo(int pos, int count);
   void MoveItemsWithoutUndo(const QList<int>& source_rows, int pos);
   void MoveItemsWithoutUndo(int start, const QList<int>& dest_rows);
 
   void RemoveItemsNotInQueue();
 
+  void InformOfCurrentSongChange(const QModelIndex& top_left, const QModelIndex& bottom_right,
+                                 const Song& metadata);
+
  private slots:
   void TracksAboutToBeDequeued(const QModelIndex&, int begin, int end);
   void TracksDequeued();
@@ -231,6 +302,7 @@ class Playlist : public QAbstractListModel {
   void SongSaveComplete();
   void ItemReloadComplete();
   void ItemsLoaded();
+  void SongInsertVetoListenerDestroyed();
 
  private:
   bool is_loading_;
@@ -261,7 +333,7 @@ class Playlist : public QAbstractListModel {
 
   bool is_shuffled_;
 
-  int scrobble_point_;
+  qint64 scrobble_point_;
   bool has_scrobbled_;
 
   PlaylistSequence* playlist_sequence_;
@@ -272,6 +344,9 @@ class Playlist : public QAbstractListModel {
   QUndoStack* undo_stack_;
 
   smart_playlists::GeneratorPtr dynamic_playlist_;
+  ColumnAlignmentMap column_alignments_;
+
+  QList<SongInsertVetoListener*> veto_listeners_;
 };
 
 QDataStream& operator <<(QDataStream&, const Playlist*);
