@@ -21,21 +21,33 @@
 #include "core/song.h"
 #include "library/librarybackend.h"
 #include "library/sqlrow.h"
+#include "playlist/songplaylistitem.h"
+#include "playlistparsers/cueparser.h"
 #include "smartplaylists/generator.h"
 
+#include <QFile>
+#include <QHash>
+#include <QMutexLocker>
 #include <QSqlQuery>
 #include <QtConcurrentMap>
 #include <QtDebug>
 
+#include <boost/bind.hpp>
+
 using smart_playlists::GeneratorPtr;
 
 using boost::shared_ptr;
 
 PlaylistBackend::PlaylistBackend(QObject* parent)
-  : QObject(parent)
+  : QObject(parent),
+    library_(NULL)
 {
 }
 
+void PlaylistBackend::SetLibrary(LibraryBackend* library) {
+  library_ = library;
+}
+
 PlaylistBackend::PlaylistList PlaylistBackend::GetAllPlaylists() {
   QMutexLocker l(db_->Mutex());
   QSqlDatabase db(db_->Connect());
@@ -44,9 +56,10 @@ PlaylistBackend::PlaylistList PlaylistBackend::GetAllPlaylists() {
 
   QSqlQuery q("SELECT ROWID, name, last_played, dynamic_playlist_type,"
               "       dynamic_playlist_data, dynamic_playlist_backend"
-              " FROM playlists", db);
+              " FROM playlists"
+              " ORDER BY ui_order", db);
   q.exec();
-  if (db_->CheckErrors(q.lastError()))
+  if (db_->CheckErrors(q))
     return ret;
 
   while (q.next()) {
@@ -73,7 +86,7 @@ PlaylistBackend::Playlist PlaylistBackend::GetPlaylist(int id) {
               " WHERE ROWID=:id", db);
   q.bindValue(":id", id);
   q.exec();
-  if (db_->CheckErrors(q.lastError()))
+  if (db_->CheckErrors(q))
     return Playlist();
 
   q.next();
@@ -93,22 +106,24 @@ QFuture<PlaylistItemPtr> PlaylistBackend::GetPlaylistItems(int playlist) {
   QMutexLocker l(db_->Mutex());
   QSqlDatabase db(db_->Connect());
 
-  QSqlQuery q("SELECT songs.ROWID, " + Song::JoinSpec("songs") + ","
-              "       magnatune_songs.ROWID, " + Song::JoinSpec("magnatune_songs") + ","
-              "       jamendo_songs.ROWID, " + Song::JoinSpec("jamendo_songs") + ","
-              "       p.type, p.url, p.title, p.artist, p.album, p.length,"
-              "       p.radio_service"
-              " FROM playlist_items AS p"
-              " LEFT JOIN songs"
-              "    ON p.library_id = songs.ROWID"
-              " LEFT JOIN magnatune_songs"
-              "    ON p.library_id = magnatune_songs.ROWID"
-              " LEFT JOIN jamendo.songs AS jamendo_songs"
-              "    ON p.library_id = jamendo_songs.ROWID"
-              " WHERE p.playlist = :playlist", db);
+  QString query = "SELECT songs.ROWID, " + Song::JoinSpec("songs") + ","
+                  "       magnatune_songs.ROWID, " + Song::JoinSpec("magnatune_songs") + ","
+                  "       jamendo_songs.ROWID, " + Song::JoinSpec("jamendo_songs") + ","
+                  "       p.type, p.url, p.title, p.artist, p.album, p.length,"
+                  "       p.radio_service, p.beginning, p.cue_path"
+                  " FROM playlist_items AS p"
+                  " LEFT JOIN songs"
+                  "    ON p.library_id = songs.ROWID"
+                  " LEFT JOIN magnatune_songs"
+                  "    ON p.library_id = magnatune_songs.ROWID"
+                  " LEFT JOIN jamendo.songs AS jamendo_songs"
+                  "    ON p.library_id = jamendo_songs.ROWID"
+                  " WHERE p.playlist = :playlist";
+  QSqlQuery q(query, db);
+
   q.bindValue(":playlist", playlist);
   q.exec();
-  if (db_->CheckErrors(q.lastError()))
+  if (db_->CheckErrors(q))
     return QFuture<PlaylistItemPtr>();
 
   QList<SqlRow> rows;
@@ -117,18 +132,74 @@ QFuture<PlaylistItemPtr> PlaylistBackend::GetPlaylistItems(int playlist) {
     rows << SqlRow(q);
   }
 
-  return QtConcurrent::mapped(rows, &PlaylistBackend::NewSongFromQuery);
+  // it's probable that we'll have a few songs associated with the
+  // same CUE so we're caching results of parsing CUEs
+  boost::shared_ptr<NewSongFromQueryState> state_ptr(new NewSongFromQueryState());
+  return QtConcurrent::mapped(rows, boost::bind(&PlaylistBackend::NewSongFromQuery, this, _1, state_ptr));
 }
 
-PlaylistItemPtr PlaylistBackend::NewSongFromQuery(const SqlRow& row) {
+PlaylistItemPtr PlaylistBackend::NewSongFromQuery(const SqlRow& row, boost::shared_ptr<NewSongFromQueryState> state) {
   // The song tables get joined first, plus one each for the song ROWIDs
   const int playlist_row = (Song::kColumns.count() + 1) * 3;
 
-  PlaylistItemPtr item(
-      PlaylistItem::NewFromType(row.value(playlist_row).toString()));
+  PlaylistItemPtr item(PlaylistItem::NewFromType(row.value(playlist_row).toString()));
   if (item) {
     item->InitFromQuery(row);
+    return RestoreCueData(item, state);
+  } else {
+    return item;
   }
+}
+
+// If song had a CUE and the CUE still exists, the metadata from it will
+// be applied here.
+PlaylistItemPtr PlaylistBackend::RestoreCueData(PlaylistItemPtr item, boost::shared_ptr<NewSongFromQueryState> state) {
+  // we need library to run a CueParser; also, this method applies only to
+  // file-type PlaylistItems
+  if(!library_ || item->type() != "File") {
+    return item;
+  }
+  CueParser cue_parser(library_);
+
+  Song song = item->Metadata();
+  // we're only interested in .cue songs here
+  if(!song.has_cue()) {
+    return item;
+  }
+
+  QString cue_path = song.cue_path();
+  // if .cue was deleted - reload the song
+  if(!QFile::exists(cue_path)) {
+    item->Reload();
+    return item;
+  }
+
+  SongList song_list;
+  {
+    QMutexLocker locker(&state->mutex_);
+
+    if(!state->cached_cues_.contains(cue_path)) {
+      QFile cue(cue_path);
+      cue.open(QIODevice::ReadOnly);
+
+      song_list = cue_parser.Load(&cue, cue_path, QDir(cue_path.section('/', 0, -2)));
+      state->cached_cues_[cue_path] = song_list;
+    } else {
+      song_list = state->cached_cues_[cue_path];
+    }
+  }
+
+  foreach(const Song& from_list, song_list) {
+    if(from_list.filename() == song.filename() &&
+       from_list.beginning_nanosec() == song.beginning_nanosec()) {
+      // we found a matching section; replace the input
+      // item with a new one containing CUE metadata
+      return PlaylistItemPtr(new SongPlaylistItem(from_list));
+    }
+  }
+
+  // there's no such section in the related .cue -> reload the song
+  item->Reload();
   return item;
 }
 
@@ -149,8 +220,8 @@ void PlaylistBackend::SavePlaylist(int playlist, const PlaylistItemList& items,
   QSqlQuery clear("DELETE FROM playlist_items WHERE playlist = :playlist", db);
   QSqlQuery insert("INSERT INTO playlist_items"
                    " (playlist, type, library_id, url, title, artist, album,"
-                   "  length, radio_service)"
-                   " VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)", db);
+                   "  length, radio_service, beginning, cue_path)"
+                   " VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", db);
   QSqlQuery update("UPDATE playlists SET "
                    "   last_played=:last_played,"
                    "   dynamic_playlist_type=:dynamic_type,"
@@ -163,7 +234,7 @@ void PlaylistBackend::SavePlaylist(int playlist, const PlaylistItemList& items,
   // Clear the existing items in the playlist
   clear.bindValue(":playlist", playlist);
   clear.exec();
-  if (db_->CheckErrors(clear.lastError()))
+  if (db_->CheckErrors(clear))
     return;
 
   // Save the new ones
@@ -172,7 +243,7 @@ void PlaylistBackend::SavePlaylist(int playlist, const PlaylistItemList& items,
     item->BindToQuery(&insert);
 
     insert.exec();
-    db_->CheckErrors(insert.lastError());
+    db_->CheckErrors(insert);
   }
 
   // Update the last played track number
@@ -188,7 +259,7 @@ void PlaylistBackend::SavePlaylist(int playlist, const PlaylistItemList& items,
   }
   update.bindValue(":playlist", playlist);
   update.exec();
-  if (db_->CheckErrors(update.lastError()))
+  if (db_->CheckErrors(update))
     return;
 
   transaction.Commit();
@@ -201,7 +272,7 @@ int PlaylistBackend::CreatePlaylist(const QString &name) {
   QSqlQuery q("INSERT INTO playlists (name) VALUES (:name)", db);
   q.bindValue(":name", name);
   q.exec();
-  if (db_->CheckErrors(q.lastError()))
+  if (db_->CheckErrors(q))
     return -1;
 
   return q.lastInsertId().toInt();
@@ -219,11 +290,11 @@ void PlaylistBackend::RemovePlaylist(int id) {
   ScopedTransaction transaction(&db);
 
   delete_playlist.exec();
-  if (db_->CheckErrors(delete_playlist.lastError()))
+  if (db_->CheckErrors(delete_playlist))
     return;
 
   delete_items.exec();
-  if (db_->CheckErrors(delete_items.lastError()))
+  if (db_->CheckErrors(delete_items))
     return;
 
   transaction.Commit();
@@ -237,7 +308,7 @@ void PlaylistBackend::RenamePlaylist(int id, const QString &new_name) {
   q.bindValue(":id", id);
 
   q.exec();
-  db_->CheckErrors(q.lastError());
+  db_->CheckErrors(q);
 }
 
 void PlaylistBackend::SetPlaylistOrder(const QList<int>& ids) {
@@ -251,7 +322,7 @@ void PlaylistBackend::SetPlaylistOrder(const QList<int>& ids) {
     q.bindValue(":index", i);
     q.bindValue(":id", ids[i]);
     q.exec();
-    if (db_->CheckErrors(q.lastError()))
+    if (db_->CheckErrors(q))
       return;
   }
 
