@@ -19,7 +19,9 @@
 #define PLAYLISTBACKEND_H
 
 #include <QFuture>
+#include <QHash>
 #include <QList>
+#include <QMutex>
 #include <QObject>
 
 #include "playlistitem.h"
@@ -28,6 +30,7 @@
 #include <boost/shared_ptr.hpp>
 
 class Database;
+class LibraryBackend;
 
 class PlaylistBackend : public QObject {
   Q_OBJECT
@@ -50,20 +53,31 @@ class PlaylistBackend : public QObject {
   PlaylistList GetAllPlaylists();
   Playlist GetPlaylist(int id);
   PlaylistItemFuture GetPlaylistItems(int playlist);
-  void SavePlaylistAsync(int playlist, const PlaylistItemList& items,
-                         int last_played, smart_playlists::GeneratorPtr dynamic);
+
   void SetPlaylistOrder(const QList<int>& ids);
 
   int CreatePlaylist(const QString& name);
-  void RemovePlaylist(int id);
+  void SavePlaylistAsync(int playlist, const PlaylistItemList& items,
+                         int last_played, smart_playlists::GeneratorPtr dynamic);
   void RenamePlaylist(int id, const QString& new_name);
+  void RemovePlaylist(int id);
+
+  void SetLibrary(LibraryBackend* library);
 
  public slots:
   void SavePlaylist(int playlist, const PlaylistItemList& items,
                     int last_played, smart_playlists::GeneratorPtr dynamic);
 
  private:
-  static PlaylistItemPtr NewSongFromQuery(const SqlRow& row);
+  struct NewSongFromQueryState {
+    QHash<QString, SongList> cached_cues_;
+    QMutex mutex_;
+  };
+
+  PlaylistItemPtr NewSongFromQuery(const SqlRow& row, boost::shared_ptr<NewSongFromQueryState> state);
+  PlaylistItemPtr RestoreCueData(PlaylistItemPtr item, boost::shared_ptr<NewSongFromQueryState> state);
+
+  LibraryBackend* library_;
 
   boost::shared_ptr<Database> db_;
 };
