@@ -67,9 +67,6 @@ PlaylistContainer::PlaylistContainer(QWidget *parent)
 
   settings_.beginGroup(kSettingsGroup);
 
-  // Icons
-  ui_->clear->setIcon(IconLoader::Load("edit-clear-locationbar-ltr"));
-
   // Tab bar
   ui_->tab_bar->setExpanding(false);
   ui_->tab_bar->setMovable(true);
@@ -78,7 +75,6 @@ PlaylistContainer::PlaylistContainer(QWidget *parent)
   ui_->tab_bar->setMaximumHeight(0);
 
   // Connections
-  connect(ui_->clear, SIGNAL(clicked()), SLOT(ClearFilter()));
   connect(ui_->tab_bar, SIGNAL(currentChanged(int)), SLOT(Save()));
   connect(ui_->tab_bar, SIGNAL(Save(int)), SLOT(SavePlaylist(int)));
 
@@ -89,13 +85,13 @@ PlaylistContainer::PlaylistContainer(QWidget *parent)
   filter->setObjectName("filter");
   ui_->horizontalLayout->addWidget(filter);
   connect(filter, SIGNAL(textChanged(QString)), SLOT(UpdateFilter()));
-  filter->SetHint(tr("Playlist search"));
+  filter->set_hint(tr("Playlist search"));
   filter_ = filter;
-
-  ui_->clear->hide();
 #else
   filter_ = ui_->filter;
   connect(ui_->filter, SIGNAL(textChanged(QString)), SLOT(UpdateFilter()));
+  connect(ui_->playlist, SIGNAL(FocusOnFilterSignal(QKeyEvent*)), SLOT(FocusOnFilter(QKeyEvent*)));
+  ui_->filter->installEventFilter(this);
 #endif
 }
 
@@ -120,11 +116,6 @@ void PlaylistContainer::SetActions(
   connect(load_playlist, SIGNAL(triggered()), SLOT(LoadPlaylist()));
 }
 
-void PlaylistContainer::ClearFilter() {
-  filter_->clear();
-  filter_->setFocus();
-}
-
 void PlaylistContainer::SetManager(PlaylistManager *manager) {
   manager_ = manager;
   ui_->tab_bar->SetManager(manager);
@@ -167,7 +158,7 @@ void PlaylistContainer::SetViewModel(Playlist* playlist) {
   emit ViewSelectionModelChanged();
 
   // Update filter
-  filter_->setText(playlist->proxy()->filterRegExp().pattern());
+  filter_->set_text(playlist->proxy()->filterRegExp().pattern());
 
   // Ensure that tab is current
   if (ui_->tab_bar->current_id() != manager_->current_id())
@@ -250,13 +241,7 @@ void PlaylistContainer::PlaylistRenamed(int id, const QString &new_name) {
 }
 
 void PlaylistContainer::NewPlaylist() {
-  QString name = QInputDialog::getText(this, tr("New playlist"),
-                                       tr("Enter a name for the new playlist"),
-                                       QLineEdit::Normal, tr("Playlist"));
-  if (name.isNull())
-    return;
-
-  manager_->New(name);
+  manager_->New(PromptForPlaylistName());
 }
 
 void PlaylistContainer::LoadPlaylist() {
@@ -347,6 +332,11 @@ void PlaylistContainer::resizeEvent(QResizeEvent* e) {
   RepositionNoMatchesLabel();
 }
 
+void PlaylistContainer::FocusOnFilter(QKeyEvent *event) {
+  ui_->filter->setFocus(Qt::OtherFocusReason);
+  QApplication::sendEvent(ui_->filter, event);
+}
+
 void PlaylistContainer::RepositionNoMatchesLabel(bool force) {
   if (!force && !no_matches_label_->isVisible())
     return;
@@ -365,3 +355,34 @@ void PlaylistContainer::RepositionNoMatchesLabel(bool force) {
 void PlaylistContainer::SelectionChanged() {
   manager_->SelectionChanged(view()->selectionModel()->selection());
 }
+
+bool PlaylistContainer::eventFilter(QObject *objectWatched, QEvent *event) {
+  if(objectWatched == ui_->filter) {
+    if (event->type() == QEvent::KeyPress) {
+      QKeyEvent *e = static_cast<QKeyEvent*>(event);
+      switch(e->key()) {
+        case Qt::Key_Up:
+        case Qt::Key_Down:
+        case Qt::Key_PageUp:
+        case Qt::Key_PageDown:
+        case Qt::Key_Return:
+        case Qt::Key_Enter:
+          view()->setFocus(Qt::OtherFocusReason);
+          QApplication::sendEvent(ui_->playlist, event);
+          return true;
+        case Qt::Key_Escape:
+          ui_->filter->LineEditInterface::clear();
+          return true;
+        default:
+          break;
+      }
+    }
+  }
+  return QWidget::eventFilter(objectWatched, event);
+}
+
+QString PlaylistContainer::PromptForPlaylistName() {
+  return QInputDialog::getText(this, tr("New playlist"),
+                                       tr("Enter a name for the new playlist"),
+                                       QLineEdit::Normal, tr("Playlist"));
+}
