@@ -44,8 +44,12 @@ public:
                   QAction* load_playlist);
   void SetManager(PlaylistManager* manager);
 
+  QString PromptForPlaylistName();
+
   PlaylistView* view() const;
 
+  bool eventFilter(QObject *objectWatched, QEvent *event);
+
 signals:
   void TabChanged(int id);
   void Rename(int id, const QString& new_name);
@@ -59,7 +63,6 @@ protected:
   void resizeEvent(QResizeEvent *);
 
 private slots:
-  void ClearFilter();
   void NewPlaylist();
   void LoadPlaylist();
   void SavePlaylist() { SavePlaylist(-1); }
@@ -81,6 +84,7 @@ private slots:
 
   void SelectionChanged();
   void UpdateFilter();
+  void FocusOnFilter(QKeyEvent *event);
 
 private:
   void UpdateActiveIcon(const QIcon& icon);
