@@ -71,6 +71,8 @@ void PlaylistItem::BindToQuery(QSqlQuery* query) const {
   query->bindValue(6, DatabaseValue(Column_Album));
   query->bindValue(7, DatabaseValue(Column_Length));
   query->bindValue(8, DatabaseValue(Column_RadioService));
+  query->bindValue(9, DatabaseValue(Column_Beginning));
+  query->bindValue(10, DatabaseValue(Column_CuePath));
 }
 
 void PlaylistItem::SetTemporaryMetadata(const Song& metadata) {
@@ -89,3 +91,39 @@ static void ReloadPlaylistItem(PlaylistItemPtr item) {
 QFuture<void> PlaylistItem::BackgroundReload() {
   return QtConcurrent::run(ReloadPlaylistItem, shared_from_this());
 }
+
+void PlaylistItem::SetBackgroundColor(short priority, const QColor& color) {
+  background_colors_[priority] = color;
+}
+bool PlaylistItem::HasBackgroundColor(short priority) const {
+  return background_colors_.contains(priority);
+}
+void PlaylistItem::RemoveBackgroundColor(short priority) {
+  background_colors_.remove(priority);
+}
+QColor PlaylistItem::GetCurrentBackgroundColor() const {
+  return background_colors_.isEmpty()
+      ? QColor()
+      : background_colors_[background_colors_.keys().last()];
+}
+bool PlaylistItem::HasCurrentBackgroundColor() const {
+  return !background_colors_.isEmpty();
+}
+
+void PlaylistItem::SetForegroundColor(short priority, const QColor& color) {
+  foreground_colors_[priority] = color;
+}
+bool PlaylistItem::HasForegroundColor(short priority) const {
+  return foreground_colors_.contains(priority);
+}
+void PlaylistItem::RemoveForegroundColor(short priority) {
+  foreground_colors_.remove(priority);
+}
+QColor PlaylistItem::GetCurrentForegroundColor() const {
+  return foreground_colors_.isEmpty()
+      ? QColor()
+      : foreground_colors_[foreground_colors_.keys().last()];
+}
+bool PlaylistItem::HasCurrentForegroundColor() const {
+  return !foreground_colors_.isEmpty();
+}
