@@ -31,7 +31,7 @@
 using smart_playlists::GeneratorPtr;
 
 PlaylistManager::PlaylistManager(TaskManager* task_manager, QObject *parent)
-  : QObject(parent),
+  : PlaylistManagerInterface(parent),
     task_manager_(task_manager),
     playlist_backend_(NULL),
     library_backend_(NULL),
@@ -70,6 +70,21 @@ void PlaylistManager::Init(LibraryBackend* library_backend,
   emit PlaylistManagerInitialized();
 }
 
+QList<Playlist*> PlaylistManager::GetAllPlaylists() const {
+  QList<Playlist*> result;
+
+  foreach(const Data& data, playlists_.values()) {
+    result.append(data.p);
+  }
+
+  return result;
+}
+
+const QItemSelection& PlaylistManager::selection(int id) const {
+  QMap<int, Data>::const_iterator it = playlists_.find(id);
+  return it->selection;
+}
+
 Playlist* PlaylistManager::AddPlaylist(int id, const QString& name) {
   Playlist* ret = new Playlist(playlist_backend_, task_manager_, library_backend_, id);
   ret->set_sequence(sequence_);
@@ -96,6 +111,9 @@ Playlist* PlaylistManager::AddPlaylist(int id, const QString& name) {
 }
 
 void PlaylistManager::New(const QString& name, const SongList& songs) {
+  if (name.isNull())
+    return;
+
   int id = playlist_backend_->CreatePlaylist(name);
 
   if (id == -1)
@@ -165,7 +183,15 @@ void PlaylistManager::Remove(int id) {
 
   playlist_backend_->RemovePlaylist(id);
 
-  int next_id = playlists_.constBegin()->p->id();
+  int next_id = -1;
+  foreach (int possible_next_id, playlists_.keys()) {
+    if (possible_next_id != id) {
+      next_id = possible_next_id;
+      break;
+    }
+  }
+  if (next_id == -1)
+    return;
 
   if (id == active_)
     SetActivePlaylist(next_id);
@@ -195,7 +221,7 @@ void PlaylistManager::SetActivePlaylist(int id) {
   // Kinda a hack: unset the current item from the old active playlist before
   // setting the new one
   if (active_ != -1 && active_ != id)
-    active()->set_current_index(-1);
+    active()->set_current_row(-1);
 
   active_ = id;
   emit ActiveChanged(active());
@@ -227,13 +253,21 @@ void PlaylistManager::SetActiveStreamMetadata(const QUrl &url, const Song &song)
   active()->SetStreamMetadata(url, song);
 }
 
+void PlaylistManager::RateCurrentSong(double rating) {
+  active()->RateSong(active()->current_index(), rating);
+}
+
+void PlaylistManager::RateCurrentSong(int rating) {
+  RateCurrentSong(rating / 5.0);
+}
+
 void PlaylistManager::ChangePlaylistOrder(const QList<int>& ids) {
   playlist_backend_->SetPlaylistOrder(ids);
 }
 
 void PlaylistManager::UpdateSummaryText() {
   int tracks = current()->rowCount();
-  quint64 seconds = 0;
+  quint64 nanoseconds = 0;
   int selected = 0;
 
   // Get the length of the selected tracks
@@ -243,9 +277,9 @@ void PlaylistManager::UpdateSummaryText() {
 
     selected += range.bottom() - range.top() + 1;
     for (int i=range.top() ; i<=range.bottom() ; ++i) {
-      int length = range.model()->index(i, Playlist::Column_Length).data().toInt();
+      qint64 length = range.model()->index(i, Playlist::Column_Length).data().toLongLong();
       if (length > 0)
-        seconds += length;
+        nanoseconds += length;
     }
   }
 
@@ -253,14 +287,14 @@ void PlaylistManager::UpdateSummaryText() {
   if (selected > 1) {
     summary += tr("%1 selected of").arg(selected) + " ";
   } else {
-    seconds = current()->GetTotalLength();
+    nanoseconds = current()->GetTotalLength();
   }
 
   // TODO: Make the plurals translatable
   summary += tracks == 1 ? tr("1 track") : tr("%1 tracks").arg(tracks);
 
-  if (seconds)
-    summary += " - [ " + Utilities::WordyTime(seconds) + " ]";
+  if (nanoseconds)
+    summary += " - [ " + Utilities::WordyTimeNanosec(nanoseconds) + " ]";
 
   emit SummaryTextChanged(summary);
 }
@@ -298,3 +332,18 @@ void PlaylistManager::PlaySmartPlaylist(GeneratorPtr generator, bool as_new, boo
 
   current()->InsertSmartPlaylist(generator);
 }
+
+// When Player has processed the new song chosen by the user...
+void PlaylistManager::SongChangeRequestProcessed(const QUrl& url, bool valid) {
+  foreach(Playlist* playlist, GetAllPlaylists()) {
+    if(playlist->ApplyValidityOnCurrentSong(url, valid)) {
+      return;
+    }
+  }
+}
+
+void PlaylistManager::InvalidateDeletedSongs() {
+  foreach(Playlist* playlist, GetAllPlaylists()) {
+    playlist->InvalidateDeletedSongs();
+  }
+}
