@@ -18,6 +18,7 @@
 #ifndef PLAYLISTMANAGER_H
 #define PLAYLISTMANAGER_H
 
+#include <QColor>
 #include <QItemSelection>
 #include <QMap>
 #include <QObject>
@@ -35,7 +36,90 @@ class TaskManager;
 class QModelIndex;
 class QUrl;
 
-class PlaylistManager : public QObject {
+class PlaylistManagerInterface : public QObject {
+  Q_OBJECT
+
+public:
+  PlaylistManagerInterface(QObject* parent)
+    : QObject(parent) {}
+
+  virtual int current_id() const = 0;
+  virtual int active_id() const = 0;
+
+  virtual Playlist* playlist(int id) const = 0;
+  virtual Playlist* current() const = 0;
+  virtual Playlist* active() const = 0;
+
+  // Returns the collection of playlists managed by this PlaylistManager.
+  virtual QList<Playlist*> GetAllPlaylists() const = 0;
+  // Grays out and reloads all deleted songs in all playlists.
+  virtual void InvalidateDeletedSongs() = 0;
+
+  virtual const QItemSelection& selection(int id) const = 0;
+  virtual const QItemSelection& current_selection() const = 0;
+  virtual const QItemSelection& active_selection() const = 0;
+
+  virtual QString name(int index) const = 0;
+
+  virtual TaskManager* task_manager() const = 0;
+  virtual LibraryBackend* library_backend() const = 0;
+  virtual PlaylistBackend* playlist_backend() const = 0;
+  virtual PlaylistSequence* sequence() const = 0;
+  virtual PlaylistParser* parser() const = 0;
+
+public slots:
+  virtual void New(const QString& name, const SongList& songs = SongList()) = 0;
+  virtual void Load(const QString& filename) = 0;
+  virtual void Save(int id, const QString& filename) = 0;
+  virtual void Rename(int id, const QString& new_name) = 0;
+  virtual void Remove(int id) = 0;
+  virtual void ChangePlaylistOrder(const QList<int>& ids) = 0;
+
+  virtual void SongChangeRequestProcessed(const QUrl& url, bool valid) = 0;
+
+  virtual void SetCurrentPlaylist(int id) = 0;
+  virtual void SetActivePlaylist(int id) = 0;
+  virtual void SetActiveToCurrent() = 0;
+
+  virtual void SelectionChanged(const QItemSelection& selection) = 0;
+
+  // Convenience slots that defer to either current() or active()
+  virtual void ClearCurrent() = 0;
+  virtual void ShuffleCurrent() = 0;
+  virtual void SetActivePlaying() = 0;
+  virtual void SetActivePaused() = 0;
+  virtual void SetActiveStopped() = 0;
+  virtual void SetActiveStreamMetadata(const QUrl& url, const Song& song) = 0;
+  // Rate current song using 0.0 - 1.0 scale.
+  virtual void RateCurrentSong(double rating) = 0;
+  // Rate current song using 0 - 5 scale.
+  virtual void RateCurrentSong(int rating) = 0;
+
+  virtual void PlaySmartPlaylist(smart_playlists::GeneratorPtr generator, bool as_new, bool clear) = 0;
+
+signals:
+  void PlaylistManagerInitialized();
+
+  void PlaylistAdded(int id, const QString& name);
+  void PlaylistRemoved(int id);
+  void PlaylistRenamed(int id, const QString& new_name);
+  void CurrentChanged(Playlist* new_playlist);
+  void ActiveChanged(Playlist* new_playlist);
+
+  void Error(const QString& message);
+  void SummaryTextChanged(const QString& summary);
+
+  // Forwarded from individual playlists
+  void CurrentSongChanged(const Song& song);
+
+  // Signals that one of manager's playlists has changed (new items, new
+  // ordering etc.) - the argument shows which.
+  void PlaylistChanged(Playlist* playlist);
+  void EditingFinished(const QModelIndex& index);
+  void PlayRequested(const QModelIndex& index);
+};
+
+class PlaylistManager : public PlaylistManagerInterface {
   Q_OBJECT
 
 public:
@@ -49,7 +133,12 @@ public:
   Playlist* current() const { return playlist(current_id()); }
   Playlist* active() const { return playlist(active_id()); }
 
-  const QItemSelection& selection(int id) const { return playlists_[id].selection; }
+  // Returns the collection of playlists managed by this PlaylistManager.
+  QList<Playlist*> GetAllPlaylists() const;
+  // Grays out and reloads all deleted songs in all playlists.
+  void InvalidateDeletedSongs();
+
+  const QItemSelection& selection(int id) const;
   const QItemSelection& current_selection() const { return selection(current_id()); }
   const QItemSelection& active_selection() const { return selection(active_id()); }
 
@@ -85,29 +174,14 @@ public slots:
   void SetActivePaused();
   void SetActiveStopped();
   void SetActiveStreamMetadata(const QUrl& url, const Song& song);
+  // Rate current song using 0.0 - 1.0 scale.
+  void RateCurrentSong(double rating);
+  // Rate current song using 0 - 5 scale.
+  void RateCurrentSong(int rating);
 
   void PlaySmartPlaylist(smart_playlists::GeneratorPtr generator, bool as_new, bool clear);
 
-signals:
-  void PlaylistManagerInitialized();
-
-  void PlaylistAdded(int id, const QString& name);
-  void PlaylistRemoved(int id);
-  void PlaylistRenamed(int id, const QString& new_name);
-  void CurrentChanged(Playlist* new_playlist);
-  void ActiveChanged(Playlist* new_playlist);
-
-  void Error(const QString& message);
-  void SummaryTextChanged(const QString& summary);
-
-  // Forwarded from individual playlists
-  void CurrentSongChanged(const Song& song);
-  
-  // Signals that one of manager's playlists has changed (new items, new
-  // ordering etc.) - the argument shows which.
-  void PlaylistChanged(Playlist* playlist);
-  void EditingFinished(const QModelIndex& index);
-  void PlayRequested(const QModelIndex& index);
+  void SongChangeRequestProcessed(const QUrl& url, bool valid);
 
 private slots:
   void OneOfPlaylistsChanged();
