@@ -60,7 +60,7 @@ void PlaylistTabBar::SetManager(PlaylistManager *manager) {
 void PlaylistTabBar::contextMenuEvent(QContextMenuEvent* e) {
   menu_index_ = tabAt(e->pos());
   rename_->setEnabled(menu_index_ != -1);
-  remove_->setEnabled(menu_index_ != -1 && count() > 1 && menu_index_ != index_of(manager_->active_id()));
+  remove_->setEnabled(menu_index_ != -1 && count() > 1);
   save_->setEnabled(menu_index_ != -1);
 
   menu_->popup(e->globalPos());
@@ -173,8 +173,7 @@ void PlaylistTabBar::TabMoved() {
 void PlaylistTabBar::dragEnterEvent(QDragEnterEvent* e) {
   if (e->mimeData()->hasUrls() ||
       e->mimeData()->hasFormat(Playlist::kRowsMimetype) ||
-      qobject_cast<const SongMimeData*>(e->mimeData()) ||
-      qobject_cast<const RadioMimeData*>(e->mimeData())) {
+      qobject_cast<const MimeData*>(e->mimeData())) {
     e->acceptProposedAction();
   }
 }
@@ -182,16 +181,14 @@ void PlaylistTabBar::dragEnterEvent(QDragEnterEvent* e) {
 void PlaylistTabBar::dragMoveEvent(QDragMoveEvent* e) {
   drag_hover_tab_ = tabAt(e->pos());
 
-  if (drag_hover_tab_ == -1) {
-    e->setDropAction(Qt::IgnoreAction);
-    e->ignore();
-    drag_hover_timer_.stop();
-  } else {
+  if (drag_hover_tab_ != -1) {
     e->setDropAction(Qt::CopyAction);
     e->accept(tabRect(drag_hover_tab_));
 
     if (!drag_hover_timer_.isActive())
       drag_hover_timer_.start(kDragHoverTimeout, this);
+  } else {
+    drag_hover_timer_.stop();
   }
 }
 
@@ -211,10 +208,16 @@ void PlaylistTabBar::timerEvent(QTimerEvent* e) {
 
 void PlaylistTabBar::dropEvent(QDropEvent* e) {
   if (drag_hover_tab_ == -1) {
-    e->ignore();
-    return;
+    const MimeData *mime_data = qobject_cast<const MimeData*>(e->mimeData());
+    if(mime_data && !mime_data->name_for_new_playlist_.isEmpty()) {
+      manager_->New(mime_data->name_for_new_playlist_);
+    } else {
+      manager_->New(tr("Playlist"));
+    }
+    setCurrentIndex(count() - 1);
+  } else {
+    setCurrentIndex(drag_hover_tab_);
   }
 
-  setCurrentIndex(drag_hover_tab_);
   manager_->current()->dropMimeData(e->mimeData(), e->proposedAction(), -1, 0, QModelIndex());
 }
