@@ -35,7 +35,6 @@
 
 #include <math.h>
 
-const char* PlaylistView::kSettingsGroup = "Playlist";
 const int PlaylistView::kStateVersion = 3;
 const int PlaylistView::kGlowIntensitySteps = 24;
 const int PlaylistView::kAutoscrollGraceTimeout = 60; // seconds
@@ -72,7 +71,7 @@ PlaylistView::PlaylistView(QWidget *parent)
     playlist_(NULL),
     header_(new PlaylistHeader(Qt::Horizontal, this)),
     setting_initial_header_layout_(false),
-    read_only_settings_(false),
+    read_only_settings_(true),
     glow_enabled_(true),
     currently_glowing_(false),
     glow_intensity_step_(0),
@@ -100,6 +99,7 @@ PlaylistView::PlaylistView(QWidget *parent)
   connect(header_, SIGNAL(sectionMoved(int,int,int)), SLOT(InvalidateCachedCurrentPixmap()));
   connect(header_, SIGNAL(SectionVisibilityChanged(int,bool)), SLOT(InvalidateCachedCurrentPixmap()));
   connect(header_, SIGNAL(StretchEnabledChanged(bool)), SLOT(SaveSettings()));
+  connect(header_, SIGNAL(ColumnAlignmentChanged()), SLOT(SaveSettings()));
   connect(header_, SIGNAL(StretchEnabledChanged(bool)), SLOT(StretchChanged(bool)));
   connect(header_, SIGNAL(MouseEntered()), SLOT(RatingHoverOut()));
 
@@ -132,6 +132,10 @@ void PlaylistView::SetItemDelegates(LibraryBackend* backend) {
       new TagCompletionItemDelegate(this, backend, Playlist::Column_Artist));
   setItemDelegateForColumn(Playlist::Column_AlbumArtist,
       new TagCompletionItemDelegate(this, backend, Playlist::Column_AlbumArtist));
+  setItemDelegateForColumn(Playlist::Column_Genre,
+      new TagCompletionItemDelegate(this, backend, Playlist::Column_Genre));
+  setItemDelegateForColumn(Playlist::Column_Composer,
+      new TagCompletionItemDelegate(this, backend, Playlist::Column_Composer));
   setItemDelegateForColumn(Playlist::Column_Length, new LengthItemDelegate(this));
   setItemDelegateForColumn(Playlist::Column_Filesize, new SizeItemDelegate(this));
   setItemDelegateForColumn(Playlist::Column_Filetype, new FileTypeItemDelegate(this));
@@ -163,6 +167,10 @@ void PlaylistView::SetPlaylist(Playlist *playlist) {
   LoadGeometry();
   ReloadSettings();
   DynamicModeChanged(playlist->is_dynamic());
+  setFocus();
+  read_only_settings_ = false;
+
+  connect(playlist_, SIGNAL(RestoreFinished()), SLOT(JumpToLastPlayedTrack()));
 
   connect(playlist_, SIGNAL(CurrentSongChanged(Song)), SLOT(MaybeAutoscroll()));
   connect(playlist_, SIGNAL(DynamicModeChanged(bool)), SLOT(DynamicModeChanged(bool)));
@@ -175,17 +183,21 @@ void PlaylistView::setModel(QAbstractItemModel *m) {
   if (model()) {
     disconnect(model(), SIGNAL(dataChanged(QModelIndex,QModelIndex)),
                this, SLOT(InvalidateCachedCurrentPixmap()));
+    disconnect(model(), SIGNAL(layoutAboutToBeChanged()),
+               this, SLOT(RatingHoverOut()));
   }
 
   QTreeView::setModel(m);
 
   connect(model(), SIGNAL(dataChanged(QModelIndex,QModelIndex)),
           this, SLOT(InvalidateCachedCurrentPixmap()));
+  connect(model(), SIGNAL(layoutAboutToBeChanged()),
+          this, SLOT(RatingHoverOut()));
 }
 
 void PlaylistView::LoadGeometry() {
   QSettings settings;
-  settings.beginGroup(kSettingsGroup);
+  settings.beginGroup(Playlist::kSettingsGroup);
 
   if (!header_->restoreState(settings.value("state").toByteArray())) {
     header_->HideSection(Playlist::Column_Disc);
@@ -244,7 +256,7 @@ void PlaylistView::SaveGeometry() {
     return;
 
   QSettings settings;
-  settings.beginGroup(kSettingsGroup);
+  settings.beginGroup(Playlist::kSettingsGroup);
   settings.setValue("state", header_->saveState());
   settings.setValue("state_version", kStateVersion);
 }
@@ -422,15 +434,30 @@ void PlaylistView::keyPressEvent(QKeyEvent* event) {
     QTreeView::keyPressEvent(event);
   } else if (state() == QAbstractItemView::EditingState) {
     QTreeView::keyPressEvent(event);
-  } else if (event->matches(QKeySequence::Delete) ||
-      event->key() == Qt::Key_Backspace) {
+  } else if (event->matches(QKeySequence::Delete)) {
     RemoveSelected();
     event->accept();
   } else if (event->key() == Qt::Key_Enter ||
-             event->key() == Qt::Key_Return ||
-             event->key() == Qt::Key_Space) {
+             event->key() == Qt::Key_Return) {
     if (currentIndex().isValid())
-      emit PlayPauseItem(currentIndex());
+      emit PlayItem(currentIndex());
+    event->accept();
+  } else if(event->modifiers() != Qt::ControlModifier //Ctrl+Space selects the item
+      && event->key() == Qt::Key_Space) {
+    emit PlayPause();
+    event->accept();
+  } else if(event->key() == Qt::Key_Left) {
+    emit SeekTrack(-1);
+    event->accept();
+  } else if(event->key() == Qt::Key_Right) {
+    emit SeekTrack(1);
+    event->accept();
+  } else if(event->modifiers() == Qt::NoModifier // No modifier keys currently pressed...
+      // ... and key pressed is something related to text
+      && ( (event->key() >= Qt::Key_A && event->key() <= Qt::Key_Z)
+        || event->key() == Qt::Key_Backspace
+        || event->key() == Qt::Key_Escape)) {
+    emit FocusOnFilterSignal(event);
     event->accept();
   } else {
     QTreeView::keyPressEvent(event);
@@ -601,12 +628,12 @@ void PlaylistView::mousePressEvent(QMouseEvent* event) {
       // Update all the selected items
       foreach (const QModelIndex& index, selectedIndexes()) {
         if (index.data(Playlist::Role_CanSetRating).toBool()) {
-          emit SongRatingSet(index, new_rating);
+          playlist_->RateSong(playlist_->proxy()->mapToSource(index), new_rating);
         }
       }
     } else {
       // Update only this item
-      emit SongRatingSet(index, new_rating);
+      playlist_->RateSong(playlist_->proxy()->mapToSource(index), new_rating);
     }
   } else {
     QTreeView::mousePressEvent(event);
@@ -645,11 +672,11 @@ void PlaylistView::MaybeAutoscroll() {
 void PlaylistView::JumpToCurrentlyPlayingTrack() {
   Q_ASSERT(playlist_);
 
-  if (playlist_->current_index() == -1)
+  if (playlist_->current_row() == -1)
     return;
 
   QModelIndex current = playlist_->proxy()->mapFromSource(
-      playlist_->index(playlist_->current_index(), 0));
+      playlist_->index(playlist_->current_row(), 0));
   if (!current.isValid())
     return;
 
@@ -661,6 +688,29 @@ void PlaylistView::JumpToCurrentlyPlayingTrack() {
   currently_autoscrolling_ = false;
 }
 
+void PlaylistView::JumpToLastPlayedTrack() {
+  Q_ASSERT(playlist_);
+
+  if (playlist_->last_played_row() == -1)
+    return;
+
+  QModelIndex last_played = playlist_->proxy()->mapFromSource(
+      playlist_->index(playlist_->last_played_row(), 0));
+  if (!last_played.isValid())
+    return;
+
+  // Select last played song
+  last_current_item_ = last_played;
+  setCurrentIndex(last_current_item_);
+
+  currently_autoscrolling_ = true;
+
+  // Scroll to the item
+  scrollTo(last_played, QAbstractItemView::PositionAtCenter);
+
+  currently_autoscrolling_ = false;
+}
+
 void PlaylistView::paintEvent(QPaintEvent* event) {
   // Reimplemented to draw the drop indicator
   // When the user is dragging some stuff over the playlist paintEvent gets
@@ -767,7 +817,7 @@ void PlaylistView::PlaylistDestroyed() {
 
 void PlaylistView::ReloadSettings() {
   QSettings s;
-  s.beginGroup(kSettingsGroup);
+  s.beginGroup(Playlist::kSettingsGroup);
   glow_enabled_ = s.value("glow_effect", true).toBool();
   header_->SetStretchEnabled(s.value("stretch", true).toBool());
 
@@ -781,6 +831,9 @@ void PlaylistView::ReloadSettings() {
     header_->SetColumnWidth(Playlist::Column_Track, 0.05);
     setting_initial_header_layout_ = false;
   }
+
+  ColumnAlignmentMap column_alignments = s.value("column_alignments").value<ColumnAlignmentMap>();
+  if (!column_alignments.isEmpty()) playlist_->set_column_alignments(column_alignments);
 }
 
 void PlaylistView::SaveSettings() {
@@ -788,9 +841,10 @@ void PlaylistView::SaveSettings() {
     return;
 
   QSettings s;
-  s.beginGroup(kSettingsGroup);
+  s.beginGroup(Playlist::kSettingsGroup);
   s.setValue("glow_effect", glow_enabled_);
   s.setValue("stretch", header_->is_stretch_enabled());
+  s.setValue("column_alignments", QVariant::fromValue(playlist_->column_alignments()));
 }
 
 void PlaylistView::StretchChanged(bool stretch) {
