@@ -59,7 +59,6 @@ class PlaylistView : public QTreeView {
  public:
   PlaylistView(QWidget* parent = 0);
 
-  static const char* kSettingsGroup;
   static const int kStateVersion;
 
   void SetItemDelegates(LibraryBackend* backend);
@@ -68,6 +67,8 @@ class PlaylistView : public QTreeView {
 
   void SetReadOnlySettings(bool read_only) { read_only_settings_ = read_only; }
 
+  Playlist* playlist() const { return playlist_; }
+
   // QTreeView
   void drawTree(QPainter* painter, const QRegion& region) const;
   void drawRow(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const;
@@ -79,13 +80,16 @@ class PlaylistView : public QTreeView {
   void StopGlowing();
   void StartGlowing();
   void JumpToCurrentlyPlayingTrack();
+  void JumpToLastPlayedTrack();
   void closeEditor(QWidget* editor, QAbstractItemDelegate::EndEditHint hint);
   void DynamicModeChanged(bool dynamic);
 
  signals:
-  void PlayPauseItem(const QModelIndex& index);
+  void PlayItem(const QModelIndex& index);
+  void PlayPause();
   void RightClicked(const QPoint& global_pos, const QModelIndex& index);
-  void SongRatingSet(const QModelIndex& index, double rating);
+  void SeekTrack(int gap);
+  void FocusOnFilterSignal(QKeyEvent *event);
 
  protected:
   void contextMenuEvent(QContextMenuEvent* e);
