@@ -21,12 +21,12 @@
 #include "core/taskmanager.h"
 
 SongLoaderInserter::SongLoaderInserter(
-    TaskManager* task_manager, LibraryBackendInterface* library, QObject *parent)
-    : QObject(parent),
-      task_manager_(task_manager),
+    TaskManager* task_manager, LibraryBackendInterface* library)
+    : task_manager_(task_manager),
       destination_(NULL),
       row_(-1),
       play_now_(true),
+      enqueue_(false),
       async_load_id_(0),
       async_progress_(0),
       library_(library) {
@@ -36,19 +36,26 @@ SongLoaderInserter::~SongLoaderInserter() {
   qDeleteAll(pending_);
 }
 
-void SongLoaderInserter::Load(Playlist *destination, int row, bool play_now,
+void SongLoaderInserter::Load(Playlist *destination,
+                              int row, bool play_now, bool enqueue,
                               const QList<QUrl> &urls) {
   destination_ = destination;
   row_ = row;
   play_now_ = play_now;
+  enqueue_ = enqueue;
+
+  connect(destination, SIGNAL(destroyed()), SLOT(DestinationDestroyed()));
 
   foreach (const QUrl& url, urls) {
     SongLoader* loader = new SongLoader(library_, this);
+
+    // we're connecting this before we're even sure if this is an async load
+    // to avoid race conditions (signal emission before we're listening to it)
+    connect(loader, SIGNAL(LoadFinished(bool)), SLOT(PendingLoadFinished(bool)));
     SongLoader::Result ret = loader->Load(url);
 
     if (ret == SongLoader::WillLoadAsync) {
       pending_.insert(loader);
-      connect(loader, SIGNAL(LoadFinished(bool)), SLOT(PendingLoadFinished(bool)));
       continue;
     }
 
@@ -68,6 +75,10 @@ void SongLoaderInserter::Load(Playlist *destination, int row, bool play_now,
   }
 }
 
+void SongLoaderInserter::DestinationDestroyed() {
+  destination_ = NULL;
+}
+
 void SongLoaderInserter::PendingLoadFinished(bool success) {
   SongLoader* loader = qobject_cast<SongLoader*>(sender());
   if (!loader || !pending_.contains(loader))
@@ -89,9 +100,9 @@ void SongLoaderInserter::PendingLoadFinished(bool success) {
 }
 
 void SongLoaderInserter::Finished() {
-  QModelIndex index = destination_->InsertSongsOrLibraryItems(songs_, row_);
-  if (play_now_)
-    emit PlayRequested(index);
+  if (destination_) {
+    destination_->InsertSongsOrLibraryItems(songs_, row_, play_now_, enqueue_);
+  }
 
   deleteLater();
 }
