@@ -34,17 +34,18 @@ class QModelIndex;
 class SongLoaderInserter : public QObject {
   Q_OBJECT
 public:
-  SongLoaderInserter(TaskManager* task_manager, LibraryBackendInterface* library, QObject* parent = 0);
+  SongLoaderInserter(TaskManager* task_manager, LibraryBackendInterface* library);
   ~SongLoaderInserter();
 
-  void Load(Playlist* destination, int row, bool play_now, const QList<QUrl>& urls);
+  void Load(Playlist* destination, int row, bool play_now, bool enqueue,
+            const QList<QUrl>& urls);
 
 signals:
   void Error(const QString& message);
-  void PlayRequested(const QModelIndex& index);
 
 private slots:
   void PendingLoadFinished(bool success);
+  void DestinationDestroyed();
 
 private:
   void Finished();
@@ -55,6 +56,7 @@ private:
   Playlist* destination_;
   int row_;
   bool play_now_;
+  bool enqueue_;
 
   SongList songs_;
 
