@@ -29,7 +29,7 @@ SongPlaylistItem::SongPlaylistItem(const QString& type)
 }
 
 SongPlaylistItem::SongPlaylistItem(const Song& song)
-  : PlaylistItem(song.filetype() == Song::Type_Stream ? "Stream" : "File"),
+  : PlaylistItem(song.is_stream() ? "Stream" : "File"),
     song_(song)
 {
 }
@@ -39,13 +39,12 @@ bool SongPlaylistItem::InitFromQuery(const SqlRow& query) {
   const int row = (Song::kColumns.count() + 1) * 3;
 
   QString filename(query.value(row + 1).toString());
+  QString title(query.value(row + 2).toString());
+  QString artist(query.value(row + 3).toString());
+  QString album(query.value(row + 4).toString());
+  qint64 length(query.value(row + 5).toLongLong());
 
   if (type() == "Stream") {
-    QString title(query.value(row + 2).toString());
-    QString artist(query.value(row + 3).toString());
-    QString album(query.value(row + 4).toString());
-    int length(query.value(row + 5).toInt());
-
     song_.set_filename(filename);
     song_.set_filetype(Song::Type_Stream);
 
@@ -53,33 +52,48 @@ bool SongPlaylistItem::InitFromQuery(const SqlRow& query) {
   } else {
     song_.InitFromFile(filename, -1);
 
-    if (!song_.is_valid())
-      return false;
+    qint64 beginning(query.value(row + 7).toLongLong());
+    QString cue_path(query.value(row + 8).toString());
+
+    // If the song was part of a cuesheet then keep the title, artist etc. that
+    // was loaded last time.
+    if (!cue_path.isEmpty()) {
+      song_.set_title(title);
+      song_.set_artist(artist);
+      song_.set_album(album);
+      song_.set_length_nanosec(length);
+    }
+
+    song_.set_beginning_nanosec(beginning);
+    song_.set_cue_path(cue_path);
   }
+
   return true;
 }
 
 QVariant SongPlaylistItem::DatabaseValue(DatabaseColumn column) const {
   switch (column) {
-    case Column_Url:    return song_.filename();
-    case Column_Title:  return song_.title();
-    case Column_Artist: return song_.artist();
-    case Column_Album:  return song_.album();
-    case Column_Length: return song_.length();
-    default:            return PlaylistItem::DatabaseValue(column);
+    case Column_Url:       return song_.filename();
+    case Column_Title:     return song_.title();
+    case Column_Artist:    return song_.artist();
+    case Column_Album:     return song_.album();
+    case Column_Length:    return song_.length_nanosec();
+    case Column_Beginning: return song_.beginning_nanosec();
+    case Column_CuePath:   return song_.cue_path();
+    default:               return PlaylistItem::DatabaseValue(column);
   }
 }
 
 QUrl SongPlaylistItem::Url() const {
-  if (QFile::exists(song_.filename())) {
-    return QUrl::fromLocalFile(song_.filename());
-  } else {
-    return song_.filename();
-  }
+  return song_.url();
 }
 
 void SongPlaylistItem::Reload() {
-  song_.InitFromFile(song_.filename(), song_.directory_id());
+  QString old_filename = song_.filename();
+  int old_directory_id = song_.directory_id();
+
+  song_ = Song();
+  song_.InitFromFile(old_filename, old_directory_id);
 }
 
 Song SongPlaylistItem::Metadata() const {
