@@ -25,7 +25,7 @@ M3UParser::M3UParser(LibraryBackendInterface* library, QObject* parent)
 {
 }
 
-SongList M3UParser::Load(QIODevice* device, const QDir& dir) const {
+SongList M3UParser::Load(QIODevice* device, const QString& playlist_path, const QDir& dir) const {
   SongList ret;
 
   M3UType type = STANDARD;
@@ -56,12 +56,8 @@ SongList M3UParser::Load(QIODevice* device, const QDir& dir) const {
           continue;
         }
       }
-    } else {
+    } else if (!line.isEmpty()) {
       Song song;
-      song.Init(current_metadata.title,
-                current_metadata.artist,
-                QString(),  // Unknown album.
-                current_metadata.length);
 
       // Track location.
       if (!ParseTrackLocation(line, dir, &song)) {
@@ -69,10 +65,16 @@ SongList M3UParser::Load(QIODevice* device, const QDir& dir) const {
       } else {
         // Load the song from the library if it's there.
         Song library_song = LoadLibrarySong(song.filename());
-        if (library_song.is_valid())
+        if (library_song.is_valid()) {
           ret << library_song;
-        else
+        } else {
+          song.Init(current_metadata.title,
+                    current_metadata.artist,
+                    QString(),  // Unknown album.
+                    current_metadata.length);
+          song.InitFromFile(song.filename(), -1);
           ret << song;
+        }
 
         current_metadata.artist.clear();
         current_metadata.title.clear();
@@ -98,7 +100,7 @@ bool M3UParser::ParseMetadata(const QString& line, M3UParser::Metadata* metadata
   if (!ok) {
     return false;
   }
-  metadata->length = length;
+  metadata->length = length * kNsecPerSec;
 
   QString track_info = info.section(',', 1);
   QStringList list = track_info.split('-');
@@ -117,7 +119,9 @@ void M3UParser::Save(const SongList &songs, QIODevice *device, const QDir &dir)
     if (song.filename().isEmpty()) {
       continue;
     }
-    QString meta = QString("#EXTINF:%1,%2 - %3\n").arg(song.length()).arg(song.artist()).arg(song.title());
+    QString meta = QString("#EXTINF:%1,%2 - %3\n")
+                   .arg(song.length_nanosec() / kNsecPerSec)
+                   .arg(song.artist()).arg(song.title());
     device->write(meta.toUtf8());
     device->write(MakeRelativeTo(song.filename(), dir).toUtf8());
     device->write("\n");
