@@ -54,7 +54,6 @@ bool ParserBase::ParseTrackLocation(const QString& filename_or_url,
     QString absolute_path = dir.absoluteFilePath(proper_path);
     song->set_filename(absolute_path);
   }
-  song->InitFromFile(song->filename(), -1);
   return true;
 }
 
@@ -80,7 +79,7 @@ QString ParserBase::MakeUrl(const QString& filename_or_url) const {
   return QUrl::fromLocalFile(filename_or_url).toString();
 }
 
-Song ParserBase::LoadLibrarySong(const QString& filename_or_url) const {
+Song ParserBase::LoadLibrarySong(const QString& filename_or_url, qint64 beginning) const {
   if (!library_)
     return Song();
 
@@ -91,13 +90,5 @@ Song ParserBase::LoadLibrarySong(const QString& filename_or_url) const {
   else
     info.setFile(filename_or_url);
 
-  LibraryQuery query;
-  query.SetColumnSpec("%songs_table.ROWID, " + Song::kColumnSpec);
-  query.AddWhere("filename", info.canonicalFilePath());
-
-  Song song;
-  if (library_->ExecQuery(&query) && query.Next()) {
-    song.InitFromQuery(query);
-  }
-  return song;
+  return library_->GetSongByFilename(info.canonicalFilePath(), beginning);
 }
