@@ -37,7 +37,14 @@ public:
 
   virtual bool TryMagic(const QByteArray& data) const = 0;
 
-  virtual SongList Load(QIODevice* device, const QDir& dir = QDir()) const = 0;
+  // Loads all songs from playlist found at path 'playlist_path' in directory 'dir'.
+  // The 'device' argument is an opened and ready to read from represantation of
+  // this playlist.
+  // This method might not return all of the songs found in the playlist. Any playlist
+  // parser may decide to leave out some entries if it finds them incomplete or invalid.
+  // This means that the final resulting SongList should be considered valid (at least
+  // from the parser's point of view).
+  virtual SongList Load(QIODevice* device, const QString& playlist_path = "", const QDir& dir = QDir()) const = 0;
   virtual void Save(const SongList& songs, QIODevice* device, const QDir& dir = QDir()) const = 0;
 
 protected:
@@ -54,9 +61,9 @@ protected:
   QString MakeUrl(const QString& filename_or_url) const;
 
   // Converts the URL or path to a canonical path and searches the library for
-  // a song with that path.  If one is found, returns it, otherwise returns an
-  // invalid song.
-  Song LoadLibrarySong(const QString& filename_or_url) const;
+  // a section of a song with that path and the given beginning. If one is found,
+  // returns it, otherwise returns an invalid song.
+  Song LoadLibrarySong(const QString& filename_or_url, qint64 beginning = 0) const;
 
 private:
   LibraryBackendInterface* library_;
