@@ -19,6 +19,7 @@
 #include "xspfparser.h"
 #include "m3uparser.h"
 #include "plsparser.h"
+#include "cueparser.h"
 #include "asxparser.h"
 #include "asxiniparser.h"
 
@@ -35,6 +36,7 @@ PlaylistParser::PlaylistParser(LibraryBackendInterface* library, QObject *parent
   parsers_ << new PLSParser(library, this);
   parsers_ << new ASXParser(library, this);
   parsers_ << new AsxIniParser(library, this);
+  parsers_ << new CueParser(library, this);
 }
 
 QStringList PlaylistParser::file_extensions() const {
@@ -87,8 +89,8 @@ ParserBase* PlaylistParser::ParserForExtension(const QString& suffix) const {
   return NULL;
 }
 
-ParserBase* PlaylistParser::MaybeGetParserForMagic(const QByteArray& data,
-                                                   const QString& mime_type) const {
+ParserBase* PlaylistParser::ParserForMagic(const QByteArray& data,
+                                           const QString& mime_type) const {
   foreach (ParserBase* p, parsers_) {
     if ((!mime_type.isEmpty() && mime_type == p->mime_type()) ||
         p->TryMagic(data))
@@ -97,11 +99,11 @@ ParserBase* PlaylistParser::MaybeGetParserForMagic(const QByteArray& data,
   return NULL;
 }
 
-SongList PlaylistParser::Load(const QString &filename, ParserBase* p) const {
+SongList PlaylistParser::LoadFromFile(const QString& filename) const {
   QFileInfo info(filename);
 
   // Find a parser that supports this file extension
-  ParserBase* parser = p ? p : ParserForExtension(info.suffix());
+  ParserBase* parser = ParserForExtension(info.suffix());
   if (!parser) {
     qWarning() << "Unknown filetype:" << filename;
     return SongList();
@@ -111,10 +113,22 @@ SongList PlaylistParser::Load(const QString &filename, ParserBase* p) const {
   QFile file(filename);
   file.open(QIODevice::ReadOnly);
 
-  return parser->Load(&file, info.absolutePath());
+  return parser->Load(&file, filename, info.absolutePath());
 }
 
-void PlaylistParser::Save(const SongList &songs, const QString &filename) const {
+SongList PlaylistParser::LoadFromDevice(QIODevice* device,
+                                        const QString& path_hint,
+                                        const QDir& dir_hint) const {
+  // Find a parser that supports this data
+  ParserBase* parser = ParserForMagic(device->peek(kMagicSize));
+  if (!parser) {
+    return SongList();
+  }
+
+  return parser->Load(device, path_hint, dir_hint);
+}
+
+void PlaylistParser::Save(const SongList& songs, const QString& filename) const {
   QFileInfo info(filename);
 
   // Find a parser that supports this file extension
