@@ -25,7 +25,7 @@ PLSParser::PLSParser(LibraryBackendInterface* library, QObject* parent)
 {
 }
 
-SongList PLSParser::Load(QIODevice *device, const QDir &dir) const {
+SongList PLSParser::Load(QIODevice *device, const QString& playlist_path, const QDir &dir) const {
   QMap<int, Song> songs;
   QRegExp n_re("\\d+$");
 
@@ -44,13 +44,18 @@ SongList PLSParser::Load(QIODevice *device, const QDir &dir) const {
 
       // Load the song from the library if it's there.
       Song library_song = LoadLibrarySong(songs[n].filename());
-      if (library_song.is_valid())
+      if (library_song.is_valid()) {
         songs[n] = library_song;
-
+      } else {
+        songs[n].InitFromFile(songs[n].filename(), -1);
+      }
     } else if (key.startsWith("title")) {
       songs[n].set_title(value);
     } else if (key.startsWith("length")) {
-      songs[n].set_length(value.toInt());
+      qint64 seconds = value.toLongLong();
+      if (seconds > 0) {
+        songs[n].set_length_nanosec(seconds * kNsecPerSec);
+      }
     }
   }
 
@@ -67,7 +72,7 @@ void PLSParser::Save(const SongList &songs, QIODevice *device, const QDir &dir)
   foreach (const Song& song, songs) {
     s << "File" << n << "=" << MakeRelativeTo(song.filename(), dir) << endl;
     s << "Title" << n << "=" << song.title() << endl;
-    s << "Length" << n << "=" << song.length() << endl;
+    s << "Length" << n << "=" << song.length_nanosec() / kNsecPerSec << endl;
     ++n;
   }
 }
