@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,12 +30,11 @@ class QDomNode;
 
 class XMLParser : public ParserBase {
  protected:
-  XMLParser(QObject* parent);
+  XMLParser(LibraryBackendInterface* library, QObject* parent);
+
   bool ParseUntilElement(QXmlStreamReader* reader, const QString& element) const;
   void IgnoreElement(QXmlStreamReader* reader) const;
 
-  QString MakeRelativeTo(const QString& filename, const QDir& dir) const;
-
   class StreamElement : public boost::noncopyable {
    public:
     StreamElement(const QString& name, QXmlStreamWriter* stream) : stream_(stream) {
