@@ -29,7 +29,7 @@ XSPFParser::XSPFParser(LibraryBackendInterface* library, QObject* parent)
 {
 }
 
-SongList XSPFParser::Load(QIODevice *device, const QDir&) const {
+SongList XSPFParser::Load(QIODevice *device, const QString& playlist_path, const QDir&) const {
   SongList ret;
 
   QXmlStreamReader reader(device);
@@ -50,7 +50,7 @@ SongList XSPFParser::Load(QIODevice *device, const QDir&) const {
 Song XSPFParser::ParseTrack(QXmlStreamReader* reader) const {
   Song song;
   QString title, artist, album;
-  int length = -1;
+  qint64 nanosec = -1;
   while (!reader->atEnd()) {
     QXmlStreamReader::TokenType type = reader->readNext();
     switch (type) {
@@ -84,9 +84,9 @@ Song XSPFParser::ParseTrack(QXmlStreamReader* reader) const {
         } else if (name == "duration") {  // in milliseconds.
           const QString& duration = reader->readElementText();
           bool ok = false;
-          length = duration.toInt(&ok) / 1000;
+          nanosec = duration.toInt(&ok) * kNsecPerMsec;
           if (!ok) {
-            length = -1;
+            nanosec = -1;
           }
         } else if (name == "image") {
           // TODO: Fetch album covers.
@@ -97,7 +97,7 @@ Song XSPFParser::ParseTrack(QXmlStreamReader* reader) const {
       }
       case QXmlStreamReader::EndElement: {
         if (reader->name() == "track") {
-          song.Init(title, artist, album, length);
+          song.Init(title, artist, album, nanosec);
           return song;
         }
       }
@@ -106,7 +106,7 @@ Song XSPFParser::ParseTrack(QXmlStreamReader* reader) const {
     }
   }
   // At least make an effort if we never find a </track>.
-  song.Init(title, artist, album, length);
+  song.Init(title, artist, album, nanosec);
   return song;
 }
 
@@ -128,8 +128,8 @@ void XSPFParser::Save(const SongList& songs, QIODevice* device, const QDir&) con
     if (!song.album().isEmpty()) {
       writer.writeTextElement("album", song.album());
     }
-    if (song.length() != -1) {
-      writer.writeTextElement("duration", QString::number(song.length() * 1000));
+    if (song.length_nanosec() != -1) {
+      writer.writeTextElement("duration", QString::number(song.length_nanosec() / kNsecPerMsec));
     }
 
     QString art = song.art_manual().isEmpty() ? song.art_automatic() : song.art_manual();
