@@ -36,7 +36,7 @@ class XSPFParser : public XMLParser {
 
   bool TryMagic(const QByteArray &data) const;
 
-  SongList Load(QIODevice *device, const QDir &dir = QDir()) const;
+  SongList Load(QIODevice *device, const QString& playlist_path = "", const QDir &dir = QDir()) const;
   void Save(const SongList &songs, QIODevice *device, const QDir &dir = QDir()) const;
 
  private:
