@@ -39,8 +39,10 @@ public:
 
   void SetIcecastModel(IcecastModel* model);
 
+public slots:
+  void FocusOnFilter(QKeyEvent* e);
+
 private slots:
-  void ClearFilter();
   void SortModeChanged(int mode);
 
 private:
