@@ -67,11 +67,10 @@ class MagnatuneService : public RadioService {
 
   static QString ReadElementText(QXmlStreamReader& reader);
 
-  RadioItem* CreateRootItem(RadioItem* parent);
-  void LazyPopulate(RadioItem* item);
+  QStandardItem* CreateRootItem();
+  void LazyPopulate(QStandardItem* item);
 
-  void ShowContextMenu(RadioItem* item, const QModelIndex& index,
-                       const QPoint& global_pos);
+  void ShowContextMenu(const QModelIndex& index, const QPoint& global_pos);
 
   QWidget* HeaderWidget() const;
 
@@ -89,30 +88,28 @@ class MagnatuneService : public RadioService {
  signals:
   void DownloadFinished(const QStringList& albums);
 
+ protected:
+  QModelIndex GetCurrentIndex();
+
  private slots:
   void UpdateTotalSongCount(int count);
   void ReloadDatabase();
   void ReloadDatabaseFinished();
 
-  void AddToPlaylist();
-  void LoadToPlaylist();
   void Download();
   void Homepage();
   void ShowConfig();
 
  private:
   void EnsureMenuCreated();
-  void AddSelectedToPlaylist(bool clear_first);
 
   Song ReadTrack(QXmlStreamReader& reader);
 
  private:
-  RadioItem* root_;
   QMenu* context_menu_;
   QModelIndex context_item_;
+  QStandardItem* root_;
 
-  QAction* add_to_playlist_;
-  QAction* load_to_playlist_;
   QAction* download_;
 
   LibraryBackend* library_backend_;
