@@ -18,15 +18,22 @@
 #ifndef RADIOMIMEDATA_H
 #define RADIOMIMEDATA_H
 
+#include "core/mimedata.h"
+
 #include <QMimeData>
+#include <QModelIndexList>
 
-class RadioItem;
+class RadioModel;
 
-class RadioMimeData : public QMimeData {
+class RadioMimeData : public MimeData {
   Q_OBJECT
 
- public:
-  QList<RadioItem*> items;
+public:
+  RadioMimeData(const RadioModel* _model)
+    : model(_model) {}
+
+  const RadioModel* model;
+  QModelIndexList indexes;
 };
 
 #endif // RADIOMIMEDATA_H
