@@ -27,17 +27,18 @@
 
 RadioPlaylistItem::RadioPlaylistItem(const QString& type)
   : PlaylistItem(type),
-    service_(NULL)
+    set_service_icon_(false)
 {
 }
 
 RadioPlaylistItem::RadioPlaylistItem(RadioService* service, const QUrl& url,
                                      const QString& title, const QString& artist)
   : PlaylistItem("Radio"),
-    service_(service),
     url_(url),
     title_(title),
-    artist_(artist)
+    artist_(artist),
+    service_name_(service->name()),
+    set_service_icon_(false)
 {
   InitMetadata();
 }
@@ -49,59 +50,70 @@ bool RadioPlaylistItem::InitFromQuery(const SqlRow& query) {
   url_ = query.value(row + 1).toString();
   title_ = query.value(row + 2).toString();
   artist_ = query.value(row + 3).toString();
-  QString service(query.value(row + 6).toString());
-
-  service_ = RadioModel::ServiceByName(service);
+  service_name_ = query.value(row + 6).toString();
 
   InitMetadata();
   return true;
 }
 
+RadioService* RadioPlaylistItem::service() const {
+  RadioService* ret = RadioModel::ServiceByName(service_name_);
+
+  if (ret && !set_service_icon_) {
+    const_cast<RadioPlaylistItem*>(this)->set_service_icon_ = true;
+
+    QString icon = ret->Icon();
+    if (!icon.isEmpty()) {
+      const_cast<RadioPlaylistItem*>(this)->metadata_.set_art_manual(icon);
+    }
+  }
+
+  return ret;
+}
+
 QVariant RadioPlaylistItem::DatabaseValue(DatabaseColumn column) const {
   switch (column) {
     case Column_Url:          return url_.toString();
     case Column_Title:        return title_;
     case Column_Artist:       return artist_;
-    case Column_RadioService: return service_->name();
+    case Column_RadioService: return service_name_;
     default:                  return PlaylistItem::DatabaseValue(column);
   }
 }
 
 void RadioPlaylistItem::InitMetadata() {
-  if (!service_)
-    metadata_.set_title(QApplication::translate("RadioPlaylistItem", "Radio service couldn't be loaded :-("));
-  else if (!title_.isEmpty())
+  if (!title_.isEmpty())
     metadata_.set_title(title_);
   else
     metadata_.set_title(url_.toString());
 
   metadata_.set_artist(artist_);
   metadata_.set_filetype(Song::Type_Stream);
-
-  if (service_) {
-    QString icon = service_->Icon();
-    if (!icon.isEmpty()) {
-      metadata_.set_art_manual(icon);
-    }
-  }
 }
 
 Song RadioPlaylistItem::Metadata() const {
+  if (!set_service_icon_) {
+    // Get the icon if we don't have it already
+    service();
+  }
+
   if (HasTemporaryMetadata())
     return temp_metadata_;
   return metadata_;
 }
 
 PlaylistItem::SpecialLoadResult RadioPlaylistItem::StartLoading() {
-  if (!service_)
+  RadioService* s = service();
+  if (!s)
     return SpecialLoadResult();
-  return service_->StartLoading(url_);
+  return s->StartLoading(url_);
 }
 
 PlaylistItem::SpecialLoadResult RadioPlaylistItem::LoadNext() {
-  if (!service_)
+  RadioService* s = service();
+  if (!s)
     return SpecialLoadResult();
-  return service_->LoadNext(url_);
+  return s->LoadNext(url_);
 }
 
 QUrl RadioPlaylistItem::Url() const {
@@ -109,7 +121,8 @@ QUrl RadioPlaylistItem::Url() const {
 }
 
 PlaylistItem::Options RadioPlaylistItem::options() const {
-  if (!service_)
+  RadioService* s = service();
+  if (!s)
     return Default;
-  return service_->playlistitem_options();
+  return s->playlistitem_options();
 }
