@@ -47,12 +47,15 @@ class RadioPlaylistItem : public PlaylistItem {
 
  private:
   void InitMetadata();
+  RadioService* service() const;
 
  private:
-  RadioService* service_;
   QUrl url_;
   QString title_;
   QString artist_;
+  QString service_name_;
+
+  bool set_service_icon_;
 
   Song metadata_;
 };
