@@ -22,7 +22,6 @@
 #include <QList>
 #include <QUrl>
 
-#include "radioitem.h"
 #include "core/song.h"
 #include "playlist/playlistitem.h"
 #include "ui/settingsdialog.h"
@@ -34,23 +33,20 @@ class LibraryFilterWidget;
 class RadioService : public QObject {
   Q_OBJECT
 
- public:
+public:
+  // Constructs a new radio service with the given name and model. The name
+  // should be user-friendly (like 'DigitallyImported' or 'Last.fm').
   RadioService(const QString& name, RadioModel* model);
   virtual ~RadioService() {}
 
   QString name() const { return name_; }
   RadioModel* model() const { return model_; }
 
-  virtual RadioItem* CreateRootItem(RadioItem* parent) = 0;
-  virtual void LazyPopulate(RadioItem* item) = 0;
+  virtual QStandardItem* CreateRootItem() = 0;
+  virtual void LazyPopulate(QStandardItem* parent) = 0;
 
-  virtual QUrl UrlForItem(const RadioItem* item) const;
-  virtual QString TitleForItem(const RadioItem* item) const;
-  virtual QString ArtistForItem(const RadioItem* item) const;
-
-  virtual void ShowContextMenu(RadioItem* item, const QModelIndex& index,
-                               const QPoint& global_pos) {
-    Q_UNUSED(item); Q_UNUSED(index); Q_UNUSED(global_pos); }
+  virtual void ShowContextMenu(const QModelIndex& index, const QPoint& global_pos) {
+    Q_UNUSED(index); Q_UNUSED(global_pos); }
 
   virtual PlaylistItem::SpecialLoadResult StartLoading(const QUrl& url);
   virtual PlaylistItem::SpecialLoadResult LoadNext(const QUrl& url);
@@ -63,18 +59,59 @@ class RadioService : public QObject {
 
   virtual QString Icon() { return QString(); }
 
- signals:
+signals:
   void AsyncLoadFinished(const PlaylistItem::SpecialLoadResult& result);
   void StreamError(const QString& message);
   void StreamMetadataFound(const QUrl& original_url, const Song& song);
   void OpenSettingsAtPage(SettingsDialog::Page page);
 
-  void AddItemToPlaylist(RadioItem* item, bool clear_first);
-  void AddItemsToPlaylist(const PlaylistItemList& items, bool clear_first);
+  void AddToPlaylistSignal(QMimeData* data);
 
- private:
+private slots:
+  void AppendToPlaylist();
+  void ReplacePlaylist();
+  void OpenInNewPlaylist();
+
+protected:
+  // Subclass provides the currently selected QModelIndex on RadioService's
+  // request.
+  virtual QModelIndex GetCurrentIndex() = 0;
+
+  // Returns all the playlist insertion related QActions (see below).
+  QList<QAction*> GetPlaylistActions();
+
+  // Returns the 'append to playlist' QAction.
+  QAction* GetAppendToPlaylistAction();
+  // Returns the 'replace playlist' QAction.
+  QAction* GetReplacePlaylistAction();
+  // Returns the 'open in new playlist' QAction.
+  QAction* GetOpenInNewPlaylistAction();
+
+  // Describes how songs should be added to playlist.
+  enum AddMode {
+    // appends songs to the current playlist
+    AddMode_Append,
+    // clears the current playlist and then appends all songs to it
+    AddMode_Replace,
+    // creates a new, empty playlist and then adds all songs to it
+    AddMode_OpenInNew
+  };
+
+  // Adds the 'index' element to playlist using the 'add_mode' mode.
+  void AddItemToPlaylist(const QModelIndex& index, AddMode add_mode);
+  // Adds the 'indexes' elements to playlist using the 'add_mode' mode.
+  void AddItemsToPlaylist(const QModelIndexList& indexes, AddMode add_mode);
+
+private:
   RadioModel* model_;
   QString name_;
+
+  QAction* append_to_playlist_;
+  QAction* replace_playlist_;
+  QAction* open_in_new_playlist_;
+  QAction* separator_;
 };
 
+Q_DECLARE_METATYPE(RadioService*);
+
 #endif // RADIOSERVICE_H
