@@ -21,8 +21,9 @@
 #include "ui_radioviewcontainer.h"
 #include "core/mergedproxymodel.h"
 
-#include <QtDebug>
+#include <QMetaMethod>
 #include <QTimeLine>
+#include <QtDebug>
 
 const int RadioViewContainer::kAnimationDuration = 500;
 
@@ -37,6 +38,7 @@ RadioViewContainer::RadioViewContainer(QWidget *parent)
 
   connect(ui_->tree, SIGNAL(collapsed(QModelIndex)), SLOT(Collapsed(QModelIndex)));
   connect(ui_->tree, SIGNAL(expanded(QModelIndex)), SLOT(Expanded(QModelIndex)));
+  connect(ui_->tree, SIGNAL(FocusOnFilterSignal(QKeyEvent*)), SLOT(FocusOnFilter(QKeyEvent*)));
 }
 
 RadioViewContainer::~RadioViewContainer() {
@@ -58,12 +60,7 @@ void RadioViewContainer::SetModel(RadioModel* model) {
 }
 
 void RadioViewContainer::ServiceChanged(const QModelIndex& index) {
-  RadioItem* item = model_->IndexToItem(
-      model_->merged_model()->FindSourceParent(index));
-  if (!item)
-    return;
-
-  RadioService* service = item->service;
+  RadioService* service = index.data(RadioModel::Role_Service).value<RadioService*>();
   if (!service || service == current_service_)
     return;
   current_service_ = service;
@@ -118,6 +115,19 @@ void RadioViewContainer::SetHeaderVisible(QWidget* header, bool visible) {
   d.animation_->start();
 }
 
+void RadioViewContainer::FocusOnFilter(QKeyEvent* event) {
+  // Beware: magic
+
+  if (current_header_) {
+    int slot = current_header_->metaObject()->indexOfSlot(
+          QMetaObject::normalizedSignature("FocusOnFilter(QKeyEvent*)"));
+    if (slot != -1) {
+      current_header_->metaObject()->method(slot).invoke(
+            current_header_, Q_ARG(QKeyEvent*, event));
+    }
+  }
+}
+
 void RadioViewContainer::SetHeaderHeight(int height) {
   QTimeLine* animation = qobject_cast<QTimeLine*>(sender());
   QWidget* header = NULL;
