@@ -19,6 +19,7 @@
 #include "ui_about.h"
 
 #include <QCoreApplication>
+#include <QPushButton>
 
 const char* About::kUrl = "http://www.clementine-player.org/";
 
@@ -37,7 +38,9 @@ About::About(QWidget *parent)
   ui_.title->setFont(title_font);
 
   authors_ << Person("David Sansome", "me@davidsansome.com")
-           << Person("John Maguire", "john.maguire@gmail.com");
+           << Person("John Maguire", "john.maguire@gmail.com")
+           << Person(QString::fromUtf8("Paweł Bara"), "keirangtp@gmail.com")
+           << Person("Arnaud Bienner", "arnaud.bienner@gmail.com");
   thanks_to_ << Person("Mark Kretschmann", "kretschmann@kde.org")
              << Person("Max Howell", "max.howell@methylblue.com")
              << Person(QString::fromUtf8("Bartłomiej Burdukiewicz"), "dev.strikeu@gmail.com")
@@ -51,6 +54,8 @@ About::About(QWidget *parent)
   qSort(thanks_to_);
 
   ui_.content->setHtml(MakeHtml());
+
+  ui_.buttonBox->button(QDialogButtonBox::Close)->setShortcut(QKeySequence::Close);
 }
 
 QString About::MakeHtml() const {
@@ -64,6 +69,8 @@ QString About::MakeHtml() const {
 
   foreach (const Person& person, thanks_to_)
     ret += "<br />" + MakeHtml(person);
+  ret += QString("<br />" + tr("All the translators") + " &lt;<a href=\"https://translations.launchpad.net/clementine\">"
+                 "https://translations.launchpad.net/clementine</a>&gt;");
 
   ret += QString("<br />%1</p>").arg(tr("...and all the Amarok contributors"));
   ret += QString("<p><b>%1</b>").arg(tr("And:"));
