@@ -23,21 +23,24 @@
 #include "library/librarybackend.h"
 #include "library/libraryquery.h"
 #include "library/sqlrow.h"
+#include "playlist/songmimedata.h"
+#include "widgets/maclineedit.h"
+#include "ui/albumcoverchoicecontroller.h"
 
-#include <QSettings>
-#include <QPainter>
-#include <QMenu>
 #include <QActionGroup>
-#include <QListWidget>
-#include <QCryptographicHash>
-#include <QDir>
-#include <QEvent>
-#include <QScrollBar>
 #include <QContextMenuEvent>
-#include <QLabel>
+#include <QEvent>
 #include <QFileDialog>
+#include <QKeySequence>
+#include <QLabel>
+#include <QListWidget>
+#include <QMenu>
 #include <QMessageBox>
+#include <QPainter>
 #include <QProgressBar>
+#include <QScrollBar>
+#include <QSettings>
+#include <QShortcut>
 #include <QTimer>
 
 const char* AlbumCoverManager::kSettingsGroup = "CoverManager";
@@ -47,10 +50,11 @@ AlbumCoverManager::AlbumCoverManager(LibraryBackend* backend, QWidget* parent,
   : QMainWindow(parent),
     constructed_(false),
     ui_(new Ui_CoverManager),
+    album_cover_choice_controller_(new AlbumCoverChoiceController(this)),
     backend_(backend),
     cover_loader_(new BackgroundThreadImplementation<AlbumCoverLoader, AlbumCoverLoader>(this)),
     cover_fetcher_(new AlbumCoverFetcher(this, network)),
-    cover_searcher_(new AlbumCoverSearcher(this)),
+    cover_searcher_(NULL),
     artist_icon_(IconLoader::Load("x-clementine-artist")),
     all_artists_icon_(IconLoader::Load("x-clementine-album")),
     context_menu_(new QMenu(this)),
@@ -64,16 +68,13 @@ AlbumCoverManager::AlbumCoverManager(LibraryBackend* backend, QWidget* parent,
 
   // Icons
   ui_->action_fetch->setIcon(IconLoader::Load("download"));
-  ui_->action_choose_manual->setIcon(IconLoader::Load("document-open"));
-  ui_->action_show_fullsize->setIcon(IconLoader::Load("zoom-in"));
-  ui_->action_unset_cover->setIcon(IconLoader::Load("list-remove"));
-  ui_->clear->setIcon(IconLoader::Load("edit-clear-locationbar-ltr"));
   ui_->view->setIcon(IconLoader::Load("view-choose"));
   ui_->fetch->setIcon(IconLoader::Load("download"));
-  ui_->action_search_manual->setIcon(IconLoader::Load("download"));
   ui_->action_add_to_playlist->setIcon(IconLoader::Load("media-playback-start"));
   ui_->action_load->setIcon(IconLoader::Load("media-playback-start"));
 
+  album_cover_choice_controller_->SetLibrary(backend_);
+
   // Get a square version of nocover.png
   QImage nocover(":/nocover.png");
   nocover = nocover.scaled(120, 120, Qt::KeepAspectRatio, Qt::SmoothTransformation);
@@ -85,9 +86,17 @@ AlbumCoverManager::AlbumCoverManager(LibraryBackend* backend, QWidget* parent,
   p.end();
   no_cover_icon_ = QPixmap::fromImage(square_nocover);
 
+  cover_searcher_ = new AlbumCoverSearcher(no_cover_icon_, this);
+
   // Set up the status bar
   statusBar()->addPermanentWidget(progress_bar_);
   progress_bar_->hide();
+
+  ui_->albums->setAttribute(Qt::WA_MacShowFocusRect, false);
+  ui_->artists->setAttribute(Qt::WA_MacShowFocusRect, false);
+
+  QShortcut* close = new QShortcut(QKeySequence::Close, this);
+  connect(close, SIGNAL(activated()), SLOT(close()));
 }
 
 AlbumCoverManager::~AlbumCoverManager() {
@@ -114,14 +123,27 @@ void AlbumCoverManager::Init() {
   ui_->view->setMenu(view_menu);
 
   // Context menu
-  context_menu_->addAction(ui_->action_show_fullsize);
-  context_menu_->addAction(ui_->action_search_manual);
-  context_menu_->addAction(ui_->action_choose_manual);
-  context_menu_->addSeparator();
-  context_menu_->addAction(ui_->action_unset_cover);
+
+  QList<QAction*> actions = album_cover_choice_controller_->GetAllActions();
+
+  connect(album_cover_choice_controller_->cover_from_file_action(),
+          SIGNAL(triggered()), this, SLOT(LoadCoverFromFile()));
+  connect(album_cover_choice_controller_->cover_to_file_action(),
+          SIGNAL(triggered()), this, SLOT(SaveCoverToFile()));
+  connect(album_cover_choice_controller_->cover_from_url_action(),
+          SIGNAL(triggered()), this, SLOT(LoadCoverFromURL()));
+  connect(album_cover_choice_controller_->search_for_cover_action(),
+          SIGNAL(triggered()), this, SLOT(SearchForCover()));
+  connect(album_cover_choice_controller_->unset_cover_action(),
+          SIGNAL(triggered()), this, SLOT(UnsetCover()));
+  connect(album_cover_choice_controller_->show_cover_action(),
+          SIGNAL(triggered()), this, SLOT(ShowCover()));
+
+  context_menu_->addActions(actions);
   context_menu_->addSeparator();
   context_menu_->addAction(ui_->action_load);
   context_menu_->addAction(ui_->action_add_to_playlist);
+
   ui_->albums->installEventFilter(this);
 
   // Connections
@@ -133,14 +155,22 @@ void AlbumCoverManager::Init() {
   connect(ui_->fetch, SIGNAL(clicked()), SLOT(FetchAlbumCovers()));
   connect(cover_fetcher_, SIGNAL(AlbumCoverFetched(quint64,QImage)),
           SLOT(AlbumCoverFetched(quint64,QImage)));
-  connect(ui_->action_show_fullsize, SIGNAL(triggered()), SLOT(ShowFullsize()));
   connect(ui_->action_fetch, SIGNAL(triggered()), SLOT(FetchSingleCover()));
-  connect(ui_->action_choose_manual, SIGNAL(triggered()), SLOT(ChooseManualCover()));
-  connect(ui_->action_unset_cover, SIGNAL(triggered()), SLOT(UnsetCover()));
   connect(ui_->albums, SIGNAL(doubleClicked(QModelIndex)), SLOT(AlbumDoubleClicked(QModelIndex)));
   connect(ui_->action_add_to_playlist, SIGNAL(triggered()), SLOT(AddSelectedToPlaylist()));
   connect(ui_->action_load, SIGNAL(triggered()), SLOT(LoadSelectedToPlaylist()));
-  connect(ui_->action_search_manual, SIGNAL(triggered()), SLOT(SearchManual()));
+
+  #ifdef Q_OS_DARWIN
+    MacLineEdit* lineedit = new MacLineEdit(ui_->filter->parentWidget());
+    lineedit->set_hint(ui_->filter->hint());
+    delete ui_->filter;
+    ui_->horizontalLayout->insertWidget(0, lineedit);
+    filter_ = lineedit;
+    connect(lineedit, SIGNAL(textChanged(QString)), SLOT(UpdateFilter()));
+    lineedit->show();
+  #else
+    filter_ = ui_->filter;
+  #endif
 
   // Restore settings
   QSettings s;
@@ -154,6 +184,7 @@ void AlbumCoverManager::Init() {
 
   cover_loader_->Start(true);
   CoverLoaderInitialised();
+  cover_searcher_->Init(cover_fetcher_);
   constructed_ = true;
 }
 
@@ -161,8 +192,6 @@ void AlbumCoverManager::CoverLoaderInitialised() {
   cover_loader_->Worker()->SetDefaultOutputImage(QImage(":nocover.png"));
   connect(cover_loader_->Worker().get(), SIGNAL(ImageLoaded(quint64,QImage)),
           SLOT(CoverImageLoaded(quint64,QImage)));
-
-  cover_searcher_->Init(cover_loader_->Worker(), cover_fetcher_);
 }
 
 void AlbumCoverManager::showEvent(QShowEvent *) {
@@ -258,6 +287,7 @@ void AlbumCoverManager::ArtistChanged(QListWidgetItem* current) {
     item->setData(Role_FirstFilename, info.first_filename);
     item->setData(Qt::TextAlignmentRole, QVariant(Qt::AlignTop | Qt::AlignHCenter));
     item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled);
+    item->setToolTip(info.artist + " - " + info.album_name);
 
     if (!info.art_automatic.isEmpty() || !info.art_manual.isEmpty()) {
       quint64 id = cover_loader_->Worker()->LoadImageAsync(
@@ -285,7 +315,7 @@ void AlbumCoverManager::CoverImageLoaded(quint64 id, const QImage &image) {
 }
 
 void AlbumCoverManager::UpdateFilter() {
-  const QString filter = ui_->filter->text().toLower();
+  const QString filter = filter_->text().toLower();
   const bool hide_with_covers = filter_without_covers_->isChecked();
   const bool hide_without_covers = filter_with_covers_->isChecked();
 
@@ -317,7 +347,8 @@ bool AlbumCoverManager::ShouldHide(
 
   QStringList query = filter.split(' ');
   foreach (const QString& s, query) {
-    if (!item.text().contains(s, Qt::CaseInsensitive)) {
+    if (!item.text().contains(s, Qt::CaseInsensitive)
+        && !item.data(Role_ArtistName).toString().contains(s, Qt::CaseInsensitive)) {
       return true;
     }
   }
@@ -366,34 +397,6 @@ void AlbumCoverManager::AlbumCoverFetched(quint64 id, const QImage &image) {
   UpdateStatusText();
 }
 
-void AlbumCoverManager::SaveAndSetCover(QListWidgetItem *item, const QImage &image) {
-  const QString artist = item->data(Role_ArtistName).toString();
-  const QString album = item->data(Role_AlbumName).toString();
-
-  // Hash the artist and album into a filename for the image
-  QCryptographicHash hash(QCryptographicHash::Sha1);
-  hash.addData(artist.toLower().toUtf8().constData());
-  hash.addData(album.toLower().toUtf8().constData());
-
-  QString filename = hash.result().toHex() + ".jpg";
-  QString path = AlbumCoverLoader::ImageCacheDir() + "/" + filename;
-
-  // Make sure this directory exists first
-  QDir dir;
-  dir.mkdir(AlbumCoverLoader::ImageCacheDir());
-
-  // Save the image to disk
-  image.save(path, "JPG");
-
-  // Save the image in the database
-  backend_->UpdateManualAlbumArtAsync(artist, album, path);
-
-  // Update the icon in our list
-  quint64 id = cover_loader_->Worker()->LoadImageAsync(QString(), path);
-  item->setData(Role_PathManual, path);
-  cover_loading_tasks_[id] = item;
-}
-
 void AlbumCoverManager::UpdateStatusText() {
   QString message = tr("Got %1 covers out of %2 (%3 failed)")
                     .arg(got_covers_).arg(jobs_).arg(missing_covers_);
@@ -437,9 +440,10 @@ bool AlbumCoverManager::eventFilter(QObject *obj, QEvent *event) {
         some_with_covers = true;
     }
 
-    ui_->action_show_fullsize->setEnabled(some_with_covers && context_menu_items_.size() == 1);
-    ui_->action_choose_manual->setEnabled(context_menu_items_.size() == 1);
-    ui_->action_unset_cover->setEnabled(some_with_covers);
+    album_cover_choice_controller_->cover_from_file_action()->setEnabled(context_menu_items_.size() == 1);
+    album_cover_choice_controller_->cover_from_url_action()->setEnabled(context_menu_items_.size() == 1);
+    album_cover_choice_controller_->show_cover_action()->setEnabled(some_with_covers && context_menu_items_.size() == 1);
+    album_cover_choice_controller_->unset_cover_action()->setEnabled(some_with_covers);
 
     QContextMenuEvent* e = static_cast<QContextMenuEvent*>(event);
     context_menu_->popup(e->globalPos());
@@ -448,26 +452,48 @@ bool AlbumCoverManager::eventFilter(QObject *obj, QEvent *event) {
   return QMainWindow::eventFilter(obj, event);
 }
 
-void AlbumCoverManager::ShowFullsize() {
-  if (context_menu_items_.size() != 1)
-    return;
-  QListWidgetItem* item = context_menu_items_[0];
+Song AlbumCoverManager::GetSingleSelectionAsSong() {
+  return context_menu_items_.size() != 1
+             ? Song()
+             : ItemAsSong(context_menu_items_[0]);
+}
+
+Song AlbumCoverManager::GetFirstSelectedAsSong() {
+  return context_menu_items_.isEmpty()
+             ? Song()
+             : ItemAsSong(context_menu_items_[0]);
+}
+
+Song AlbumCoverManager::ItemAsSong(QListWidgetItem* item) {
+  Song result;
 
   QString title = item->data(Role_AlbumName).toString();
   if (!item->data(Role_ArtistName).toString().isNull())
-    title = item->data(Role_ArtistName).toString() + " - " + title;
+    result.set_title(item->data(Role_ArtistName).toString() + " - " + title);
+  else
+    result.set_title(title);
 
-  QDialog* dialog = new QDialog(this);
-  dialog->setAttribute(Qt::WA_DeleteOnClose, true);
-  dialog->setWindowTitle(title);
+  result.set_artist(item->data(Role_ArtistName).toString());
+  result.set_album(item->data(Role_AlbumName).toString());
 
-  QLabel* label = new QLabel(dialog);
-  label->setPixmap(AlbumCoverLoader::TryLoadPixmap(
-      item->data(Role_PathAutomatic).toString(),
-      item->data(Role_PathManual).toString()));
+  result.set_filename(item->data(Role_FirstFilename).toString());
 
-  dialog->resize(label->pixmap()->size());
-  dialog->show();
+  result.set_art_automatic(item->data(Role_PathAutomatic).toString());
+  result.set_art_manual(item->data(Role_PathManual).toString());
+
+  // force validity
+  result.set_valid(true);
+  result.set_id(0);
+
+  return result;
+}
+
+void AlbumCoverManager::ShowCover() {
+  Song song = GetSingleSelectionAsSong();
+  if(!song.is_valid())
+    return;
+
+  album_cover_choice_controller_->ShowCover(song);
 }
 
 void AlbumCoverManager::FetchSingleCover() {
@@ -483,41 +509,106 @@ void AlbumCoverManager::FetchSingleCover() {
   UpdateStatusText();
 }
 
-void AlbumCoverManager::ChooseManualCover() {
-  if (context_menu_items_.size() != 1)
-    return;
-  QListWidgetItem* item = context_menu_items_[0];
 
-  QString cover = QFileDialog::getOpenFileName(
-      this, tr("Choose manual cover"), item->data(Role_PathAutomatic).toString(),
-      tr("Images (*.png *.jpg *.jpeg *.bmp *.gif *.xpm *.pbm *.pgm *.ppm *.xbm *.tiff)") + ";;" +
-      tr("All files (*)"));
-  if (cover.isNull())
-    return;
-
-  // Can we load the image?
-  QImage image(cover);
-  if (image.isNull())
-    return;
-
-  // Update database
-  backend_->UpdateManualAlbumArtAsync(item->data(Role_ArtistName).toString(),
-                                      item->data(Role_AlbumName).toString(),
-                                      cover);
-
-  // Update the icon in our list
+void AlbumCoverManager::UpdateCoverInList(QListWidgetItem* item, const QString& cover) {
   quint64 id = cover_loader_->Worker()->LoadImageAsync(QString(), cover);
   item->setData(Role_PathManual, cover);
   cover_loading_tasks_[id] = item;
 }
 
+void AlbumCoverManager::LoadCoverFromFile() {
+  Song song = GetSingleSelectionAsSong();
+  if(!song.is_valid())
+    return;
+
+  QListWidgetItem* item = context_menu_items_[0];
+
+  QString cover = album_cover_choice_controller_->LoadCoverFromFile(&song);
+
+  if (!cover.isEmpty()) {
+    UpdateCoverInList(item, cover);
+  }
+}
+
+void AlbumCoverManager::SaveCoverToFile() {
+  Song song = GetSingleSelectionAsSong();
+  if(!song.is_valid())
+    return;
+
+  QImage image;
+
+  // load the image from disk
+  if(song.has_manually_unset_cover()) {
+    image = QImage(":/nocover.png");
+  } else {
+    if(!song.art_manual().isEmpty() && QFile::exists(song.art_manual())) {
+      image = QImage(song.art_manual());
+    } else if(!song.art_automatic().isEmpty() && QFile::exists(song.art_automatic())) {
+      image = QImage(song.art_automatic());
+    } else {
+      image = QImage(":/nocover.png");
+    }
+  }
+
+  album_cover_choice_controller_->SaveCoverToFile(song, image);
+}
+
+void AlbumCoverManager::LoadCoverFromURL() {
+  Song song = GetSingleSelectionAsSong();
+  if(!song.is_valid())
+    return;
+
+  QListWidgetItem* item = context_menu_items_[0];
+
+  QString cover = album_cover_choice_controller_->LoadCoverFromURL(&song);
+
+  if (!cover.isEmpty()) {
+    UpdateCoverInList(item, cover);
+  }
+}
+
+void AlbumCoverManager::SearchForCover() {
+  Song song = GetFirstSelectedAsSong();
+  if(!song.is_valid())
+    return;
+
+  QListWidgetItem* item = context_menu_items_[0];
+
+  QString cover = album_cover_choice_controller_->SearchForCover(&song);
+  if (cover.isEmpty())
+    return;
+
+  // force the found cover on all of the selected items
+  foreach (QListWidgetItem* current, context_menu_items_) {
+    // don't save the first one twice
+    if(current != item) {
+      Song current_song = ItemAsSong(current);
+      album_cover_choice_controller_->SaveCover(&current_song, cover);
+    }
+
+    UpdateCoverInList(current, cover);
+  }
+}
+
 void AlbumCoverManager::UnsetCover() {
-  foreach (QListWidgetItem* item, context_menu_items_) {
-    item->setIcon(no_cover_icon_);
-    item->setData(Role_PathManual, AlbumCoverLoader::kManuallyUnsetCover);
-    backend_->UpdateManualAlbumArtAsync(item->data(Role_ArtistName).toString(),
-                                        item->data(Role_AlbumName).toString(),
-                                        AlbumCoverLoader::kManuallyUnsetCover);
+  Song song = GetFirstSelectedAsSong();
+  if(!song.is_valid())
+    return;
+
+  QListWidgetItem* item = context_menu_items_[0];
+
+  QString cover = album_cover_choice_controller_->UnsetCover(&song);
+
+  // force the 'none' cover on all of the selected items
+  foreach (QListWidgetItem* current, context_menu_items_) {
+    current->setIcon(no_cover_icon_);
+    current->setData(Role_PathManual, cover);
+
+    // don't save the first one twice
+    if(current != item) {
+      Song current_song = ItemAsSong(current);
+      album_cover_choice_controller_->SaveCover(&current_song, cover);
+    }
   }
 }
 
@@ -553,35 +644,48 @@ SongList AlbumCoverManager::GetSongsInAlbums(const QModelIndexList& indexes) con
   return ret;
 }
 
+SongMimeData* AlbumCoverManager::GetMimeDataForAlbums(const QModelIndexList& indexes) const {
+  SongList songs = GetSongsInAlbums(indexes);
+  if (songs.isEmpty())
+    return NULL;
+
+  SongMimeData* data = new SongMimeData;
+  data->backend = backend_;
+  data->songs = songs;
+  return data;
+}
+
 void AlbumCoverManager::AlbumDoubleClicked(const QModelIndex &index) {
-  emit SongsDoubleClicked(GetSongsInAlbum(index));
+  SongMimeData* data = GetMimeDataForAlbums(QModelIndexList() << index);
+  if (data) {
+    data->from_doubleclick_ = true;
+    emit AddToPlaylist(data);
+  }
 }
 
 void AlbumCoverManager::AddSelectedToPlaylist() {
-  emit AddSongsToPlaylist(GetSongsInAlbums(
-      ui_->albums->selectionModel()->selectedIndexes()));
+  emit AddToPlaylist(GetMimeDataForAlbums(ui_->albums->selectionModel()->selectedIndexes()));
 }
 
 void AlbumCoverManager::LoadSelectedToPlaylist() {
-  emit LoadSongsToPlaylist(GetSongsInAlbums(
-      ui_->albums->selectionModel()->selectedIndexes()));
-}
-
-void AlbumCoverManager::SearchManual() {
-  if (context_menu_items_.isEmpty())
-    return;
-
-  // Get something sensible to stick in the search box
-  QString query = context_menu_items_[0]->data(Role_ArtistName).toString();
-  if (!query.isEmpty())
-    query += " ";
-  query += context_menu_items_[0]->data(Role_AlbumName).toString();
-
-  QImage image = cover_searcher_->Exec(query);
-  if (image.isNull())
-    return;
-
-  foreach (QListWidgetItem* item, context_menu_items_) {
-    SaveAndSetCover(item, image);
+  SongMimeData* data = GetMimeDataForAlbums(ui_->albums->selectionModel()->selectedIndexes());
+  if (data) {
+    data->clear_first_ = true;
+    emit AddToPlaylist(data);
   }
 }
+
+void AlbumCoverManager::SaveAndSetCover(QListWidgetItem *item, const QImage &image) {
+  const QString artist = item->data(Role_ArtistName).toString();
+  const QString album = item->data(Role_AlbumName).toString();
+
+  QString path = album_cover_choice_controller_->SaveCoverInCache(artist, album, image);
+
+  // Save the image in the database
+  backend_->UpdateManualAlbumArtAsync(artist, album, path);
+
+  // Update the icon in our list
+  quint64 id = cover_loader_->Worker()->LoadImageAsync(QString(), path);
+  item->setData(Role_PathManual, path);
+  cover_loading_tasks_[id] = item;
+}
