@@ -28,9 +28,12 @@
 #include "core/backgroundthread.h"
 #include "core/song.h"
 
-class LibraryBackend;
+class AlbumCoverChoiceController;
 class AlbumCoverFetcher;
 class AlbumCoverSearcher;
+class LibraryBackend;
+class LineEditInterface;
+class SongMimeData;
 class Ui_CoverManager;
 
 class QListWidgetItem;
@@ -55,11 +58,10 @@ class AlbumCoverManager : public QMainWindow {
 
   SongList GetSongsInAlbum(const QModelIndex& index) const;
   SongList GetSongsInAlbums(const QModelIndexList& indexes) const;
+  SongMimeData* GetMimeDataForAlbums(const QModelIndexList& indexes) const;
 
  signals:
-  void AddSongsToPlaylist(const SongList& songs);
-  void LoadSongsToPlaylist(const SongList& songs);
-  void SongsDoubleClicked(const SongList& songs);
+  void AddToPlaylist(QMimeData* data);
 
  protected:
   void showEvent(QShowEvent *);
@@ -78,17 +80,22 @@ class AlbumCoverManager : public QMainWindow {
   void AlbumCoverFetched(quint64 id, const QImage& image);
 
   // On the context menu
-  void ShowFullsize();
   void FetchSingleCover();
-  void SearchManual();
-  void ChooseManualCover();
+
+  void LoadCoverFromFile();
+  void SaveCoverToFile();
+  void LoadCoverFromURL();
+  void SearchForCover();
   void UnsetCover();
+  void ShowCover();
 
   // For adding albums to the playlist
   void AlbumDoubleClicked(const QModelIndex& index);
   void AddSelectedToPlaylist();
   void LoadSelectedToPlaylist();
 
+  void UpdateCoverInList(QListWidgetItem* item, const QString& cover);
+
  private:
   enum ArtistItemType {
     All_Artists,
@@ -110,6 +117,19 @@ class AlbumCoverManager : public QMainWindow {
     Hide_WithoutCovers,
   };
 
+  QString InitialPathForOpenCoverDialog(const QString& path_automatic, const QString& first_file_name) const;
+
+  // Returns the selected element in form of a Song ready to be used 
+  // by AlbumCoverChoiceController or invalid song if there's nothing
+  // or multiple elements selected.
+  Song GetSingleSelectionAsSong();
+  // Returns the first of the selected elements in form of a Song ready
+  // to be used by AlbumCoverChoiceController or invalid song if there's nothing
+  // selected.
+  Song GetFirstSelectedAsSong();
+
+  Song ItemAsSong(QListWidgetItem* item);
+
   void CancelRequests();
 
   void UpdateStatusText();
@@ -120,6 +140,8 @@ class AlbumCoverManager : public QMainWindow {
   bool constructed_;
 
   Ui_CoverManager* ui_;
+
+  AlbumCoverChoiceController* album_cover_choice_controller_;
   LibraryBackend* backend_;
 
   QAction* filter_all_;
@@ -146,6 +168,8 @@ class AlbumCoverManager : public QMainWindow {
   int got_covers_;
   int missing_covers_;
 
+  LineEditInterface* filter_;
+
   FRIEND_TEST(AlbumCoverManagerTest, HidesItemsWithCover);
   FRIEND_TEST(AlbumCoverManagerTest, HidesItemsWithoutCover);
   FRIEND_TEST(AlbumCoverManagerTest, HidesItemsWithFilter);
