@@ -19,23 +19,25 @@
 #define ALBUMCOVERSEARCHER_H
 
 #include "core/albumcoverfetcher.h"
+#include "core/backgroundthread.h"
 
 #include <QDialog>
+#include <QIcon>
 
 #include <boost/shared_ptr.hpp>
 
 class AlbumCoverLoader;
-class AlbumCoverManager;
 class Ui_AlbumCoverSearcher;
 
 class QListWidgetItem;
 class QModelIndex;
 
+// This is a dialog that lets the user search for album covers
 class AlbumCoverSearcher : public QDialog {
   Q_OBJECT
 
 public:
-  AlbumCoverSearcher(AlbumCoverManager* parent);
+  AlbumCoverSearcher(const QIcon& no_cover_icon, QWidget* parent);
   ~AlbumCoverSearcher();
 
   enum Role {
@@ -43,8 +45,7 @@ public:
     Role_ImageRequestId,
   };
 
-  void Init(boost::shared_ptr<AlbumCoverLoader> loader,
-            AlbumCoverFetcher* fetcher);
+  void Init(AlbumCoverFetcher* fetcher);
 
   QImage Exec(const QString& query);
 
@@ -61,8 +62,8 @@ private slots:
 private:
   Ui_AlbumCoverSearcher* ui_;
 
-  AlbumCoverManager* manager_;
-  boost::shared_ptr<AlbumCoverLoader> loader_;
+  QIcon no_cover_icon_;
+  BackgroundThread<AlbumCoverLoader>* loader_;
   AlbumCoverFetcher* fetcher_;
 
   quint64 id_;
