@@ -33,8 +33,8 @@
 
 class About;
 class AddStreamDialog;
-class AlbumCoverManager;
 class ArtistInfoView;
+class ArtLoader;
 class BackgroundStreams;
 class CommandlineOptions;
 class Database;
@@ -48,6 +48,7 @@ class GlobalShortcuts;
 class GroupByDialog;
 class Library;
 class LibraryViewContainer;
+class MimeData;
 class MultiLoadingIndicator;
 class OrganiseDialog;
 class OSD;
@@ -58,23 +59,41 @@ class QueueManager;
 class RadioItem;
 class RadioModel;
 class RadioViewContainer;
+class Remote;
+class ScriptDialog;
+class ScriptManager;
 class Song;
 class SongInfoBase;
 class SongInfoView;
 class SystemTrayIcon;
+class TagFetcher;
 class TaskManager;
+class TrackSelectionDialog;
 class TranscodeDialog;
 class VisualisationContainer;
 class WiimotedevShortcuts;
+class Windows7ThumbBar;
 class Ui_MainWindow;
 
 class QSortFilterProxyModel;
 
+#ifdef HAVE_LIBLASTFM
+  class AlbumCoverManager;
+#endif
+
 class MainWindow : public QMainWindow, public PlatformInterface {
   Q_OBJECT
 
  public:
-  MainWindow(Engine::Type engine, QWidget *parent = 0);
+  MainWindow(BackgroundThread<Database>* database,
+             TaskManager* task_manager,
+             PlaylistManager* playlists,
+             RadioModel* radio_model,
+             Player* player,
+             SystemTrayIcon* tray_icon,
+             OSD* osd,
+             ArtLoader* art_loader,
+             QWidget *parent = 0);
   ~MainWindow();
 
   static const char* kSettingsGroup;
@@ -88,6 +107,21 @@ class MainWindow : public QMainWindow, public PlatformInterface {
     Startup_AlwaysHide = 3,
   };
 
+  // Don't change the values
+  enum AddBehaviour {
+    AddBehaviour_Append = 1,
+    AddBehaviour_Enqueue = 2,
+    AddBehaviour_Load = 3,
+    AddBehaviour_OpenInNew = 4
+  };
+
+  // Don't change the values
+  enum PlayBehaviour {
+    PlayBehaviour_Never = 1,
+    PlayBehaviour_IfStopped = 2,
+    PlayBehaviour_Always = 3,
+  };
+
   void SetHiddenInTray(bool hidden);
   void CommandlineOptionsReceived(const CommandlineOptions& options);
 
@@ -95,6 +129,10 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void resizeEvent(QResizeEvent* event);
   void closeEvent(QCloseEvent* event);
 
+#ifdef Q_OS_WIN32
+  bool winEvent(MSG* message, long* result);
+#endif
+
   // PlatformInterface
   void Activate();
   bool LoadUrl(const QString& url);
@@ -116,11 +154,13 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void PlaylistQueue();
   void PlaylistRemoveCurrent();
   void PlaylistEditFinished(const QModelIndex& index);
-  void PlaylistSongRated(const QModelIndex& index, double rating);
   void EditTracks();
+  void EditTagDialogAccepted();
   void RenumberTracks();
   void SelectionSetValue();
   void EditValue();
+  void AutoCompleteTags();
+  void AutoCompleteTagsAccepted();
   void PlaylistUndoRedoChanged(QAction* undo, QAction* redo);
 
   void PlaylistCopyToLibrary();
@@ -128,6 +168,9 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void PlaylistCopyToDevice();
   void PlaylistOrganiseSelected(bool copy);
   void PlaylistDelete();
+  void PlaylistOpenInBrowser();
+
+  void ChangeLibraryQueryMode(QAction* action);
 
   void PlayIndex(const QModelIndex& index);
   void StopAfterCurrent();
@@ -135,42 +178,29 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void SongChanged(const Song& song);
   void VolumeChanged(int volume);
 
-  void LoadLibraryItemToPlaylist(const QModelIndexList& indexes);
-  void AddLibraryItemToPlaylist(const QModelIndexList& indexes);
-  void LibraryItemDoubleClicked(const QModelIndex& index);
-
-  void LoadFilesToPlaylist(const QList<QUrl>& urls);
-  void AddFilesToPlaylist(const QList<QUrl>& urls);
-  void FilesDoubleClicked(const QList<QUrl>& urls);
-
   void CopyFilesToLibrary(const QList<QUrl>& urls);
   void MoveFilesToLibrary(const QList<QUrl>& urls);
   void CopyFilesToDevice(const QList<QUrl>& urls);
 
-  void LoadLibrarySongsToPlaylist(const SongList& songs);
-  void AddLibrarySongsToPlaylist(const SongList& songs);
-  void LibrarySongsDoubleClicked(const SongList& songs);
-
-  void LoadDeviceSongsToPlaylist(const SongList& songs);
-  void AddDeviceSongsToPlaylist(const SongList& songs);
-  void DeviceSongsDoubleClicked(const SongList& songs);
+  void AddToPlaylist(QMimeData* data);
+  void AddToPlaylist(QAction* action);
 
   void VolumeWheelEvent(int delta);
   void ToggleShowHide();
 
   void UpdateTrackPosition();
 
-  void RadioDoubleClick(const QModelIndex& index);
-  void InsertRadioItem(RadioItem*, bool clear_first);
-  void InsertRadioItems(const PlaylistItemList& items, bool clear_first);
-  void ScrobblingEnabledChanged(bool value);
   void LastFMButtonVisibilityChanged(bool value);
+#ifdef HAVE_LIBLASTFM
+  void ScrobblingEnabledChanged(bool value);
   void Love();
+#endif
 
   void TaskCountChanged(int count);
 
   void ShowLibraryConfig();
   void ReloadSettings();
+  void ReloadAllSettings();
   void SetHiddenInTray() { SetHiddenInTray(true); }
 
   void AddFile();
@@ -186,7 +216,9 @@ class MainWindow : public QMainWindow, public PlatformInterface {
 
   void SongSaveComplete();
 
+#ifdef HAVE_LIBLASTFM
   void ShowCoverManager();
+#endif
   void ShowAboutDialog();
   void ShowTranscodeDialog();
   void ShowErrorDialog(const QString& message);
@@ -197,6 +229,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void OpenSettingsDialog();
   void OpenSettingsDialogAtPage(SettingsDialog::Page page);
   void ShowSongInfoConfig();
+  void ShowScriptDialog();
 
   void SaveGeometry();
 
@@ -204,17 +237,21 @@ class MainWindow : public QMainWindow, public PlatformInterface {
 
   void DeleteFinished(const SongList& songs_with_errors);
 
+  void Raise();
+
+  void Exit();
+
  private:
-  void AddFilesToPlaylist(bool clear_first, const QList<QUrl>& urls);
-  void AddLibraryItemToPlaylist(bool clear_first, const QModelIndexList& indexes);
-  void AddLibrarySongsToPlaylist(bool clear_first, const SongList& songs);
-  void AddSmartPlaylistToPlaylist(bool clear_first, smart_playlists::GeneratorPtr gen);
-  void AddDeviceSongsToPlaylist(bool clear_first, const SongList& songs);
-  void AddUrls(bool play_now, const QList<QUrl>& urls);
   void ConnectInfoView(SongInfoBase* view);
 
+  void ApplyAddBehaviour(AddBehaviour b, MimeData* data) const;
+  void ApplyPlayBehaviour(PlayBehaviour b, MimeData* data) const;
+
+  void CheckFullRescanRevisions();
+
  private:
   Ui_MainWindow* ui_;
+  Windows7ThumbBar* thumbbar_;
 
   SystemTrayIcon* tray_icon_;
   OSD* osd_;
@@ -229,6 +266,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   Player* player_;
   Library* library_;
   GlobalShortcuts* global_shortcuts_;
+  Remote* remote_;
 
   DeviceManager* devices_;
 
@@ -241,21 +279,34 @@ class MainWindow : public QMainWindow, public PlatformInterface {
 
   boost::scoped_ptr<SettingsDialog> settings_dialog_;
   boost::scoped_ptr<AddStreamDialog> add_stream_dialog_;
+#ifdef HAVE_LIBLASTFM
   boost::scoped_ptr<AlbumCoverManager> cover_manager_;
+#endif
   boost::scoped_ptr<Equalizer> equalizer_;
   boost::scoped_ptr<TranscodeDialog> transcode_dialog_;
   boost::scoped_ptr<ErrorDialog> error_dialog_;
   boost::scoped_ptr<OrganiseDialog> organise_dialog_;
   boost::scoped_ptr<QueueManager> queue_manager_;
 
+  boost::scoped_ptr<TagFetcher> tag_fetcher_;
+  boost::scoped_ptr<TrackSelectionDialog> track_selection_dialog_;
+  PlaylistItemList autocomplete_tag_items_;
+
 #ifdef ENABLE_VISUALISATIONS
   boost::scoped_ptr<VisualisationContainer> visualisation_;
 #endif
 
-#ifdef ENABLE_WIIMOTEDEV
+#ifdef HAVE_WIIMOTEDEV
   boost::scoped_ptr<WiimotedevShortcuts> wiimotedev_shortcuts_;
 #endif
 
+  ScriptManager* scripts_;
+  boost::scoped_ptr<ScriptDialog> script_dialog_;
+
+  QAction* library_show_all_;
+  QAction* library_show_duplicates_;
+  QAction* library_show_untagged_;
+
   QMenu* playlist_menu_;
   QAction* playlist_play_pause_;
   QAction* playlist_stop_after_;
@@ -265,7 +316,9 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   QAction* playlist_move_to_library_;
   QAction* playlist_copy_to_device_;
   QAction* playlist_delete_;
+  QAction* playlist_open_in_browser_;
   QAction* playlist_queue_;
+  QAction* playlist_add_to_another_;
   QModelIndex playlist_menu_index_;
 
   QSortFilterProxyModel* library_sort_model_;
@@ -274,7 +327,9 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   QSettings settings_;
 
   bool was_maximized_;
-  bool autoclear_playlist_;
+  AddBehaviour doubleclick_addmode_;
+  PlayBehaviour doubleclick_playmode_;
+  PlayBehaviour menu_playmode_;
 
   BackgroundStreams* background_streams_;
 };
