@@ -17,25 +17,32 @@
 
 #include "config.h"
 #include "core/backgroundstreams.h"
+#include "core/networkproxyfactory.h"
 #include "iconloader.h"
 #include "mainwindow.h"
 #include "settingsdialog.h"
 #include "engines/enginebase.h"
+#include "engines/gstengine.h"
 #include "playlist/playlistview.h"
+#include "songinfo/songinfofetcher.h"
 #include "songinfo/songinfotextview.h"
 #include "widgets/osd.h"
 #include "widgets/osdpretty.h"
 
 #include "ui_settingsdialog.h"
 
-#ifdef ENABLE_WIIMOTEDEV
-#include "ui/wiimotedevshortcutsconfig.h"
-#include "ui_wiimotedevshortcutsconfig.h"
-#include "wiimotedev/shortcuts.h"
+#ifdef HAVE_LIBLASTFM
+# include "radio/lastfmconfig.h"
 #endif
 
-#ifdef HAVE_GSTREAMER
-# include "engines/gstengine.h"
+#ifdef HAVE_WIIMOTEDEV
+# include "ui/wiimotedevshortcutsconfig.h"
+# include "ui_wiimotedevshortcutsconfig.h"
+# include "wiimotedev/shortcuts.h"
+#endif
+
+#ifdef HAVE_REMOTE
+# include "remote/remoteconfig.h"
 #endif
 
 #include <QColorDialog>
@@ -95,6 +102,41 @@ SettingsDialog::SettingsDialog(BackgroundStreams* streams, QWidget* parent)
   pretty_popup_->SetMessage(tr("OSD Preview"), tr("Drag to reposition"),
                             QImage(":nocover.png"));
 
+#ifdef HAVE_LIBLASTFM
+  ui_->list->insertItem(Page_Lastfm, tr("Last.fm"));
+  ui_->list->item(Page_Lastfm)->setIcon(QIcon(":/last.fm/as.png"));
+
+  QWidget* lastfm_page = new QWidget;
+  QVBoxLayout* lastfm_layout = new QVBoxLayout;
+  lastfm_layout->setContentsMargins(0, 0, 0, 0);
+  lastfm_config_ = new LastFMConfig;
+  lastfm_layout->addWidget(lastfm_config_);
+  lastfm_page->setLayout(lastfm_layout);
+
+  ui_->stacked_widget->insertWidget(Page_Lastfm, lastfm_page);
+
+  connect(lastfm_config_, SIGNAL(ValidationComplete(bool)), SLOT(ValidationComplete(bool)));
+#endif
+
+#ifdef HAVE_REMOTE
+  ui_->list->insertItem(Page_Remote, tr("Remote Control"));
+  ui_->list->item(Page_Remote)->setIcon(IconLoader::Load("network-server"));
+
+  QWidget* remote_page = new QWidget;
+  QVBoxLayout* remote_layout = new QVBoxLayout;
+  remote_layout->setContentsMargins(0, 0, 0, 0);
+
+  remote_config_ = new RemoteConfig;
+  remote_layout->addWidget(remote_config_);
+  remote_page->setLayout(remote_layout);
+
+  ui_->stacked_widget->insertWidget(Page_Remote, remote_page);
+
+  connect(remote_config_, SIGNAL(ValidationComplete(bool)), SLOT(ValidationComplete(bool)));
+#endif
+
+  connect(ui_->magnatune, SIGNAL(ValidationComplete(bool)), SLOT(ValidationComplete(bool)));
+
   // Icons
   ui_->list->item(Page_Playback)->setIcon(IconLoader::Load("media-playback-start"));
   ui_->list->item(Page_SongInformation)->setIcon(IconLoader::Load("view-media-lyrics"));
@@ -102,12 +144,13 @@ SettingsDialog::SettingsDialog(BackgroundStreams* streams, QWidget* parent)
   ui_->list->item(Page_Notifications)->setIcon(IconLoader::Load("help-hint"));
   ui_->list->item(Page_Library)->setIcon(IconLoader::Load("folder-sound"));
   ui_->list->item(Page_BackgroundStreams)->setIcon(QIcon(":/icons/32x32/weather-showers-scattered.png"));
+  ui_->list->item(Page_Proxy)->setIcon(IconLoader::Load("applications-internet"));
 
   AddStreams();
 
-#ifdef ENABLE_WIIMOTEDEV
+#ifdef HAVE_WIIMOTEDEV
   // Wiimotedev page
-  ui_->list->addItem("Wiimotedev");
+  ui_->list->insertItem(Page_Wiimotedev, "Wiimotedev");
   ui_->list->item(Page_Wiimotedev)->setIcon(QIcon(":/icons/32x32/wiimotedev.png"));
 
   QWidget* wiimotedev_page = new QWidget(this);
@@ -127,9 +170,7 @@ SettingsDialog::SettingsDialog(BackgroundStreams* streams, QWidget* parent)
   connect(ui_->fading_cross, SIGNAL(toggled(bool)), SLOT(FadingOptionsChanged()));
   connect(ui_->fading_out, SIGNAL(toggled(bool)), SLOT(FadingOptionsChanged()));
   connect(ui_->fading_auto, SIGNAL(toggled(bool)), SLOT(FadingOptionsChanged()));
-#ifdef HAVE_GSTREAMER
   connect(ui_->gst_plugin, SIGNAL(currentIndexChanged(int)), SLOT(GstPluginChanged(int)));
-#endif
 
   connect(ui_->replaygain_preamp, SIGNAL(valueChanged(int)), SLOT(RgPreampChanged(int)));
   ui_->replaygain_preamp_label->setMinimumWidth(
@@ -139,6 +180,19 @@ SettingsDialog::SettingsDialog(BackgroundStreams* streams, QWidget* parent)
   // Behaviour
   connect(ui_->b_show_tray_icon_, SIGNAL(toggled(bool)), SLOT(ShowTrayIconToggled(bool)));
 
+  ui_->doubleclick_addmode->setItemData(0, MainWindow::AddBehaviour_Append);
+  ui_->doubleclick_addmode->setItemData(1, MainWindow::AddBehaviour_Load);
+  ui_->doubleclick_addmode->setItemData(2, MainWindow::AddBehaviour_OpenInNew);
+  ui_->doubleclick_addmode->setItemData(3, MainWindow::AddBehaviour_Enqueue);
+
+  ui_->doubleclick_playmode->setItemData(0, MainWindow::PlayBehaviour_Never);
+  ui_->doubleclick_playmode->setItemData(1, MainWindow::PlayBehaviour_IfStopped);
+  ui_->doubleclick_playmode->setItemData(2, MainWindow::PlayBehaviour_Always);
+
+  ui_->menu_playmode->setItemData(0, MainWindow::PlayBehaviour_Never);
+  ui_->menu_playmode->setItemData(1, MainWindow::PlayBehaviour_IfStopped);
+  ui_->menu_playmode->setItemData(2, MainWindow::PlayBehaviour_Always);
+
   // Populate the language combo box.  We do this by looking at all the
   // compiled in translations.
   QDir dir(":/translations/");
@@ -177,9 +231,6 @@ SettingsDialog::SettingsDialog(BackgroundStreams* streams, QWidget* parent)
   }
 #endif
 
-  // Last.fm
-  connect(ui_->lastfm, SIGNAL(ValidationComplete(bool)), SLOT(LastFMValidationComplete(bool)));
-
   // List box
   connect(ui_->list, SIGNAL(currentTextChanged(QString)), SLOT(CurrentTextChanged(QString)));
   ui_->list->setCurrentRow(Page_Playback);
@@ -211,6 +262,8 @@ SettingsDialog::SettingsDialog(BackgroundStreams* streams, QWidget* parent)
   ui_->b_show_tray_icon_->setEnabled(false);
   ui_->startup_group_->setEnabled(false);
 #endif
+
+  ui_->buttonBox->button(QDialogButtonBox::Cancel)->setShortcut(QKeySequence::Close);
 }
 
 SettingsDialog::~SettingsDialog() {
@@ -230,7 +283,7 @@ void SettingsDialog::SetGlobalShortcutManager(GlobalShortcuts *manager) {
   ui_->global_shortcuts->SetManager(manager);
 }
 
-void SettingsDialog::LastFMValidationComplete(bool success) {
+void SettingsDialog::ValidationComplete(bool success) {
   ui_->buttonBox->setEnabled(true);
 
   if (success)
@@ -238,26 +291,56 @@ void SettingsDialog::LastFMValidationComplete(bool success) {
 }
 
 void SettingsDialog::accept() {
-  if (ui_->lastfm->NeedsValidation()) {
-    ui_->lastfm->Validate();
+#ifdef HAVE_LIBLASTFM
+  if (lastfm_config_->NeedsValidation()) {
+    lastfm_config_->Validate();
     ui_->buttonBox->setEnabled(false);
     return;
   } else {
-    ui_->lastfm->Save();
+    lastfm_config_->Save();
+  }
+#endif
+
+#ifdef HAVE_REMOTE
+  if (remote_config_->NeedsValidation()) {
+    remote_config_->Validate();
+    ui_->buttonBox->setEnabled(false);
+    return;
+  } else {
+    remote_config_->Save();
+  }
+#endif
+
+  if (ui_->magnatune->NeedsValidation()) {
+    ui_->magnatune->Validate();
+    ui_->buttonBox->setEnabled(false);
+    return;
+  } else {
+    ui_->magnatune->Save();
   }
 
   QSettings s;
 
   // Behaviour
-  MainWindow::StartupBehaviour behaviour;
+  MainWindow::StartupBehaviour behaviour = MainWindow::Startup_Remember;
   if (ui_->b_always_hide_->isChecked()) behaviour = MainWindow::Startup_AlwaysHide;
   if (ui_->b_always_show_->isChecked()) behaviour = MainWindow::Startup_AlwaysShow;
   if (ui_->b_remember_->isChecked())    behaviour = MainWindow::Startup_Remember;
 
+  MainWindow::AddBehaviour doubleclick_addmode = MainWindow::AddBehaviour(
+    ui_->doubleclick_addmode->itemData(ui_->doubleclick_addmode->currentIndex()).toInt());
+  MainWindow::PlayBehaviour doubleclick_playmode = MainWindow::PlayBehaviour(
+    ui_->doubleclick_playmode->itemData(ui_->doubleclick_playmode->currentIndex()).toInt());
+  MainWindow::PlayBehaviour menu_playmode = MainWindow::PlayBehaviour(
+    ui_->menu_playmode->itemData(ui_->menu_playmode->currentIndex()).toInt());
+
   s.beginGroup(MainWindow::kSettingsGroup);
   s.setValue("showtray", ui_->b_show_tray_icon_->isChecked());
   s.setValue("keeprunning", ui_->b_keep_running_->isChecked());
   s.setValue("startupbehaviour", int(behaviour));
+  s.setValue("doubleclick_addmode", doubleclick_addmode);
+  s.setValue("doubleclick_playmode", doubleclick_playmode);
+  s.setValue("menu_playmode", menu_playmode);
   s.endGroup();
 
   s.beginGroup("General");
@@ -266,8 +349,9 @@ void SettingsDialog::accept() {
   s.endGroup();
 
   // Playback
-  s.beginGroup(PlaylistView::kSettingsGroup);
+  s.beginGroup(Playlist::kSettingsGroup);
   s.setValue("glow_effect", ui_->current_glow->isChecked());
+  s.setValue("greyoutdeleted", ui_->b_grey_out_deleted_->isChecked());
   s.endGroup();
 
   s.beginGroup(Engine::Base::kSettingsGroup);
@@ -275,9 +359,9 @@ void SettingsDialog::accept() {
   s.setValue("FadeoutDuration", ui_->fading_duration->value());
   s.setValue("CrossfadeEnabled", ui_->fading_cross->isChecked());
   s.setValue("AutoCrossfadeEnabled", ui_->fading_auto->isChecked());
+  s.setValue("NoCrossfadeSameAlbum", ui_->fading_samealbum->isChecked());
   s.endGroup();
 
-#ifdef HAVE_GSTREAMER
   s.beginGroup(GstEngine::kSettingsGroup);
   s.setValue("sink", ui_->gst_plugin->itemData(ui_->gst_plugin->currentIndex()).toString());
   s.setValue("device", ui_->gst_device->text());
@@ -287,17 +371,17 @@ void SettingsDialog::accept() {
   s.setValue("rgcompression", ui_->replaygain_compression->isChecked());
   s.setValue("bufferduration", ui_->buffer_duration->value());
   s.endGroup();
-#endif
 
   // Song info
   s.beginGroup(SongInfoTextView::kSettingsGroup);
   s.setValue("font_size", ui_->song_info_font_preview->font().pointSizeF());
+  s.setValue("timeout", ui_->song_info_timeout->value());
   s.endGroup();
 
   ui_->lyric_settings->Save();
 
   // Wii remotes
-#ifdef ENABLE_WIIMOTEDEV
+#ifdef HAVE_WIIMOTEDEV
   s.beginGroup(WiimotedevShortcuts::kActionsGroup);
   s.remove("");
   foreach (const WiimotedevShortcutsConfig::Shortcut& shortcut, wiimotedev_config_->actions_)
@@ -325,6 +409,7 @@ void SettingsDialog::accept() {
   s.setValue("Behaviour", int(osd_behaviour));
   s.setValue("Timeout", ui_->notifications_duration->value() * 1000);
   s.setValue("ShowOnVolumeChange", ui_->notifications_volume->isChecked());
+  s.setValue("ShowOnPlayModeChange", ui_->notifications_play_mode->isChecked());
   s.setValue("ShowArt", ui_->notifications_art->isChecked());
   s.endGroup();
 
@@ -336,6 +421,23 @@ void SettingsDialog::accept() {
   s.setValue("popup_pos", pretty_popup_->popup_pos());
   s.endGroup();
 
+  // Network proxy
+  NetworkProxyFactory::Mode mode = NetworkProxyFactory::Mode_System;
+  if      (ui_->proxy_direct->isChecked()) mode = NetworkProxyFactory::Mode_Direct;
+  else if (ui_->proxy_system->isChecked()) mode = NetworkProxyFactory::Mode_System;
+  else if (ui_->proxy_manual->isChecked()) mode = NetworkProxyFactory::Mode_Manual;
+
+  s.beginGroup(NetworkProxyFactory::kSettingsGroup);
+  s.setValue("mode", mode);
+  s.setValue("type", ui_->proxy_type->currentIndex() == 0 ?
+             QNetworkProxy::HttpProxy : QNetworkProxy::Socks5Proxy);
+  s.setValue("hostname", ui_->proxy_hostname->text());
+  s.setValue("port", ui_->proxy_port->value());
+  s.setValue("use_authentication", ui_->proxy_auth->isChecked());
+  s.setValue("username", ui_->proxy_username->text());
+  s.setValue("password", ui_->proxy_password->text());
+  s.endGroup();
+
   ui_->library_config->Save();
   ui_->magnatune->Save();
   ui_->global_shortcuts->Save();
@@ -343,13 +445,14 @@ void SettingsDialog::accept() {
   streams_->SaveStreams();
 
   QDialog::accept();
+
+  NetworkProxyFactory::Instance()->ReloadSettings();
 }
 
 void SettingsDialog::showEvent(QShowEvent*) {
   QSettings s;
   loading_settings_ = true;
 
-#ifdef HAVE_GSTREAMER
   if (ui_->gst_plugin->count() <= 1 && gst_engine_) {
     GstEngine::PluginDetailsList list = gst_engine_->GetOutputsList();
 
@@ -363,13 +466,18 @@ void SettingsDialog::showEvent(QShowEvent*) {
     ui_->gst_group->setEnabled(true);
     ui_->replaygain_group->setEnabled(true);
   }
-#endif // HAVE_GSTREAMER
 
   // Behaviour
   s.beginGroup(MainWindow::kSettingsGroup);
   ui_->b_show_tray_icon_->setChecked(s.value("showtray", true).toBool());
   ui_->b_keep_running_->setChecked(s.value("keeprunning",
       ui_->b_show_tray_icon_->isChecked()).toBool());
+  ui_->doubleclick_addmode->setCurrentIndex(ui_->doubleclick_addmode->findData(
+      s.value("doubleclick_addmode", MainWindow::AddBehaviour_Append).toInt()));
+  ui_->doubleclick_playmode->setCurrentIndex(ui_->doubleclick_playmode->findData(
+      s.value("doubleclick_playmode", MainWindow::PlayBehaviour_IfStopped).toInt()));
+  ui_->menu_playmode->setCurrentIndex(ui_->menu_playmode->findData(
+      s.value("menu_playmode", MainWindow::PlayBehaviour_IfStopped).toInt()));
 
   MainWindow::StartupBehaviour behaviour = MainWindow::StartupBehaviour(
       s.value("startupbehaviour", MainWindow::Startup_Remember).toInt());
@@ -392,12 +500,20 @@ void SettingsDialog::showEvent(QShowEvent*) {
   s.beginGroup(SongInfoTextView::kSettingsGroup);
   ui_->song_info_font_size->setValue(
       s.value("font_size", SongInfoTextView::kDefaultFontSize).toReal());
+  ui_->song_info_timeout->setValue(
+      s.value("timeout", SongInfoFetcher::kDefaultTimeoutDuration).toInt());
   s.endGroup();
 
   ui_->lyric_settings->Load();
 
   // Last.fm
-  ui_->lastfm->Load();
+#ifdef HAVE_LIBLASTFM
+  lastfm_config_->Load();
+#endif
+
+#ifdef HAVE_REMOTE
+  remote_config_->Load();
+#endif
 
   // Magnatune
   ui_->magnatune->Load();
@@ -406,8 +522,9 @@ void SettingsDialog::showEvent(QShowEvent*) {
   ui_->global_shortcuts->Load();
 
   // Playback
-  s.beginGroup(PlaylistView::kSettingsGroup);
+  s.beginGroup(Playlist::kSettingsGroup);
   ui_->current_glow->setChecked(s.value("glow_effect", true).toBool());
+  ui_->b_grey_out_deleted_->setChecked(s.value("greyoutdeleted", false).toBool());
   s.endGroup();
 
   s.beginGroup(Engine::Base::kSettingsGroup);
@@ -415,9 +532,9 @@ void SettingsDialog::showEvent(QShowEvent*) {
   ui_->fading_cross->setChecked(s.value("CrossfadeEnabled", true).toBool());
   ui_->fading_auto->setChecked(s.value("AutoCrossfadeEnabled", false).toBool());
   ui_->fading_duration->setValue(s.value("FadeoutDuration", 2000).toInt());
+  ui_->fading_samealbum->setChecked(s.value("NoCrossfadeSameAlbum", true).toBool());
   s.endGroup();
 
-#ifdef HAVE_GSTREAMER
   s.beginGroup(GstEngine::kSettingsGroup);
   QString sink = s.value("sink", GstEngine::kAutoSink).toString();
   ui_->gst_plugin->setCurrentIndex(0);
@@ -434,7 +551,6 @@ void SettingsDialog::showEvent(QShowEvent*) {
   ui_->replaygain_compression->setChecked(s.value("rgcompression", true).toBool());
   ui_->buffer_duration->setValue(s.value("bufferduration", 1000).toInt());
   s.endGroup();
-#endif
 
   // Notifications
   s.beginGroup(OSD::kSettingsGroup);
@@ -465,6 +581,7 @@ void SettingsDialog::showEvent(QShowEvent*) {
   }
   ui_->notifications_duration->setValue(s.value("Timeout", 5000).toInt() / 1000);
   ui_->notifications_volume->setChecked(s.value("ShowOnVolumeChange", false).toBool());
+  ui_->notifications_play_mode->setChecked(s.value("ShowOnPlayModeChange", true).toBool());
   ui_->notifications_art->setChecked(s.value("ShowArt", true).toBool());
   s.endGroup();
 
@@ -484,6 +601,34 @@ void SettingsDialog::showEvent(QShowEvent*) {
 
   ui_->library_config->Load();
 
+  // Network proxy
+  s.beginGroup(NetworkProxyFactory::kSettingsGroup);
+  NetworkProxyFactory::Mode mode = NetworkProxyFactory::Mode(
+      s.value("mode", NetworkProxyFactory::Mode_System).toInt());
+  switch (mode) {
+  case NetworkProxyFactory::Mode_Manual:
+    ui_->proxy_manual->setChecked(true);
+    break;
+
+  case NetworkProxyFactory::Mode_Direct:
+    ui_->proxy_direct->setChecked(true);
+    break;
+
+  case NetworkProxyFactory::Mode_System:
+  default:
+    ui_->proxy_system->setChecked(true);
+    break;
+  }
+
+  ui_->proxy_type->setCurrentIndex(s.value("type", QNetworkProxy::HttpProxy)
+      .toInt() == QNetworkProxy::HttpProxy ? 0 : 1);
+  ui_->proxy_hostname->setText(s.value("hostname").toString());
+  ui_->proxy_port->setValue(s.value("port").toInt());
+  ui_->proxy_auth->setChecked(s.value("use_authentication", false).toBool());
+  ui_->proxy_username->setText(s.value("username").toString());
+  ui_->proxy_password->setText(s.value("password").toString());
+  s.endGroup();
+
   loading_settings_ = false;
 }
 
@@ -554,14 +699,12 @@ void SettingsDialog::ShowTrayIconToggled(bool on) {
 }
 
 void SettingsDialog::GstPluginChanged(int index) {
-#ifdef HAVE_GSTREAMER
   QString name = ui_->gst_plugin->itemData(index).toString();
 
   bool enabled = GstEngine::DoesThisSinkSupportChangingTheOutputDeviceToAUserEditableString(name);
 
   ui_->gst_device->setEnabled(enabled);
   ui_->gst_device_label->setEnabled(enabled);
-#endif // HAVE_GSTREAMER
 }
 
 void SettingsDialog::RgPreampChanged(int value) {
@@ -586,7 +729,7 @@ void SettingsDialog::SetSongInfoView(SongInfoView* view) {
   ui_->lyric_settings->SetSongInfoView(view);
 }
 
-void SettingsDialog::SongInfoFontSizeChanged(qreal value) {
+void SettingsDialog::SongInfoFontSizeChanged(double value) {
   QFont font;
   font.setPointSizeF(value);
 
