@@ -35,9 +35,15 @@ class OSDPretty;
 class SongInfoView;
 class Ui_SettingsDialog;
 
-#ifdef ENABLE_WIIMOTEDEV
+#ifdef HAVE_LIBLASTFM
+  class LastFMConfig;
+#endif
+#ifdef HAVE_WIIMOTEDEV
   class WiimotedevShortcutsConfig;
 #endif
+#ifdef HAVE_REMOTE
+  class RemoteConfig;
+#endif
 
 class GstEngine;
 
@@ -55,10 +61,16 @@ class SettingsDialog : public QDialog {
     Page_GlobalShortcuts,
     Page_Notifications,
     Page_Library,
+#ifdef HAVE_LIBLASTFM
     Page_Lastfm,
+#endif
     Page_Magnatune,
     Page_BackgroundStreams,
-#ifdef ENABLE_WIIMOTEDEV
+    Page_Proxy,
+#ifdef HAVE_REMOTE
+    Page_Remote,
+#endif
+#ifdef HAVE_WIIMOTEDEV
     Page_Wiimotedev,
 #endif
   };
@@ -85,7 +97,7 @@ class SettingsDialog : public QDialog {
  private slots:
   void CurrentTextChanged(const QString& text);
   void NotificationTypeChanged();
-  void LastFMValidationComplete(bool success);
+  void ValidationComplete(bool success);
 
   void PrettyOpacityChanged(int value);
   void PrettyColorPresetChanged(int index);
@@ -105,8 +117,14 @@ class SettingsDialog : public QDialog {
   void StreamVolumeChanged(int value);
 
  private:
-#ifdef ENABLE_WIIMOTEDEV
+#ifdef HAVE_LIBLASTFM
+  LastFMConfig* lastfm_config_;
+#endif
+#ifdef HAVE_WIIMOTEDEV
   WiimotedevShortcutsConfig* wiimotedev_config_;
+#endif
+#ifdef HAVE_REMOTE
+  RemoteConfig* remote_config_;
 #endif
   const GstEngine* gst_engine_;
 
@@ -119,7 +137,7 @@ class SettingsDialog : public QDialog {
 
   BackgroundStreams* streams_;
 
-#ifdef ENABLE_WIIMOTEDEV
+#ifdef HAVE_WIIMOTEDEV
  signals:
   void SetWiimotedevInterfaceActived(bool);
 #endif
