@@ -24,15 +24,15 @@
 #include "ui/iconloader.h"
 #include "ui/screensaver.h"
 
-#include <QHBoxLayout>
-#include <QSettings>
 #include <QGLWidget>
-#include <QtDebug>
 #include <QGraphicsProxyWidget>
-#include <QMenu>
-#include <QSignalMapper>
-
+#include <QHBoxLayout>
 #include <QLabel>
+#include <QMenu>
+#include <QSettings>
+#include <QShortcut>
+#include <QSignalMapper>
+#include <QtDebug>
 
 const char* VisualisationContainer::kSettingsGroup = "Visualisations";
 const int VisualisationContainer::kDefaultWidth = 828;
@@ -59,6 +59,11 @@ VisualisationContainer::VisualisationContainer(QWidget *parent)
   }
   fps_ = s.value("fps", kDefaultFps).toInt();
   size_ = s.value("size", kDefaultTextureSize).toInt();
+
+  QShortcut* close = new QShortcut(QKeySequence::Close, this);
+  connect(close, SIGNAL(activated()), SLOT(close()));
+
+  setMinimumSize(64, 64);
 }
 
 void VisualisationContainer::Init() {
@@ -127,10 +132,8 @@ void VisualisationContainer::AddMenuItem(const QString &name, int value, int def
 void VisualisationContainer::SetEngine(GstEngine* engine) {
   engine_ = engine;
 
-#ifdef HAVE_GSTREAMER
   if (isVisible())
     engine_->AddBufferConsumer(vis_);
-#endif
 }
 
 void VisualisationContainer::showEvent(QShowEvent* e) {
@@ -142,20 +145,16 @@ void VisualisationContainer::showEvent(QShowEvent* e) {
   QGraphicsView::showEvent(e);
   update_timer_.start(1000 / fps_, this);
 
-#ifdef HAVE_GSTREAMER
   if (engine_)
     engine_->AddBufferConsumer(vis_);
-#endif
 }
 
 void VisualisationContainer::hideEvent(QHideEvent* e) {
   QGraphicsView::hideEvent(e);
   update_timer_.stop();
 
-#ifdef HAVE_GSTREAMER
   if (engine_)
     engine_->RemoveBufferConsumer(vis_);
-#endif
 }
 
 void VisualisationContainer::resizeEvent(QResizeEvent* e) {
