@@ -19,9 +19,11 @@
 #include "ui_fileview.h"
 #include "core/deletefiles.h"
 #include "core/filesystemmusicstorage.h"
+#include "core/mimedata.h"
 #include "ui/iconloader.h"
 #include "ui/organiseerrordialog.h"
 
+#include <QKeyEvent>
 #include <QFileSystemModel>
 #include <QMessageBox>
 #include <QScrollBar>
@@ -53,8 +55,7 @@ FileView::FileView(QWidget* parent)
 
   connect(ui_->list, SIGNAL(activated(QModelIndex)), SLOT(ItemActivated(QModelIndex)));
   connect(ui_->list, SIGNAL(doubleClicked(QModelIndex)), SLOT(ItemDoubleClick(QModelIndex)));
-  connect(ui_->list, SIGNAL(Load(QList<QUrl>)), SIGNAL(Load(QList<QUrl>)));
-  connect(ui_->list, SIGNAL(AddToPlaylist(QList<QUrl>)), SIGNAL(AddToPlaylist(QList<QUrl>)));
+  connect(ui_->list, SIGNAL(AddToPlaylist(QMimeData*)), SIGNAL(AddToPlaylist(QMimeData*)));
   connect(ui_->list, SIGNAL(CopyToLibrary(QList<QUrl>)), SIGNAL(CopyToLibrary(QList<QUrl>)));
   connect(ui_->list, SIGNAL(MoveToLibrary(QList<QUrl>)), SIGNAL(MoveToLibrary(QList<QUrl>)));
   connect(ui_->list, SIGNAL(CopyToDevice(QList<QUrl>)), SIGNAL(CopyToDevice(QList<QUrl>)));
@@ -131,7 +132,14 @@ void FileView::ItemDoubleClick(const QModelIndex& index) {
   if (model_->isDir(index))
     return;
 
-  emit DoubleClicked(QList<QUrl>() << QUrl::fromLocalFile(model_->filePath(index)));
+  QString file_path = model_->filePath(index);
+
+  MimeData* data = new MimeData;
+  data->from_doubleclick_ = true;
+  data->setUrls(QList<QUrl>() << QUrl::fromLocalFile(file_path));
+  data->name_for_new_playlist_ = file_path;
+
+  emit AddToPlaylist(data);
 }
 
 
@@ -198,3 +206,14 @@ void FileView::showEvent(QShowEvent* e) {
   if (!lazy_set_path_.isEmpty())
     ChangeFilePathWithoutUndo(lazy_set_path_);
 }
+
+void FileView::keyPressEvent(QKeyEvent* e) {
+  switch (e->key()) {
+    case Qt::Key_Back:
+    case Qt::Key_Backspace:
+      ui_->up->click();
+      break;
+  }
+
+  QWidget::keyPressEvent(e);
+}
