@@ -16,21 +16,24 @@
 */
 
 #include "fileviewlist.h"
+#include "core/mimedata.h"
 #include "ui/iconloader.h"
 
-#include <QMenu>
 #include <QContextMenuEvent>
 #include <QFileSystemModel>
+#include <QMenu>
 #include <QtDebug>
 
 FileViewList::FileViewList(QWidget* parent)
     : QListView(parent),
       menu_(new QMenu(this))
 {
-  menu_->addAction(IconLoader::Load("media-playback-start"), tr("Load"),
-                   this, SLOT(LoadSlot()));
-  menu_->addAction(IconLoader::Load("media-playback-start"), tr("Add to playlist"),
+  menu_->addAction(IconLoader::Load("media-playback-start"), tr("Append to current playlist"),
                    this, SLOT(AddToPlaylistSlot()));
+  menu_->addAction(IconLoader::Load("media-playback-start"), tr("Replace current playlist"),
+                   this, SLOT(LoadSlot()));
+  menu_->addAction(IconLoader::Load("document-new"), tr("Open in new playlist"),
+                   this, SLOT(OpenInNewPlaylistSlot()));
   menu_->addSeparator();
   menu_->addAction(IconLoader::Load("edit-copy"), tr("Copy to library..."),
                    this, SLOT(CopyToLibrarySlot()));
@@ -55,11 +58,27 @@ QList<QUrl> FileViewList::UrlListFromSelection() const {
   foreach (const QModelIndex& index, menu_selection_.indexes()) {
     if (index.column() == 0)
       urls << QUrl::fromLocalFile(
-          static_cast<QFileSystemModel*>(model())->filePath(index));
+          static_cast<QFileSystemModel*>(model())->fileInfo(index).canonicalFilePath());
   }
   return urls;
 }
 
+MimeData* FileViewList::MimeDataFromSelection() const {
+  MimeData* data = new MimeData;
+  data->setUrls(UrlListFromSelection());
+
+  QList<QString> filenames = FilenamesFromSelection();
+  // if just one folder selected - use it's path as the new playlist's name
+  if(filenames.size() == 1 && QFileInfo(filenames.first()).isDir()) {
+    data->name_for_new_playlist_ = filenames.first();
+  // otherwise, use the current root path
+  } else {
+    data->name_for_new_playlist_ = static_cast<QFileSystemModel*>(model())->rootPath();
+  }
+
+  return data;
+}
+
 QStringList FileViewList::FilenamesFromSelection() const {
   QStringList filenames;
   foreach (const QModelIndex& index, menu_selection_.indexes()) {
@@ -70,11 +89,19 @@ QStringList FileViewList::FilenamesFromSelection() const {
 }
 
 void FileViewList::LoadSlot() {
-  emit Load(UrlListFromSelection());
+  MimeData* data = MimeDataFromSelection();
+  data->clear_first_ = true;
+  emit AddToPlaylist(data);
 }
 
 void FileViewList::AddToPlaylistSlot() {
-  emit AddToPlaylist(UrlListFromSelection());
+  emit AddToPlaylist(MimeDataFromSelection());
+}
+
+void FileViewList::OpenInNewPlaylistSlot() {
+  MimeData* data = MimeDataFromSelection();
+  data->open_in_new_playlist_ = true;
+  emit AddToPlaylist(data);
 }
 
 void FileViewList::CopyToLibrarySlot() {
@@ -92,3 +119,18 @@ void FileViewList::CopyToDeviceSlot() {
 void FileViewList::DeleteSlot() {
   emit Delete(FilenamesFromSelection());
 }
+
+void FileViewList::mousePressEvent(QMouseEvent* e) {
+  QListView::mousePressEvent(e);
+
+  //enqueue to playlist with middleClick
+  if (e->button() == Qt::MidButton) {
+    //we need to update the menu selection
+    menu_selection_ = selectionModel()->selection();
+
+    MimeData* data = new MimeData;
+    data->setUrls(UrlListFromSelection());
+    data->enqueue_now_ = true;
+    emit AddToPlaylist(data);
+  }
+}
