@@ -21,15 +21,18 @@
 #include <QListView>
 #include <QUrl>
 
+class MimeData;
+
 class FileViewList : public QListView {
   Q_OBJECT
 
  public:
   FileViewList(QWidget* parent = 0);
 
+  void mousePressEvent(QMouseEvent* e);
+
  signals:
-  void Load(const QList<QUrl>& urls);
-  void AddToPlaylist(const QList<QUrl>& urls);
+  void AddToPlaylist(QMimeData* data);
   void CopyToLibrary(const QList<QUrl>& urls);
   void MoveToLibrary(const QList<QUrl>& urls);
   void CopyToDevice(const QList<QUrl>& urls);
@@ -41,13 +44,15 @@ class FileViewList : public QListView {
  private slots:
   void LoadSlot();
   void AddToPlaylistSlot();
+  void OpenInNewPlaylistSlot();
   void CopyToLibrarySlot();
   void MoveToLibrarySlot();
   void CopyToDeviceSlot();
   void DeleteSlot();
 
-  QList<QUrl> UrlListFromSelection() const;
   QStringList FilenamesFromSelection() const;
+  QList<QUrl> UrlListFromSelection() const;
+  MimeData* MimeDataFromSelection() const;
 
  private:
   QMenu* menu_;
