@@ -39,6 +39,8 @@ class SearchTargetWrapper {
 
   void SetHint(const QString& hint);
 
+  void SetEnabled(bool enabled);
+
  private:
   NSSearchField* search_;
   SearchTarget* target_;
@@ -88,7 +90,9 @@ void SearchTargetWrapper::SetHint(const QString& hint) {
   [t release];
 }
 
-
+void SearchTargetWrapper::SetEnabled(bool enabled) {
+  [search_ setEnabled:enabled];
+}
 
 MacLineEdit::MacLineEdit(QWidget* parent)
     : QMacCocoaViewContainer(0, parent),
@@ -121,10 +125,14 @@ QString MacLineEdit::text() const {
   return wrapper_->text();
 }
 
-void MacLineEdit::setText(const QString& text) {
+void MacLineEdit::set_text(const QString& text) {
   wrapper_->setText(text);
 }
 
-void MacLineEdit::SetHint(const QString& hint) {
+void MacLineEdit::set_enabled(bool enabled) {
+  wrapper_->SetEnabled(enabled);
+}
+
+void MacLineEdit::set_hint(const QString& hint) {
   wrapper_->SetHint(hint);
 }
