@@ -15,9 +15,13 @@
    along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "fullscreenhypnotoad.h"
 #include "nowplayingwidget.h"
 #include "core/albumcoverloader.h"
 #include "core/kittenloader.h"
+#include "library/librarybackend.h"
+#include "ui/albumcoverchoicecontroller.h"
+#include "ui/iconloader.h"
 
 #include <QMenu>
 #include <QMovie>
@@ -52,6 +56,7 @@ const int NowPlayingWidget::kTopBorder = 4;
 
 NowPlayingWidget::NowPlayingWidget(QWidget *parent)
   : QWidget(parent),
+    album_cover_choice_controller_(new AlbumCoverChoiceController(this)),
     cover_loader_(new BackgroundThreadImplementation<AlbumCoverLoader, AlbumCoverLoader>(this)),
     kitten_loader_(NULL),
     mode_(SmallSongDetails),
@@ -65,7 +70,7 @@ NowPlayingWidget::NowPlayingWidget(QWidget *parent)
     load_cover_id_(0),
     details_(new QTextDocument(this)),
     previous_track_opacity_(0.0),
-    hypnotoad_(NULL),
+    bask_in_his_glory_action_(NULL),
     aww_(false)
 {
   // Load settings
@@ -73,6 +78,9 @@ NowPlayingWidget::NowPlayingWidget(QWidget *parent)
   s.beginGroup(kSettingsGroup);
   mode_ = Mode(s.value("mode", SmallSongDetails).toInt());
 
+  // Accept drops for setting album art
+  setAcceptDrops(true);
+
   // Context menu
   QActionGroup* mode_group = new QActionGroup(this);
   QSignalMapper* mode_mapper = new QSignalMapper(this);
@@ -82,11 +90,34 @@ NowPlayingWidget::NowPlayingWidget(QWidget *parent)
 
   menu_->addActions(mode_group->actions());
   menu_->addSeparator();
+
+  QList<QAction*> actions = album_cover_choice_controller_->GetAllActions();
+
+  connect(album_cover_choice_controller_->cover_from_file_action(),
+          SIGNAL(triggered()), this, SLOT(LoadCoverFromFile()));
+  connect(album_cover_choice_controller_->cover_to_file_action(),
+          SIGNAL(triggered()), this, SLOT(SaveCoverToFile()));
+  connect(album_cover_choice_controller_->cover_from_url_action(),
+          SIGNAL(triggered()), this, SLOT(LoadCoverFromURL()));
+  connect(album_cover_choice_controller_->search_for_cover_action(),
+          SIGNAL(triggered()), this, SLOT(SearchForCover()));
+  connect(album_cover_choice_controller_->unset_cover_action(),
+          SIGNAL(triggered()), this, SLOT(UnsetCover()));
+  connect(album_cover_choice_controller_->show_cover_action(),
+          SIGNAL(triggered()), this, SLOT(ShowCover()));
+
+  menu_->addActions(actions);
+  menu_->addSeparator();
   above_statusbar_action_ = menu_->addAction(tr("Show above status bar"));
+
   above_statusbar_action_->setCheckable(true);
   connect(above_statusbar_action_, SIGNAL(toggled(bool)), SLOT(ShowAboveStatusBar(bool)));
   above_statusbar_action_->setChecked(s.value("above_status_bar", false).toBool());
 
+  bask_in_his_glory_action_ = menu_->addAction(tr("ALL GLORY TO THE HYPNOTOAD"));
+  bask_in_his_glory_action_->setVisible(false);
+  connect(bask_in_his_glory_action_, SIGNAL(triggered()), SLOT(Bask()));
+
   // Animations
   connect(show_hide_animation_, SIGNAL(frameChanged(int)), SLOT(SetHeight(int)));
   setMaximumHeight(0);
@@ -99,6 +130,9 @@ NowPlayingWidget::NowPlayingWidget(QWidget *parent)
   connect(cover_loader_, SIGNAL(Initialised()), SLOT(CoverLoaderInitialised()));
 }
 
+NowPlayingWidget::~NowPlayingWidget() {
+}
+
 void NowPlayingWidget::CreateModeAction(Mode mode, const QString &text, QActionGroup *group, QSignalMapper* mapper) {
   QAction* action = new QAction(text, group);
   action->setCheckable(true);
@@ -112,7 +146,7 @@ void NowPlayingWidget::CreateModeAction(Mode mode, const QString &text, QActionG
 void NowPlayingWidget::set_ideal_height(int height) {
   small_ideal_height_ = height;
   UpdateHeight(aww_
-      ?  kitten_loader_->Worker().get()
+      ? kitten_loader_->Worker().get()
       : cover_loader_->Worker().get());
 }
 
@@ -125,8 +159,8 @@ void NowPlayingWidget::CoverLoaderInitialised() {
       static_cast<BackgroundThread<AlbumCoverLoader>*>(sender());
   UpdateHeight(loader->Worker().get());
   loader->Worker()->SetPadOutputImage(true);
-  connect(loader->Worker().get(), SIGNAL(ImageLoaded(quint64,QImage)),
-          SLOT(AlbumArtLoaded(quint64,QImage)));
+  connect(loader->Worker().get(), SIGNAL(ImageLoaded(quint64,QImage,QImage)),
+          SLOT(AlbumArtLoaded(quint64,QImage,QImage)));
 }
 
 void NowPlayingWidget::UpdateHeight(AlbumCoverLoader* loader) {
@@ -174,7 +208,7 @@ void NowPlayingWidget::NowPlaying(const Song& metadata) {
 
   // Loads the cover too.
   UpdateHeight(aww_
-      ?  kitten_loader_->Worker().get()
+      ? kitten_loader_->Worker().get()
       : cover_loader_->Worker().get());
   UpdateDetailsText();
 
@@ -215,11 +249,12 @@ void NowPlayingWidget::UpdateDetailsText() {
   details_->setHtml(html);
 }
 
-void NowPlayingWidget::AlbumArtLoaded(quint64 id, const QImage& image) {
+void NowPlayingWidget::AlbumArtLoaded(quint64 id, const QImage& scaled, const QImage& original) {
   if (id != load_cover_id_)
     return;
 
-  cover_ = QPixmap::fromImage(image);
+  cover_ = QPixmap::fromImage(scaled);
+  original_ = original;
   update();
 
   // Were we waiting for this cover to load before we started fading?
@@ -316,7 +351,7 @@ void NowPlayingWidget::FadePreviousTrack(qreal value) {
 void NowPlayingWidget::SetMode(int mode) {
   mode_ = Mode(mode);
   UpdateHeight(aww_
-      ?  kitten_loader_->Worker().get()
+      ? kitten_loader_->Worker().get()
       : cover_loader_->Worker().get());
   UpdateDetailsText();
   update();
@@ -329,13 +364,38 @@ void NowPlayingWidget::SetMode(int mode) {
 void NowPlayingWidget::resizeEvent(QResizeEvent* e) {
   if (visible_ && mode_ == LargeSongDetails && e->oldSize().width() != e->size().width()) {
     UpdateHeight(aww_
-        ?  kitten_loader_->Worker().get()
+        ? kitten_loader_->Worker().get()
         : cover_loader_->Worker().get());
     UpdateDetailsText();
   }
 }
 
 void NowPlayingWidget::contextMenuEvent(QContextMenuEvent* e) {
+  // initial 'enabled' values depending on the kitty mode
+  album_cover_choice_controller_->cover_from_file_action()->setEnabled(!aww_);
+  album_cover_choice_controller_->cover_to_file_action()->setEnabled(aww_);
+  album_cover_choice_controller_->cover_from_url_action()->setEnabled(!aww_);
+  album_cover_choice_controller_->search_for_cover_action()->setEnabled(!aww_);
+  album_cover_choice_controller_->unset_cover_action()->setEnabled(!aww_);
+  album_cover_choice_controller_->show_cover_action()->setEnabled(!aww_);
+
+  // some special cases
+  if (!aww_) {
+  #ifndef HAVE_LIBLASTFM
+    album_cover_choice_controller_->cover_from_file_action()->setEnabled(false);
+    album_cover_choice_controller_->search_for_cover_action()->setEnabled(false);
+  #endif
+
+    const bool art_is_not_set = metadata_.has_manually_unset_cover()
+        || (metadata_.art_automatic().isEmpty() && metadata_.art_manual().isEmpty());
+
+    album_cover_choice_controller_->unset_cover_action()->setEnabled(!art_is_not_set);
+    album_cover_choice_controller_->show_cover_action()->setEnabled(!art_is_not_set);
+  }
+
+  bask_in_his_glory_action_->setVisible(hypnotoad_);
+
+  // show the menu
   menu_->popup(mapToGlobal(e->pos()));
 }
 
@@ -353,13 +413,12 @@ bool NowPlayingWidget::show_above_status_bar() const {
 
 void NowPlayingWidget::AllHail(bool hypnotoad) {
   if (hypnotoad) {
-    hypnotoad_ = new QMovie(kHypnotoadPath, QByteArray(), this);
-    connect(hypnotoad_, SIGNAL(updated(const QRect&)), SLOT(update()));
+    hypnotoad_.reset(new QMovie(kHypnotoadPath, QByteArray(), this));
+    connect(hypnotoad_.get(), SIGNAL(updated(const QRect&)), SLOT(update()));
     hypnotoad_->start();
     update();
   } else {
-    delete hypnotoad_;
-    hypnotoad_ = NULL;
+    hypnotoad_.reset();
     update();
   }
 }
@@ -375,3 +434,61 @@ void NowPlayingWidget::EnableKittens(bool aww) {
 
   aww_ = aww;
 }
+
+void NowPlayingWidget::LoadCoverFromFile() {
+  QString cover = album_cover_choice_controller_->LoadCoverFromFile(&metadata_);
+
+  if(!cover.isEmpty())
+    NowPlaying(metadata_);
+}
+
+void NowPlayingWidget::LoadCoverFromURL() {
+  QString cover = album_cover_choice_controller_->LoadCoverFromURL(&metadata_);
+
+  if(!cover.isEmpty())
+    NowPlaying(metadata_);
+}
+
+void NowPlayingWidget::SearchForCover() {
+  QString cover = album_cover_choice_controller_->SearchForCover(&metadata_);
+
+  if(!cover.isEmpty())
+    NowPlaying(metadata_);
+}
+
+void NowPlayingWidget::SaveCoverToFile() {
+  album_cover_choice_controller_->SaveCoverToFile(metadata_, original_);
+}
+
+void NowPlayingWidget::UnsetCover() {
+  album_cover_choice_controller_->UnsetCover(&metadata_);
+  NowPlaying(metadata_);
+}
+
+void NowPlayingWidget::ShowCover() {
+  album_cover_choice_controller_->ShowCover(metadata_);
+}
+
+void NowPlayingWidget::SetLibraryBackend(LibraryBackend* backend) {
+  album_cover_choice_controller_->SetLibrary(backend);
+}
+
+void NowPlayingWidget::Bask() {
+  big_hypnotoad_.reset(new FullscreenHypnotoad);
+  big_hypnotoad_->showFullScreen();
+}
+
+void NowPlayingWidget::dragEnterEvent(QDragEnterEvent* e) {
+  if (AlbumCoverChoiceController::CanAcceptDrag(e)) {
+    e->acceptProposedAction();
+  }
+
+  QWidget::dragEnterEvent(e);
+}
+
+void NowPlayingWidget::dropEvent(QDropEvent* e) {
+  album_cover_choice_controller_->SaveCover(&metadata_, e);
+  NowPlaying(metadata_);
+
+  QWidget::dropEvent(e);
+}
