@@ -23,8 +23,14 @@
 
 #include <QWidget>
 
-class AlbumCoverLoader;
+#include <boost/scoped_ptr.hpp>
 
+class AlbumCoverChoiceController;
+class AlbumCoverLoader;
+class FullscreenHypnotoad;
+class LibraryBackend;
+
+class QAction;
 class QActionGroup;
 class QMenu;
 class QMovie;
@@ -37,6 +43,7 @@ class NowPlayingWidget : public QWidget {
 
 public:
   NowPlayingWidget(QWidget* parent = 0);
+  ~NowPlayingWidget();
 
   static const char* kSettingsGroup;
   static const int kPadding;
@@ -52,8 +59,9 @@ public:
     LargeSongDetails = 1,
   };
 
-  void set_ideal_height(int height);
+  void SetLibraryBackend(LibraryBackend* backend);
 
+  void set_ideal_height(int height);
   bool show_above_status_bar() const;
 
   QSize sizeHint() const;
@@ -71,19 +79,30 @@ protected:
   void paintEvent(QPaintEvent* e);
   void resizeEvent(QResizeEvent*);
   void contextMenuEvent(QContextMenuEvent* e);
+  void dragEnterEvent(QDragEnterEvent* e);
+  void dropEvent(QDropEvent* e);
 
 private slots:
   void SetMode(int mode);
   void ShowAboveStatusBar(bool above);
 
   void CoverLoaderInitialised();
-  void AlbumArtLoaded(quint64 id, const QImage& image);
+  void AlbumArtLoaded(quint64 id, const QImage& scaled, const QImage& original);
 
   void SetVisible(bool visible);
   void SetHeight(int height);
 
   void FadePreviousTrack(qreal value);
 
+  void LoadCoverFromFile();
+  void SaveCoverToFile();
+  void LoadCoverFromURL();
+  void SearchForCover();
+  void UnsetCover();
+  void ShowCover();
+
+  void Bask();
+
 private:
   void CreateModeAction(Mode mode, const QString& text, QActionGroup* group,
                         QSignalMapper* mapper);
@@ -92,12 +111,15 @@ private:
   void DrawContents(QPainter* p);
 
 private:
+  AlbumCoverChoiceController* album_cover_choice_controller_;
+
   BackgroundThread<AlbumCoverLoader>* cover_loader_;
   BackgroundThread<AlbumCoverLoader>* kitten_loader_;
 
   Mode mode_;
 
   QMenu* menu_;
+
   QAction* above_statusbar_action_;
 
   bool visible_;
@@ -111,6 +133,8 @@ private:
   Song metadata_;
   quint64 load_cover_id_;
   QPixmap cover_;
+  // A copy of the original, unscaled album cover.
+  QImage original_;
   QTextDocument* details_;
 
   // Holds the last track while we're fading to the new track
@@ -118,7 +142,9 @@ private:
   qreal previous_track_opacity_;
 
   static const char* kHypnotoadPath;
-  QMovie* hypnotoad_;
+  QAction* bask_in_his_glory_action_;
+  boost::scoped_ptr<QMovie> hypnotoad_;
+  boost::scoped_ptr<FullscreenHypnotoad> big_hypnotoad_;
 
   bool aww_;
 };
