@@ -22,10 +22,12 @@
 #include <QImage>
 #include <QObject>
 
+#include "config.h"
 #include "engines/engine_fwd.h"
 #include "core/albumcoverloader.h"
 #include "core/backgroundthread.h"
 #include "core/song.h"
+#include "playlist/playlistsequence.h"
 
 class OrgFreedesktopNotificationsInterface;
 class OSDPretty;
@@ -33,7 +35,7 @@ class SystemTrayIcon;
 
 class QDBusPendingCallWatcher;
 
-#ifdef Q_WS_X11
+#ifdef HAVE_DBUS
 # include <QDBusArgument>
 # include <boost/scoped_ptr.hpp>
 
@@ -71,8 +73,10 @@ class OSD : public QObject {
   void PlaylistFinished();
   void VolumeChanged(int value);
   void MagnatuneDownloadFinished(const QStringList& albums);
+  void RepeatModeChanged(PlaylistSequence::RepeatMode mode);
+  void ShuffleModeChanged(PlaylistSequence::ShuffleMode mode);
 
-#ifdef ENABLE_WIIMOTEDEV
+#ifdef HAVE_WIIMOTEDEV
   void WiiremoteActived(int id);
   void WiiremoteDeactived(int id);
   void WiiremoteConnected(int id);
@@ -113,6 +117,7 @@ class OSD : public QObject {
   Behaviour behaviour_;
   bool show_on_volume_change_;
   bool show_art_;
+  bool show_on_play_mode_change_;
 
   bool force_show_next_;
   bool ignore_next_stopped_;
@@ -127,7 +132,7 @@ class OSD : public QObject {
   GrowlNotificationWrapper* wrapper_;
 #endif  // Q_OS_DARWIN
 
-#ifdef Q_WS_X11
+#ifdef HAVE_DBUS
   boost::scoped_ptr<OrgFreedesktopNotificationsInterface> interface_;
   uint notification_id_;
   QDateTime last_notification_time_;
