@@ -66,8 +66,31 @@ TEST_F(ASXParserTest, ParsesMoreThanOneTrackFromXML) {
   ASSERT_EQ(2, songs.length());
   EXPECT_EQ("http://example.com/foo.mp3", songs[0].filename());
   EXPECT_EQ("http://example.com/bar.mp3", songs[1].filename());
-  EXPECT_EQ(Song::Type_Stream, songs[0].filetype());
-  EXPECT_EQ(Song::Type_Stream, songs[1].filetype());
+  EXPECT_TRUE(songs[0].is_stream());
+  EXPECT_TRUE(songs[1].is_stream());
+}
+
+TEST_F(ASXParserTest, ParsesBrokenXmlEntities) {
+  QByteArray data =
+      "<asx version = \"3.0\">"
+      "<Title>DI.fm</Title>"
+      "<Author>Digitally Imported Radio</Author>"
+      "<Copyright>2006 Digitally Imported., Inc</Copyright>"
+      "<MoreInfo href=\"http://www.di.fm/\" />"
+      "  <entry>"
+      "    <PARAM name=\"HTMLView\" value=\"http://www.di.fm/classictrance/pro/mini_playlist.html\"/>"
+      "    <ref href = \"mms://72.26.204.105/classictrance128k?user=h&pass=xxxxxxxxxxxxxxx\"/>"
+      "    <Title>Classic Trance</Title>"
+      "    <Author>Digitally Imported Premium</Author>"
+      "  </entry>"
+      "</asx>";
+
+  QBuffer buffer(&data);
+  buffer.open(QIODevice::ReadOnly);
+  ASXParser parser(NULL);
+  SongList songs = parser.Load(&buffer);
+  ASSERT_EQ(1, songs.length());
+  EXPECT_EQ("mms://72.26.204.105/classictrance128k?user=h&pass=xxxxxxxxxxxxxxx", songs[0].filename());
 }
 
 TEST_F(ASXParserTest, SavesSong) {
@@ -79,7 +102,7 @@ TEST_F(ASXParserTest, SavesSong) {
   one.set_filename("http://www.example.com/foo.mp3");
   one.set_filetype(Song::Type_Stream);
   one.set_title("foo");
-  one.set_length(123);
+  one.set_length_nanosec(123 * kNsecPerSec);
   one.set_artist("bar");
   SongList songs;
   songs << one;
