@@ -47,7 +47,7 @@ TEST_F(M3UParserTest, ParsesMetadata) {
   ASSERT_TRUE(parser_.ParseMetadata(line, &metadata));
   EXPECT_EQ("Foo artist", metadata.artist.toStdString());
   EXPECT_EQ("Foo track", metadata.title.toStdString());
-  EXPECT_EQ(123, metadata.length);
+  EXPECT_EQ(123 * kNsecPerSec, metadata.length);
 }
 
 TEST_F(M3UParserTest, ParsesTrackLocation) {
@@ -57,7 +57,10 @@ TEST_F(M3UParserTest, ParsesTrackLocation) {
   Song song(&taglib_);
   QString line(temp.fileName());
   ASSERT_TRUE(parser_.ParseTrackLocation(line, QDir(), &song));
-  EXPECT_EQ(temp.fileName(), song.filename());
+  ASSERT_EQ(temp.fileName(), song.filename());
+
+  song.InitFromFile(song.filename(), -1);
+
   EXPECT_EQ("foo", song.title());
   EXPECT_EQ("bar", song.artist());
   EXPECT_EQ("baz", song.album());
@@ -72,7 +75,10 @@ TEST_F(M3UParserTest, ParsesTrackLocationRelative) {
   QString line(info.fileName());
   Song song(&taglib_);
   ASSERT_TRUE(parser.ParseTrackLocation(line, info.dir(), &song));
-  EXPECT_EQ(temp.fileName(), song.filename());
+  ASSERT_EQ(temp.fileName(), song.filename());
+
+  song.InitFromFile(song.filename(), -1);
+
   EXPECT_EQ("foo", song.title());
 }
 
@@ -95,7 +101,7 @@ TEST_F(M3UParserTest, ParsesSongsFromDevice) {
   Song s = songs[0];
   EXPECT_EQ("Some Artist", s.artist());
   EXPECT_EQ("Some Title", s.title());
-  EXPECT_EQ(123, s.length());
+  EXPECT_EQ(123 * kNsecPerSec, s.length_nanosec());
   EXPECT_PRED_FORMAT2(::testing::IsSubstring,
       "http://foo.com/bar/somefile.mp3", s.filename().toStdString());
 }
@@ -106,14 +112,14 @@ TEST_F(M3UParserTest, ParsesNonExtendedM3U) {
   QBuffer buffer(&data);
   buffer.open(QIODevice::ReadOnly);
   M3UParser parser(NULL);
-  SongList songs = parser.Load(&buffer, QDir("somedir"));
+  SongList songs = parser.Load(&buffer, "", QDir("somedir"));
   ASSERT_EQ(2, songs.size());
   EXPECT_PRED_FORMAT2(::testing::IsSubstring,
       "http://foo.com/bar/somefile.mp3", songs[0].filename().toStdString());
   EXPECT_PRED_FORMAT2(::testing::IsSubstring,
       "http://baz.com/thing.mp3", songs[1].filename().toStdString());
-  EXPECT_EQ(-1, songs[0].length());
-  EXPECT_EQ(-1, songs[1].length());
+  EXPECT_EQ(-1, songs[0].length_nanosec());
+  EXPECT_EQ(-1, songs[1].length_nanosec());
   EXPECT_TRUE(songs[0].artist().isEmpty());
 }
 
@@ -121,12 +127,12 @@ TEST_F(M3UParserTest, ParsesActualM3U) {
   QFile file(":testdata/test.m3u");
   file.open(QIODevice::ReadOnly);
   M3UParser parser(NULL);
-  SongList songs = parser.Load(&file, QDir("somedir"));
+  SongList songs = parser.Load(&file, "", QDir("somedir"));
   ASSERT_EQ(239, songs.size());
   EXPECT_EQ("gravity", songs[0].title());
-  EXPECT_EQ(203, songs[0].length());
+  EXPECT_EQ(203 * kNsecPerSec, songs[0].length_nanosec());
   EXPECT_EQ(QString::fromUtf8("ほっぴンちょっぴン"), songs.back().title());
-  EXPECT_EQ(85, songs.back().length());
+  EXPECT_EQ(85 * kNsecPerSec, songs.back().length_nanosec());
 }
 
 TEST_F(M3UParserTest, SavesSong) {
@@ -137,7 +143,7 @@ TEST_F(M3UParserTest, SavesSong) {
   one.set_filetype(Song::Type_Stream);
   one.set_title("foo");
   one.set_artist("bar");
-  one.set_length(123);
+  one.set_length_nanosec(123 * kNsecPerSec);
   one.set_filename("http://www.example.com/foo.mp3");
   SongList songs;
   songs << one;
