@@ -15,10 +15,13 @@
    along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "config.h"
 #include "core/encoding.h"
 #include "core/song.h"
+#ifdef HAVE_LIBLASTFM
 #include "radio/fixlastfm.h"
 #include <lastfm/Track>
+#endif
 
 #include "gmock/gmock.h"
 #include "gtest/gtest.h"
@@ -29,7 +32,7 @@
 #include <QTemporaryFile>
 #include <QTextCodec>
 
-#include <taglib/id3v2tag.h>
+#include <id3v2tag.h>
 
 namespace {
 
@@ -44,6 +47,7 @@ class SongTest : public ::testing::Test {
 };
 
 
+#ifdef HAVE_LIBLASTFM
 TEST_F(SongTest, InitsFromLastFM) {
   Song song;
   lastfm::MutableTrack track;
@@ -58,6 +62,7 @@ TEST_F(SongTest, InitsFromLastFM) {
   EXPECT_EQ("Baz", song.album());
   EXPECT_EQ("Bar", song.artist());
 }
+#endif // HAVE_LIBLASTFM
 
 TEST_F(SongTest, InitsFromFile) {
   QTemporaryFile temp;
