@@ -53,6 +53,9 @@ protected:
     library_.reset(new MockLibraryBackend);
     loader_.reset(new SongLoader(library_.get()));
     loader_->set_timeout(20000);
+
+    // the thing we return is not really important
+    EXPECT_CALL(*library_.get(), GetSongByFilename(_, _)).WillRepeatedly(Return(Song()));
   }
 
   void LoadLocalDirectory(const QString& dir);
@@ -108,7 +111,7 @@ TEST_F(SongLoaderTest, LoadLocalPls) {
   // Check the song got loaded
   ASSERT_EQ(1, loader_->songs().count());
   EXPECT_EQ("Title", loader_->songs()[0].title());
-  EXPECT_EQ(123, loader_->songs()[0].length());
+  EXPECT_EQ(123 * kNsecPerSec, loader_->songs()[0].length_nanosec());
 }
 
 TEST_F(SongLoaderTest, LoadLocalM3U) {
