@@ -50,7 +50,7 @@ TEST_F(XSPFParserTest, ParsesOneTrackFromXML) {
   EXPECT_EQ("Bar", song.artist());
   EXPECT_EQ("Baz", song.album());
   EXPECT_EQ("http://example.com/foo.mp3", song.filename());
-  EXPECT_EQ(60, song.length());
+  EXPECT_EQ(60 * kNsecPerSec, song.length_nanosec());
   EXPECT_TRUE(song.is_valid());
 }
 
@@ -71,8 +71,8 @@ TEST_F(XSPFParserTest, ParsesMoreThanOneTrackFromXML) {
   ASSERT_EQ(2, songs.length());
   EXPECT_EQ("http://example.com/foo.mp3", songs[0].filename());
   EXPECT_EQ("http://example.com/bar.mp3", songs[1].filename());
-  EXPECT_EQ(Song::Type_Stream, songs[0].filetype());
-  EXPECT_EQ(Song::Type_Stream, songs[1].filetype());
+  EXPECT_TRUE(songs[0].is_stream());
+  EXPECT_TRUE(songs[1].is_stream());
 }
 
 TEST_F(XSPFParserTest, IgnoresInvalidLength) {
@@ -91,7 +91,7 @@ TEST_F(XSPFParserTest, IgnoresInvalidLength) {
   XSPFParser parser(NULL);
   SongList songs = parser.Load(&buffer);
   ASSERT_EQ(1, songs.length());
-  EXPECT_EQ(-1, songs[0].length());
+  EXPECT_EQ(-1, songs[0].length_nanosec());
 }
 
 TEST_F(XSPFParserTest, SavesSong) {
@@ -103,7 +103,7 @@ TEST_F(XSPFParserTest, SavesSong) {
   one.set_filename("http://www.example.com/foo.mp3");
   one.set_filetype(Song::Type_Stream);
   one.set_title("foo");
-  one.set_length(123);
+  one.set_length_nanosec(123 * kNsecPerSec);
   one.set_artist("bar");
   SongList songs;
   songs << one;
@@ -124,7 +124,7 @@ TEST_F(XSPFParserTest, SavesLocalFile) {
   one.set_filename("/bar/foo.mp3");
   one.set_filetype(Song::Type_Mpeg);
   one.set_title("foo");
-  one.set_length(123);
+  one.set_length_nanosec(123 * kNsecPerSec);
   one.set_artist("bar");
   SongList songs;
   songs << one;
